﻿#ifndef AIFACEOBJECT_H
#define AIFACEOBJECT_H

#include <QString>
#include <QMap>
#include <QVector>
#include <QRect>

#include "src/inc/aifacepch.h"

#include "lib/sdk/VideoNetClient/include/VideoNetClient_Common.h"
#include <Windows.h>

using AIFaceErcode = enum AIFaceErcode
{
	AFErcode_None,

	AFErcode_UserAlreadyExist,
	AFErcode_UserNotExist,
	AFErcode_MACNotExist,
	AFErcode_ChannelNotExist,
	AFErcode_PortNotExist,
};

using DeviceType = enum DeviceType
{
	DeviceType_Unknown,

	DeviceType_DVR,
	DeviceType_NVR,
	DeviceType_IPC,
	DeviceType_IPD,
};

using DeviceInfo = struct DeviceInfo
{
	DeviceInfo()=default;
	DeviceInfo(DeviceType dt,QString dn,QString m,QString i,QString r,QString btime):
		deviceType(dt),
		deviceName(dn),
		mac(m),
		ip(i),
		romVersion(r),
		bootTime(btime)
	{}
	~DeviceInfo()=default;

	bool operator == (const DeviceInfo &deviceInfo)
	{
		if(deviceType==deviceInfo.deviceType&&deviceName==deviceInfo.deviceName&&mac==deviceInfo.mac&&ip==deviceInfo.ip&&romVersion==deviceInfo.romVersion)
			return true;
		else
			return false;
	}

	DeviceType deviceType{DeviceType_Unknown};
	QString deviceName;
	QString mac;
	QString ip;
	QString romVersion;
	QString bootTime;
};

using DeviceEventType = enum DeviceEventType
{
	DeviceEvent_AlarmNotice,						//报警事件
	DeviceEvent_DeviceOffline,						//设备离线（心跳丢失，网络断开）
	DeviceEvent_DeviceOnline,						//设备上线（网络重连成功）
	DeviceEvent_UserDisconnect,					//用户登出
	DeviceEvent_StreamDisconnect,				//远程流媒体断开
	DeviceEvent_DiskGroupManage,				//硬盘组管理事件
	DeviceEvent_HistoryStreamNotice,			//历史流事件通知
	DeviceEvent_RealStreamConnect,				//实时流连接
	DeviceEvent_RealStreamDisconnect,			//实时流断开
	DeviceEvent_VoiceStreamConnect,			//语音流连接
	DeviceEvent_VoiceStreamDisconnect,		//语音流断开
	DeviceEvent_HistoryStreamCreate,			//历史流创建
	DeviceEvent_HistoryStreamDestory,			//历史流销毁
	DeviceEvent_HistoryStreamConnect,		//历史流连接
	DeviceEvent_HistoryStreamDisconnect,	//历史流断开
};

using UserInfo = struct UserInfo
{
	UserInfo()=default;
	UserInfo(HUSER h,QString m,QString i):
		hUser(h),
		mac(m),
		ip(i)
	{}
	~UserInfo()=default;

	//设备登录用户句柄
	HUSER hUser{INVALID_HANDLE_VALUE};

	//设备MAC
	QString mac;
	//设备IP
	QString ip;

	//设备实时流句柄
	QMap<int,HSTREAM> channelRealStream;
	//实时流播放端口
	QMap<int,int> channelRealStreamPort;
	//在流上绘制矩形
	QMap<int,QVector<QRect>> channelRealStreamRects;
};

using FaceLibraryItem = struct FaceLibraryItem
{
	QString libraryID;
	QString libraryName;
	QString imageCount;
	QString createTime;
};
using FaceLibraryList = QVector<FaceLibraryItem>;

#ifdef UniqueID_Support
using FaceLibraryItemInfo = struct FaceLibraryItemInfo
{
	QString name;
	QString sex;
	QString certificateType;
	QString certificateID;
	QString uniqueID;
	QString libraryID;
	QString country;
	QString address;
	QString birthday;
	QString faceImageID;
	QString modelStatus;
};
#else
	using FaceLibraryItemInfo = struct FaceLibraryItemInfo
	{
		QString name;
		QString sex;
		QString certificateType;
		QString certificateID;
		QString personID;
		QString libraryID;
		QString country;
		QString address;
		QString birthday;
		QString faceImageID;
		QString modelStatus;
	};
#endif
using FaceLibraryItemInfoList = QVector<FaceLibraryItemInfo>;

using RecoRuleFaceLibraryNode = struct 	RecoRuleFaceLibraryNode
{
	QString id;
	QString name;
	QString enable;
};

using RecoRule = struct RecoRule
{
	QString isEnable;
	QString recognitionRule;
	QString compareLimit;
	QString identificationThreshold;
	QVector<RecoRuleFaceLibraryNode> faceLibraryNodes;
	QString alarm1Linkage;
	QString alarm2Linkage;
	QString recordPlan;
	QVector<QString> recordTimeBlocks;
};
using RecoRuleList = QVector<RecoRule>;

using RecognitionRecord = struct RecognitionRecord
{
	QString logID;
	QString personID;
	QString libraryName;
	QString time;
	QString isPass;
	QString snapshotPath;
	QString similar;
	QString ruleID;
	QString peopleName;
};
using RecognitionRecordList = QVector<RecognitionRecord>;

using RecognitionRecordDetail = struct RecognitionRecordDetail
{
	QString name;
	QString sex;
	QString certificateType;
	QString certificateID;
	QString personID;
	QString country;
	QString address;
	QString birthday;
	QString recoResult;
};

#endif // AIFACEOBJECT_H
