﻿#ifndef DEVICEINFO_H
#define DEVICEINFO_H

#include "src/inc/aifaceobject.h"
#include "lib/sdk/VideoNetClient/include/VideoNetClient_Common.h"

#include <QString>
#include <QMap>

class DeviceAnalysiser
{
public:
	static DeviceAnalysiser *GetInstance()
	{
		static DeviceAnalysiser deviceAnalysiser;
		return &deviceAnalysiser;
	}

protected:
	DeviceAnalysiser();

public:
	/**
	 * @brief getDeviceType		解析设备类型
	 * @param config					[in]设备探测函数回调结构体指针
	 * @return
	 */
	DeviceType getDeviceType(LPDeviceProbeConfigV2 config);

	/**
	 * @brief getDeviceName		解析设备名称
	 * @param config					[in]设备探测函数回调结构体指针
	 * @return
	 */
	QString getDeviceName(LPDeviceProbeConfigV2 config);

	/**
	 * @brief getIPAddress			解析IP地址
	 * @param config					[in]设备探测函数回调结构体指针
	 * @return
	 */
	QString getIPAddress(LPDeviceProbeConfigV2 config);

	/**
	 * @brief getMACAddress		解析MAC地址
	 * @param config					[in]设备探测函数回调结构体指针
	 * @return
	 */
	QString getMACAddress(LPDeviceProbeConfigV2 config);

	/**
	 * @brief getROMVersion		解析固件版本
	 * @param config					[in]设备探测函数回调结构体指针
	 * @return
	 */
	QString getROMVersion(LPDeviceProbeConfigV2 config);

	/**
	 * @brief getBootTime			解析设备启动时间
	 * @param config					[in]设备探测函数回调结构体指针
	 * @return
	 */
	QString getBootTime(LPDeviceProbeConfigV2 config);

private:
	/**
	 * @brief readDeviceNameDatabase		//读取设备型号与名称对照表
	 */
	void readDeviceNameDatabase();

private:
	QMap<int,QString> m_ipcDeviceName;
	QMap<int,QString> m_ipdDeviceName;
	QMap<int,QString> m_nvrDeviceName;
	QMap<int,QString> m_dvrDeviceName;
};

#endif // DEVICEINFO_H
