﻿#ifndef DEVICEMANAGER_H
#define DEVICEMANAGER_H

#include <QVector>
#include <QMap>

#include <mutex>

#include "src/inc/aifaceobject.h"

class DeviceManager
{
public:
	static DeviceManager *GetInstance()
	{
		static DeviceManager deviceManager;
		return &deviceManager;
	}

protected:
	DeviceManager();

public:
	/**
	 * @brief addDevice			添加设备
	 * @param deviceInfo		[in]设备信息
	 */
	void addDevice(const DeviceInfo &deviceInfo);

	/**
	 * @brief removeDevice		删除设备
	 * @param mac					[in]设备MAC地址
	 */
	void removeDevice(const QString &mac);

	/**
	 * @brief getDevices			得到探测到的设备
	 * @brief devices				[out]设备列表
	 * @return
	 */
	void getDevices(QVector<DeviceInfo> &devices);

	/**
	  * @brief getIPCDevices		得到探测到的IPC设备
	  * @brief ipcDevices			[out]IPC设备列表
	  * @return
	  */
	 void getIPCDevices(QVector<DeviceInfo> &ipcDevices);

	 /**
	  * @brief getIPDDevices		得到探测到的IPD设备
	  * @brief ipdDevices			[out]IPD设备列表
	  * @return
	  */
	 void getIPDDevices(QVector<DeviceInfo> &ipdDevices);

	 /**
	  * @brief getNVRDevices		得到探测到的NVR设备
	  * @brief nvrDevices			[out]NVR设备列表
	  * @return
	  */
	 void getNVRDevices(QVector<DeviceInfo> &nvrDevices);

	 /**
	  * @brief getDVRDevices		得到探测到的DVR设备
	  * @brief dvrDevices			[out]DVR设备列表
	  * @return
	  */
	 void getDVRDevices(QVector<DeviceInfo> &dvrDevices);

	 /**
	  * @brief getIPbyMAC		根据MAC得到IP地址
	  * @param mac				[in]MAC地址
	  * @return
	  */
	 QString getIPbyMAC(const QString &mac);

	 /**
	  * @brief clearDevices		清除探测到的设备
	  */
	 void clearDevices();

public:
	 template<typename... Args>
	 void addDevice(Args... args)
	 {
		if(auto l=m_devices.lock())
		{
			DeviceInfo deviceInfo(std::forward<Args>(args)...);

			if(false==m_devices.devices.contains(deviceInfo.mac))
				m_devices.devices.insert(deviceInfo.mac,deviceInfo);
		}
	 }

private:
	struct
	{
		QMap<QString,DeviceInfo> devices;
		std::unique_lock<std::mutex> lock()
		{
			return std::unique_lock<std::mutex>(m);
		}
	private:
		std::mutex m;
	}m_devices;

private:
	/**
	 * @brief getDevices			//获取某一类型的设备
	 * @param deviceType		[in]设备类型
	 * @param devices			[out]设备列表
	 */
	void getDevices(const DeviceType &deviceType,QVector<DeviceInfo> &devices);
};

#endif // DEVICEMANAGER_H
