﻿#ifndef SNAPCONFIGWIDGET_H
#define SNAPCONFIGWIDGET_H

#include "src/inc/aifacepch.h"

#include <QGroupBox>
#include <QLabel>
#include <QComboBox>
#include <QLineEdit>
#include <QFont>

using SnapMode = enum SnapMode
{
	SnapMode_Unknown=-1,

	SnapMode_Auto,
	SnapMode_Optimal,
	SnapMode_Auick,
	SnapMode_Custom,
};

class SnapConfigWidget : public QGroupBox
{
	Q_OBJECT
public:
	explicit SnapConfigWidget(QWidget *parent = nullptr);

	void setUserInfo(const QString &ip,const QString &username,const QString &password);

	/**
	 * @brief setSnapMode		设置抓拍模式
	 * @param snapMode			[in]抓拍模式
	 */
	void setSnapMode(const SnapMode &snapMode);

private:
	struct SnapModeGui
	{
		SnapModeGui()=default;
		virtual ~SnapModeGui()=default;

		virtual void show()=0;
		virtual void hide()=0;
	};
	struct SnapModeAutoGui : public SnapModeGui
	{
		void show() override
		{
			numberShots->show();
			numberShotsBox->show();
			snapIntervals->show();
			snapIntervalsBox->show();
			mapConfig->show();
			mapConfigBox->show();
		}
		void hide() override
		{
			numberShots->hide();
			numberShotsBox->hide();
			snapIntervals->hide();
			snapIntervalsBox->hide();
			mapConfig->hide();
			mapConfigBox->hide();
		}

		QLabel *numberShots{nullptr};
		QComboBox *numberShotsBox{nullptr};
		QLabel *snapIntervals{nullptr};
		QComboBox *snapIntervalsBox{nullptr};
		QLabel *mapConfig{nullptr};
		QComboBox *mapConfigBox{nullptr};
	};
	struct SnapModeOptimalGui:public SnapModeGui
	{
		void show() override
		{
			numberShots->show();
			numberShotsBox->show();
			mapConfig->show();
			mapConfigBox->show();
		}
		void hide() override
		{
			numberShots->hide();
			numberShotsBox->hide();
			mapConfig->hide();
			mapConfigBox->hide();
		}

		QLabel *numberShots{nullptr};
		QComboBox *numberShotsBox{nullptr};
		QLabel *mapConfig{nullptr};
		QComboBox *mapConfigBox{nullptr};
	};
	struct SnapModeAuickGui:public SnapModeGui
	{
		void show() override
		{
			numberShots->show();
			numberShotsBox->show();
			mapConfig->show();
			mapConfigBox->show();
		}
		void hide() override
		{
			numberShots->hide();
			numberShotsBox->hide();
			mapConfig->hide();
			mapConfigBox->hide();
		}

		QLabel *numberShots{nullptr};
		QComboBox *numberShotsBox{nullptr};
		QLabel *mapConfig{nullptr};
		QComboBox *mapConfigBox{nullptr};
	};
	struct SnapModeCustomGui:public SnapModeGui
	{
		void show() override
		{
			numberShots->show();
			numberShotsBox->show();
			mapConfig->show();
			qualityThreshold->show();
			qualityThresholdEdit->show();
			obliqueAngle->show();
			obliqueAngleEdit->show();
			pitchAngle->show();
			pitchAngleEdit->show();
			sideAngle->show();
			sideAngleEdit->show();
			fuzzyDegree->show();
			fuzzyDegreeEdit->show();
			mapConfigBox->show();
			mapConfigBox->show();
		}
		void hide() override
		{
			numberShots->hide();
			numberShotsBox->hide();
			mapConfig->hide();
			qualityThreshold->hide();
			qualityThresholdEdit->hide();
			obliqueAngle->hide();
			obliqueAngleEdit->hide();
			pitchAngle->hide();
			pitchAngleEdit->hide();
			sideAngle->hide();
			sideAngleEdit->hide();
			fuzzyDegree->hide();
			fuzzyDegreeEdit->hide();
			mapConfigBox->hide();
			mapConfigBox->hide();
		}

		QLabel *numberShots{nullptr};
		QComboBox *numberShotsBox{nullptr};
		QLabel *qualityThreshold{nullptr};
		QLineEdit *qualityThresholdEdit{nullptr};
		QLabel *obliqueAngle{nullptr};
		QLineEdit *obliqueAngleEdit{nullptr};
		QLabel *pitchAngle{nullptr};
		QLineEdit *pitchAngleEdit{nullptr};
		QLabel *sideAngle{nullptr};
		QLineEdit *sideAngleEdit{nullptr};
		QLabel *fuzzyDegree{nullptr};
		QLineEdit *fuzzyDegreeEdit{nullptr};
		QLabel *mapConfig{nullptr};
		QComboBox *mapConfigBox{nullptr};
	};

	QMap<SnapMode,SnapModeGui *> m_snapModeGuis;
	QFont m_font{"Microsoft YaHei",static_cast<int>(adaptx(10))};
	QComboBox *m_snapBox{nullptr};

	QString m_ip;
	QString m_username;
	QString m_password;

	SnapMode m_currentSnapMode{SnapMode_Unknown};

	//CGI字符串与网页字符串对应表
	QMap<QString,QString> m_mapConfigTable{{"face_only","face map"},{"half_body","half body"},{"whole_body","full body"}};

private:
	void initGui();

	void initSnapModeAutoGui();
	void initSnapModeOptimalGui();
	void initSnapModeAuickGui();
	void initSnapModeCustomGui();

signals:

public slots:
	void slot_save();
};

#endif // SNAPCONFIGWIDGET_H
