﻿#ifndef FACELIBRARYITEMSHOWWIDGET_H
#define FACELIBRARYITEMSHOWWIDGET_H

#include "src/inc/aifaceobject.h"

#include <QWidget>

QT_BEGIN_NAMESPACE
class QTableWidget;
class QLabel;
class QComboBox;
class QPushButton;
QT_END_NAMESPACE

class FaceLibraryItemShowWidget : public QWidget
{
	Q_OBJECT
public:
	explicit FaceLibraryItemShowWidget(QWidget *parent = nullptr);

	/**
	 * @brief init					初始化
	 * @param mac			[in]设备mac地址
	 * @param ip				[in]设备IP地址
	 * @param username	[in]用户名
	 * @param password	[in]密码
	 */
	void init(const QString &mac,const QString &ip,const QString &username,const QString &password);

	/**
	 * @brief setSourceLibraryID		设置源脸库ID
	 * @param faceLibraryID				[in]脸库ID
	 */
	void setSourceLibraryID(const QString &faceLibraryID,const QString &faceLibraryName);

	/**
	 * @brief setTargetLibraryID			设置目标脸库ID
	 * @param faceLibraryID				[in]脸库ID
	 */
	void setTargetLibraryID(const QString &faceLibraryID);

private:
	QLabel *m_libraryName{nullptr};
	QTableWidget *m_tableWidget{nullptr};

	QString m_sourceFaceLibraryID;
	QString m_targetFaceLibraryID;

	QString m_ip;
	QString m_username;
	QString m_password;

	FaceLibraryItemInfoList m_infos;

	QComboBox *m_pageSize{nullptr};
	QComboBox *m_page{nullptr};
	QLabel *m_sumPageNum{nullptr};

private:
	/**
	 * @brief initGui		创建表格
	 */
	void initGui();

	/**
	 * @brief updatePageInfo		更新页信息
	 * 以下为搜索字段
	 * @param searchFlag										[in]当前是否处于搜索状态，如果否，则以下值都无效
	 * @param name												[in]名字
	 * @param sex													[in]性别
	 * @param certificateType								[in]证件类型
	 * @param certificateID									[in]证件ID
	 * @param country											[in]国家
	 * @param address											[in]地址
	 * @param startBirthday									[in]生日起始时间
	 * @param stopBirthday									[in]生日停止时间
	 */
	void updatePageInfo(const bool &searchFlag=false, const QString &Name="",const QString &sex="unknown",const QString &certificateType="other",const QString &certificateID="",const QString &country="",const QString address="",const QString &startBirthday="00000000",const QString &stopBirthday="00000000");

	/**
	 * @brief updateItems		更新表格数据
	 * 以下为搜索字段
	 * @param searchFlag										[in]当前是否处于搜索状态，如果否，则以下值都无效
	 * @param name												[in]名字
	 * @param sex													[in]性别
	 * @param certificateType								[in]证件类型
	 * @param certificateID									[in]证件ID
	 * @param country											[in]国家
	 * @param address											[in]地址
	 * @param startBirthday									[in]生日起始时间
	 * @param stopBirthday									[in]生日停止时间
	 */
	void updateItems(const int &pageSize,const int &pageIndex,
					 const bool &searchFlag=false, const QString &Name="",const QString &sex="unknown",const QString &certificateType="other",const QString &certificateID="",const QString &country="",const QString address="",const QString &startBirthday="00000000",const QString &stopBirthday="00000000");

signals:
	void signal_updateFaceLibraryItem();
	void signal_updateFaceLibrary();
};

#endif // FACELIBRARYITEMSHOWWIDGET_H
