﻿#ifndef RECOGNIZERECORDWIDGET_H
#define RECOGNIZERECORDWIDGET_H

#include <QWidget>

QT_BEGIN_NAMESPACE
class QTableWidget;
class QLabel;
class QComboBox;
QT_END_NAMESPACE

class RecognizeRecordWidget : public QWidget
{
	Q_OBJECT
public:
	explicit RecognizeRecordWidget(QWidget *parent = nullptr);

	/**
	 * @brief init					初始化
	 * @param mac			[in]设备mac地址
	 * @param ip				[in]设备IP地址
	 * @param username	[in]用户名
	 * @param password	[in]密码
	 */
	void initUserInfo(const QString &mac,const QString &ip,const QString &username,const QString &password);

private:
	/**
	 * @brief initGui		初始化界面
	 */
	void initGui();

	/**
	 * @brief updatePageInfo		更新页信息
	 * 以下为搜索字段
	 * @param searchFlag							[in]是否为搜索模式
	 * @param libraryID								[in]脸库ID
	 * @param peopleName						[in]人名
	 * @param startTime								[in]起始时间
	 * @param endTime								[in]结束时间
	 * @param recognitionResultType			[in]识别结果
	 * @param ruleID									[in]规则ID
	 */
	void updatePageInfo(const bool &searchFlag=false, const QString &libraryID="-1",const QString &peopleName="",const QString &startTime="00000000",const QString &endTime="00000000",const QString &recognitionResultType="all",const QString &ruleID="-1");

	/**
	 * @brief updateRecord						更新记录
	 * @param pageSize								[in]每页记录数量
	 * @param pageIndex							[in]页码
	 * 以下为搜索字段
	 * @param searchFlag							[in]是否为搜索模式
	 * @param libraryID								[in]脸库ID
	 * @param peopleName						[in]人名
	 * @param startTime								[in]起始时间
	 * @param endTime								[in]结束时间
	 * @param recognitionResultType			[in]识别结果
	 * @param ruleID									[in]规则ID
	 */
	void updateRecord(const int &pageSize,const int &pageIndex,const bool &searchFlag=false, const QString &libraryID="-1",const QString &peopleName="",const QString &startTime="000000000000",const QString &endTime="000000000000",const QString &recognitionResultType="all",const QString &ruleID="-1");

private:
	QString m_ip;
	QString m_username;
	QString m_password;

	QTableWidget *m_tableWidget{nullptr};

	QComboBox *m_pageSize{nullptr};
	QComboBox *m_page{nullptr};
	QLabel *m_sumPageNum{nullptr};

	using SearchContent = struct SearchContent
	{
		void clear()
		{
			searchFlag = false;
			libraryID.clear();
			peopleName.clear();
			startTime.clear();
			endTime.clear();
			recognizeType.clear();
			ruleID.clear();
		}

		bool searchFlag{false};

		QString libraryID;
		QString peopleName;
		QString startTime;
		QString endTime;
		QString recognizeType;
		QString ruleID;
	};
	SearchContent m_searchContent;

signals:

public slots:
	void slot_search();
	void slot_refresh();
};

#endif // RECOGNIZERECORDWIDGET_H
