﻿#ifndef SNAPSHOTWIDGET_H
#define SNAPSHOTWIDGET_H

#include <QWidget>

QT_BEGIN_NAMESPACE
class QScrollArea;
class QListWidget;
QT_END_NAMESPACE

class SnapshotWidget : public QWidget
{
	Q_OBJECT
public:
	static SnapshotWidget *GetInstance(QWidget *parent = nullptr)
	{
		if(nullptr == m_snapshotWidget)
			m_snapshotWidget = new SnapshotWidget(parent);
		return m_snapshotWidget;
	}

protected:
	SnapshotWidget(QWidget *parent = nullptr);

private:
	static SnapshotWidget *m_snapshotWidget;

public:
	/**
	 * @brief init					初始化
	 * @param mac			[in]设备mac地址
	 * @param ip				[in]设备IP地址
	 * @param username	[in]用户名
	 * @param password	[in]密码
	 */
	void init(const QString &mac,const QString &ip,const QString &username,const QString &password);

	/**
	 * @brief addBasicPeopleDetectEvent		添加基础人形检测事件
	 * @param eventInfo								[in]事件信息
	 */
	void addBasicPeopleDetectEvent(const QString &eventInfo);

	/**
	 * @brief addBasicFaceDetectEvent			添加基础人脸检测事件
	 * @param eventInfo								[in]人脸事件
	 */
	void addBasicFaceDetectEvent(const QString &eventInfo);

	/**
	 * @brief addAdvancedFaceDetectEvent		添加高级人脸探测事件
	 * @param time											[in]时间
	 * @param imageData									[in]图片
	 */
	void addAdvancedFaceDetectEvent(const QString &time,const QString &faceTemperature,const QString &trackID,const QByteArray &imageData);

	/**
	 * @brief addAdvancedFaceRecognizeEvent		添加高级人脸识别时间
	 * @param time												[in]时间
	 * @param name												[in]姓名
	 * @param sex													[in]性别
	 * @param faceTemperature								[in]体温
	 * @param imageData										[in]图片数据
	 */
	void addAdvancedFaceRecognizeEvent(const QString &time,const QString &name,const QString &sex,const QString &faceTemperature,const QByteArray &imageData);

	/**
	 * @brief addImage				添加图片
	 * @param imageData			[in]图片数据
	 */
	void addImage(const QByteArray &imageData);

	/**
	 * @brief showGui		显示界面
	 */
	void showGui();

	/**
	 * @brief hideGui			隐藏界面
	 */
	void hideGui();

	/**
	 * @brief initGui		初始化界面
	 */
	void initGui();

private:
	QListWidget *m_basicEventWidget{nullptr};
	QScrollArea *m_advancedFaceDetectEventArea{nullptr};
	QWidget *m_advancedFaceDetectEventWidget{nullptr};
	QScrollArea *m_advancedFaceRecognizeEventArea{nullptr};
	QWidget *m_advancedFaceRecognizeEventWidget{nullptr};

	int m_detectImageCount{0};
	int m_recognizeImageCount{0};
	int m_imageSize{0};

	QString m_ip;
	QString m_username;
	QString m_password;

signals:
	void signal_addBasicPeopleDetectEvent(QString);
	void signal_addBasicFaceDetectEvent(QString);
	void signal_addAdvancedFaceDetectEvent(QString,QString,QString,QByteArray);
	void signal_addAdvancedFaceRecognizeImage(QString,QString,QString,QString,QByteArray);

private slots:
	void slot_addBasicPeopleDetectEvent(QString);
	void slot_addBasicFaceDetectEvent(QString);
	void slot_addAdvancedFaceDetectEvent(QString,QString,QString,QByteArray);
	void slot_addAdvancedFaceRecognizeImage(QString,QString,QString,QString,QByteArray imageData);
};

#endif // SNAPSHOTWIDGET_H
