﻿#ifndef AIFACEPACKAGE_H
#define AIFACEPACKAGE_H

#include "src/inc/httpmanager/httpmanagerpch.h"

#include <QObject>
#include <QString>

class AIFacePackage : public QObject
{
	Q_OBJECT
public:
	static AIFacePackage *GetInstance()
	{
		static AIFacePackage aiFacePackage;
		return &aiFacePackage;
	}

protected:
	AIFacePackage();

public:
	/**
	 * @brief isSupportAIFace				判断设备是否支持人脸功能
	 * @param ip								[in]设备IP地址
	 * 	@param username					[in]用户名
	 * @param password					[in]密码
	 * @param isSupport					[out]是否支持AIFace
	 * @return
	 */
	HttpManagerErcode isSupportAIFace(const QString &ip,const QString &username,const QString &password,bool &isSupport);

	/**
	 * @brief getStreamResolution		得到设备主码流的分辨率
	 * @param ip								[in]IP地址
	 * @param username					[in]用户名
	 * @param password					[in]密码
	 * @param width							[out]宽
	 * @param height							[out]高
	 * @return
	 */
	HttpManagerErcode getStreamResolution(const QString &ip,const QString &username,const QString &password,int &width,int &height);

	/**
	 * @brief getMaxFaceResolution	得到最大人脸大小
	 * @param ip								[in]IP地址
	 * @param username					[in]用户名
	 * @param password					[in]密码
	 * @param width							[out]宽
	 * @param height							[out]高
	 * @return
	 */
	HttpManagerErcode getMaxFaceResolutionLimit(const QString &ip,const QString &username,const QString &password,int &width,int &height);

	/**
	 * @brief getMaxFaceResolution	得到最小人脸大小
	 * @param ip								[in]IP地址
	 * @param username					[in]用户名
	 * @param password					[in]密码
	 * @param width							[out]宽
	 * @param height							[out]高
	 * @return
	 */
	HttpManagerErcode getMinFaceResolutionLimit(const QString &ip,const QString &username,const QString &password,int &width,int &height);

	/**
	 * @brief getCurrentDetectRegion		得到当前探测区域
	 * @param ip										[in]IP地址
	 * @param username							[in]用户名
	 * @param password							[in]密码
	 * @param x											[out]x
	 * @param y											[out]y
	 * @param width									[out]宽
	 * @param height									[out]高
	 * @return
	 */
	HttpManagerErcode getCurrentDetectRegion(const QString &ip,const QString &username,const QString &password,int &x,int &y,int &width,int &height);

	/**
	 * @brief getMaxFaceResolution	得到当前最大人脸大小
	 * @param ip								[in]IP地址
	 * @param username					[in]用户名
	 * @param password					[in]密码
	 * @param width							[out]宽
	 * @param height							[out]高
	 * @return
	 */
	HttpManagerErcode getCurrentMaxFaceResolution(const QString &ip,const QString &username,const QString &password,int &width,int &height);

	/**
	 * @brief getMaxFaceResolution	得到当前最小人脸大小
	 * @param ip								[in]IP地址
	 * @param username					[in]用户名
	 * @param password					[in]密码
	 * @param width							[out]宽
	 * @param height							[out]高
	 * @return
	 */
	HttpManagerErcode getCurrentMinFaceResolution(const QString &ip,const QString &username,const QString &password,int &width,int &height);

	/**
	 * @brief setFaceDetectRegion		设置人脸检测区域
	 * @param ip								[in]IP地址
	 * @param username					[in]用户名
	 * @param password					[in]密码
	 * @param x									[in]x坐标
	 * @param y									[in]y坐标
	 * @param width							[in]宽度
	 * @param height							[in]高度
	 * @return
	 */
	HttpManagerErcode setFaceDetectRegion(const QString &ip,const QString username,const QString &password,const int &x,const int &y,const int &width,const int &height);

	/**
	 * @brief setMaxFaceSize			设置最大人脸大小
	 * @param ip								[in]IP地址
	 * @param username					[in]用户名
	 * @param password					[in]密码
	 * @param maxWidth					[in]宽度
	 * @param maxHeight				[in]高度
	 * 	@param minWidth					[in]宽度
	 * @param minHeight					[in]高度
	 * @return
	 */
	HttpManagerErcode setFaceFilterSize(const QString &ip,const QString username,const QString &password,const int &maxWidth,const int &maxHeight,const int &minWidth,const int &minHeight);

	/**
	 * @brief getSnapConfig				获取抓拍配置
	 * @param ip								[in]IP地址
	 * @param username					[in]用户名
	 * @param password					[in]密码
	 * @param values							[out]值
	 * @return
	 */
	HttpManagerErcode getCurrentSnapConfig(const QString &ip,const QString &username,const QString &password, QVector<QString> &values);

	/**
	 * @brief getSnapConfig				获取抓拍配置
	 * @param ip								[in]IP地址
	 * @param username					[in]用户名
	 * @param password					[in]密码
	 * @param values							[out]值
	 * @return
	 */
	HttpManagerErcode getSnapConfig(const QString &ip,const QString &username,const QString &password,const QString &snapShotMode, QVector<QString> &values);

	/**
	 * @brief setSnapConfig				设置抓拍配置
	 * @param ip								[in]IP地址
	 * @param username					[in]用户名
	 * @param password					[in]密码
	 * @param values							[in]值
	 * @return
	 */
	HttpManagerErcode setSnapConfig(const QString &ip,const QString &username,const QString &password,const QVector<QString> &values);

	/**
	 * @brief getOverlayConfig			获取Overlay信息
	 * @param ip								[in]IP地址
	 * @param username					[in]用户名
	 * @param password					[in]密码
	 * @param value							[in]值
	 * @return
	 */
	HttpManagerErcode getOverlayConfig(const QString &ip,const QString &username,const QString &password,QString &value);

	/**
	 * @brief setOverlayConfig			设置Overlay信息
	 * @param ip								[in]IP地址
	 * @param username					[in]用户名
	 * @param password					[in]密码
	 * @param value							[in]值
	 * @return
	 */
	HttpManagerErcode setOverlayConfig(const QString &ip,const QString &username,const QString &password,const QString &value);

private:
	/**
	 * @brief setHeader		设置HTTP请求头
	 * @param ip				[in]IP地址
	 * @param username	[in]用户名
	 * @param password	[in]密码
	 */
	void setHeader(const QString &ip,const QString &username,const QString &password);
};

#endif // AIFACEPACKAGE_H
