﻿#ifndef FACELIBRARYPACKAGE_H
#define FACELIBRARYPACKAGE_H

#include "src/inc/aifacepch.h"
#include "src/inc/aifaceobject.h"
#include "src/inc/httpmanager/httpmanagerpch.h"

#include <QVector>
#include <QString>

class FaceLibraryPackage
{
public:
	static FaceLibraryPackage *GetInstance()
	{
		static FaceLibraryPackage faceLibraryPackage;
		return &faceLibraryPackage;
	}

	/**
	 * @brief getFaceLibraryList		得到所有的脸库项目
	 * @param ip								[in]IP地址
	 * @param username					[in]用户名
	 * @param password					[in]密码
	 * @param items							[out]所有脸库项目
	 * @return
	 */
	HttpManagerErcode getFaceLibraryList(const QString &ip,const QString &username,const QString &password,FaceLibraryList &items);

	/**
	 * @brief getRecordRecordCount		得到识别记录总数
	 * @param ip										[in]设备IP地址
	 * @param username							[in]用户名
	 * @param password							[in]密码
	 * @param libraryID							[in]脸库ID
	 * @param recordCount						[out]总数
	 * 以下为搜索字段
	 * @param name												[in]名字
	 * @param sex													[in]性别
	 * @param certificateType								[in]证件类型
	 * @param certificateID									[in]证件ID
	 * @param country											[in]国家
	 * @param address											[in]地址
	 * @param startBirthday									[in]生日起始时间
	 * @param stopBirthday									[in]生日停止时间
	 * @return
	 */
	HttpManagerErcode getFaceLibraryCount(const QString &ip,const QString &username,const QString &password,const QString &libraryID,QString &itemCount,
							 const QString &Name="",const QString &sex="unknown",const QString &certificateType="other",const QString &certificateID="",const QString &country="",const QString address="",const QString &startBirthday="00000000",const QString &stopBirthday="00000000");

	/**
	 * @brief getFaceLibraryByLibraryID					获取某一脸库下的所有项目
	 * @param ip													[in]IP地址
	 * @param username										[in]用户名
	 * @param password										[in]密码
	 * @param libraryID											[in]脸库ID
	 * @param pageSize											[in]页面容纳条目数量
	 * @param pageIndex										[in]第几页
	 * @param infos												[out]所有条目
	 * 以下为搜索字段
	 * @param name												[in]名字
	 * @param sex													[in]性别
	 * @param certificateType								[in]证件类型
	 * @param certificateID									[in]证件ID
	 * @param country											[in]国家
	 * @param address											[in]地址
	 * @param startBirthday									[in]生日起始时间
	 * @param stopBirthday									[in]生日停止时间
	 * @return
	 */
	HttpManagerErcode getFaceLibraryByLibraryID(const QString &ip,const QString &username,const QString &password,const QString &libraryID,const int &pageSize,const int &pageIndex,FaceLibraryItemInfoList &infos,
								   const QString &Name="",const QString &sex="unknown",const QString &certificateType="other",const QString &certificateID="",const QString &country="",const QString address="",const QString &startBirthday="00000000",const QString &stopBirthday="00000000"
								   );


#ifdef UniqueID_Support
	/**
	 * @brief getPersonImageInfo						获取人物图片
	 * @param ip												[in]IP地址
	 * @param username									[in]用户名
	 * @param password									[in]密码
	 * @param uniqueID										[in]唯一ID
	 * @param imageID										[in]图片ID
	 * @param imageData									[out]图片数据
	 * @return
	 */
	HttpManagerErcode getPersonImageInfo(const QString &ip,const QString &username,const QString &password,const QString &uniqueID,const QString &imageID,QByteArray &imageData);
#else
	/**
	 * @brief getPersonImageInfo						获取人物图片
	 * @param ip												[in]IP地址
	 * @param username									[in]用户名
	 * @param password									[in]密码
	 * @param personID										[in]人物ID
	 * @param imageID										[in]图片ID
	 * @param imageData									[out]图片数据
	 * @return
	 */
	HttpManagerErcode getPersonImageInfo(const QString &ip,const QString &username,const QString &password,const QString &personID,const QString &imageID,QByteArray &imageData);
#endif

	/**
	 * @brief setPersonInfo			设置人物信息
	 * @param ip							[in]IP地址
	 * @param username				[in]用户名
	 * @param password				[in]密码
	 * @param info							[in]人物信息
	 * @return
	 */
	HttpManagerErcode setPersonInfo(const QString &ip,const QString &username,const QString &password,const FaceLibraryItemInfo &info);

	/**
	 * @brief setPersonImage			设置人物图片信息
	 * @param ip							[in]IP地址
	 * @param username				[in]用户名
	 * @param password				[in]密码
	 * @param personID					[in]人物ID
	 * @param imageID					[in]图片ID
	 * @param imageName			[in]图片名称
	 * @param imageData				[in]图片信息
	 * @return
	 */
	HttpManagerErcode setPersonImage(const QString &ip,const QString &username,const QString &password,const QString &personID,const QString &imageID,const QString &imageName,const QByteArray &imageData);

	/**
	 * @brief deletePersonInfo			删除人物信息
	 * @param ip							[in]IP地址
	 * @param username				[in]用户名
	 * @param password				[in]密码
	 * @param libraryID					[in]脸库ID
	 * @param personID					[in]人物ID
	 * @return
	 */
	HttpManagerErcode deletePersonInfo(const QString &ip,const QString &username,const QString &password,const QVector<QVector<QString>> &datas,const QString &isDeleteFromAllLibrary);

	/**
	 * @brief setPersonInfo			设置人物信息
	 * @param ip							[in]IP地址
	 * @param username				[in]用户名
	 * @param password				[in]密码
	 * @param info							[in]人物信息
	 * @param imageData				[in]人物图片
	 * @param uniqueID
	 * @return
	 */
	HttpManagerErcode addPersonInfo(const QString &ip,const QString &username,const QString &password,const FaceLibraryItemInfo &info,const QByteArray &imageData,QString &personID, QString &uniqueID);

	/**
	 * @brief deleteFaceLibrary		设置脸库信息
	 * @param ip							[in]IP地址
	 * @param username				[in]用户名
	 * @param password				[in]密码
	 * @param groupID					[in]脸库ID
	 * @return
	 */
	HttpManagerErcode deleteFaceLibrary(const QString &ip,const QString &username,const QString &password,const QString &groupID,const QString &isDelBindFace="false");

	/**
	 * @brief editFaceLibraryName		编辑脸库名称
	 * @param ip								[in]IP地址
	 * @param username					[in]用户名
	 * @param password					[in]密码
	 * @param groupID						[in]脸库ID
	 * @param groupName				[in]脸库名称
	 * @return
	 */
	HttpManagerErcode editFaceLibraryName(const QString &ip,const QString &username,const QString &password,const QString &groupID,const QString &groupName);

	/**
	 * @brief editFaceLibraryName		添加脸库
	 * @param ip								[in]IP地址
	 * @param username					[in]用户名
	 * @param password					[in]密码
	 * @param groupName				[in]脸库名称
	 * @return
	 */
	HttpManagerErcode addFaceLibraryName(const QString &ip,const QString &username,const QString &password,const QString &name);

	/**
	 * @brief editFaceLibraryName		添加脸库
	 * @param ip								[in]IP地址
	 * @param username					[in]用户名
	 * @param password					[in]密码
	 * QVector
	 * @param srcGroup						[in]源脸库ID
	 * @param dstGroup					[in]目标脸库ID
	 * @param personID						[in]人物信息ID
	 * @return
	 */
	HttpManagerErcode bindPersonToFaceLibrary(const QString &ip,const QString &username,const QString &password,const QVector<QVector<QString>> &datas);

	/**
	 * @brief exportFaceLibrary			导出脸库
	 * @param ip								[in]IP地址
	 * @param username					[in]用户名
	 * @param password					[in]密码
	 * @param data							[out]二进制数据
	 * @return
	 */
	HttpManagerErcode exportFaceLibrary(const QString &ip,const QString &username,const QString &password,QByteArray &data);

	/**
	 * @brief exportFaceLibrary			导出脸库
	 * @param ip								[in]IP地址
	 * @param username					[in]用户名
	 * @param password					[in]密码
	 * @param filename						[in]文件名
	 * @param data							[in]二进制数据
	 * @return
	 */
	HttpManagerErcode importFaceLibrary(const QString &ip,const QString &username,const QString &password,const QString &filename, const QByteArray &data);

protected:
	FaceLibraryPackage();

private:
	/**
	 * @brief setHeader		设置HTTP请求头
	 * @param ip					[in]IP地址
	 * @param username		[in]用户名
	 * @param password		[in]密码
	 */
	void setHeader(const QString &ip,const QString &username,const QString &password);
};

#endif // FACELIBRARYPACKAGE_H
