﻿#ifndef RECOGNIZERECORDPACKAGE_H
#define RECOGNIZERECORDPACKAGE_H

#include "src/inc/aifaceobject.h"
#include "src/inc/httpmanager/httpmanagerpch.h"

class RecognizeRecordPackage
{
public:
	static RecognizeRecordPackage *GetInstance()
	{
		static RecognizeRecordPackage recognizeRecordPackage;
		return &recognizeRecordPackage;
	}

protected:
	RecognizeRecordPackage();

public:
	/**
	 * @brief getRecordRecordCount			得到识别记录总数
	 * @param ip										[in]设备IP地址
	 * @param username							[in]用户名
	 * @param password							[in]密码
	 * @param recordCount						[out]总数
	 * 以下为搜索字段
	 * @param libraryID								[in]脸库ID
	 * @param peopleName						[in]人名
	 * @param startTime								[in]起始时间
	 * @param endTime								[in]结束时间
	 * @param recognitionResultType			[in]识别结果
	 * @param ruleID									[in]规则ID
	 * @return
	 */
	HttpManagerErcode getRecordRecordCount(const QString &ip,const QString &username,const QString &password,QString &recordCount,
							  const QString &libraryID="-1",const QString &peopleName="",const QString &startTime="00000000",const QString &endTime="00000000",const QString &recognitionResultType="all",const QString &ruleID="-1");

	/**
	 * @brief getRecognizeRecord				得到人脸识别记录
	 * @param ip										[in]设备IP地址
	 * @param username							[in]用户名
	 * @param password							[in]密码
	 * @param pageSIze								[in]每页记录数量
	 * @param pageIndex							[in]页码
	 * @param recordList							[out]记录
	 * 	以下为搜索字段
	 * @param libraryID								[in]脸库ID
	 * @param peopleName						[in]人名
	 * @param startTime								[in]起始时间
	 * @param endTime								[in]结束时间
	 * @param recognitionResultType			[in]识别结果
	 * @param ruleID									[in]规则ID
	 */
	HttpManagerErcode getRecognizeRecord(const QString &ip,const QString &username,const QString &password,const QString &pageSize, const QString &pageIndex, RecognitionRecordList &recordList,
							const QString &libraryID="-1",const QString &peopleName="",const QString &startTime="000000000000",const QString &endTime="000000000000",const QString &recognitionResultType="all",const QString &ruleID="-1");

	/**
	 * @brief getRecognizeRecordDetail		得到识别具体记录
	 * @param ip										[in]设备IP地址
	 * @param username							[in]用户名
	 * @param password							[in]密码
	 * @param logID									[in]记录ID
	 * @param detail									[out]识别详细记录
	 * @return
	 */
	HttpManagerErcode getRecognizeRecordDetail(const QString &ip,const QString &username,const QString &password,const QString &logID,RecognitionRecordDetail &detail);

	/**
	 * @brief getSnapshot							得到抓拍图片
	 * @param ip										[in]设备IP地址
	 * @param username							[in]用户名
	 * @param password							[in]密码
	 * @param snapshotPath						[in]图片路径
	 * @param image									[out]图片二进制数据
	 * @return
	 */
	HttpManagerErcode getSnapshot(const QString &ip,const QString &username,const QString &password,const QString &snapshotPath,QByteArray &image);

	/**
	 * @brief getSnapshot							得到抓拍图片
	 * @param ip										[in]设备IP地址
	 * @param username							[in]用户名
	 * @param password							[in]密码
	 * @param logID									[in]logID
	 * @param image									[out]图片二进制数据
	 * @return
	 */
	HttpManagerErcode getComparedImage(const QString &ip,const QString &username,const QString &password,const QString &logID,QByteArray &image);

private:
	/**
	 * @brief setHeader		设置HTTP请求头
	 * @param ip					[in]IP地址
	 * @param username		[in]用户名
	 * @param password		[in]密码
	 */
	void setHeader(const QString &ip,const QString &username,const QString &password);
};

#endif // RECOGNIZERECORDPACKAGE_H
