﻿#ifndef RESTCLIENT_H
#define RESTCLIENT_H

#include "src/inc/restclient/restobject.h"

#include <QObject>
#include <QScopedPointer>
#include <QNetworkAccessManager>

class RestClient : public QObject
{
	Q_OBJECT
public:
	/**
	 * @brief GetInstance
	 * @return
	 */
	static RestClient *GetInstance()
	{
		static RestClient restClient;
		return &restClient;
	}

protected:
	RestClient();

public:
	/**
	 * @brief setUrl		设置URL
	 * @param url		[in]URL
	 */
	void setUrl(const QString &url);

	/**
	 * @brief addRequestParam		设置请求参数
	 * @param values						[in]请求参数
	 */
	void addRequestParam(const QString &key,const QString &value);

	/**
	 * @brief setResquestBody		设置请求体
	 * @param body						[in]请求体
	 */
	void addRequestParam(const QString &key,const int &value);

	/**
	 * @brief setRequestBody			设置请求体
	 * @param body						[in]请求体
	 */
	void setRequestBody(const QByteArray &body);

	/**
	 * @brief sendRequest			发送HTTP请求
	 * @param requestType		[in]HTTP请求类型
	 *  @param waitTime				[in]等待时间，-1代表无限
	 */
	int sendRequest(const RequestType &requestType, QByteArray &response, const int &waitTime=INT_MAX);

	/**
	 * @brief clear		清除请求信息
	 */
	void clear();

	/**
	 * @brief getResponse			得到HTTP响应
	 * @param responseValue		[out]HTTP响应体
	 * @param waitTime				[in]等待时间，-1代表无限
	 */
//	bool getResponse(QByteArray &responseValue,const int &waitTime=-1);

private:
	QString m_url;
	QVector<QPair<QString,QString>> m_params;
	QByteArray m_body;

	QScopedPointer<QNetworkAccessManager> m_manager{nullptr};
};

#endif // RESTCLIENT_H
