﻿#ifndef USERINFO_H
#define USERINFO_H

#include "src/inc/aifacepch.h"
#include "src/inc/aifaceobject.h"
#include "lib/sdk/VideoNetClient/include/VideoNetClient_Common.h"

#include <QMap>

class UserManager
{
public:
	static UserManager *GetInstance()
	{
		static UserManager userInfo;
		return &userInfo;
	}

protected:
	UserManager();

public:
	/**
	 * @brief addUser			添加用户句柄
	 * @param userID			[in]用户ID
	 * @param userInfo		[in]用户句柄
	 * @return
	 */
	AIFaceErcode addUser(const int &userID,const UserInfo &userInfo);

	/**
	 * @brief removeUser		移除用户
	 * @param userID			[in]用户ID
	 * @return
	 */
	AIFaceErcode removeUser(const int &userID);

	/**
	 * @brief getHUser		得到用户句柄
	 * @param userID			[in]用户ID
	 * @param hUser			[out]用户句柄
	 * @return
	 */
	AIFaceErcode getHUser(const int &userID,HUSER &hUser);

	/**
	 * @brief getMAC			根据用户句柄得到MAC地址
	 * @param hUser			[in]用户句柄
	 * @param mac			[out]mac地址
	 * @return
	 */
	AIFaceErcode getMAC(const HUSER &hUser,QString &mac);

	/**
	 * @brief isDeviceLogin		获取设备UserID
	 * @param mac					[in]设备MAC地址
	 * @return
	 */
	AIFaceErcode getDeviceUserID(const QString &mac,int &userID);

	/**
	 * @brief addRealStreamHandle		添加实时流句柄
	 * @param userID								[in]用户ID
	 * @param channelID						[in]通道
	 * @param hStream							[in]实时流句柄
	 * @return
	 */
	AIFaceErcode setRealStreamHandle(const int &userID,const int &channelID,const HSTREAM &hStream);

	/**
	 * @brief getRealStreamHandle		获取实时流句柄
	 * @param userID							[in]用户ID
	 * @param channelID					[in]通道
	 * @param hStream						[out]实时流句柄
	 * @return
	 */
	AIFaceErcode getRealStreamHandle(const int &userID,const int &channelID,HSTREAM &hStream);

	/**
	 * @brief setRealStreamPort			设置实时流播放端口
	 * @param userID							[in]用户ID
	 * @param channelID					[in]通道ID
	 * @param port							[in]端口
	 * @return
	 */
	AIFaceErcode setRealStreamPort(const int &userID,const int &channelID,const int &port);

	/**
	 * @brief getRealStreamPort			得到实时流播放端口
	 * @param userID							[in]用户ID
	 * @param channelID					[in]通道ID
	 * @param port							[out]端口
	 * @return
	 */
	AIFaceErcode getRealStreamPort(const int &userID,const int &channelID,int &port);

	/**
	 * @brief addRectOnStream			添加要在流上绘制的矩形
	 * @param userID							[in]用户ID
	 * @param channelID					[in]通道ID
	 * @param rect								[in]矩形
	 * @return
	 */
	AIFaceErcode addRectOnStream(const int &userID,const int &channelID,const QRect &rect);

	/**
	 * @brief clearRectsOnStream		清除所有将会绘制在流上的矩形
	 * @param userID							[in]用户ID
	 * @param channelID					[in]通道ID
	 * @return
	 */
	AIFaceErcode clearRectsOnStream(const int &userID,const int &channelID);

	/**
	 * @brief getRectsByPort				得到所有将会在流上绘制的矩形
	 * @param port							[in]Port
	 * @param rects							[out]矩形
	 * @return
	 */
	AIFaceErcode getRectsByPort(const int &port,QVector<QRect> &rects);

public:
	template<typename... Args>
	AIFaceErcode addUser(const int &userID,Args... args)
	{
		if(true==m_userInfo.contains(userID))
			check_ercode(AFErcode_UserAlreadyExist,"UserID already exist!")

		m_userInfo.insert(userID,UserInfo(std::forward<Args>(args)...));

		return AFErcode_None;
	}

private:
	QMap<int,UserInfo> m_userInfo;
};

#endif // USERINFO_H
