﻿#include "src/inc/devicemanager/deviceanalysiser.h"

#include "src/inc/aifacepch.h"

#include <QFileInfo>
#include <QList>

DeviceAnalysiser::DeviceAnalysiser()
{
	readDeviceNameDatabase();
}

DeviceType DeviceAnalysiser::getDeviceType(LPDeviceProbeConfigV2 config)
{
	DeviceType deviceType{DeviceType_Unknown};

	int bytDeviceType = config->m_bytDevType;

	//如果没有给出设备名字，则根据m_bytDevType判断
	if(true == m_ipcDeviceName.contains(bytDeviceType))
		deviceType = DeviceType_IPC;
	else if(true == m_ipdDeviceName.contains(bytDeviceType))
		deviceType = DeviceType_IPD;
	else if(true == m_nvrDeviceName.contains(bytDeviceType))
		deviceType = DeviceType_NVR;
	else if(true == m_dvrDeviceName.contains(bytDeviceType))
		deviceType = DeviceType_DVR;

	if(DeviceType_Unknown != deviceType)
		return deviceType;

	//根据m_cDeviceName判断设备类型
	QString deviceName(config->m_cDeviceName);
	if(true == deviceName.contains("IPC"))
		deviceType = DeviceType_IPC;
	else if(true == deviceName.contains("NVR"))
		deviceType = DeviceType_NVR;
	else if(true == deviceName.contains("HVR"))
		deviceType = DeviceType_DVR;
	else if(true == deviceName.contains("IPD"))
		deviceType = DeviceType_IPD;

	if(DeviceType_Unknown != deviceType)
		return deviceType;

	QString allName = config->m_szTypeName;
	if(true == allName.contains("IPG"))
		deviceType = DeviceType_IPC;
	else if(true == allName.contains("AHB"))
		deviceType = DeviceType_DVR;
	else if(true == allName.contains("NBD"))
		deviceType = DeviceType_NVR;

	return deviceType;
}

QString DeviceAnalysiser::getDeviceName(LPDeviceProbeConfigV2 config)
{
	QString deviceName;

	int bytDevType=config->m_bytDevType;

	//获取m_szTypeNamed的值
	deviceName=config->m_szTypeName;

	//如果m_szTypeName无值，根据m_bytDevType判断
	if(true==deviceName.isEmpty())
	{
		if(true==m_ipcDeviceName.contains(bytDevType))
			deviceName=m_ipcDeviceName[bytDevType];
		else if(true==m_ipdDeviceName.contains(bytDevType))
			deviceName=m_ipdDeviceName[bytDevType];
		else if(true==m_nvrDeviceName.contains(bytDevType))
			deviceName=m_nvrDeviceName[bytDevType];
		else if(true==m_nvrDeviceName.contains(bytDevType))
			deviceName=m_dvrDeviceName[bytDevType];
	}

	return deviceName;
}

QString DeviceAnalysiser::getIPAddress(LPDeviceProbeConfigV2 config)
{
	QString ip;

	for(int i=0;i<IP_ADDRESS_LEN;++i)
	{
		if('\0'==config->m_bytDevIP[i])
			break;

		ip+=config->m_bytDevIP[i];
	}

	return ip;
}

QString DeviceAnalysiser::getMACAddress(LPDeviceProbeConfigV2 config)
{
	QString mac;

	for (int i = 0; i < MAC_BINARY_ADDRESS_LEN;++i)
	{
		char m[3]{ 0,0,0 };
		sprintf(m, "%02x", config->m_bytDevMac[i]);
		mac += m;
		if (i  ==  MAC_BINARY_ADDRESS_LEN - 1);
		else
			mac += ":";
	}

	return mac;
}

QString DeviceAnalysiser::getROMVersion(LPDeviceProbeConfigV2 config)
{
	QString romVersion;

	//尝试从m_szSoftwareVersion得到固件版本
	romVersion=config->m_szSoftwareVersion;

	//如果m_szSoftwareVersion为空，则从m_bytDeviceVersion获取信息
	if(true==romVersion.isEmpty())
		romVersion=config->m_bytDeviceVersion;

	return romVersion;
}

QString DeviceAnalysiser::getBootTime(LPDeviceProbeConfigV2 config)
{
	QString bootTime;

	int hour = config->m_dwDeviceBootTime/3600;
	int minute = (config->m_dwDeviceBootTime%3600)/60;
	int second = (config->m_dwDeviceBootTime)%60;

	bootTime = QString::number(hour)+"h"+QString::number(minute)+"m"+QString::number(second)+"s";

	return bootTime;
}

void DeviceAnalysiser::readDeviceNameDatabase()
{
	//判断文件是否存在
	QFileInfo fileInfo("./config/boardtype.ini");
	if(false == fileInfo.exists())
	{
		message_output("./config/boardtype.ini not exist!");
		return;
	}

	//打开文件
	QFile boardType("./config/boardtype.ini");
	boardType.open(QIODevice::ReadOnly|QIODevice::Text);

	//读取某一个设备类型下的版型信息
	const auto readDeviceNameInfo = [&boardType](QMap<int,QString> &deviceName)
	{
		QString line;
		while(true)
		{
			line=boardType.readLine();
			if(true == line.isEmpty())
				break;

			QStringList deviceTypeNameInfo;
			deviceTypeNameInfo=line.split("=");
			if(deviceTypeNameInfo.length() != 2)
				return;

			char *str{nullptr};
			deviceName.insert(strtol(deviceTypeNameInfo[0].toStdString().c_str(), &str, 16),deviceTypeNameInfo.at(1));
		}
	};

	QString line;
	while(false == boardType.atEnd())
	{
		line=boardType.readLine();
		if(true == line.isEmpty())
			continue;

		if(true == line.contains("[IPC]"))		//读取IPC版型信息
			readDeviceNameInfo(m_ipcDeviceName);
		else if(true == line.contains("[IPD]"))			//读取IPD版型信息
			readDeviceNameInfo(m_ipdDeviceName);
		else if(true == line.contains("[DVR]"))		//读取DVR版型信息
			readDeviceNameInfo(m_dvrDeviceName);
		else if(true == line.contains("[NVR]"))		//读取NVR版型信息
			readDeviceNameInfo(m_nvrDeviceName);
	}
}
