﻿#include "src/inc/devicemanager/devicemanager.h"

#include <stdlib.h>

DeviceManager::DeviceManager()
{
}

void DeviceManager::addDevice(const DeviceInfo &deviceInfo)
{
	if(auto l = m_devices.lock())
	{
		if(false==m_devices.devices.contains(deviceInfo.mac))
			m_devices.devices.insert(deviceInfo.mac,deviceInfo);
	}
}

void DeviceManager::removeDevice(const QString &mac)
{
	if(auto l = m_devices.lock())
	{
		if(true==m_devices.devices.contains(mac))
			m_devices.devices.remove(mac);
	}
}

void DeviceManager::getDevices(QVector<DeviceInfo> &devices)
{
	if(auto l = m_devices.lock())
	{
		for(const DeviceInfo &deviceInfo:m_devices.devices)
			devices.append(deviceInfo);
	}
}

void DeviceManager::getIPCDevices(QVector<DeviceInfo> &ipcDevices)
{
	getDevices(DeviceType_IPC,ipcDevices);
}

void DeviceManager::getIPDDevices(QVector<DeviceInfo> &ipdDevices)
{
	getDevices(DeviceType_IPD,ipdDevices);
}

void DeviceManager::getNVRDevices(QVector<DeviceInfo> &nvrDevices)
{
	getDevices(DeviceType_NVR,nvrDevices);
}

void DeviceManager::getDVRDevices(QVector<DeviceInfo> &dvrDevices)
{
	getDevices(DeviceType_DVR,dvrDevices);
}

QString DeviceManager::getIPbyMAC(const QString &mac)
{
	if(auto l = m_devices.lock())
	{
		if(true==m_devices.devices.contains(mac))
			return m_devices.devices[mac].ip;
	}

	return QString();
}

void DeviceManager::clearDevices()
{
	if(auto l = m_devices.lock())
	{
		m_devices.devices.clear();
	}
}



void DeviceManager::getDevices(const DeviceType &deviceType, QVector<DeviceInfo> &devices)
{
	if(auto l = m_devices.lock())
	{
		for(const DeviceInfo &deviceInfo:m_devices.devices)
		{
			if(deviceType==deviceInfo.deviceType)
				devices.append(deviceInfo);
		}
	}
}
