﻿#include "src/inc/gui/facedetect/aifaceconfigwidget.h"

#include "src/inc/aifacepch.h"
#include "src/inc/package/videopackage.h"
#include "src/inc/httpmanager/aifacepackage.h"
#include "src/inc/gui/facedetect/aifaceregion.h"
#include "src/inc/gui/facedetect/snapconfigwidget.h"
#include "src/inc/gui/facedetect/dxregion.h"

#include <QGroupBox>
#include <QLabel>
#include <QFont>
#include <QLineEdit>
#include <QPushButton>
#include <QStyleOption>
#include <QPainter>
#include <QMouseEvent>
#include <QComboBox>

AIFaceConfigWidget::AIFaceConfigWidget(QWidget *parent) : QWidget(parent)
{
	this->setGeometry(adaptxywh(100,100,1300,700));
//	this->setAttribute(Qt::WA_DeleteOnClose);
	this->setFocusPolicy(Qt::StrongFocus);

	//创建GUI
	initAreaConfigGUI();
}

AIFaceConfigWidget::~AIFaceConfigWidget()
{
	delete m_aiFaceRegion;
	m_aiFaceRegion = nullptr;

	unInit();
}

void AIFaceConfigWidget::init(const QString &mac,const QString &ip,const QString &username,const QString &password)
{
	int ercode{0};

	m_ip = ip;
	m_username = username;
	m_password = password;

	//登录设备
	ercode = vncp::VideoPackage::GetInstance()->loginDevice(mac,m_ip,6060,m_username,m_password,m_userID);
	check_ercode_no_return(ercode,"Login failed!")

	//播放实时流
#ifdef USE_QT_WIDGET
	ercode = vncp::VideoPackage::GetInstance()->startRealStream(m_userID,0,m_videoWidget);
	check_ercode_no_return(ercode,"Start real stream failed!")
#else
	ercode = vncp::VideoPackage::GetInstance()->startRealStream(m_userID,0,m_dxRegion);
	check_ercode_no_return(ercode,"Start real stream failed!")
#endif

	m_dxRegion->setUserID(m_userID);

	//获取视频流分辨率
	int resolutionWidth{0};
	int resolutionHeight{0};
	ercode = AIFacePackage::GetInstance()->getStreamResolution(m_ip,m_username,m_password,resolutionWidth,resolutionHeight);
	check_http_ercode_no_return(ercode,"Get Stream Resolution failed!")

	m_streamResolutionWidth = resolutionWidth;
	m_streamResolutionHeight = resolutionHeight;
	m_defaultFaceDetectRect={0,0,resolutionWidth,resolutionHeight};

	//获取最大人脸区域配置信息（万分比）
	int maxFaceWidth{0};
	int maxFaceHeight{0};
	ercode = AIFacePackage::GetInstance()->getMaxFaceResolutionLimit(m_ip,m_username,m_password,maxFaceWidth,maxFaceHeight);
	check_http_ercode_no_return(ercode,"Get Max Face Resolution Limit Failed!")
	m_defaultMaxFaceSize={maxFaceWidth,maxFaceHeight};

	//获取最小人脸配置区域信息（万分比）
	int minFaceWidth{0};
	int minFaceHeight{0};
	ercode = AIFacePackage::GetInstance()->getMinFaceResolutionLimit(m_ip,m_username,m_password,minFaceWidth,minFaceHeight);
	check_http_ercode_no_return(ercode,"Get Min Face Resolution Limit Failed!")
	m_defaultMinFaceSize={minFaceWidth,minFaceHeight};

#ifdef USE_QT_WIDGET
	//创建用于绘制人脸探测区域框，最大人脸框，最小人脸框的窗体
	m_aiFaceRegion = new AIFaceRegion;
	check_pointer_no_return(m_aiFaceRegion)
	m_aiFaceRegion->setGeometry(this->x(),this->y()+(this->frameGeometry().height()-this->geometry().height()),adaptx(900),this->height());
	m_aiFaceRegion->setStreamSize(resolutionWidth,resolutionHeight);
	m_aiFaceRegion->setMaxFaceSizeLimit(rateToResolution(resolutionWidth,maxFaceWidth),rateToResolution(resolutionHeight,maxFaceHeight));
	m_aiFaceRegion->setMinFaceSizeLimit(rateToResolution(resolutionWidth,minFaceWidth),rateToResolution(resolutionHeight,minFaceHeight));
	m_aiFaceRegion->init();
	m_aiFaceRegion->show();
	//最大最小人脸框修改后，通知数值修改
	connect(m_aiFaceRegion,&AIFaceRegion::signal_maxFaceRegionChange,this,&AIFaceConfigWidget::slot_maxFaceRegionChange);
	connect(m_aiFaceRegion,&AIFaceRegion::signal_minFaceRegionChange,this,&AIFaceConfigWidget::slot_minFaceRegionChange);
#else
	m_dxRegion->setStreamSize(QSize(resolutionWidth,resolutionHeight));
	m_dxRegion->setMaxFaceSizeLimit(QSize(rateToResolution(resolutionWidth,maxFaceWidth),rateToResolution(resolutionHeight,maxFaceHeight)));
	m_dxRegion->setMinFaceSizeLimit(QSize(rateToResolution(resolutionWidth,minFaceWidth),rateToResolution(resolutionHeight,minFaceHeight)));
	m_defaultMaxFaceSize=QSize(rateToResolution(resolutionWidth,maxFaceWidth),rateToResolution(resolutionHeight,maxFaceHeight));
	m_defaultMinFaceSize=QSize(rateToResolution(resolutionWidth,minFaceWidth),rateToResolution(resolutionHeight,minFaceHeight));

#endif

	//得到当前探测区域配置
	int x{0},y{0},width{0},height{0};
	ercode = AIFacePackage::GetInstance()->getCurrentDetectRegion(m_ip,m_username,m_password,x,y,width,height);
	check_http_ercode_no_return(ercode,"Get current detect region failed!");
#ifdef USE_QT_WIDGET
	m_aiFaceRegion->setCurrentDetectRect(rateToResolution(resolutionWidth,x),rateToResolution(resolutionHeight,y),rateToResolution(resolutionWidth,width),rateToResolution(resolutionHeight,height));
#else
	m_dxRegion->setDetectRect(QRect(rateToResolution(resolutionWidth,x),rateToResolution(resolutionHeight,y),rateToResolution(resolutionWidth,width),rateToResolution(resolutionHeight,height)));
#endif

	//得到当前最大人脸区域配置（万分比）
	int currentMaxFaceWidth{0};
	int currentMaxFaceHeight{0};
	ercode = AIFacePackage::GetInstance()->getCurrentMaxFaceResolution(m_ip,m_username,m_password,currentMaxFaceWidth,currentMaxFaceHeight);
	check_http_ercode_no_return(ercode,"Get current max face resolution failed!");
	//计算分辨率角度的最大人脸区域大小
	qreal realMaxWidth = rateToResolution(resolutionWidth,currentMaxFaceWidth);
	qreal realMaxHeight = rateToResolution(resolutionHeight,currentMaxFaceHeight);
#ifdef USE_QT_WIDGET
	//设置最大人脸区域框
	m_aiFaceRegion->setCurrentMaxFaceSize(realMaxWidth,realMaxHeight);
#else
	m_dxRegion->setMaxFaceSize(QSize(realMaxWidth,realMaxHeight));
#endif
	//设置最大人脸区域数值
	m_maxFaceSizeWidth->setText(QString::number(static_cast<int>(realMaxWidth)));
	m_maxFaceSizeHeight->setText(QString::number(static_cast<int>(realMaxHeight)));

	//得到当前最小人脸区域配置（万分比）
	int currentMinFaceWidth{0};
	int currentMinFaceHeight{0};
	ercode = AIFacePackage::GetInstance()->getCurrentMinFaceResolution(m_ip,m_username,m_password,currentMinFaceWidth,currentMinFaceHeight);
	check_http_ercode_no_return(ercode,"Get current min face resolution failed!");
	//计算分辨率角度的最小人脸区域大小
	qreal realMinWidth = rateToResolution(resolutionWidth,currentMinFaceWidth);
	qreal realMinHeight = rateToResolution(resolutionHeight,currentMinFaceHeight);
#ifdef USE_QT_WIDGET
	//设置最小人脸区域框
	m_aiFaceRegion->setCurrentMinFaceSize(realMinWidth,realMinHeight);
#else
	m_dxRegion->setMinFaceSize(QSize(realMinWidth,realMinHeight));
#endif
	//设置最小人脸区域数值
	m_minFaceSizeWidth->setText(QString::number(static_cast<int>(realMinWidth)));
	m_minFaceSizeHeight->setText(QString::number(static_cast<int>(realMinHeight)));

	m_dxRegion->updateAllRect();

	//初始化SnapConfig窗体
	initFaceSnapGUI();

	//初始化InformationOverlay窗体
	initOverlayGui();
}

void AIFaceConfigWidget::unInit()
{
	//停止实时流
	int ercode{0};
	ercode = vncp::VideoPackage::GetInstance()->stopRealStream(m_userID,0);
	check_ercode_no_return(ercode,"Stop real stream failed!");
	//退出设备登录
	ercode = vncp::VideoPackage::GetInstance()->logoutDevice(m_userID);
	check_ercode_no_return(ercode,"Logout device failed!");
}

#ifdef USE_QT_WIDGET
void AIFaceConfigWidget::moveEvent(QMoveEvent *event)
{
	Q_UNUSED(event)

	if(nullptr != m_aiFaceRegion)
		m_aiFaceRegion->setGeometry(this->geometry().x(),this->geometry().y(),adaptx(900),this->geometry().height());
}

void AIFaceConfigWidget::showEvent(QShowEvent *event)
{
	Q_UNUSED(event)

	if(nullptr != m_aiFaceRegion)
	{
		if(true == m_aiFaceRegion->isHidden())
			m_aiFaceRegion->show();
	}
}

void AIFaceConfigWidget::hideEvent(QHideEvent *event)
{
	Q_UNUSED(event)

	if(nullptr != m_aiFaceRegion)
	{
		if(false == m_aiFaceRegion->isHidden())
			m_aiFaceRegion->hide();
	}
}
#endif

void AIFaceConfigWidget::initAreaConfigGUI()
{
#ifdef USE_QT_WIDGET
	//创建用于绘制视频流的窗体
	m_videoWidget = new QWidget(this);
	check_pointer_no_return(m_videoWidget)
	m_videoWidget->setGeometry(adaptxywh(50,50,800,600));
	m_videoWidget->setUpdatesEnabled(false);
#else
	m_dxRegion = new DXRegion(this);
	check_pointer_no_return(m_dxRegion)
	connect(m_dxRegion,&DXRegion::signal_maxFaceRegionChange,this,&AIFaceConfigWidget::slot_maxFaceRegionChange);
	connect(m_dxRegion,&DXRegion::signal_minFaceRegionChange,this,&AIFaceConfigWidget::slot_minFaceRegionChange);
#endif

	QFont font("Microsoft YaHei",adaptx(10));

	QGroupBox *areaConfigBox{nullptr};
	areaConfigBox = new QGroupBox(this);
	check_pointer_no_return(areaConfigBox)
	areaConfigBox->setGeometry(adaptxywh(900,50,350,175));
	areaConfigBox->setFont(font);
	areaConfigBox->setTitle("Area Config");

	//人脸检测区域
	QLabel *faceDetectRegion{nullptr};
	faceDetectRegion = new QLabel(areaConfigBox);
	check_pointer_no_return(faceDetectRegion)
	faceDetectRegion->setGeometry(adaptxywh(20,25,25,25));
	faceDetectRegion->setStyleSheet("background-color:red;");

	QLabel *faceDetectLabel{nullptr};
	faceDetectLabel = new QLabel(areaConfigBox);
	check_pointer_no_return(faceDetectLabel)
	faceDetectLabel->setGeometry(adaptxywh(60,25,200,25));
	faceDetectLabel->setFont(font);
	faceDetectLabel->setText("Face Detect Region");

	QPushButton *faceDetectRegionDefault{nullptr};
	faceDetectRegionDefault = new QPushButton(areaConfigBox);
	check_pointer_no_return(faceDetectRegionDefault)
	faceDetectRegionDefault->setGeometry(adaptxywh(270,25,60,25));
	faceDetectRegionDefault->setFont(font);
	faceDetectRegionDefault->setText("Default");
	connect(faceDetectRegionDefault,&QPushButton::released,[&]()
	{
		m_dxRegion->setDetectRect(m_defaultFaceDetectRect);
		m_dxRegion->updateAllRect();
	});
\
	//最大人脸大小
	QLabel *maxFaceSize{nullptr};
	maxFaceSize = new QLabel(areaConfigBox);
	check_pointer_no_return(maxFaceSize)
	maxFaceSize->setGeometry(adaptxywh(20,60,25,25));
	maxFaceSize->setStyleSheet("background-color:green;");

	QLabel *maxFaceSizeLabel{nullptr};
	maxFaceSizeLabel = new QLabel(areaConfigBox);
	check_pointer_no_return(maxFaceSizeLabel)
	maxFaceSizeLabel->setGeometry(adaptxywh(60,60,200,25));
	maxFaceSizeLabel->setFont(font);
	maxFaceSizeLabel->setText("Max Face Size");

	m_maxFaceSizeWidth = new QLineEdit(areaConfigBox);
	check_pointer_no_return(m_maxFaceSizeWidth)
	m_maxFaceSizeWidth->setGeometry(adaptxywh(160,60,40,25));
	m_maxFaceSizeWidth->setFont(font);
	m_maxFaceSizeWidth->setText("450");
	m_maxFaceSizeWidth->setFocusPolicy(Qt::NoFocus);

	QLabel *maxFaceSizeX{nullptr};
	maxFaceSizeX = new QLabel(areaConfigBox);
	check_pointer_no_return(maxFaceSizeX)
	maxFaceSizeX->setGeometry(adaptxywh(200,60,20,25));
	maxFaceSizeX->setAlignment(Qt::AlignCenter);
	maxFaceSizeX->setText("*");

	m_maxFaceSizeHeight = new QLineEdit(areaConfigBox);
	check_pointer_no_return(m_maxFaceSizeHeight)
	m_maxFaceSizeHeight->setGeometry(adaptxywh(220,60,40,25));
	m_maxFaceSizeHeight->setFont(font);
	m_maxFaceSizeHeight->setText("440");
	m_maxFaceSizeHeight->setFocusPolicy(Qt::NoFocus);

	QPushButton *maxFaceSizeDefault{nullptr};
	maxFaceSizeDefault = new QPushButton(areaConfigBox);
	check_pointer_no_return(maxFaceSizeDefault)
	maxFaceSizeDefault->setGeometry(adaptxywh(270,60,60,25));
	maxFaceSizeDefault->setFont(font);
	maxFaceSizeDefault->setText("Default");
	connect(maxFaceSizeDefault,&QPushButton::released,[&]()
	{
		m_maxFaceSizeWidth->setText(QString::number(m_defaultMaxFaceSize.width()));
		m_maxFaceSizeHeight->setText(QString::number(m_defaultMaxFaceSize.height()));
		m_dxRegion->setMaxFaceSize(m_defaultMaxFaceSize);
		m_dxRegion->updateAllRect();
	});

	//最小人脸大小
	QLabel *minFaceSize{nullptr};
	minFaceSize = new QLabel(areaConfigBox);
	check_pointer_no_return(minFaceSize)
	minFaceSize->setGeometry(adaptxywh(20,95,25,25));
	minFaceSize->setStyleSheet("background-color:blue;");

	QLabel *minFaceSizeLabel{nullptr};
	minFaceSizeLabel = new QLabel(areaConfigBox);
	check_pointer_no_return(minFaceSizeLabel)
	minFaceSizeLabel->setGeometry(adaptxywh(60,95,200,25));
	minFaceSizeLabel->setFont(font);
	minFaceSizeLabel->setText("Min Face Size");

	m_minFaceSizeWidth = new QLineEdit(areaConfigBox);
	check_pointer_no_return(m_minFaceSizeWidth)
	m_minFaceSizeWidth->setGeometry(adaptxywh(160,95,40,25));
	m_minFaceSizeWidth->setFont(font);
	m_minFaceSizeWidth->setText("60");
	m_minFaceSizeWidth->setFocusPolicy(Qt::NoFocus);

	QLabel *minFaceSizeX{nullptr};
	minFaceSizeX = new QLabel(areaConfigBox);
	check_pointer_no_return(minFaceSizeX)
	minFaceSizeX->setGeometry(adaptxywh(200,95,20,25));
	minFaceSizeX->setAlignment(Qt::AlignCenter);
	minFaceSizeX->setText("*");

	m_minFaceSizeHeight = new QLineEdit(areaConfigBox);
	check_pointer_no_return(m_minFaceSizeHeight)
	m_minFaceSizeHeight->setGeometry(adaptxywh(220,95,40,25));
	m_minFaceSizeHeight->setFont(font);
	m_minFaceSizeHeight->setText("59");
	m_minFaceSizeHeight->setFocusPolicy(Qt::NoFocus);

	QPushButton *minFaceSizeDefault{nullptr};
	minFaceSizeDefault = new QPushButton(areaConfigBox);
	check_pointer_no_return(minFaceSizeDefault)
	minFaceSizeDefault->setGeometry(adaptxywh(270,95,60,25));
	minFaceSizeDefault->setFont(font);
	minFaceSizeDefault->setText("Default");
	connect(minFaceSizeDefault,&QPushButton::released,[&]()
	{
		m_minFaceSizeWidth->setText(QString::number(m_defaultMinFaceSize.width()));
		m_minFaceSizeHeight->setText(QString::number(m_defaultMinFaceSize.height()));
		m_dxRegion->setMinFaceSize(m_defaultMinFaceSize);
		m_dxRegion->updateAllRect();
	});

	//配置保存按钮
	QPushButton *areaConfigSave{nullptr};
	areaConfigSave = new QPushButton(areaConfigBox);
	check_pointer_no_return(areaConfigSave)
	areaConfigSave->setGeometry(adaptxywh(350/2-40,135,80,25));
	areaConfigSave->setFont(font);
	areaConfigSave->setText("Save");
	connect(areaConfigSave,&QPushButton::released,this,&AIFaceConfigWidget::slot_saveRegionConfig);
}

void AIFaceConfigWidget::initFaceSnapGUI()
{
	m_snapConfigWidget = new SnapConfigWidget(this);
	check_pointer_no_return(m_snapConfigWidget)
	m_snapConfigWidget->setUserInfo(m_ip,m_username,m_password);
	m_snapConfigWidget->show();
}

void AIFaceConfigWidget::initOverlayGui()
{
	QFont font("Microsoft YaHei",adaptx(10));

	//初始化overlay界面
	QGroupBox *overlayGroupBox{nullptr};
	overlayGroupBox = new QGroupBox(this);
	check_pointer_no_return(overlayGroupBox)
	overlayGroupBox->setGeometry(adaptxywh(900,590,350,65));
	overlayGroupBox->setTitle("Overlay Config");
	overlayGroupBox->setFont(font);
	overlayGroupBox->show();

	QLabel *overlayTrack{nullptr};
	overlayTrack = new QLabel(overlayGroupBox);
	check_pointer_no_return(overlayTrack)
	overlayTrack->setGeometry(adaptxywh(20,25,100,25));
	overlayTrack->setText("Overlay Track");
	overlayTrack->setFont(font);
	overlayTrack->show();

	QString value;
	HttpManagerErcode ercode = AIFacePackage::GetInstance()->getOverlayConfig(m_ip,m_username,m_password,value);
	check_http_ercode_no_return(ercode,"Get overlay config failed!")
	value = ("false" == value?"Disable":"Enable");

	QComboBox *overlayBox{nullptr};
	overlayBox = new QComboBox(overlayGroupBox);
	check_pointer_no_return(overlayBox)
	overlayBox->setGeometry(adaptxywh(120,25,120,25));
	QStringList values{"Enable","Disable"};
	overlayBox->addItems(values);
	overlayBox->setFont(font);
	overlayBox->setCurrentText(value);
	overlayBox->show();

	QPushButton *areaConfigSave{nullptr};
	areaConfigSave = new QPushButton(overlayGroupBox);
	check_pointer_no_return(areaConfigSave)
	areaConfigSave->setGeometry(adaptxywh(250,25,80,25));
	areaConfigSave->setFont(font);
	areaConfigSave->setText("Save");
	areaConfigSave->show();
	connect(areaConfigSave,&QPushButton::released,[=]()
	{
		func_lock

		QString value = ("Enable" == overlayBox->currentText()?"true":"false");
		HttpManagerErcode ercode = AIFacePackage::GetInstance()->setOverlayConfig(m_ip,m_username,m_password,value);
		check_http_ercode_no_return(ercode,"Set overlay config failed")

		message_output("Save overlay config successfully!");
	});
}

void AIFaceConfigWidget::slot_maxFaceRegionChange(QSizeF size)
{
	m_maxFaceSizeWidth->setText(QString::number(static_cast<int>(size.width()+0.5)));
	m_maxFaceSizeHeight->setText(QString::number(static_cast<int>(size.height()+0.5)));
}

void AIFaceConfigWidget::slot_minFaceRegionChange(QSizeF size)
{
	m_minFaceSizeWidth->setText(QString::number(static_cast<int>(size.width()+0.5)));
	m_minFaceSizeHeight->setText(QString::number(static_cast<int>(size.height()+0.5)));
}

void AIFaceConfigWidget::slot_saveRegionConfig()
{
	func_lock

	HttpManagerErcode ercode{HttpErcode_None};

	//得到人脸探测区域大小（分辨率）
#ifdef USE_QT_WIDGET
	QRectF faceRegion;
	m_aiFaceRegion->getFaceDetectRegion(faceRegion);
#else
	QRect faceRegion;
	m_dxRegion->getFaceDetectRegion(faceRegion);
//	qDebug()<<faceRegion;
#endif
	ercode = AIFacePackage::GetInstance()->setFaceDetectRegion(m_ip,m_username,m_password,
															   resolutionToRate(m_streamResolutionWidth,faceRegion.x()),
															   resolutionToRate(m_streamResolutionHeight,faceRegion.y()),
															   resolutionToRate(m_streamResolutionWidth,faceRegion.width()),
															   resolutionToRate(m_streamResolutionHeight,faceRegion.height()));
	check_http_ercode_no_return(ercode,"Set face detect region failed!");

	//获取当前最大最小人脸大小
#ifdef USE_QT_WIDGET
	QSizeF maxFaceSize;
	QSizeF minFaceSize;
	m_aiFaceRegion->getMaxFaceSize(maxFaceSize);
	m_aiFaceRegion->getMinFaceSize(minFaceSize);
#else
	QSize maxFaceSize;
	QSize minFaceSize;
	m_dxRegion->getMaxFaceSize(maxFaceSize);
	m_dxRegion->getMinFaceSize(minFaceSize);
#endif
	ercode = AIFacePackage::GetInstance()->setFaceFilterSize(m_ip,m_username,m_password,
															 resolutionToRate(m_streamResolutionWidth,maxFaceSize.width()),
															 resolutionToRate(m_streamResolutionHeight,maxFaceSize.height()),
															 resolutionToRate(m_streamResolutionWidth,minFaceSize.width()),
															 resolutionToRate(m_streamResolutionHeight,minFaceSize.height()));
	check_http_ercode_no_return(ercode,"Set face filter size failed!");

	message_output("Save area region config successfully!");
}




