﻿#ifndef AIFACECONFIGWIDGET_H
#define AIFACECONFIGWIDGET_H

#include <QObject>
#include <QWidget>
#include <QLabel>
#include <QComboBox>

QT_BEGIN_NAMESPACE
class QLineEdit;
class AIFaceRegion;
class SnapConfigWidget;
class DXRegion;
QT_END_NAMESPACE

class AIFaceConfigWidget : public QWidget
{
	Q_OBJECT
public:
	explicit AIFaceConfigWidget(QWidget *parent = nullptr);
	~AIFaceConfigWidget();

	/**
	 * @brief init					初始化
	 * @param mac			[in]设备mac地址
	 * @param ip				[in]设备IP地址
	 * @param username	[in]用户名
	 * @param password	[in]密码
	 */
	void init(const QString &mac,const QString &ip,const QString &username,const QString &password);

	/**
	 * @brief unInit			反初始化
	 */
	void unInit();

protected:
#ifdef USE_QT_WIDGET
	void moveEvent(QMoveEvent *event) override;

	void showEvent(QShowEvent *event) override;
	void hideEvent(QHideEvent *event) override;
#endif

private:
	//初始化人脸区域配置GUI
	void initAreaConfigGUI();
	//初始化抓拍配置GUI
	void initFaceSnapGUI();
	//初始化叠加层配置GUI
	void initOverlayGui();

private:
#ifdef USE_QT_WIDGET
	//用于绘制实时流的窗体
	QWidget *m_videoWidget{nullptr};
#endif
	//登录设备后返回的用户ID
	int m_userID{-1};
	//用于绘制人脸配置区域
	AIFaceRegion *m_aiFaceRegion{nullptr};

	QLineEdit *m_maxFaceSizeWidth{nullptr};
	QLineEdit *m_maxFaceSizeHeight{nullptr};
	QLineEdit *m_minFaceSizeWidth{nullptr};
	QLineEdit *m_minFaceSizeHeight{nullptr};

	QString m_ip;
	QString m_username;
	QString m_password;

	SnapConfigWidget *m_snapConfigWidget{nullptr};

	qreal m_streamResolutionWidth{0};
	qreal m_streamResolutionHeight{0};

	//用于绘制实时流的窗体
	DXRegion *m_dxRegion{nullptr};

	QRect m_defaultFaceDetectRect;
	QSize m_defaultMaxFaceSize;
	QSize m_defaultMinFaceSize;

signals:

public slots:
	void slot_maxFaceRegionChange(QSizeF size);
	void slot_minFaceRegionChange(QSizeF size);

	void slot_saveRegionConfig();
};

#endif // AIFACECONFIGWIDGET_H
