﻿#include "src/inc/gui/facedetect/aifaceregion.h"

#include "src/inc/aifacepch.h"
#include "src/inc/gui/facedetect/aifaceregionitem.h"

#include <QLabel>
#include <QPainter>
#include <QMouseEvent>
#include <QPushButton>
#include <QGraphicsScene>
#include <QGraphicsView>

AIFaceRegion::AIFaceRegion(QWidget *parent) : QWidget(parent)
{
	this->setWindowFlags(Qt::FramelessWindowHint|Qt::WindowStaysOnTopHint|Qt::SubWindow);
	this->setAttribute(Qt::WA_TranslucentBackground);
//	this->setAttribute(Qt::WA_DeleteOnClose);
}

void AIFaceRegion::init()
{
	//创建图形视图框架，用来绘制人脸探测框，最大人脸框，最小人脸框
	m_scene = new QGraphicsScene(this);
	check_pointer_no_return(m_scene)
	m_scene->setSceneRect(this->rect());

	m_view = new QGraphicsView(this);
	check_pointer_no_return(m_view)
	m_view->setGeometry(this->rect());
	m_view->setScene(m_scene);
	m_view->setStyleSheet("background: transparent;border:0px");
	m_view->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	m_view->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

	//设置探测区域，最大最小人脸框
	m_aiFaceRegionItem = new AIFaceRegionItem;
	check_pointer_no_return(m_aiFaceRegionItem)
	m_aiFaceRegionItem->setItemSize(QSizeF(this->width(),this->height()));
	m_aiFaceRegionItem->setStreamSize(m_streamSize);
	m_aiFaceRegionItem->setMaxFaceSizeLimit(m_maxFaceRectMaxSize);
	m_aiFaceRegionItem->setMinFaceSizeLimit(m_minFaceRectMaxSize);
	m_aiFaceRegionItem->init();
	connect(m_aiFaceRegionItem,&AIFaceRegionItem::signal_maxFaceRegionChange,this,&AIFaceRegion::signal_maxFaceRegionChange);
	connect(m_aiFaceRegionItem,&AIFaceRegionItem::signal_minFaceRegionChange,this,&AIFaceRegion::signal_minFaceRegionChange);

	m_scene->addItem(m_aiFaceRegionItem);
}

void AIFaceRegion::setStreamSize(const qreal &width, const qreal &height)
{
	m_streamSize = {width,height};
}

void AIFaceRegion::setMaxFaceSizeLimit(const qreal &width, const qreal &height)
{
	m_maxFaceRectMaxSize = {width,height};
}

void AIFaceRegion::setMinFaceSizeLimit(const qreal &width, const qreal &height)
{
	m_minFaceRectMaxSize = {width,height};
}

void AIFaceRegion::setCurrentDetectRect(const qreal &x, const qreal &y, const qreal &width, const qreal &height)
{
	m_aiFaceRegionItem->setCurrentDetectRegion({x,y,width,height});
}

void AIFaceRegion::setCurrentMaxFaceSize(const qreal &width, const qreal &height)
{
	m_aiFaceRegionItem->setCurrentMaxFaceSize({width,height});
}

void AIFaceRegion::setCurrentMinFaceSize(const qreal &width, const qreal &height)
{
	m_aiFaceRegionItem->setCurrentMinFaceSize({width,height});
}

void AIFaceRegion::getFaceDetectRegion(QRectF &rect)
{
	m_aiFaceRegionItem->getFaceDetectRect(rect);
}

void AIFaceRegion::getMaxFaceSize(QSizeF &size)
{
	m_aiFaceRegionItem->getMaxFaceSize(size);
}

void AIFaceRegion::getMinFaceSize(QSizeF &size)
{
	m_aiFaceRegionItem->getMinFaceSize(size);
}

//void AIFaceRegion::paintEvent(QPaintEvent *event)
//{

//	QPainter painter(this);

//	painter.fillRect(this->rect(),QColor(0,0,0,1));		//混合最小透明度，使本全透明窗体鼠标事件不会穿透到下层窗体

//	painter.setPen(QPen(Qt::red,adaptx(2)));
//	painter.drawRect(m_faceDetectRect);
//	painter.save();
//	painter.setBrush(Qt::red);
//	painter.drawRect(m_faceDetectRectTop);
//	painter.drawRect(m_faceDetectRectLeft);
//	painter.drawRect(m_faceDetectRectRight);
//	painter.drawRect(m_faceDetectRectBottom);
//	painter.restore();

//	painter.setPen(QPen(Qt::green,adaptx(2)));
//	painter.drawRect(m_maxFaceRect);
//	painter.save();
//	painter.setBrush(Qt::green);
//	painter.drawRect(m_maxFaceRectTop);
//	painter.drawRect(m_maxFaceRectLeft);
//	painter.drawRect(m_maxFaceRectRight);
//	painter.drawRect(m_maxFaceRectBottom);
//	painter.restore();

//	painter.setPen(QPen(Qt::blue,adaptx(2)));
//	painter.drawRect(m_minFaceRect);
//	painter.save();
//	painter.setBrush(Qt::blue);
//	painter.drawRect(m_minFaceRectTop);
//	painter.drawRect(m_minFaceRectLeft);
//	painter.drawRect(m_minFaceRectRight);
//	painter.drawRect(m_minFaceRectBottom);
//	painter.restore();
//}

//void AIFaceRegion::mousePressEvent(QMouseEvent *event)
//{
//	qDebug()<<"region press";

//	m_pressPos=event->pos();
//	m_lastMovePos=event->pos();

//	if(true == m_faceDetectRectTop.contains(m_pressPos))
//		m_selectRect=FaceDetectRectTop;
//	else if(true == m_faceDetectRectLeft.contains(m_pressPos))
//		m_selectRect=FaceDetectRectLeft;
//	else if(true == m_faceDetectRectRight.contains(m_pressPos))
//		m_selectRect=FaceDetectRectRight;
//	else if(true == m_faceDetectRectBottom.contains(m_pressPos))
//		m_selectRect=FaceDetectRectBottom;
//	else if(true == m_maxFaceRectTop.contains(m_pressPos))
//		m_selectRect=MaxFaceRectTop;
//	else if(true == m_maxFaceRectLeft.contains(m_pressPos))
//		m_selectRect=MaxFaceRectLeft;
//	else if(true == m_maxFaceRectRight.contains(m_pressPos))
//		m_selectRect=MaxFaceRectRight;
//	else if(true == m_maxFaceRectBottom.contains(m_pressPos))
//		m_selectRect=MaxFaceRectBottom;
//	else if(true == m_minFaceRectTop.contains(m_pressPos))
//		m_selectRect=MinFaceRectTop;
//	else if(true == m_minFaceRectLeft.contains(m_pressPos))
//		m_selectRect=MinFaceRectLeft;
//	else if(true == m_minFaceRectRight.contains(m_pressPos))
//		m_selectRect=MinFaceRectRight;
//	else if(true == m_minFaceRectBottom.contains(m_pressPos))
//		m_selectRect=MinFaceRectBottom;
//	else if(true == m_minFaceRect.contains(m_pressPos))
//		m_selectRect=MinFaceRect;
//	else if(true == m_maxFaceRect.contains(m_pressPos))
//		m_selectRect=MaxFaceRect;
//	else if(true == m_faceDetectRect.contains(m_pressPos))
//		m_selectRect=FaceDetectRect;

	//如果鼠标在最小人脸区域按下，取各边向内2像素，因为鼠标在矩形边缘按下时用来调整矩形大小，留一定像素距离判断鼠标是否在矩形边缘按下
//	if(true == QRect(m_minFaceRect.x()+adaptx(2),m_minFaceRect.y()+adapty(2),m_minFaceRect.width()-adaptx(4),m_minFaceRect.height()-adapty(4)).contains(event->pos()))
//		m_selectRect=MinFaceRect;
//	else if(true == QRect(m_minFaceRect.x(),m_minFaceRect.y()-adapty(2),m_minFaceRect.width(),))
//	else if(true == QRect(m_maxFaceRect.x()+adaptx(2),m_maxFaceRect.y()+adapty(2),m_maxFaceRect.width()-adaptx(4),m_maxFaceRect.height()-adapty(4)).contains(event->pos()))
//		m_selectRect=MaxFaceRect;
//	else if(true == QRect(m_faceDetectRect.x()+adaptx(2),m_faceDetectRect.y()+adapty(2),m_faceDetectRect.width()-adaptx(4),m_faceDetectRect.height()-adapty(4)).contains(event->pos()))
//		m_selectRect=FaceDetectRect;
//}

//void AIFaceRegion::mouseMoveEvent(QMouseEvent *event)
//{
//	qDebug()<<"region move";

//	QPoint movePos=event->pos();
//	QPoint length = movePos-m_lastMovePos;

//	const int adjustRectWidth=adaptx(originAdjustRectWidth);
//	const int adjustRectHeght=adapty(originAdjustRectHeight);

//	switch(m_selectRect)
//	{
//	case MinFaceRect:
////		m_minFaceRect.moveTo(m_minFaceRect.topLeft()+event->pos());
//		break;
//	case MinFaceRectTop:
//		if(m_minFaceRect.height()>=m_minFaceRectMaxSize.height()&&length.y()<0);		//如果最小人脸区域的高度已经到达最大，用户行为还想扩大
//		else
//		{
//			if(qAbs(m_minFaceRect.top()+length.y()-m_minFaceRect.bottom())>m_minFaceRectMaxSize.height())
//			{
//				m_minFaceRect.setTop(m_minFaceRect.bottom()-m_minFaceRectMaxSize.height());
//				m_minFaceRectTop=QRect(m_minFaceRect.x()+m_minFaceRect.width()/2-adjustRectWidth,m_minFaceRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			}
//			else
//			{
//				//上调节块移动
//				m_minFaceRectTop.moveTop(m_minFaceRectTop.y()+length.y());
//				//最小人脸区域改变大小
//				m_minFaceRect.setY(m_minFaceRect.y()+length.y());
//			}

//			//左调节块移动
//			m_minFaceRectLeft=QRect(m_minFaceRect.x()-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			//右调节块移动
//			m_minFaceRectRight=QRect(m_minFaceRect.x()+m_minFaceRect.width()-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);

//			emit signal_minFaceRegionChange(m_minFaceRect.size());
//		}
//		break;
//	case MinFaceRectLeft:
//		if(m_minFaceRect.width()>=m_minFaceRectMaxSize.width()&&length.x()<0);		//如果最小人脸区域的宽度已经到达最大，用户行为还想扩大
//		else
//		{
//			if(qAbs(m_minFaceRect.left()+length.x()-m_minFaceRect.right())>m_minFaceRectMaxSize.width())
//			{
//				m_minFaceRect.setLeft(m_minFaceRect.right()-m_minFaceRectMaxSize.width());
//				m_minFaceRectLeft=QRect(m_minFaceRect.x()-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			}
//			else
//			{
//				//左调节块移动
//				m_minFaceRectLeft.moveLeft(m_minFaceRectLeft.x()+length.x());
//				//最小人脸区域改变大小
//				m_minFaceRect.setX(m_minFaceRect.x()+length.x());
//			}

//			//上调节块移动
//			m_minFaceRectTop=QRect(m_minFaceRect.x()+m_minFaceRect.width()/2-adjustRectWidth,m_minFaceRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			//下调节块移动
//			m_minFaceRectBottom=QRect(m_minFaceRect.x()+m_minFaceRect.width()/2-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);

//			emit signal_minFaceRegionChange(m_minFaceRect.size());
//		}
//		break;
//	case MinFaceRectRight:
//		if(m_minFaceRect.width()>=m_minFaceRectMaxSize.width()&&length.x()>0);		//如果最小人脸区域的宽度已经到达最大，用户行为还想扩大
//		else
//		{
//			if(m_minFaceRect.right()+length.x()-m_minFaceRect.x()>m_minFaceRectMaxSize.width())		//如果拖动的大小超过了最大区域，直接将其设置到最大值
//			{
//				m_minFaceRect.setRight(m_minFaceRect.x()+m_minFaceRectMaxSize.width());
//				m_minFaceRectRight=QRect(m_minFaceRect.x()+m_minFaceRect.width()-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			}
//			else
//			{
//				//右调节块移动
//				m_minFaceRectRight.moveLeft(m_minFaceRectRight.x()+length.x());
//				//最小人脸区域改变大小
//				m_minFaceRect.setRight(m_minFaceRect.right()+length.x());
//			}
//			//上调节块移动
//			m_minFaceRectTop=QRect(m_minFaceRect.x()+m_minFaceRect.width()/2-adjustRectWidth,m_minFaceRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			//下调节块移动
//			m_minFaceRectBottom=QRect(m_minFaceRect.x()+m_minFaceRect.width()/2-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);

//			emit signal_minFaceRegionChange(m_minFaceRect.size());
//		}
//		break;
//	case MinFaceRectBottom:
//		if(m_minFaceRect.height()>=m_minFaceRectMaxSize.height()&&length.y()>0);		//如果最小人脸区域的高度已经到达最大，用户行为还想扩大
//		else
//		{
//			if(m_minFaceRect.bottom()+length.y()-m_minFaceRect.y()>m_minFaceRectMaxSize.height())
//			{
//				m_minFaceRect.setBottom(m_minFaceRect.y()+m_minFaceRectMaxSize.height());
//				m_minFaceRectBottom=QRect(m_minFaceRect.x()+m_minFaceRect.width()/2-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			}
//			else
//			{
//				//上调节块移动
//				m_minFaceRectBottom.moveBottom(m_minFaceRectBottom.bottom()+length.y());
//				//最小人脸区域改变大小
//				m_minFaceRect.setBottom(m_minFaceRect.bottom()+length.y());
//			}
//			//左调节块移动
//			m_minFaceRectLeft=QRect(m_minFaceRect.x()-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			//右调节块移动
//			m_minFaceRectRight=QRect(m_minFaceRect.x()+m_minFaceRect.width()-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);

//			emit signal_minFaceRegionChange(m_minFaceRect.size());
//		}
//		break;
//	case MaxFaceRect:
//		break;
//	case MaxFaceRectTop:
//		if(m_maxFaceRect.height()>=m_maxFaceRectMaxSize.height()&&length.y()<0);		//如果最小人脸区域的高度已经到达最大，用户行为还想扩大
//		else
//		{
//			if(qAbs(m_maxFaceRect.top()+length.y()-m_maxFaceRect.bottom())>m_maxFaceRectMaxSize.height())
//			{
//				m_maxFaceRect.setTop(m_maxFaceRect.bottom()-m_maxFaceRectMaxSize.height());
//				m_maxFaceRectTop=QRect(m_maxFaceRect.x()+m_maxFaceRect.width()/2-adjustRectWidth,m_maxFaceRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			}
//			else
//			{
//				//上调节块移动
//				m_maxFaceRectTop.moveTop(m_maxFaceRectTop.y()+length.y());
//				//最小人脸区域改变大小
//				m_maxFaceRect.setY(m_maxFaceRect.y()+length.y());
//			}
//			//左调节块移动
//			m_maxFaceRectLeft=QRect(m_maxFaceRect.x()-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			//右调节块移动
//			m_maxFaceRectRight=QRect(m_maxFaceRect.x()+m_maxFaceRect.width()-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);

//			emit signal_maxFaceRegionChange(m_minFaceRect.size());
//		}
//		break;
//	case MaxFaceRectLeft:
//		if(m_maxFaceRect.width()>=m_maxFaceRectMaxSize.width()&&length.x()<0);		//如果最小人脸区域的宽度已经到达最大，用户行为还想扩大
//		else
//		{
//			if(qAbs(m_maxFaceRect.left()+length.x()-m_maxFaceRect.right())>m_maxFaceRectMaxSize.width())
//			{
//				m_maxFaceRect.setLeft(m_maxFaceRect.right()-m_maxFaceRectMaxSize.width());
//				m_maxFaceRectLeft=QRect(m_maxFaceRect.x()-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			}
//			else
//			{
//				//左调节块移动
//				m_maxFaceRectLeft.moveLeft(m_maxFaceRectLeft.x()+length.x());
//				//最小人脸区域改变大小
//				m_maxFaceRect.setX(m_maxFaceRect.x()+length.x());
//			}
//			//上调节块移动
//			m_maxFaceRectTop=QRect(m_maxFaceRect.x()+m_maxFaceRect.width()/2-adjustRectWidth,m_maxFaceRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			//下调节块移动
//			m_maxFaceRectBottom=QRect(m_maxFaceRect.x()+m_maxFaceRect.width()/2-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);

//			emit signal_maxFaceRegionChange(m_minFaceRect.size());
//		}
//		break;
//	case MaxFaceRectRight:
//		if(m_maxFaceRect.width()>=m_maxFaceRectMaxSize.width()&&length.x()>0);		//如果最小人脸区域的宽度已经到达最大，用户行为还想扩大
//		else
//		{
//			if(m_maxFaceRect.right()+length.x()-m_maxFaceRect.x()>m_maxFaceRectMaxSize.width())		//如果拖动的大小超过了最大区域，直接将其设置到最大值
//			{
//				m_maxFaceRect.setRight(m_maxFaceRect.x()+m_maxFaceRectMaxSize.width());
//				m_maxFaceRectRight=QRect(m_maxFaceRect.x()+m_maxFaceRect.width()-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			}
//			else
//			{
//				//右调节块移动
//				m_maxFaceRectRight.moveLeft(m_maxFaceRectRight.x()+length.x());
//				//最小人脸区域改变大小
//				m_maxFaceRect.setRight(m_maxFaceRect.right()+length.x());
//			}
//			//上调节块移动
//			m_maxFaceRectTop=QRect(m_maxFaceRect.x()+m_maxFaceRect.width()/2-adjustRectWidth,m_maxFaceRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			//下调节块移动
//			m_maxFaceRectBottom=QRect(m_maxFaceRect.x()+m_maxFaceRect.width()/2-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);

//			emit signal_maxFaceRegionChange(m_minFaceRect.size());
//		}
//		break;
//	case MaxFaceRectBottom:
//		if(m_maxFaceRect.height()>=m_maxFaceRectMaxSize.height()&&length.y()>0);		//如果最小人脸区域的高度已经到达最大，用户行为还想扩大
//		else
//		{
//			if(m_maxFaceRect.bottom()+length.y()-m_maxFaceRect.y()>m_maxFaceRectMaxSize.height())
//			{
//				m_maxFaceRect.setBottom(m_maxFaceRect.y()+m_maxFaceRectMaxSize.height());
//				m_maxFaceRectBottom=QRect(m_maxFaceRect.x()+m_maxFaceRect.width()/2-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			}
//			else
//			{
//				//上调节块移动
//				m_maxFaceRectBottom.moveBottom(m_maxFaceRectBottom.bottom()+length.y());
//				//最小人脸区域改变大小
//				m_maxFaceRect.setBottom(m_maxFaceRect.bottom()+length.y());
//			}

//			//左调节块移动
//			m_maxFaceRectLeft=QRect(m_minFaceRect.x()-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			//右调节块移动
//			m_maxFaceRectRight=QRect(m_maxFaceRect.x()+m_maxFaceRect.width()-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);

//			emit signal_maxFaceRegionChange(m_minFaceRect.size());
//		}
//		break;
//	case FaceDetectRect:
//		break;
//	case FaceDetectRectTop:
//		if(m_faceDetectRect.height()>=m_faceDetectRectMaxSize.height()&&length.y()<0);		//如果最小人脸区域的高度已经到达最大，用户行为还想扩大
//		else
//		{
//			if(qAbs(m_faceDetectRect.top()+length.y()-m_faceDetectRect.bottom())>m_faceDetectRectMaxSize.height())
//			{
//				m_faceDetectRect.setTop(m_faceDetectRect.bottom()-m_faceDetectRectMaxSize.height());
//				m_faceDetectRectTop=QRect(m_faceDetectRect.x()+m_faceDetectRect.width()/2-adjustRectWidth,m_faceDetectRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			}
//			else
//			{
//				//上调节块移动
//				m_faceDetectRectTop.moveTop(m_faceDetectRectTop.y()+length.y());
//				//最小人脸区域改变大小
//				m_faceDetectRect.setY(m_faceDetectRect.y()+length.y());
//			}

//			//左调节块移动
//			m_faceDetectRectLeft=QRect(m_faceDetectRect.x()-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			//右调节块移动
//			m_faceDetectRectRight=QRect(m_faceDetectRect.x()+m_faceDetectRect.width()-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//		}
//		break;
//	case FaceDetectRectLeft:
//		if(m_faceDetectRect.width()>=m_faceDetectRectMaxSize.width()&&length.x()<0);		//如果最小人脸区域的宽度已经到达最大，用户行为还想扩大
//		else
//		{
//			if(qAbs(m_faceDetectRect.left()+length.x()-m_faceDetectRect.right())>m_faceDetectRectMaxSize.width())
//			{
//				m_faceDetectRect.setLeft(m_faceDetectRect.right()-m_faceDetectRectMaxSize.width());
//				m_faceDetectRectLeft=QRect(m_faceDetectRect.x()-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			}
//			else
//			{
//				//左调节块移动
//				m_faceDetectRectLeft.moveLeft(m_faceDetectRectLeft.x()+length.x());
//				//最小人脸区域改变大小
//				m_faceDetectRect.setX(m_faceDetectRect.x()+length.x());
//			}

//			//上调节块移动
//			m_faceDetectRectTop=QRect(m_faceDetectRect.x()+m_faceDetectRect.width()/2-adjustRectWidth,m_faceDetectRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			//下调节块移动
//			m_faceDetectRectBottom=QRect(m_faceDetectRect.x()+m_faceDetectRect.width()/2-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//		}
//		break;
//	case FaceDetectRectRight:
//		if(m_faceDetectRect.width()>=m_faceDetectRectMaxSize.width()&&length.x()>0);		//如果最小人脸区域的宽度已经到达最大，用户行为还想扩大
//		else
//		{
//			if(m_faceDetectRect.right()+length.x()-m_faceDetectRect.x()>m_faceDetectRectMaxSize.width())		//如果拖动的大小超过了最大区域，直接将其设置到最大值
//			{
//				m_faceDetectRect.setRight(m_faceDetectRect.x()+m_faceDetectRectMaxSize.width());
//				m_faceDetectRectRight=QRect(m_faceDetectRect.x()+m_faceDetectRect.width()-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			}
//			else
//			{
//				//右调节块移动
//				m_faceDetectRectRight.moveLeft(m_faceDetectRectRight.x()+length.x());
//				//最小人脸区域改变大小
//				m_faceDetectRect.setRight(m_faceDetectRect.right()+length.x());
//			}

//			//上调节块移动
//			m_faceDetectRectTop=QRect(m_faceDetectRect.x()+m_faceDetectRect.width()/2-adjustRectWidth,m_faceDetectRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			//下调节块移动
//			m_faceDetectRectBottom=QRect(m_faceDetectRect.x()+m_faceDetectRect.width()/2-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//		}
//		break;
//	case FaceDetectRectBottom:
//		if(m_faceDetectRect.height()>=m_faceDetectRectMaxSize.height()&&length.y()>0);		//如果最小人脸区域的高度已经到达最大，用户行为还想扩大
//		else
//		{
//			if(m_faceDetectRect.bottom()+length.y()-m_faceDetectRect.y()>m_faceDetectRectMaxSize.height())
//			{
//				m_faceDetectRect.setBottom(m_faceDetectRect.y()+m_faceDetectRectMaxSize.height());
//				m_faceDetectRectBottom=QRect(m_faceDetectRect.x()+m_faceDetectRect.width()/2-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			}
//			else
//			{
//				//上调节块移动
//				m_faceDetectRectBottom.moveBottom(m_faceDetectRectBottom.bottom()+length.y());
//				//最小人脸区域改变大小
//				m_faceDetectRect.setBottom(m_faceDetectRect.bottom()+length.y());
//			}

//			//左调节块移动
//			m_faceDetectRectLeft=QRect(m_faceDetectRect.x()-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//			//右调节块移动
//			m_faceDetectRectRight=QRect(m_faceDetectRect.x()+m_faceDetectRect.width()-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
//		}
//		break;
//	default:
//		break;
//	}

//	update();

//	m_lastMovePos=movePos;
//}

//void AIFaceRegion::mouseReleaseEvent(QMouseEvent *event)
//{
//	qDebug()<<"region release";
//}

//bool AIFaceRegion::nativeEvent(const QByteArray &eventType, void *message, long *result)
//{
//	MSG *msg=(MSG *)message;
//	switch(msg->message)
//	{
//	case WM_PAINT:
//		qDebug()<<"paint";
//		return false;
//	}
//}
