#ifndef AIFACEREGION_H
#define AIFACEREGION_H

#include <QWidget>
#include <QRect>
#include <QSize>

QT_BEGIN_NAMESPACE
class QGraphicsScene;
class QGraphicsView;
class AIFaceRegionItem;
QT_END_NAMESPACE

class AIFaceRegion : public QWidget
{
	Q_OBJECT
public:
	explicit AIFaceRegion(QWidget *parent = nullptr);

	/**
	 * @brief init		初始化界面元素
	 */
	void init();

	/**
	 * @brief setStreamSize		设置实时流分辨率大小
	 * @param width					[in]宽度
	 * @param height					[in]高度
	 */
	void setStreamSize(const qreal &width,const qreal &height);

	/**
	 * @brief setMaxFaceSize		设置最大人脸识别大小限制
	 * @param width					[in]宽度
	 * @param height					[in]高度
	 */
	void setMaxFaceSizeLimit(const qreal &width,const qreal &height);

	/**
	 * @brief setMinFaceSize		设置最小人脸识别大小限制
	 * @param width					[in]宽度
	 * @param height					[in]高度
	 */
	void setMinFaceSizeLimit(const qreal &width,const qreal &height);

	/**
	 * @brief setCurrentDetectRect		设置当前人脸探测区域
	 * @param x										[in]x
	 * @param y										[in]y
	 * @param width								[in]宽度
	 * @param height								[in]高度
	 */
	void setCurrentDetectRect(const qreal &x,const qreal &y,const qreal &width,const qreal &height);

	/**
	 * @brief setCurrentMaxFaceSize			设置当前人脸最大识别大小
	 * @param width									[in]宽度
	 * @param height									[in]高度
	 */
	void setCurrentMaxFaceSize(const qreal &width,const qreal &height);

	/**
	 * @brief setCurrentMinFaceSize			设置当前最小人脸识别大小
	 * @param width									[in]宽度
	 * @param height									[in]高度
	 */
	void setCurrentMinFaceSize(const qreal &width,const qreal &height);

	/**
	 * @brief setFaceDetectRegion		设置人脸探测区域
	 * @param rect								[out]区域
	 */
	void getFaceDetectRegion(QRectF &rect);

	/**
	 * @brief getMaxFaceSize			得到最大人脸区域
	 * @param size							[out]大小
	 */
	void getMaxFaceSize(QSizeF &size);

	/**
	 * @brief getMinFaceSize			得到最小人脸识别区域
	 * @param size							[out]大小
	 */
	void getMinFaceSize(QSizeF &size);

protected:
//	void paintEvent(QPaintEvent *event) override;
//	bool nativeEvent(const QByteArray &eventType, void *message, long *result) override;

//	void mousePressEvent(QMouseEvent *event) override;
//	void mouseMoveEvent(QMouseEvent *event) override;
//	void mouseReleaseEvent(QMouseEvent *event) override;

private:
	QSizeF m_streamSize;
	QSizeF m_maxFaceRectMaxSize;
	QSizeF m_minFaceRectMaxSize;

	QPoint m_pressPos;
	QPoint m_lastMovePos;

	QGraphicsScene *m_scene{nullptr};
	QGraphicsView *m_view{nullptr};
	AIFaceRegionItem *m_aiFaceRegionItem{nullptr};

signals:
	void signal_maxFaceRegionChange(QSizeF);
	void signal_minFaceRegionChange(QSizeF);

public slots:
};

#endif // AIFACEREGION_H
