﻿#include "src/inc/gui/facedetect/aifaceregionitem.h"

#include "src/inc/aifacepch.h"

#include <QPainter>
#include <QPen>
#include <QGraphicsSceneMouseEvent>

const qreal originAdjustRectWidth{3.0};
const qreal originAdjustRectHeight{3.0};

AIFaceRegionItem::AIFaceRegionItem()
{

}

void AIFaceRegionItem::setItemSize(const QSizeF &size)
{
	m_itemSize = size;
}

void AIFaceRegionItem::setStreamSize(const QSizeF &size)
{
	m_streamSize = size;

	m_xRate = adaptx(800)/m_streamSize.width();
	m_yRate = adapty(600)/m_streamSize.height();
}

void AIFaceRegionItem::setMaxFaceSizeLimit(const QSizeF &size)
{
	m_maxFaceRectMaxSize = QSizeF(size.width()*m_xRate,size.height()*m_yRate);
}

void AIFaceRegionItem::setMinFaceSizeLimit(const QSizeF &size)
{
	m_minFaceRectMaxSize = QSizeF(size.width()*m_xRate,size.height()*m_yRate);
}

void AIFaceRegionItem::setCurrentDetectRegion(const QRectF &rect)
{
	const qreal adjustRectWidth = adaptx(originAdjustRectWidth);
	const qreal adjustRectHeght = adapty(originAdjustRectHeight);

	m_faceDetectRect = QRectF(rect.x()*m_xRate+adaptx(50),rect.y()*m_yRate+adapty(50),rect.width()*m_xRate,rect.height()*m_yRate);
	m_faceDetectRectTop = QRectF(m_faceDetectRect.x()+m_faceDetectRect.width()/2-adjustRectWidth,m_faceDetectRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
	m_faceDetectRectBottom = QRectF(m_faceDetectRect.x()+m_faceDetectRect.width()/2-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
	m_faceDetectRectLeft = QRectF(m_faceDetectRect.x()-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
	m_faceDetectRectRight = QRectF(m_faceDetectRect.x()+m_faceDetectRect.width()-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);

	update();
}

void AIFaceRegionItem::setCurrentMaxFaceSize(const QSizeF &size)
{
	const qreal adjustRectWidth = adaptx(originAdjustRectWidth);
	const qreal adjustRectHeght = adapty(originAdjustRectHeight);

	m_maxFaceRect = QRectF(adaptx(50+800/2-size.width()*m_xRate/2),adapty(50+600/2-size.height()*m_yRate/2),size.width()*m_xRate,size.height()*m_yRate);
	m_maxFaceRectTop = QRectF(m_maxFaceRect.x()+m_maxFaceRect.width()/2-adjustRectWidth,m_maxFaceRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
	m_maxFaceRectBottom = QRectF(m_maxFaceRect.x()+m_maxFaceRect.width()/2-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
	m_maxFaceRectLeft = QRectF(m_maxFaceRect.x()-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
	m_maxFaceRectRight = QRectF(m_maxFaceRect.x()+m_maxFaceRect.width()-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);

	update();
}

void AIFaceRegionItem::setCurrentMinFaceSize(const QSizeF &size)
{
	const qreal adjustRectWidth = adaptx(originAdjustRectWidth);
	const qreal adjustRectHeght = adapty(originAdjustRectHeight);

	m_minFaceRect = QRectF(adaptxywh(50+800/2-size.width()*m_xRate/2,50+600/2-size.height()*m_yRate/2,size.width()*m_xRate,size.height()*m_yRate));
	m_minFaceRectTop = QRectF(m_minFaceRect.x()+m_minFaceRect.width()/2-adjustRectWidth,m_minFaceRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
	m_minFaceRectBottom = QRectF(m_minFaceRect.x()+m_minFaceRect.width()/2-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
	m_minFaceRectLeft = QRectF(m_minFaceRect.x()-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
	m_minFaceRectRight = QRectF(m_minFaceRect.x()+m_minFaceRect.width()-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);

	update();
}

void AIFaceRegionItem::init()
{
	const qreal adjustRectWidth = adaptx(originAdjustRectWidth);
	const qreal adjustRectHeght = adapty(originAdjustRectHeight);

	m_faceDetectRectMaxSize = QSizeF(adaptxy(800,600));

	m_faceDetectRect = QRectF(adaptxywh(50,50,800,600));
	m_faceDetectRectTop = QRectF(m_faceDetectRect.x()+m_faceDetectRect.width()/2-adjustRectWidth,m_faceDetectRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
	m_faceDetectRectBottom = QRectF(m_faceDetectRect.x()+m_faceDetectRect.width()/2-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
	m_faceDetectRectLeft = QRectF(m_faceDetectRect.x()-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
	m_faceDetectRectRight = QRectF(m_faceDetectRect.x()+m_faceDetectRect.width()-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);

	m_maxFaceRect = QRectF(adaptx(50+800/2-m_maxFaceRectMaxSize.width()/2),adapty(50+600/2-m_maxFaceRectMaxSize.height()/2),m_maxFaceRectMaxSize.width(),m_maxFaceRectMaxSize.height());
	m_maxFaceRectTop = QRectF(m_maxFaceRect.x()+m_maxFaceRect.width()/2-adjustRectWidth,m_maxFaceRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
	m_maxFaceRectBottom = QRectF(m_maxFaceRect.x()+m_maxFaceRect.width()/2-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
	m_maxFaceRectLeft = QRectF(m_maxFaceRect.x()-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
	m_maxFaceRectRight = QRectF(m_maxFaceRect.x()+m_maxFaceRect.width()-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);

	m_minFaceRect = QRectF(adaptxywh(50+800/2-m_minFaceRectMaxSize.width()/2,50+600/2-m_minFaceRectMaxSize.height()/2,m_minFaceRectMaxSize.width(),m_minFaceRectMaxSize.height()));
	m_minFaceRectTop = QRectF(m_minFaceRect.x()+m_minFaceRect.width()/2-adjustRectWidth,m_minFaceRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
	m_minFaceRectBottom = QRectF(m_minFaceRect.x()+m_minFaceRect.width()/2-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
	m_minFaceRectLeft = QRectF(m_minFaceRect.x()-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
	m_minFaceRectRight = QRectF(m_minFaceRect.x()+m_minFaceRect.width()-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
}

void AIFaceRegionItem::getFaceDetectRect(QRectF &rect)
{
	rect = {(m_faceDetectRect.x()-50)/m_xRate,(m_faceDetectRect.y()-50)/m_yRate,m_faceDetectRect.width()/m_xRate,m_faceDetectRect.height()/m_yRate};
}

void AIFaceRegionItem::getMaxFaceSize(QSizeF &size)
{
	size = {m_maxFaceRect.width()/m_xRate,m_maxFaceRect.height()/m_yRate};
}

void AIFaceRegionItem::getMinFaceSize(QSizeF &size)
{
	size = {m_minFaceRect.width()/m_xRate,m_minFaceRect.height()/m_yRate};
}

QRectF AIFaceRegionItem::boundingRect() const
{
	return QRectF(0,0,m_itemSize.width(),m_itemSize.height());
}

void AIFaceRegionItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
	Q_UNUSED(option)
	Q_UNUSED(widget)

	painter->fillRect(this->boundingRect(),QColor(0,0,0,1));		//混合最小透明度，使本全透明窗体鼠标事件不会穿透到下层窗体

	painter->setPen(QPen(Qt::red,adaptx(2)));
	painter->drawRect(m_faceDetectRect);
	painter->save();
	painter->setBrush(Qt::red);
	painter->drawRect(m_faceDetectRectTop);
	painter->drawRect(m_faceDetectRectLeft);
	painter->drawRect(m_faceDetectRectRight);
	painter->drawRect(m_faceDetectRectBottom);
	painter->restore();

	painter->setPen(QPen(Qt::green,adaptx(2)));
	painter->drawRect(m_maxFaceRect);
	painter->save();
	painter->setBrush(Qt::green);
	painter->drawRect(m_maxFaceRectTop);
	painter->drawRect(m_maxFaceRectLeft);
	painter->drawRect(m_maxFaceRectRight);
	painter->drawRect(m_maxFaceRectBottom);
	painter->restore();

	painter->setPen(QPen(Qt::blue,adaptx(2)));
	painter->drawRect(m_minFaceRect);
	painter->save();
	painter->setBrush(Qt::blue);
	painter->drawRect(m_minFaceRectTop);
	painter->drawRect(m_minFaceRectLeft);
	painter->drawRect(m_minFaceRectRight);
	painter->drawRect(m_minFaceRectBottom);
	painter->restore();
}

void AIFaceRegionItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
	m_pressPos = event->pos();
	m_lastMovePos = event->pos();

	if(true == m_faceDetectRectTop.contains(m_pressPos))
		m_selectRect = FaceDetectRectTop;
	else if(true == m_faceDetectRectLeft.contains(m_pressPos))
		m_selectRect = FaceDetectRectLeft;
	else if(true == m_faceDetectRectRight.contains(m_pressPos))
		m_selectRect = FaceDetectRectRight;
	else if(true == m_faceDetectRectBottom.contains(m_pressPos))
		m_selectRect = FaceDetectRectBottom;
	else if(true == m_maxFaceRectTop.contains(m_pressPos))
		m_selectRect = MaxFaceRectTop;
	else if(true == m_maxFaceRectLeft.contains(m_pressPos))
		m_selectRect = MaxFaceRectLeft;
	else if(true == m_maxFaceRectRight.contains(m_pressPos))
		m_selectRect = MaxFaceRectRight;
	else if(true == m_maxFaceRectBottom.contains(m_pressPos))
		m_selectRect = MaxFaceRectBottom;
	else if(true == m_minFaceRectTop.contains(m_pressPos))
		m_selectRect = MinFaceRectTop;
	else if(true == m_minFaceRectLeft.contains(m_pressPos))
		m_selectRect = MinFaceRectLeft;
	else if(true == m_minFaceRectRight.contains(m_pressPos))
		m_selectRect = MinFaceRectRight;
	else if(true  ==  m_minFaceRectBottom.contains(m_pressPos))
		m_selectRect = MinFaceRectBottom;
	else if(true == m_minFaceRect.contains(m_pressPos))
		m_selectRect = MinFaceRect;
	else if(true == m_maxFaceRect.contains(m_pressPos))
		m_selectRect = MaxFaceRect;
	else if(true == m_faceDetectRect.contains(m_pressPos))
		m_selectRect = FaceDetectRect;
}

void AIFaceRegionItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
	QPointF movePos = event->pos();
	QPointF length = movePos-m_lastMovePos;

	const qreal adjustRectWidth = adaptx(originAdjustRectWidth);
	const qreal adjustRectHeght = adapty(originAdjustRectHeight);

	switch(m_selectRect)
	{
	case MinFaceRect:
	{
		QRectF futureRect = m_minFaceRect;
		futureRect.moveTo(futureRect.topLeft()+length);
		if(futureRect.top()<adapty(50))
			futureRect.moveTop(adapty(50));
		if(futureRect.left()<adaptx(50))
			futureRect.moveLeft(adaptx(50));
		if(futureRect.right()>adaptx(50+800))
			futureRect.moveRight(adaptx(50+800));
		if(futureRect.bottom()>adapty(50+600))
			futureRect.moveBottom(adapty(50+600));

		m_minFaceRect = futureRect;
		m_minFaceRectTop = QRectF(m_minFaceRect.x()+m_minFaceRect.width()/2-adjustRectWidth,m_minFaceRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
		m_minFaceRectBottom = QRectF(m_minFaceRect.x()+m_minFaceRect.width()/2-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
		m_minFaceRectLeft = QRectF(m_minFaceRect.x()-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
		m_minFaceRectRight = QRectF(m_minFaceRect.x()+m_minFaceRect.width()-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
	}
	break;
	case MinFaceRectTop:
		if(m_minFaceRect.height()<=m_minFaceRectMaxSize.height()&&length.y()>0);		//如果最小人脸区域的高度已经到达最小，用户行为还想缩小
		else if(m_minFaceRect.height()>=m_maxFaceRect.height()&&length.y()<0);			//如果最小人脸区域的高度大于等于最大人脸区域的高度，但用户行为还想扩大
		else
		{
			if(qAbs(m_minFaceRect.top()+length.y()-m_minFaceRect.bottom())<m_minFaceRectMaxSize.height())		//如果调整后的高度将会小于限制值
			{
				m_minFaceRect.setTop(m_minFaceRect.bottom()-m_minFaceRectMaxSize.height());
				m_minFaceRectTop = QRectF(m_minFaceRect.x()+m_minFaceRect.width()/2-adjustRectWidth,m_minFaceRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			}
			else if(qAbs(m_minFaceRect.top()+length.y()-m_minFaceRect.bottom())>m_maxFaceRect.height())		//如果调整后的高度将会大于最大人脸框高度，直接将高度设置为最大人脸框的高度
			{
				m_minFaceRect.setTop(m_minFaceRect.bottom()-m_maxFaceRect.height());
				m_minFaceRectTop = QRectF(m_minFaceRect.x()+m_minFaceRect.width()/2-adjustRectWidth,m_minFaceRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			}
			else
			{
				//上调节块移动
				m_minFaceRectTop.moveTop(m_minFaceRectTop.y()+length.y());
				//最小人脸区域改变大小
				m_minFaceRect.setY(m_minFaceRect.y()+length.y());
			}

			//左调节块移动
			m_minFaceRectLeft = QRectF(m_minFaceRect.x()-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			//右调节块移动
			m_minFaceRectRight = QRectF(m_minFaceRect.x()+m_minFaceRect.width()-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);

			emit signal_minFaceRegionChange(QSizeF(m_minFaceRect.width()/m_xRate,m_minFaceRect.height()/m_yRate));
		}
		break;
	case MinFaceRectLeft:
		if(m_minFaceRect.width()<=m_minFaceRectMaxSize.width()&&length.x()>0);		//如果最小人脸区域的宽度已经到达最小限制值，用户行为还想缩小
		else if(m_minFaceRect.width()>=m_maxFaceRect.width()&&length.x()<0);				//如果最小人脸区域的宽度已经大于等于最大人脸区域的宽度，用户行为还想扩大
		else
		{
			if(qAbs(m_minFaceRect.left()+length.x()-m_minFaceRect.right())<m_minFaceRectMaxSize.width())		//如果最小人脸区域调整后将会小于限制值，直接将最小人脸区域宽度调整为限制值
			{
				m_minFaceRect.setLeft(m_minFaceRect.right()-m_minFaceRectMaxSize.width());
				m_minFaceRectLeft = QRectF(m_minFaceRect.x()-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			}
			else if(qAbs(m_minFaceRect.left()+length.x()-m_minFaceRect.right())>m_maxFaceRect.width())		//如果调整后的宽度将会大于最大人脸区域的宽度,直接将宽度设置为最大人脸区域的宽度
			{
				m_minFaceRect.setLeft(m_minFaceRect.right()-m_maxFaceRect.width());
				m_minFaceRectLeft = QRectF(m_minFaceRect.x()-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			}
			else
			{
				//左调节块移动
				m_minFaceRectLeft.moveLeft(m_minFaceRectLeft.x()+length.x());
				//最小人脸区域改变大小
				m_minFaceRect.setX(m_minFaceRect.x()+length.x());
			}

			//上调节块移动
			m_minFaceRectTop = QRectF(m_minFaceRect.x()+m_minFaceRect.width()/2-adjustRectWidth,m_minFaceRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			//下调节块移动
			m_minFaceRectBottom = QRectF(m_minFaceRect.x()+m_minFaceRect.width()/2-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);

			emit signal_minFaceRegionChange(QSizeF(m_minFaceRect.width()/m_xRate,m_minFaceRect.height()/m_yRate));
		}
		break;
	case MinFaceRectRight:
		if(m_minFaceRect.width()<=m_minFaceRectMaxSize.width()&&length.x()<0);		//如果最小人脸区域的宽度已经达到最小限制值，用户行为还想缩小
		else if(m_minFaceRect.width()>=m_maxFaceRect.width()&&length.x()>0);		//如果最小人脸区域的宽度大于等于最大人脸区域的宽度
		else
		{
			if(m_minFaceRect.right()+length.x()-m_minFaceRect.x()<m_minFaceRectMaxSize.width())		//如果拖动的大小将会超过最小区域限制值，直接将其设置为限制值
			{
				m_minFaceRect.setRight(m_minFaceRect.x()+m_minFaceRectMaxSize.width());
				m_minFaceRectRight = QRectF(m_minFaceRect.x()+m_minFaceRect.width()-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			}
			else if(m_minFaceRect.right()+length.x()-m_minFaceRect.x()>m_maxFaceRect.width())		//如果调整后将会超过最大人脸区域的大小，直接将宽度设置为最大人脸区域的宽度
			{
				m_minFaceRect.setRight(m_minFaceRect.x()+m_maxFaceRect.width());
				m_minFaceRectRight = QRectF(m_minFaceRect.x()+m_minFaceRect.width()-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			}
			else
			{
				//右调节块移动
				m_minFaceRectRight.moveLeft(m_minFaceRectRight.x()+length.x());
				//最小人脸区域改变大小
				m_minFaceRect.setRight(m_minFaceRect.right()+length.x());
			}
			//上调节块移动
			m_minFaceRectTop = QRectF(m_minFaceRect.x()+m_minFaceRect.width()/2-adjustRectWidth,m_minFaceRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			//下调节块移动
			m_minFaceRectBottom = QRectF(m_minFaceRect.x()+m_minFaceRect.width()/2-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);

			emit signal_minFaceRegionChange(QSizeF(m_minFaceRect.width()/m_xRate,m_minFaceRect.height()/m_yRate));
		}
		break;
	case MinFaceRectBottom:
		if(m_minFaceRect.height()<=m_minFaceRectMaxSize.height()&&length.y()<0);		//如果最小人脸区域的高度已经到达最小限制值，用户行为还想缩小
		else if(m_minFaceRect.height()>=m_maxFaceRect.height()&&length.y()>0);
		else
		{
			if(m_minFaceRect.bottom()+length.y()-m_minFaceRect.y()<m_minFaceRectMaxSize.height())		//如果最小人脸区域的高度已经到达最小限制值,直接设为限制值高度
			{
				m_minFaceRect.setBottom(m_minFaceRect.y()+m_minFaceRectMaxSize.height());
				m_minFaceRectBottom = QRectF(m_minFaceRect.x()+m_minFaceRect.width()/2-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			}
			else if(m_minFaceRect.bottom()+length.y()-m_minFaceRect.y()>m_maxFaceRect.height())
			{
				m_minFaceRect.setBottom(m_minFaceRect.top()+m_maxFaceRect.height());
				m_minFaceRectBottom = QRectF(m_minFaceRect.x()+m_minFaceRect.width()/2-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			}
			else
			{
				//上调节块移动
				m_minFaceRectBottom.moveBottom(m_minFaceRectBottom.bottom()+length.y());
				//最小人脸区域改变大小
				m_minFaceRect.setBottom(m_minFaceRect.bottom()+length.y());
			}
			//左调节块移动
			m_minFaceRectLeft = QRectF(m_minFaceRect.x()-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			//右调节块移动
			m_minFaceRectRight = QRectF(m_minFaceRect.x()+m_minFaceRect.width()-adjustRectWidth,m_minFaceRect.y()+m_minFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);

			emit signal_minFaceRegionChange(QSizeF(m_minFaceRect.width()/m_xRate,m_minFaceRect.height()/m_yRate));
		}
		break;
	case MaxFaceRect:
	{
		QRectF futureRect = m_maxFaceRect;
		futureRect.moveTo(futureRect.topLeft()+length);
		if(futureRect.top()<adapty(50))
			futureRect.moveTop(adapty(50));
		if(futureRect.left()<adaptx(50))
			futureRect.moveLeft(adaptx(50));
		if(futureRect.right()>adaptx(50+800))
			futureRect.moveRight(adaptx(50+800));
		if(futureRect.bottom()>adapty(50+600))
			futureRect.moveBottom(adapty(50+600));

		m_maxFaceRect = futureRect;
		m_maxFaceRectTop = QRectF(m_maxFaceRect.x()+m_maxFaceRect.width()/2-adjustRectWidth,m_maxFaceRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
		m_maxFaceRectBottom = QRectF(m_maxFaceRect.x()+m_maxFaceRect.width()/2-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
		m_maxFaceRectLeft = QRectF(m_maxFaceRect.x()-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
		m_maxFaceRectRight = QRectF(m_maxFaceRect.x()+m_maxFaceRect.width()-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
	}
		break;
	case MaxFaceRectTop:
		if(m_maxFaceRect.height()>=m_maxFaceRectMaxSize.height()&&length.y()<0);		//如果最大人脸区域的高度已经到达最大，用户行为还想扩大
		else if(m_maxFaceRect.height()<=m_minFaceRect.height()&&length.y()>0);			//如果最大人脸区域的高度已经达到最小（与最小人脸高度持平）
		else
		{
			if(qAbs(m_maxFaceRect.top()+length.y()-m_maxFaceRect.bottom())>m_maxFaceRectMaxSize.height())		//如果调整后高度超出限制，则直接将高度设置为最大高度
			{
				m_maxFaceRect.setTop(m_maxFaceRect.bottom()-m_maxFaceRectMaxSize.height());
				m_maxFaceRectTop = QRectF(m_maxFaceRect.x()+m_maxFaceRect.width()/2-adjustRectWidth,m_maxFaceRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			}
			else if(qAbs(m_maxFaceRect.top()+length.y()-m_maxFaceRect.bottom())<m_minFaceRect.height())			//如果调整之后高度将小于最小人脸区域大小，那么直接将高度设置为最小人脸高度
			{
				m_maxFaceRect.setTop(m_maxFaceRect.bottom()-m_minFaceRect.height());
				m_maxFaceRectTop = QRectF(m_maxFaceRect.x()+m_maxFaceRect.width()/2-adjustRectWidth,m_maxFaceRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			}
			else
			{
				//上调节块移动
				m_maxFaceRectTop.moveTop(m_maxFaceRectTop.y()+length.y());
				//最小人脸区域改变大小
				m_maxFaceRect.setY(m_maxFaceRect.y()+length.y());
			}

			//左调节块移动
			m_maxFaceRectLeft = QRectF(m_maxFaceRect.x()-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			//右调节块移动
			m_maxFaceRectRight = QRectF(m_maxFaceRect.x()+m_maxFaceRect.width()-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);

			emit signal_maxFaceRegionChange(QSizeF(m_maxFaceRect.width()/m_xRate,m_maxFaceRect.height()/m_yRate));
		}
		break;
	case MaxFaceRectLeft:
		if(m_maxFaceRect.width()>=m_maxFaceRectMaxSize.width()&&length.x()<0);		//如果最大人脸区域的宽度已经到达最大，用户行为还想扩大
		else if(m_maxFaceRect.width()<=m_minFaceRect.width()&&length.x()>0);		//如果最大人脸区域的宽度已经达到最小（与最小人脸宽度持平）
		else
		{
			if(qAbs(m_maxFaceRect.left()+length.x()-m_maxFaceRect.right())>m_maxFaceRectMaxSize.width())
			{
				m_maxFaceRect.setLeft(m_maxFaceRect.right()-m_maxFaceRectMaxSize.width());
				m_maxFaceRectLeft = QRectF(m_maxFaceRect.x()-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			}
			else if(qAbs(m_maxFaceRect.left()+length.x()-m_maxFaceRect.right())<m_minFaceRect.width())			//如果调整后最大人脸区域的宽度将会比最小人脸区域的宽度还小，则直接将宽度设置为最小人脸区域的宽度
			{
				m_maxFaceRect.setLeft(m_maxFaceRect.right()-m_minFaceRect.width());
				m_maxFaceRectLeft = QRectF(m_maxFaceRect.x()-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			}
			else
			{
				//左调节块移动
				m_maxFaceRectLeft.moveLeft(m_maxFaceRectLeft.x()+length.x());
				//最小人脸区域改变大小
				m_maxFaceRect.setX(m_maxFaceRect.x()+length.x());
			}

			//上调节块移动
			m_maxFaceRectTop = QRectF(m_maxFaceRect.x()+m_maxFaceRect.width()/2-adjustRectWidth,m_maxFaceRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			//下调节块移动
			m_maxFaceRectBottom = QRectF(m_maxFaceRect.x()+m_maxFaceRect.width()/2-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);

			emit signal_maxFaceRegionChange(QSizeF(m_maxFaceRect.width()/m_xRate,m_maxFaceRect.height()/m_yRate));
		}
		break;
	case MaxFaceRectRight:
		if(m_maxFaceRect.width()>=m_maxFaceRectMaxSize.width()&&length.x()>0);		//如果最小人脸区域的宽度已经到达最大，用户行为还想扩大
		else if(m_maxFaceRect.width()<=m_minFaceRect.width()&&length.x()<0);
		else
		{
			if(m_maxFaceRect.right()+length.x()-m_maxFaceRect.x()>m_maxFaceRectMaxSize.width())		//如果拖动的大小超过了最大区域，直接将其设置到最大值
			{
				m_maxFaceRect.setRight(m_maxFaceRect.x()+m_maxFaceRectMaxSize.width());
				m_maxFaceRectRight = QRectF(m_maxFaceRect.x()+m_maxFaceRect.width()-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			}
			else if(m_maxFaceRect.right()+length.x()-m_maxFaceRect.x()<m_minFaceRect.width())
			{
				m_maxFaceRect.setRight(m_maxFaceRect.left()+m_minFaceRect.width());
				m_maxFaceRectRight = QRectF(m_maxFaceRect.x()+m_maxFaceRect.width()-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			}
			else
			{
				//右调节块移动
				m_maxFaceRectRight.moveLeft(m_maxFaceRectRight.x()+length.x());
				//最小人脸区域改变大小
				m_maxFaceRect.setRight(m_maxFaceRect.right()+length.x());
			}
			//上调节块移动
			m_maxFaceRectTop = QRectF(m_maxFaceRect.x()+m_maxFaceRect.width()/2-adjustRectWidth,m_maxFaceRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			//下调节块移动
			m_maxFaceRectBottom = QRectF(m_maxFaceRect.x()+m_maxFaceRect.width()/2-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);

			emit signal_maxFaceRegionChange(QSizeF(m_maxFaceRect.width()/m_xRate,m_maxFaceRect.height()/m_yRate));
		}
		break;
	case MaxFaceRectBottom:
		if(m_maxFaceRect.height()>=m_maxFaceRectMaxSize.height()&&length.y()>0);		//如果最小人脸区域的高度已经到达最大，用户行为还想扩大
		else if(m_maxFaceRect.height()<=m_minFaceRect.height()&&length.y()<0);
		else
		{
			if(m_maxFaceRect.bottom()+length.y()-m_maxFaceRect.y()>m_maxFaceRectMaxSize.height())
			{
				m_maxFaceRect.setBottom(m_maxFaceRect.y()+m_maxFaceRectMaxSize.height());
				m_maxFaceRectBottom = QRectF(m_maxFaceRect.x()+m_maxFaceRect.width()/2-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			}
			else if(m_maxFaceRect.bottom()+length.y()-m_maxFaceRect.y()<m_minFaceRect.height())
			{
				m_maxFaceRect.setBottom(m_maxFaceRect.top()+m_minFaceRect.height());
				m_maxFaceRectBottom = QRectF(m_maxFaceRect.x()+m_maxFaceRect.width()/2-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			}
			else
			{
				//上调节块移动
				m_maxFaceRectBottom.moveBottom(m_maxFaceRectBottom.bottom()+length.y());
				//最小人脸区域改变大小
				m_maxFaceRect.setBottom(m_maxFaceRect.bottom()+length.y());
			}

			//左调节块移动
			m_maxFaceRectLeft = QRectF(m_maxFaceRect.x()-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			//右调节块移动
			m_maxFaceRectRight = QRectF(m_maxFaceRect.x()+m_maxFaceRect.width()-adjustRectWidth,m_maxFaceRect.y()+m_maxFaceRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);

			emit signal_maxFaceRegionChange(QSizeF(m_maxFaceRect.width()/m_xRate,m_maxFaceRect.height()/m_yRate));
		}
		break;
	case FaceDetectRect:
	{
		QRectF futureRect = m_faceDetectRect;
		futureRect.moveTo(futureRect.topLeft()+length);
		if(futureRect.top()<adapty(50))
			futureRect.moveTop(adapty(50));
		if(futureRect.left()<adaptx(50))
			futureRect.moveLeft(adaptx(50));
		if(futureRect.right()>adaptx(50+800))
			futureRect.moveRight(adaptx(50+800));
		if(futureRect.bottom()>adapty(50+600))
			futureRect.moveBottom(adapty(50+600));

		m_faceDetectRect = futureRect;
		m_faceDetectRectTop = QRectF(m_faceDetectRect.x()+m_faceDetectRect.width()/2-adjustRectWidth,m_faceDetectRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
		m_faceDetectRectBottom = QRectF(m_faceDetectRect.x()+m_faceDetectRect.width()/2-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
		m_faceDetectRectLeft = QRectF(m_faceDetectRect.x()-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
		m_faceDetectRectRight = QRectF(m_faceDetectRect.x()+m_faceDetectRect.width()-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
	}
		break;
	case FaceDetectRectTop:
		if(m_faceDetectRect.height()>=m_faceDetectRectMaxSize.height()&&length.y()<0);		//如果最小人脸区域的高度已经到达最大，用户行为还想扩大
		else
		{
			if(qAbs(m_faceDetectRect.top()+length.y()-m_faceDetectRect.bottom())>m_faceDetectRectMaxSize.height())
			{
				m_faceDetectRect.setTop(m_faceDetectRect.bottom()-m_faceDetectRectMaxSize.height());
				m_faceDetectRectTop = QRectF(m_faceDetectRect.x()+m_faceDetectRect.width()/2-adjustRectWidth,m_faceDetectRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			}
			else
			{
				//上调节块移动
				m_faceDetectRectTop.moveTop(m_faceDetectRectTop.y()+length.y());
				//最小人脸区域改变大小
				m_faceDetectRect.setY(m_faceDetectRect.y()+length.y());
			}

			//左调节块移动
			m_faceDetectRectLeft = QRectF(m_faceDetectRect.x()-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			//右调节块移动
			m_faceDetectRectRight = QRectF(m_faceDetectRect.x()+m_faceDetectRect.width()-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
		}
		break;
	case FaceDetectRectLeft:
		if(m_faceDetectRect.width()>=m_faceDetectRectMaxSize.width()&&length.x()<0);		//如果最小人脸区域的宽度已经到达最大，用户行为还想扩大
		else
		{
			if(qAbs(m_faceDetectRect.left()+length.x()-m_faceDetectRect.right())>m_faceDetectRectMaxSize.width())
			{
				m_faceDetectRect.setLeft(m_faceDetectRect.right()-m_faceDetectRectMaxSize.width());
				m_faceDetectRectLeft = QRectF(m_faceDetectRect.x()-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			}
			else
			{
				//左调节块移动
				m_faceDetectRectLeft.moveLeft(m_faceDetectRectLeft.x()+length.x());
				//最小人脸区域改变大小
				m_faceDetectRect.setX(m_faceDetectRect.x()+length.x());
			}

			//上调节块移动
			m_faceDetectRectTop = QRectF(m_faceDetectRect.x()+m_faceDetectRect.width()/2-adjustRectWidth,m_faceDetectRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			//下调节块移动
			m_faceDetectRectBottom = QRectF(m_faceDetectRect.x()+m_faceDetectRect.width()/2-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
		}
		break;
	case FaceDetectRectRight:
		if(m_faceDetectRect.width()>=m_faceDetectRectMaxSize.width()&&length.x()>0);		//如果最小人脸区域的宽度已经到达最大，用户行为还想扩大
		else
		{
			if(m_faceDetectRect.right()+length.x()-m_faceDetectRect.x()>m_faceDetectRectMaxSize.width())		//如果拖动的大小超过了最大区域，直接将其设置到最大值
			{
				m_faceDetectRect.setRight(m_faceDetectRect.x()+m_faceDetectRectMaxSize.width());
				m_faceDetectRectRight = QRectF(m_faceDetectRect.x()+m_faceDetectRect.width()-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			}
			else
			{
				//右调节块移动
				m_faceDetectRectRight.moveLeft(m_faceDetectRectRight.x()+length.x());
				//最小人脸区域改变大小
				m_faceDetectRect.setRight(m_faceDetectRect.right()+length.x());
			}

			//上调节块移动
			m_faceDetectRectTop = QRectF(m_faceDetectRect.x()+m_faceDetectRect.width()/2-adjustRectWidth,m_faceDetectRect.y()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			//下调节块移动
			m_faceDetectRectBottom = QRectF(m_faceDetectRect.x()+m_faceDetectRect.width()/2-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
		}
		break;
	case FaceDetectRectBottom:
		if(m_faceDetectRect.height()>=m_faceDetectRectMaxSize.height()&&length.y()>0);		//如果最小人脸区域的高度已经到达最大，用户行为还想扩大
		else
		{
			if(m_faceDetectRect.bottom()+length.y()-m_faceDetectRect.y()>m_faceDetectRectMaxSize.height())
			{
				m_faceDetectRect.setBottom(m_faceDetectRect.y()+m_faceDetectRectMaxSize.height());
				m_faceDetectRectBottom = QRectF(m_faceDetectRect.x()+m_faceDetectRect.width()/2-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			}
			else
			{
				//上调节块移动
				m_faceDetectRectBottom.moveBottom(m_faceDetectRectBottom.bottom()+length.y());
				//最小人脸区域改变大小
				m_faceDetectRect.setBottom(m_faceDetectRect.bottom()+length.y());
			}

			//左调节块移动
			m_faceDetectRectLeft = QRectF(m_faceDetectRect.x()-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
			//右调节块移动
			m_faceDetectRectRight = QRectF(m_faceDetectRect.x()+m_faceDetectRect.width()-adjustRectWidth,m_faceDetectRect.y()+m_faceDetectRect.height()/2-adjustRectHeght,adjustRectWidth*2,adjustRectHeght*2);
		}
		break;
	default:
		break;
	}

	update();

	m_lastMovePos = movePos;
}
