﻿#ifndef AIFACEREGIONITEM_H
#define AIFACEREGIONITEM_H

#include <QGraphicsObject>
#include <QRectF>
#include <QSizeF>

class AIFaceRegionItem : public QGraphicsObject
{
	Q_OBJECT
public:
	AIFaceRegionItem();

	/**
	 * @brief 设置Item boundingRect大小
	 */
	void setItemSize(const QSizeF &size);

	/**
	 * @brief setStreamSize
	 * @param size
	 */
	void setStreamSize(const QSizeF &size);

	/**
	 * @brief setDefaultMaxFaceSize			设置最大人脸区域大小限制
	 * @param size										[in]大小
	 */
	void setMaxFaceSizeLimit(const QSizeF &size);

	/**
	 * @brief setDefaultMinFaceSize			设置最小人脸区域大小限制
	 * @param size										[in]大小
	 */
	void setMinFaceSizeLimit(const QSizeF &size);

	/**
	 * @brief setCurrentDetectRegion			设置当前人脸探测区域
	 * @param rect
	 */
	void setCurrentDetectRegion(const QRectF &rect);

	/**
	 * @brief setCurrentMaxFaceSize			设置当前最大人脸区域
	 * @param size										[in]大小
	 */
	void setCurrentMaxFaceSize(const QSizeF &size);

	/**
	 * @brief setCurrentMinFaceSize			设置当前最小人脸区域
	 * @param size										[in]大小
	 */
	void setCurrentMinFaceSize(const QSizeF &size);

	/**
	 * @brief init		初始化界面元素
	 */
	void init();

	/**
	 * @brief getFaceDetectRect		得到人脸区域大小
	 * @param rect							[out]区域
	 */
	void getFaceDetectRect(QRectF &rect);

	/**
	 * @brief getMaxFaceSize			得到最大人脸区域
	 * @param size							[out]大小
	 */
	void getMaxFaceSize(QSizeF &size);

	/**
	 * @brief getMinFaceSize			得到最小人脸区域
	 * @param size							[out]大小
	 */
	void getMinFaceSize(QSizeF &size);

protected:
	QRectF boundingRect() const override;
	void paint(QPainter *painter, const QStyleOptionGraphicsItem *option,QWidget *widget) override;

	void mousePressEvent(QGraphicsSceneMouseEvent *event) override;
	void mouseMoveEvent(QGraphicsSceneMouseEvent *event) override;

private:
	QRectF m_faceDetectRect;
	QRectF m_faceDetectRectTop;
	QRectF m_faceDetectRectBottom;
	QRectF m_faceDetectRectLeft;
	QRectF m_faceDetectRectRight;
	QRectF m_maxFaceRect;
	QPointF m_maxFaceRectCenter;
	QRectF m_maxFaceRectTop;
	QRectF m_maxFaceRectBottom;
	QRectF m_maxFaceRectLeft;
	QRectF m_maxFaceRectRight;
	QRectF m_minFaceRect;
	QPointF m_minFaceRectCenter;
	QRectF m_minFaceRectTop;
	QRectF m_minFaceRectBottom;
	QRectF m_minFaceRectLeft;
	QRectF m_minFaceRectRight;
	using SelectRect = enum SelectRect
	{
		UnknownRect,

		FaceDetectRect,
		FaceDetectRectTop,
		FaceDetectRectBottom,
		FaceDetectRectLeft,
		FaceDetectRectRight,
		MaxFaceRect,
		MaxFaceRectTop,
		MaxFaceRectBottom,
		MaxFaceRectLeft,
		MaxFaceRectRight,
		MinFaceRect,
		MinFaceRectTop,
		MinFaceRectBottom,
		MinFaceRectLeft,
		MinFaceRectRight,
	};
	SelectRect m_selectRect{UnknownRect};

	QSizeF m_faceDetectRectMaxSize;
	QSizeF m_maxFaceRectMaxSize;		//最小人脸大小限制值
	QSizeF m_minFaceRectMaxSize;		//最大人脸大小限制值

	QPointF m_pressPos;
	QPointF m_lastMovePos;

	QSizeF m_itemSize;

	QSizeF m_streamSize;
	qreal m_xRate{0.0};
	qreal m_yRate{0.0};

signals:
	void signal_maxFaceRegionChange(QSizeF);
	void signal_minFaceRegionChange(QSizeF);
};

#endif // AIFACEREGIONITEM_H
