﻿#include "src/inc/gui/facedetect/dxregion.h"

#include <QDebug>
#include <QMouseEvent>

#include "src/inc/aifacepch.h"
#include "src/inc/package/videopackage.h"

DXRegion::DXRegion(QWidget *parent) : QWidget(parent)
{
	this->setGeometry(adaptxywh(50,50,800,600));
	this->setUpdatesEnabled(false);
}

void DXRegion::setUserID(const int &userID)
{
	m_userID = userID;
}

void DXRegion::setStreamSize(const QSize &streamSize)
{
	m_streamSize = streamSize;

	m_xRate = adaptx(800)/m_streamSize.width();
	m_yRate = adapty(600)/m_streamSize.height();
}

void DXRegion::setMaxFaceSizeLimit(const QSize &size)
{
	m_originMaxFaceRectMaxSize = size;
	m_maxFaceRectMaxSize = QSize(size.width()*m_xRate,size.height()*m_yRate);
}

void DXRegion::setMinFaceSizeLimit(const QSize &size)
{
	m_originMinFaceRectMaxSize = size;
	m_minFaceRectMaxSize = QSize(size.width()*m_xRate,size.height()*m_yRate);
}

void DXRegion::setDetectRect(const QRect &detectRect)
{
	m_faceDetectRect=QRect(detectRect.x()*m_xRate,detectRect.y()*m_yRate,detectRect.width()*m_xRate,detectRect.height()*m_yRate);
}

void DXRegion::setMaxFaceSize(const QSize &maxFaceSize)
{
	int x = adaptx(800)/2 - maxFaceSize.width()*m_xRate/2;
	int y = adaptx(600)/2 - maxFaceSize.height()*m_yRate/2;

	m_maxFaceRect = {x,y,static_cast<int>(maxFaceSize.width()*m_xRate),static_cast<int>(maxFaceSize.height()*m_yRate)};
}

void DXRegion::setMinFaceSize(const QSize &minFaceSize)
{
	int x = adaptx(800)/2 - minFaceSize.width()*m_xRate/2;
	int y = adaptx(600)/2 - minFaceSize.height()*m_yRate/2;

	m_minFaceRect = {x,y,static_cast<int>(minFaceSize.width()*m_xRate),static_cast<int>(minFaceSize.height()*m_yRate)};
}

void DXRegion::updateAllRect()
{
	QVector<QRect> rects{m_faceDetectRect,m_maxFaceRect,m_minFaceRect};
	int ercode = vncp::VideoPackage::GetInstance()->addRectOnStream(m_userID,0,rects);
	check_ercode_no_return(ercode,"Add rect on stream failed");
}

void DXRegion::getFaceDetectRegion(QRect &detectRegion)
{
	const auto reviseWidth = [this](const qreal &value)->int
	{
		if(value < 0)
			return 0;
		if(value > m_streamSize.width())
			return m_streamSize.width();

		return static_cast<int>(value);
	};
	const auto reviseHeight = [this](const qreal &value)->int
	{
		if(value < 0)
			return 0;
		if(value > m_streamSize.height())
			return m_streamSize.height();

		return static_cast<int>(value);
	};

	int x = reviseWidth(m_faceDetectRect.x()/m_xRate);
	int y = reviseHeight(m_faceDetectRect.y()/m_yRate);
	int width = reviseWidth(m_faceDetectRect.width()/m_xRate);
	int height = reviseHeight(m_faceDetectRect.height()/m_yRate);
	detectRegion = QRect(x,y,width,height);
}

void DXRegion::getMaxFaceSize(QSize &size)
{
//	const auto reviseWidth = [this](const qreal &value)->int
//	{
//		if(value < m_minFaceRect.width())
//			return m_minFaceRect.width();
//		if(value > m_originMaxFaceRectMaxSize.width())
//			return m_originMaxFaceRectMaxSize.width();

//		return static_cast<int>(value);
//	};
//	const auto reviseHeight = [this](const qreal &value)->int
//	{
//		if(value < m_minFaceRectMaxSize.height())
//			return m_minFaceRectMaxSize.height();
//		if(value > m_originMaxFaceRectMaxSize.height())
//			return m_originMaxFaceRectMaxSize.height();

//		return static_cast<int>(value);
//	};

//	int width = reviseWidth(m_maxFaceRect.width()/m_xRate);
//	int height = reviseHeight(m_maxFaceRect.height()/m_yRate);
//	size = QSize(width,height);

	size = QSize(m_maxFaceRect.width()/m_xRate,m_maxFaceRect.height()/m_yRate);
}

void DXRegion::getMinFaceSize(QSize &size)
{
//	const auto reviseWidth = [this](const qreal &value)->int
//	{
//		if(value > m_maxFaceRect.width())
//			return m_maxFaceRect.width();
//		if(value < m_originMinFaceRectMaxSize.width())
//			return m_originMinFaceRectMaxSize.width();

//		return static_cast<int>(value);
//	};
//	const auto reviseHeight = [this](const qreal &value)->int
//	{
//		if(value > m_maxFaceRect.height())
//			return m_maxFaceRect.height();
//		if(value < m_originMinFaceRectMaxSize.height())
//			return m_originMinFaceRectMaxSize.height();

//		return static_cast<int>(value);
//	};

//	int width = reviseWidth(m_minFaceRect.width()/m_xRate);
//	int height = reviseHeight(m_minFaceRect.height()/m_yRate);
//	size = QSize(width,height);

	size = QSize(m_minFaceRect.width()/m_xRate,m_minFaceRect.height()/m_yRate);
}

void DXRegion::mousePressEvent(QMouseEvent *event)
{
	m_pressPos = event->pos();
	m_pressFaceDetectRect=m_faceDetectRect;
	m_pressMaxFaceRect=m_maxFaceRect;
	m_pressMinFaceRect=m_minFaceRect;

	const int distance{4};

	if(m_pressPos.x()>m_faceDetectRect.topLeft().x()&&m_pressPos.x()<m_faceDetectRect.topRight().x()&&abs(m_pressPos.y()-m_faceDetectRect.top())<distance)
		m_selectRect = FaceDetectRectTop;
	else if(m_pressPos.y()>m_faceDetectRect.topLeft().y()&&m_pressPos.y()<m_faceDetectRect.bottomLeft().y()&&abs(m_pressPos.x()-m_faceDetectRect.left())<distance)
		m_selectRect = FaceDetectRectLeft;
	else if(m_pressPos.y()>m_faceDetectRect.topRight().y()&&m_pressPos.y()<m_faceDetectRect.bottomRight().y()&&abs(m_pressPos.x()-m_faceDetectRect.right())<distance)
		m_selectRect = FaceDetectRectRight;
	else if(m_pressPos.x()>m_faceDetectRect.bottomLeft().x()&&m_pressPos.x()<m_faceDetectRect.bottomRight().x()&&abs(m_pressPos.y()-m_faceDetectRect.bottom())<distance)
		m_selectRect = FaceDetectRectBottom;
	else if(m_pressPos.x()>m_maxFaceRect.topLeft().x()&&m_pressPos.x()<m_maxFaceRect.topRight().x()&&abs(m_pressPos.y()-m_maxFaceRect.top())<distance)
		m_selectRect = MaxFaceRectTop;
	else if(m_pressPos.y()>m_maxFaceRect.topLeft().y()&&m_pressPos.y()<m_maxFaceRect.bottomLeft().y()&&abs(m_pressPos.x()-m_maxFaceRect.left())<distance)
		m_selectRect = MaxFaceRectLeft;
	else if(m_pressPos.y()>m_maxFaceRect.topRight().y()&&m_pressPos.y()<m_maxFaceRect.bottomRight().y()&&abs(m_pressPos.x()-m_maxFaceRect.right())<distance)
		m_selectRect = MaxFaceRectRight;
	else if(m_pressPos.x()>m_maxFaceRect.bottomLeft().x()&&m_pressPos.x()<m_maxFaceRect.bottomRight().x()&&abs(m_pressPos.y()-m_maxFaceRect.bottom())<distance)
		m_selectRect = MaxFaceRectBottom;
	else if(m_pressPos.x()>m_minFaceRect.topLeft().x()&&m_pressPos.x()<m_minFaceRect.topRight().x()&&abs(m_pressPos.y()-m_minFaceRect.top())<distance)
		m_selectRect = MinFaceRectTop;
	else if(m_pressPos.y()>m_minFaceRect.topLeft().y()&&m_pressPos.y()<m_minFaceRect.bottomLeft().y()&&abs(m_pressPos.x()-m_minFaceRect.left())<distance)
		m_selectRect = MinFaceRectLeft;
	else if(m_pressPos.y()>m_minFaceRect.topRight().y()&&m_pressPos.y()<m_minFaceRect.bottomRight().y()&&abs(m_pressPos.x()-m_minFaceRect.right())<distance)
		m_selectRect = MinFaceRectRight;
	else if(m_pressPos.x()>m_minFaceRect.bottomLeft().x()&&m_pressPos.x()<m_minFaceRect.bottomRight().x()&&abs(m_pressPos.y()-m_minFaceRect.bottom())<distance)
		m_selectRect = MinFaceRectBottom;
	else if(true == m_minFaceRect.contains(m_pressPos))
		m_selectRect = MinFaceRect;
	else if(true == m_maxFaceRect.contains(m_pressPos))
		m_selectRect = MaxFaceRect;
	else if(true == m_faceDetectRect.contains(m_pressPos))
		m_selectRect = FaceDetectRect;
}

void DXRegion::mouseMoveEvent(QMouseEvent *event)
{
	QPoint movePos = event->pos();

	const int minFaceDetectRectHeight = adapty(10);
	const int minFaceDetectRectWidth = adaptx(10);

	switch(m_selectRect)
	{
	case MinFaceRectTop:
	{
		int futureHeight = m_minFaceRect.bottom()-movePos.y();

		//如果调整后的最小人脸检测区域将会大于当前最大人脸区域
		if(futureHeight > m_maxFaceRect.height())
			m_minFaceRect.setTop(m_minFaceRect.bottom()-(m_maxFaceRect.height()-2));
		else if(movePos.y() < 0)
			m_minFaceRect.setTop(0);
		//如果调整后的最小人脸检测区域将会小于最小人脸区域限制
		else if(futureHeight < m_minFaceRectMaxSize.height())
			m_minFaceRect.setTop(m_minFaceRect.bottom()-m_minFaceRectMaxSize.height());
		else
			m_minFaceRect.setTop(movePos.y());
	}
		break;
	case MinFaceRectBottom:
	{
		int futureHeight = movePos.y()-m_minFaceRect.top();

		//如果调整后的最小人脸检测区域将会大于当前最大人脸区域
		if(futureHeight > m_maxFaceRect.height())
			m_minFaceRect.setBottom(m_minFaceRect.top()+(m_maxFaceRect.height()-2));
		else if(movePos.y()>this->height())
			m_minFaceRect.setBottom(this->height());
		//如果调整后的最小人脸检测区域将会小于最小人脸区域限制
		else if(futureHeight < m_minFaceRectMaxSize.height())
			m_minFaceRect.setBottom(m_minFaceRect.top()+m_minFaceRectMaxSize.height());
		else
			m_minFaceRect.setBottom(movePos.y());
	}
		break;
	case MinFaceRectLeft:
	{
		int futureWidth = m_minFaceRect.right()-movePos.x();

		//如果调整后的最小人脸检测区域将会大于当前最大人脸区域
		if(futureWidth > m_maxFaceRect.width())
			m_minFaceRect.setLeft(m_minFaceRect.right()-(m_maxFaceRect.width()-2));
		else if(movePos.x()<0)
			m_minFaceRect.setLeft(0);
		//如果调整后的最小人脸检测区域将会小于最小人脸区域限制
		else if(futureWidth < m_minFaceRectMaxSize.width())
			m_minFaceRect.setLeft(m_minFaceRect.right()-m_minFaceRectMaxSize.width());
		else
			m_minFaceRect.setLeft(movePos.x());
	}
		break;
	case MinFaceRectRight:
	{
		int futureWidth = movePos.x() - m_minFaceRect.left();

		//如果调整后的最小人脸检测区域将会大于当前最大人脸区域
		if(futureWidth > m_maxFaceRect.width())
			m_minFaceRect.setRight(m_minFaceRect.left()+(m_maxFaceRect.width()-2));
		else if(movePos.x()>this->width())
			m_minFaceRect.setRight(this->width());
		//如果调整后的最小人脸检测区域将会小于最小人脸区域限制
		else if(futureWidth < m_minFaceRectMaxSize.width())
			m_minFaceRect.setRight(m_minFaceRect.left()+m_minFaceRectMaxSize.width());
		else
			m_minFaceRect.setRight(movePos.x());
	}
		break;
	case MaxFaceRectTop:
	{
		int futureHeight = m_maxFaceRect.bottom()-movePos.y();

		//如果调整后的最大人脸检测区域将会小于当前最小人脸区域
		if(futureHeight < m_minFaceRect.height())
			m_maxFaceRect.setTop(m_maxFaceRect.bottom()-(m_minFaceRect.height()+2));
		//如果调整后的最大人脸检测区域将会大于最大人脸区域限制
		else if(futureHeight > m_maxFaceRectMaxSize.height())
			m_maxFaceRect.setTop(m_maxFaceRect.bottom()-m_maxFaceRectMaxSize.height());
		else if(movePos.y()<0)
			m_maxFaceRect.setTop(0);
		else
			m_maxFaceRect.setTop(movePos.y());
	}
		break;
	case MaxFaceRectLeft:
	{
		int futureWidth = m_maxFaceRect.right()-movePos.x();

		//如果调整后的最大人脸检测区域将会小于当前最小人脸区域
		if(futureWidth < m_minFaceRect.width())
			m_maxFaceRect.setLeft(m_maxFaceRect.right()-(m_minFaceRect.width()+2));
		//如果调整后的最大人脸检测区域将会大于最大人脸区域限制
		else if(futureWidth > m_maxFaceRectMaxSize.width())
			m_maxFaceRect.setLeft(m_maxFaceRect.right()-m_maxFaceRectMaxSize.width());
		else if(movePos.x()<0)
			m_maxFaceRect.setLeft(0);
		else
			m_maxFaceRect.setLeft(movePos.x());
	}
		break;
	case MaxFaceRectRight:
	{
		int futureWidth = movePos.x() - m_maxFaceRect.left();

		//如果调整后的最大人脸检测区域将会小于当前最小人脸区域
		if(futureWidth < m_minFaceRect.width())
			m_maxFaceRect.setRight(m_maxFaceRect.left()+(m_minFaceRect.width()+2));
		//如果调整后的最大人脸检测区域将会大于最大人脸区域限制
		else if(futureWidth > m_maxFaceRectMaxSize.width())
			m_maxFaceRect.setRight(m_maxFaceRect.left()+m_maxFaceRectMaxSize.width());
		else if(movePos.x()>this->width())
			m_maxFaceRect.setRight(this->width());
		else
			m_maxFaceRect.setRight(movePos.x());
	}
		break;
	case MaxFaceRectBottom:
	{
		int futureHeight = movePos.y()-m_maxFaceRect.top();

		//如果调整后的最大人脸检测区域将会小于当前最小人脸区域
		if(futureHeight < m_minFaceRect.height())
			m_maxFaceRect.setBottom(m_maxFaceRect.top()+(m_minFaceRect.height()+2));
		//如果调整后的最大人脸检测区域将会大于最大人脸区域限制
		else if(futureHeight > m_maxFaceRectMaxSize.height())
			m_maxFaceRect.setBottom(m_maxFaceRect.top()+m_maxFaceRectMaxSize.height());
		else if(movePos.y()>this->height())
			m_maxFaceRect.setBottom(this->height());
		else
			m_maxFaceRect.setBottom(movePos.y());
	}
		break;
	case FaceDetectRectTop:
	{
		int futureHeight = m_faceDetectRect.bottom()-movePos.y();

		if(futureHeight<minFaceDetectRectHeight)
			m_faceDetectRect.setTop(m_faceDetectRect.bottom()+minFaceDetectRectHeight);
		else if(movePos.y()<0)
			m_faceDetectRect.setTop(0);
		else
			m_faceDetectRect.setTop(movePos.y());
	}
		break;
	case FaceDetectRectBottom:
	{
		int futureHeight = movePos.y()-m_faceDetectRect.top();

		if(futureHeight<minFaceDetectRectHeight)
			m_faceDetectRect.setBottom(m_faceDetectRect.top()+minFaceDetectRectHeight);
		else if(movePos.y()>this->height())
			m_faceDetectRect.setBottom(this->height());
		else
			m_faceDetectRect.setBottom(movePos.y());
	}
		break;
	case FaceDetectRectLeft:
	{
		int futureWidth = m_maxFaceRect.right()-movePos.x();

		if(futureWidth<minFaceDetectRectWidth)
			m_faceDetectRect.setLeft(m_faceDetectRect.right()-minFaceDetectRectWidth);
		else if(movePos.x()<0)
			m_faceDetectRect.setLeft(0);
		else
			m_faceDetectRect.setLeft(movePos.x());
	}
		break;
	case FaceDetectRectRight:
	{
		int futureWidth = movePos.x()-m_faceDetectRect.left();

		if(futureWidth<minFaceDetectRectWidth)
			m_faceDetectRect.setRight(m_faceDetectRect.left()+minFaceDetectRectWidth);
		else if(movePos.x()>this->width())
			m_faceDetectRect.setRight(this->width());
		else
			m_faceDetectRect.setRight(movePos.x());
	}
		break;
	case MinFaceRect:
	{
		QRect futureRect = m_minFaceRect;
		 futureRect.moveTo(m_pressMinFaceRect.topLeft()+(movePos-m_pressPos));

		 bool flag{true};
		 if(futureRect.x()<0)
		 {
			 flag=false;
			 m_minFaceRect.moveLeft(0);
		 }
		 if(futureRect.y()<0)
		 {
			 flag=false;
			 m_minFaceRect.moveTop(0);
		 }
		 if(futureRect.bottom()>this->height())
		 {
			 flag=false;
			 m_minFaceRect.moveBottom(this->height());
		 }
		if(futureRect.right()>this->width())
		{
			flag=false;
			m_minFaceRect.moveRight(this->width());
		}

		if(true==flag)
			m_minFaceRect.moveTo(futureRect.topLeft());
	}
		break;
	case MaxFaceRect:
	{
		QRect futureRect = m_maxFaceRect;
		 futureRect.moveTo(m_pressMaxFaceRect.topLeft()+(movePos-m_pressPos));

		 bool flag{true};
		 if(futureRect.x()<0)
		 {
			 flag=false;
			 m_maxFaceRect.moveLeft(0);
		 }
		 if(futureRect.y()<0)
		 {
			 flag=false;
			 m_maxFaceRect.moveTop(0);
		 }
		 if(futureRect.bottom()>this->height())
		 {
			 flag=false;
			 m_maxFaceRect.moveBottom(this->height());
		 }
		if(futureRect.right()>this->width())
		{
			flag=false;
			m_maxFaceRect.moveRight(this->width());
		}

		if(true==flag)
			m_maxFaceRect.moveTo(futureRect.topLeft());
	}
		break;
	case FaceDetectRect:
	{
		QRect futureRect = m_faceDetectRect;
		 futureRect.moveTo(m_pressFaceDetectRect.topLeft()+(movePos-m_pressPos));

		 bool flag{true};
		 if(futureRect.x()<0)
		 {
			 flag=false;
			 m_faceDetectRect.moveLeft(0);
		 }
		 if(futureRect.y()<0)
		 {
			 flag=false;
			 m_faceDetectRect.moveTop(0);
		 }
		 if(futureRect.bottom()>this->height())
		 {
			 flag=false;
			 m_faceDetectRect.moveBottom(this->height());
		 }
		if(futureRect.right()>this->width())
		{
			flag=false;
			m_faceDetectRect.moveRight(this->width());
		}

		if(true==flag)
			m_faceDetectRect.moveTo(futureRect.topLeft());
	}
		break;
	default:
		break;
	}

	updateAllRect();

	QSize maxFaceSize;
	getMaxFaceSize(maxFaceSize);
	QSize minFaceSize;
	getMinFaceSize(minFaceSize);

	signal_maxFaceRegionChange(maxFaceSize);
	signal_minFaceRegionChange(minFaceSize);
}

void DXRegion::mouseReleaseEvent(QMouseEvent *event)
{

}
