﻿#ifndef DXREGION_H
#define DXREGION_H

#include <QWidget>

class DXRegion : public QWidget
{
	Q_OBJECT
public:
	explicit DXRegion(QWidget *parent = nullptr);

	/**
	 * @brief setUserID			设置用户ID
	 * @param userID				[in]用户ID
	 */
	void setUserID(const int &userID);

	/**
	 * @brief setStreamSize		设置流大小
	 * @param streamSize			[in]流大小
	 */
	void setStreamSize(const QSize &streamSize);

	/**
	 * @brief setMaxFaceSizeLimit		设置最大人脸限制
	 * @param size								[in]最大人脸大小限制值
	 */
	void setMaxFaceSizeLimit(const QSize &size);

	/**
	 * @brief setMinFaceSizeLimit		设置最小人脸限制值
	 * @param size								[in]最小人脸限制值
	 */
	void setMinFaceSizeLimit(const QSize &size);

	/**
	 * @brief setDetectRect		设置探测区域
	 * @param detectRect			[in]探测区域
	 */
	void setDetectRect(const QRect &detectRect);

	/**
	 * @brief setMaxFaceSize		设置最大人脸大小
	 * @param maxFaceSize		[in]最大人脸大小
	 */
	void setMaxFaceSize(const QSize &maxFaceSize);

	/**
	 * @brief setMinFaceSize		设置最小人脸大小
	 * @param minFaceSize		[in]最小人脸大小
	 */
	void setMinFaceSize(const QSize &minFaceSize);

	/**
	 * @brief updateAllRect
	 */
	void updateAllRect();

	/**
	 * @brief getFaceDetectRegion		得到人脸探测区域
	 * @param detectRegion				[out]
	 */
	void getFaceDetectRegion(QRect &detectRegion);

	void getMaxFaceSize(QSize &size);

	void getMinFaceSize(QSize &size);

protected:
	void mousePressEvent(QMouseEvent *event) override;
	void mouseMoveEvent(QMouseEvent *event) override;
	void mouseReleaseEvent(QMouseEvent *event) override;

private:
	QRect m_faceDetectRect{-1,-1,0,0};
	QRect m_maxFaceRect{-1,-1,0,0};
	QRect m_minFaceRect{-1,-1,0,0};
	QSizeF m_faceDetectRectMaxSize;
	QSize m_originMaxFaceRectMaxSize;
	QSizeF m_maxFaceRectMaxSize;		//最小人脸大小限制值
	QSize m_originMinFaceRectMaxSize;
	QSizeF m_minFaceRectMaxSize;		//最大人脸大小限制值

	QRect m_pressFaceDetectRect;
	QRect m_pressMaxFaceRect;
	QRect m_pressMinFaceRect;

	QSize m_streamSize;
	qreal m_xRate{1};
	qreal m_yRate{1};

	int m_userID{-1};

	using SelectRect = enum SelectRect
	{
		UnknownRect,

		FaceDetectRect,
		FaceDetectRectTop,
		FaceDetectRectBottom,
		FaceDetectRectLeft,
		FaceDetectRectRight,
		MaxFaceRect,
		MaxFaceRectTop,
		MaxFaceRectBottom,
		MaxFaceRectLeft,
		MaxFaceRectRight,
		MinFaceRect,
		MinFaceRectTop,
		MinFaceRectBottom,
		MinFaceRectLeft,
		MinFaceRectRight,
	};
	SelectRect m_selectRect{UnknownRect};

	QPoint m_pressPos;
	QPoint m_lastMovePos;

signals:
	void signal_maxFaceRegionChange(QSizeF);
	void signal_minFaceRegionChange(QSizeF);

public slots:
};

#endif // DXREGION_H
