﻿#include "src/inc/gui/facedetect/snapconfigwidget.h"

#include "src/inc/httpmanager/aifacepackage.h"

#include <QPushButton>

SnapConfigWidget::SnapConfigWidget(QWidget *parent) : QGroupBox(parent)
{
	this->setGeometry(adaptxywh(900,240,350,340));
	this->setFont(m_font);
	this->setTitle("Snap Config");

	initGui();
}

void SnapConfigWidget::setUserInfo(const QString &ip, const QString &username, const QString &password)
{
	m_ip = ip;
	m_username = username;
	m_password = password;

	//隐藏所有界面
	for(SnapModeGui *gui:m_snapModeGuis)
		gui->hide();

	//获取当前的模式及模式下子项的值
	QVector<QString> values;
	HttpManagerErcode ercode = AIFacePackage::GetInstance()->getCurrentSnapConfig(m_ip,m_username,m_password,values);
	check_http_ercode_no_return(ercode,"Get current snap config failed!")

	if("auto" == values[0])
	{
		m_snapBox->setCurrentIndex(0);
		m_currentSnapMode = SnapMode_Auto;
		SnapModeAutoGui *autoGui = dynamic_cast<SnapModeAutoGui *>(m_snapModeGuis[SnapMode_Auto]);
		autoGui->numberShotsBox->setCurrentText(values[1]);
		autoGui->snapIntervalsBox->setCurrentText(values[2]);
		autoGui->mapConfigBox->setCurrentText(m_mapConfigTable.value(values[3]));

		autoGui->show();
	}
	else if("optimal" == values[0])
	{
		m_snapBox->setCurrentIndex(1);
		m_currentSnapMode = SnapMode_Optimal;
		SnapModeOptimalGui *optimalGui = dynamic_cast<SnapModeOptimalGui *>(m_snapModeGuis[SnapMode_Optimal]);
		optimalGui->numberShotsBox->setCurrentText(values[1]);
		optimalGui->mapConfigBox->setCurrentText(m_mapConfigTable.value(values[2]));
		optimalGui->show();
	}
	else if("fast" == values[0])
	{
		m_snapBox->setCurrentIndex(2);
		m_currentSnapMode = SnapMode_Auick;
		SnapModeAuickGui *auickGui = dynamic_cast<SnapModeAuickGui *>(m_snapModeGuis[SnapMode_Auick]);
		auickGui->numberShotsBox->setCurrentText(values[1]);
		auickGui->mapConfigBox->setCurrentText(m_mapConfigTable.value(values[2]));
		auickGui->show();
	}
	else if("custom" == values[0])
	{
		m_snapBox->setCurrentIndex(3);
		m_currentSnapMode = SnapMode_Custom;
		SnapModeCustomGui *customGui = dynamic_cast<SnapModeCustomGui *>(m_snapModeGuis[SnapMode_Custom]);
		customGui->numberShotsBox->setCurrentText(values[1]);
		customGui->qualityThresholdEdit->setText(values[2]);
		customGui->obliqueAngleEdit->setText(values[3]);
		customGui->pitchAngleEdit->setText(values[4]);
		customGui->sideAngleEdit->setText(values[5]);
		customGui->fuzzyDegreeEdit->setText(values[6]);
		customGui->mapConfigBox->setCurrentText(m_mapConfigTable.value(values[7]));
		customGui->show();
	}
}

void SnapConfigWidget::setSnapMode(const SnapMode &snapMode)
{
	m_currentSnapMode = snapMode;

	HttpManagerErcode ercode{HttpErcode_None};

	switch(snapMode)
	{
	case SnapMode_Auto:
	{
		QVector<QString> values;
		ercode = AIFacePackage::GetInstance()->getSnapConfig(m_ip,m_username,m_password,"auto",values);
		check_http_ercode_no_return(ercode,"Get snap config failed! mode:optimal")

		SnapModeAutoGui *autoGui = dynamic_cast<SnapModeAutoGui *>(m_snapModeGuis[SnapMode_Auto]);
		autoGui->numberShotsBox->setCurrentText(values[0]);
		autoGui->snapIntervalsBox->setCurrentText(values[1]);
		autoGui->mapConfigBox->setCurrentText(m_mapConfigTable.value(values[2]));
	}
		break;
	case SnapMode_Optimal:
	{
		QVector<QString> values;
		ercode = AIFacePackage::GetInstance()->getSnapConfig(m_ip,m_username,m_password,"optimal",values);
		check_http_ercode_no_return(ercode,"Get optimal config failed! mode:optimal")

		SnapModeOptimalGui *optimalGui = dynamic_cast<SnapModeOptimalGui *>(m_snapModeGuis[SnapMode_Optimal]);
		optimalGui->numberShotsBox->setCurrentText(values[0]);
		optimalGui->mapConfigBox->setCurrentText(m_mapConfigTable.value(values[1]));
	}
		break;
	case SnapMode_Auick:
	{
		QVector<QString> values;
		ercode = AIFacePackage::GetInstance()->getSnapConfig(m_ip,m_username,m_password,"fast",values);
		check_http_ercode_no_return(ercode,"Get snap config! mode:auick")

		SnapModeAuickGui *auickGui = dynamic_cast<SnapModeAuickGui *>(m_snapModeGuis[SnapMode_Auick]);
		auickGui->numberShotsBox->setCurrentText(values[0]);
		auickGui->mapConfigBox->setCurrentText(m_mapConfigTable.value(values[1]));
	}
		break;
	case SnapMode_Custom:
	{
		QVector<QString> values;
		ercode = AIFacePackage::GetInstance()->getSnapConfig(m_ip,m_username,m_password,"custom",values);
		check_http_ercode_no_return(ercode,"Get snap config failed, mode:custom")

		SnapModeCustomGui *customGui = dynamic_cast<SnapModeCustomGui *>(m_snapModeGuis[SnapMode_Custom]);
		customGui->numberShotsBox->setCurrentText(values[0]);
		customGui->qualityThresholdEdit->setText(values[1]);
		customGui->obliqueAngleEdit->setText(values[2]);
		customGui->pitchAngleEdit->setText(values[3]);
		customGui->sideAngleEdit->setText(values[4]);
		customGui->fuzzyDegreeEdit->setText(values[5]);
		customGui->mapConfigBox->setCurrentText(m_mapConfigTable.value(values[6]));
	}
		break;
	default:
		break;
	}

	QMap<SnapMode,SnapModeGui *>::iterator iter;
	for(iter=m_snapModeGuis.begin();iter!=m_snapModeGuis.end();++iter)
	{
		if(snapMode==iter.key())
			iter.value()->show();
		else
			iter.value()->hide();
	}
}

void SnapConfigWidget::initGui()
{
	//抓拍模式
	QLabel *snapMode = new QLabel(this);
	check_pointer_no_return(snapMode)
	snapMode->setGeometry(adaptxywh(20,20,100,25));
	snapMode->setFont(m_font);
	snapMode->setText("Snap Mode");

	m_snapBox = new QComboBox(this);
	check_pointer_no_return(m_snapBox)
	m_snapBox->setGeometry(adaptxywh(120,20,200,25));
	QStringList snapModeItems{"auto","optimal","auick","custom"};
	m_snapBox->addItems(snapModeItems);
	//连接模式切换信号
	using comboBoxIntSignal=void (QComboBox::*)(int);
	connect(m_snapBox,static_cast<comboBoxIntSignal>(&QComboBox::currentIndexChanged),this,[&](int index)
	{
		setSnapMode(static_cast<SnapMode>(index));
	});

	//创建各模式界面
	initSnapModeAutoGui();
	initSnapModeOptimalGui();
	initSnapModeAuickGui();
	initSnapModeCustomGui();

	//创建保存按钮
	QPushButton *save = new QPushButton(this);
	save->setGeometry(adaptxywh(350/2-80/2,305,80,25));
	save->setText("save");
	connect(save,&QPushButton::released,this,&SnapConfigWidget::slot_save);
}

void SnapConfigWidget::initSnapModeAutoGui()
{
	SnapModeAutoGui *snapModeAutoGui = new SnapModeAutoGui;
	check_pointer_no_return(snapModeAutoGui)

	snapModeAutoGui->numberShots = new QLabel(this);
	check_pointer_no_return(snapModeAutoGui->numberShots);
	snapModeAutoGui->numberShots->setGeometry(adaptxywh(20,60,100,25));
	snapModeAutoGui->numberShots->setFont(m_font);
	snapModeAutoGui->numberShots->setText("Number Shots");

	snapModeAutoGui->numberShotsBox = new QComboBox(this);
	check_pointer_no_return(snapModeAutoGui->numberShotsBox)
	snapModeAutoGui->numberShotsBox->setGeometry(adaptxywh(120,60,200,25));
	QStringList numberShotItems{"1","2","3"};
	snapModeAutoGui->numberShotsBox->addItems(numberShotItems);

	snapModeAutoGui->snapIntervals = new QLabel(this);
	check_pointer_no_return(snapModeAutoGui->snapIntervals)
	snapModeAutoGui->snapIntervals->setGeometry(adaptxywh(20,95,100,25));
	snapModeAutoGui->snapIntervals->setFont(m_font);
	snapModeAutoGui->snapIntervals->setText("Snap Interval");

	snapModeAutoGui->snapIntervalsBox = new QComboBox(this);
	check_pointer_no_return(snapModeAutoGui->snapIntervalsBox)
	snapModeAutoGui->snapIntervalsBox->setGeometry(adaptxywh(120,95,200,25));
	QStringList snapIntervalsItems{"200","500","1000","2000"};
	snapModeAutoGui->snapIntervalsBox->addItems(snapIntervalsItems);

	snapModeAutoGui->mapConfig = new QLabel(this);
	check_pointer_no_return(snapModeAutoGui->mapConfig)
	snapModeAutoGui->mapConfig->setGeometry(adaptxywh(20,130,100,25));
	snapModeAutoGui->mapConfig->setFont(m_font);
	snapModeAutoGui->mapConfig->setText("Map Config");

	snapModeAutoGui->mapConfigBox = new QComboBox(this);
	check_pointer_no_return(snapModeAutoGui->mapConfigBox)
	snapModeAutoGui->mapConfigBox->setGeometry(adaptxywh(120,130,200,25));
	QStringList mapConfigItems{"face map","half body","full body"};
	snapModeAutoGui->mapConfigBox->addItems(mapConfigItems);

	snapModeAutoGui->hide();
	m_snapModeGuis[SnapMode_Auto] = snapModeAutoGui;
}

void SnapConfigWidget::initSnapModeOptimalGui()
{
	SnapModeOptimalGui *snapModeOptimalGui = new SnapModeOptimalGui;
	check_pointer_no_return(snapModeOptimalGui)

	snapModeOptimalGui->numberShots = new QLabel(this);
	check_pointer_no_return(snapModeOptimalGui->numberShots)
	snapModeOptimalGui->numberShots->setGeometry(adaptxywh(20,60,100,25));
	snapModeOptimalGui->numberShots->setFont(m_font);
	snapModeOptimalGui->numberShots->setText("Number Shots");

	snapModeOptimalGui->numberShotsBox = new QComboBox(this);
	check_pointer_no_return(snapModeOptimalGui->numberShotsBox)
	snapModeOptimalGui->numberShotsBox->setGeometry(adaptxywh(120,60,200,25));
	QStringList numberShotItems{"1","2","3"};
	snapModeOptimalGui->numberShotsBox->addItems(numberShotItems);

	snapModeOptimalGui->mapConfig = new QLabel(this);
	check_pointer_no_return(snapModeOptimalGui->mapConfig)
	snapModeOptimalGui->mapConfig->setGeometry(adaptxywh(20,95,100,25));
	snapModeOptimalGui->mapConfig->setFont(m_font);
	snapModeOptimalGui->mapConfig->setText("Snap Interval");

	snapModeOptimalGui->mapConfigBox = new QComboBox(this);
	check_pointer_no_return(snapModeOptimalGui->mapConfigBox)
	snapModeOptimalGui->mapConfigBox->setGeometry(adaptxywh(120,95,200,25));
	QStringList mapConfigItems{"face map","half body","full body"};
	snapModeOptimalGui->mapConfigBox->addItems(mapConfigItems);

	snapModeOptimalGui->hide();
	m_snapModeGuis[SnapMode_Optimal] = snapModeOptimalGui;
}

void SnapConfigWidget::initSnapModeAuickGui()
{
	SnapModeAuickGui *snapModeAuickGui = new SnapModeAuickGui;
	check_pointer_no_return(snapModeAuickGui)

	snapModeAuickGui->numberShots = new QLabel(this);
	check_pointer_no_return(snapModeAuickGui->numberShots)
	snapModeAuickGui->numberShots->setGeometry(adaptxywh(20,60,100,25));
	snapModeAuickGui->numberShots->setFont(m_font);
	snapModeAuickGui->numberShots->setText("Number Shots");

	snapModeAuickGui->numberShotsBox = new QComboBox(this);
	check_pointer_no_return(snapModeAuickGui->numberShotsBox)
	snapModeAuickGui->numberShotsBox->setGeometry(adaptxywh(120,60,200,25));
	QStringList numberShotItems{"1","2","3"};
	snapModeAuickGui->numberShotsBox->addItems(numberShotItems);

	snapModeAuickGui->mapConfig = new QLabel(this);
	check_pointer_no_return(snapModeAuickGui->mapConfig)
	snapModeAuickGui->mapConfig->setGeometry(adaptxywh(20,95,100,25));
	snapModeAuickGui->mapConfig->setFont(m_font);
	snapModeAuickGui->mapConfig->setText("Snap Interval");

	snapModeAuickGui->mapConfigBox = new QComboBox(this);
	check_pointer_no_return(snapModeAuickGui->mapConfigBox)
	snapModeAuickGui->mapConfigBox->setGeometry(adaptxywh(120,95,200,25));
	QStringList mapConfigItems{"face map","half body","full body"};
	snapModeAuickGui->mapConfigBox->addItems(mapConfigItems);

	snapModeAuickGui->hide();
	m_snapModeGuis[SnapMode_Auick] = snapModeAuickGui;
}

void SnapConfigWidget::initSnapModeCustomGui()
{
	SnapModeCustomGui *snapModeCustomGui = new SnapModeCustomGui;
	check_pointer_no_return(snapModeCustomGui)

	snapModeCustomGui->numberShots = new QLabel(this);
	check_pointer_no_return(snapModeCustomGui->numberShots)
	snapModeCustomGui->numberShots->setGeometry(adaptxywh(20,60,100,25));
	snapModeCustomGui->numberShots->setFont(m_font);
	snapModeCustomGui->numberShots->setText("Number Shots");

	snapModeCustomGui->numberShotsBox = new QComboBox(this);
	check_pointer_no_return(snapModeCustomGui->numberShotsBox)
	snapModeCustomGui->numberShotsBox->setGeometry(adaptxywh(120,60,200,25));
	QStringList numberShotItems{"1","2","3"};
	snapModeCustomGui->numberShotsBox->addItems(numberShotItems);

	snapModeCustomGui->qualityThreshold = new QLabel(this);
	check_pointer_no_return(snapModeCustomGui->qualityThreshold)
	snapModeCustomGui->qualityThreshold->setGeometry(adaptxywh(20,95,100,25));
	snapModeCustomGui->qualityThreshold->setFont(m_font);
	snapModeCustomGui->qualityThreshold->setText("Snap Interval");

	snapModeCustomGui->qualityThresholdEdit = new QLineEdit(this);
	check_pointer_no_return(snapModeCustomGui->qualityThresholdEdit)
	snapModeCustomGui->qualityThresholdEdit->setGeometry(adaptxywh(120,95,200,25));
	snapModeCustomGui->qualityThresholdEdit->setText("");

	snapModeCustomGui->obliqueAngle = new QLabel(this);
	check_pointer_no_return(snapModeCustomGui->obliqueAngle)
	snapModeCustomGui->obliqueAngle->setGeometry(adaptxywh(20,130,100,25));
	snapModeCustomGui->obliqueAngle->setFont(m_font);
	snapModeCustomGui->obliqueAngle->setText("Oblique Angle");

	snapModeCustomGui->obliqueAngleEdit = new QLineEdit(this);
	check_pointer_no_return(snapModeCustomGui->obliqueAngleEdit)
	snapModeCustomGui->obliqueAngleEdit->setGeometry(adaptxywh(120,130,200,25));
	snapModeCustomGui->obliqueAngleEdit->setText("");

	snapModeCustomGui->pitchAngle = new QLabel(this);
	check_pointer_no_return(snapModeCustomGui->pitchAngle)
	snapModeCustomGui->pitchAngle->setGeometry(adaptxywh(20,165,100,25));
	snapModeCustomGui->pitchAngle->setFont(m_font);
	snapModeCustomGui->pitchAngle->setText("Pitch Angle");

	snapModeCustomGui->pitchAngleEdit = new QLineEdit(this);
	check_pointer_no_return(snapModeCustomGui->pitchAngleEdit)
	snapModeCustomGui->pitchAngleEdit->setGeometry(adaptxywh(120,165,200,25));
	snapModeCustomGui->pitchAngleEdit->setText("");

	snapModeCustomGui->sideAngle = new QLabel(this);
	check_pointer_no_return(snapModeCustomGui->sideAngle)
	snapModeCustomGui->sideAngle->setGeometry(adaptxywh(20,200,100,25));
	snapModeCustomGui->sideAngle->setFont(m_font);
	snapModeCustomGui->sideAngle->setText("Side Angle");

	snapModeCustomGui->sideAngleEdit = new QLineEdit(this);
	check_pointer_no_return(snapModeCustomGui->sideAngleEdit)
	snapModeCustomGui->sideAngleEdit->setGeometry(adaptxywh(120,200,200,25));
	snapModeCustomGui->sideAngleEdit ->setText("");

	snapModeCustomGui->fuzzyDegree = new QLabel(this);
	check_pointer_no_return(snapModeCustomGui->fuzzyDegree)
	snapModeCustomGui->fuzzyDegree->setGeometry(adaptxywh(20,235,100,25));
	snapModeCustomGui->fuzzyDegree->setFont(m_font);
	snapModeCustomGui->fuzzyDegree->setText("Fuzzy Angle");

	snapModeCustomGui->fuzzyDegreeEdit = new QLineEdit(this);
	check_pointer_no_return(snapModeCustomGui->fuzzyDegreeEdit)
	snapModeCustomGui->fuzzyDegreeEdit->setGeometry(adaptxywh(120,235,200,25));
	snapModeCustomGui->fuzzyDegreeEdit ->setText("");

	snapModeCustomGui->mapConfig = new QLabel(this);
	check_pointer_no_return(snapModeCustomGui->mapConfig)
	snapModeCustomGui->mapConfig->setGeometry(adaptxywh(20,270,100,25));
	snapModeCustomGui->mapConfig->setFont(m_font);
	snapModeCustomGui->mapConfig->setText("Map Interval");

	snapModeCustomGui->mapConfigBox = new QComboBox(this);
	check_pointer_no_return(snapModeCustomGui->mapConfigBox)
	snapModeCustomGui->mapConfigBox->setGeometry(adaptxywh(120,270,200,25));
	QStringList mapConfigItems{"face map","half body","full body"};
	snapModeCustomGui->mapConfigBox->addItems(mapConfigItems);

	snapModeCustomGui->hide();
	m_snapModeGuis[SnapMode_Custom] = snapModeCustomGui;
}

void SnapConfigWidget::slot_save()
{
	SnapModeAutoGui *autoGui = dynamic_cast<SnapModeAutoGui *>(m_snapModeGuis[SnapMode_Auto]);
	SnapModeOptimalGui *optimalGui = dynamic_cast<SnapModeOptimalGui *>(m_snapModeGuis[SnapMode_Optimal]);
	SnapModeAuickGui *auickGui = dynamic_cast<SnapModeAuickGui *>(m_snapModeGuis[SnapMode_Auick]);
	SnapModeCustomGui *customGui = dynamic_cast<SnapModeCustomGui *>(m_snapModeGuis[SnapMode_Custom]);

	QVector<QString> values;
	//SnapshotMode
	values.append(m_snapBox->currentText());
	//CutoutMode
	switch(m_currentSnapMode)
	{
	case SnapMode_Auto:
		values.append(m_mapConfigTable.key(autoGui->mapConfigBox->currentText()));
		break;
	case SnapMode_Optimal:
		values.append(m_mapConfigTable.key(optimalGui->mapConfigBox->currentText()));
		break;
	case SnapMode_Auick:
		values.append(m_mapConfigTable.key(auickGui->mapConfigBox->currentText()));
		break;
	case SnapMode_Custom:
		values.append(m_mapConfigTable.key(customGui->mapConfigBox->currentText()));
		break;
	default:
		break;
	}
	//ModeAuto-SnapshotCount
	values.append(autoGui->numberShotsBox->currentText());
	//ModeAuto-SnapshotInterval
	values.append(autoGui->snapIntervalsBox->currentText());
	//ModeFast-SnapshotCount
	values.append(auickGui->numberShotsBox->currentText());
	//ModeOptimal-SnapshotCount
	values.append(optimalGui->numberShotsBox->currentText());
	//ModeCustom-SnapshotCount
	values.append(customGui->numberShotsBox->currentText());
	//ModeCustom-PicQualityThreshold
	values.append(customGui->qualityThresholdEdit->text());
	//ModeCustom-ObliqueAngle
	values.append(customGui->obliqueAngleEdit->text());
	//ModeCustom-PitchAngle
	values.append(customGui->pitchAngleEdit->text());
	//ModeCustom-SideAngle
	values.append(customGui->sideAngleEdit->text());
	//ModeCustom-FuzzyDegree
	values.append(customGui->fuzzyDegreeEdit->text());

	HttpManagerErcode ercode = AIFacePackage::GetInstance()->setSnapConfig(m_ip,m_username,m_password,values);
	check_http_ercode_no_return(ercode,"Set snap config failed!")

	message_output("Save snap config successfully!");
}
