﻿#include "src/inc/gui/facelibrary/facelibrary.h"

#include "src/inc/aifacepch.h"
#include "src/inc/package/videopackage.h"
#include "src/inc/httpmanager/facelibrarypackage.h"
#include "src/inc/gui/facelibrary/facelibraryitemwidget.h"
#include "src/inc/gui/facelibrary/namewidget.h"

#include <QTableWidget>
#include <QHeaderView>
#include <QPushButton>
#include <QLabel>
#include <QLineEdit>
#include <QFileDialog>
#include <QThread>

FaceLibrary::FaceLibrary(QWidget *parent) : QWidget(parent)
{
	this->setGeometry(adaptxywh(100,100,1300,700));
//	this->setAttribute(Qt::WA_DeleteOnClose);
	this->setWindowTitle("Face Library");

	initGui();
}

FaceLibrary::~FaceLibrary()
{
//	vncp::VideoPackage::GetInstance()->logoutDevice(m_userID);

//	UserManager::GetInstance()->removeUser(m_userID);
}

void FaceLibrary::init(const QString &mac, const QString &ip, const QString &username, const QString &password)
{
	m_ip = ip;
	m_username = username;
	m_password = password;

	//	vncp::VideoPackage::GetInstance()->loginDevice(mac,ip,6060,username,password,m_userID);
}

void FaceLibrary::initGui()
{
	QFont font("Microsoft YaHei",adaptx(10));

	//脸库表
	m_faceLibraryTable = new QTableWidget(10,6,this);
	check_pointer_no_return(m_faceLibraryTable)
	m_faceLibraryTable->setGeometry(adaptxywh(50,100,1200,550));
	QStringList header{"Name","Image Count","Create Time","Manage","Delete","Edit"};
	m_faceLibraryTable->setHorizontalHeaderLabels(header);
	m_faceLibraryTable->setSelectionBehavior(QAbstractItemView::SelectRows);
	m_faceLibraryTable->setSelectionMode(QAbstractItemView::SingleSelection);
//	m_faceLibraryTable->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
//	m_faceLibraryTable->setColumnWidth(0,adaptx(100));
	m_faceLibraryTable->setColumnWidth(0,adaptx(500));
	m_faceLibraryTable->setColumnWidth(1,adaptx(100));
	m_faceLibraryTable->setColumnWidth(2,adaptx(276));
	m_faceLibraryTable->setColumnWidth(3,adaptx(100));
	m_faceLibraryTable->setColumnWidth(4,adaptx(100));
	m_faceLibraryTable->setColumnWidth(5,adaptx(100));

	//导出按钮
	QPushButton *exportFaceLibrary{nullptr};
	exportFaceLibrary = new QPushButton(this);
	check_pointer_no_return(exportFaceLibrary)
	exportFaceLibrary->setGeometry(adaptxywh(50+1200-440,60,100,30));
	exportFaceLibrary->setFont(font);
	exportFaceLibrary->setText("Export");
	connect(exportFaceLibrary,&QPushButton::released,this,&FaceLibrary::slot_export);

	//导入按钮
	QPushButton *importFaceLibrary{nullptr};
	importFaceLibrary = new QPushButton(this);
	check_pointer_no_return(importFaceLibrary)
	importFaceLibrary->setGeometry(adaptxywh(50+1200-330,60,100,30));
	importFaceLibrary->setFont(font);
	importFaceLibrary->setText("Import");
	connect(importFaceLibrary,&QPushButton::released,this,&FaceLibrary::slot_import);

	//添加按钮
	QPushButton *add{nullptr};
	add = new QPushButton(this);
	check_pointer_no_return(add)
	add->setGeometry(adaptxywh(50+1200-220,60,100,30));
	add->setFont(font);
	add->setText("Add");
	connect(add,&QPushButton::released,this,&FaceLibrary::slot_add);

	//删除全部按钮
	QPushButton *removeAll{nullptr};
	removeAll = new QPushButton(this);
	check_pointer_no_return(removeAll)
	removeAll->setGeometry(adaptxywh(50+1200-110,60,100,30));
	removeAll->setFont(font);
	removeAll->setText("Delete All");
	connect(removeAll,&QPushButton::released,this,&FaceLibrary::slot_removeAll);
}

void FaceLibrary::updateFaceLibraryInfo()
{
	func_lock

	m_faceLibraryTable->clearContents();

	//获取所有的脸库信息
//	FaceLibraryList items;
	m_allFaceLibrary.clear();
	HttpManagerErcode ercode = FaceLibraryPackage::GetInstance()->getFaceLibraryList(m_ip,m_username,m_password,m_allFaceLibrary);
	check_http_ercode_no_return(ercode,"Get face library list failed");

	//将脸库信息添加到界面上
	FaceLibraryList::iterator iter;
	for(iter = m_allFaceLibrary.begin();iter != m_allFaceLibrary.end();++iter)
	{
		int offset = iter-m_allFaceLibrary.begin();

		//脸库ID
//		QTableWidgetItem *libraryIDItem{nullptr};
//		libraryIDItem = new QTableWidgetItem(iter->libraryID);
//		check_pointer_no_return(libraryIDItem)
//		libraryIDItem->setFlags(libraryIDItem->flags() & (~Qt::ItemIsEditable));
//		m_faceLibraryTable->setItem(offset,0,libraryIDItem);

		//脸库名称
		QTableWidgetItem *libraryName{nullptr};
		libraryName = new QTableWidgetItem("unBindGroup"==iter->libraryName?QStringLiteral("未绑定脸库"):iter->libraryName);
		check_pointer_no_return(libraryName)
		libraryName->setFlags(libraryName->flags() & (~Qt::ItemIsEditable));
		m_faceLibraryTable->setItem(offset,0,libraryName);

		//脸库里的人员信息数量
		QTableWidgetItem *imageCountItem{nullptr};
		imageCountItem = new QTableWidgetItem(iter->imageCount);
		check_pointer_no_return(imageCountItem)
		imageCountItem->setFlags(imageCountItem->flags() & (~Qt::ItemIsEditable));
		m_faceLibraryTable->setItem(offset,1,imageCountItem);

		//脸库创建时间
		QTableWidgetItem *createTimeItem{nullptr};
		createTimeItem = new QTableWidgetItem(iter->createTime);
		check_pointer_no_return(createTimeItem)
		createTimeItem->setFlags(createTimeItem->flags() & (~Qt::ItemIsEditable));
		m_faceLibraryTable->setItem(offset,2,createTimeItem);

		//管理脸库按钮
		QPushButton *manage{nullptr};
		manage= new QPushButton(this);
		check_pointer_no_return(manage)
		manage->setText("manage");
		m_faceLibraryTable->setCellWidget(offset,3,manage);
		connect(manage,&QPushButton::released,this,&FaceLibrary::slot_manage);

		if(0 == offset)
			continue;

		//删除某个脸库按钮
		QPushButton *remove{nullptr};
		remove = new QPushButton(this);
		check_pointer_no_return(remove)
		remove->setText("Delete");
		m_faceLibraryTable->setCellWidget(offset,4,remove);
		connect(remove,&QPushButton::released,this,&FaceLibrary::slot_remove);

		//编辑脸库名称按钮
		QPushButton *edit{nullptr};
		edit = new QPushButton(this);
		check_pointer_no_return(edit)
		edit->setText("Edit");
		m_faceLibraryTable->setCellWidget(offset,5,edit);
		connect(edit,&QPushButton::released,this,&FaceLibrary::slot_edit);
	}
}

void FaceLibrary::slot_manage()
{
	func_lock

	if(nullptr == sender())
		return;

	QPushButton *manage = qobject_cast<QPushButton *>(sender());
	QModelIndex index = m_faceLibraryTable->indexAt(QPoint(manage->frameGeometry().x(),manage->frameGeometry().y()));
	int row = index.row();

	FaceLibraryItemWidget *itemWidget = new FaceLibraryItemWidget;
	connect(itemWidget,&FaceLibraryItemWidget::signal_updateFaceLibrary,[this]()
	{
		updateFaceLibraryInfo();
	});
	itemWidget->init("",m_ip,m_username,m_password);
	itemWidget->setFaceLibraryID(m_allFaceLibrary.at(row).libraryID,m_allFaceLibrary.at(row).libraryName);
	//当前脸库是“未绑定脸库”，启用绑定按钮
	if(0 == row)
		itemWidget->setUseForUnBindGroup(true);
	else
		itemWidget->setUseForUnBindGroup(false);
	itemWidget->show();
}

void FaceLibrary::slot_remove()
{
	func_lock

	warning_box("Confirm delete?")

	if(nullptr == sender())
		return;

	QPushButton *remove = qobject_cast<QPushButton *>(sender());
	QModelIndex index = m_faceLibraryTable->indexAt(QPoint(remove->frameGeometry().x(),remove->frameGeometry().y()));
	int row = index.row();

	HttpManagerErcode ercode = FaceLibraryPackage::GetInstance()->deleteFaceLibrary(m_ip,m_username,m_password,m_allFaceLibrary.at(row).libraryID);
	check_http_ercode_no_return(ercode,"Delete face library failed")

	message_output("Delete successfully!");

	updateFaceLibraryInfo();
}

void FaceLibrary::slot_edit()
{
	func_lock

	if(nullptr == sender())
		return;

	//获取点击的行数
	QPushButton *edit = qobject_cast<QPushButton *>(sender());
	QModelIndex index = m_faceLibraryTable->indexAt(QPoint(edit->frameGeometry().x(),edit->frameGeometry().y()));
	int row = index.row();

	NameWidget *nameWidget = new NameWidget;
	nameWidget->show();
	for(const FaceLibraryItem &item:m_allFaceLibrary)
		nameWidget->addExistFaceLibraryName(item.libraryName);
	nameWidget->addExistFaceLibraryName(QStringLiteral("未绑定脸库"));
	connect(nameWidget,&NameWidget::signal_name,[this,row](QString name)
	{
		func_lock

		HttpManagerErcode ercode = FaceLibraryPackage::GetInstance()->editFaceLibraryName(m_ip,m_username,m_password,m_allFaceLibrary.at(row).libraryID,name);
		check_http_ercode_no_return(ercode,"Edit face library name failed!");

		message_output("Edit face library name successfully!");

		updateFaceLibraryInfo();
	});

//	QFont font("微软雅黑",adaptx(10));

//	QWidget *editWidget = new QWidget;
//	editWidget->setAttribute(Qt::WA_DeleteOnClose);
//	editWidget->setGeometry(adaptxywh(100+1300/2-150,100+700/2-50,300,100));

//	QLabel *nameEditLabel = new QLabel(editWidget);
//	nameEditLabel->setFont(font);
//	nameEditLabel->setGeometry(adaptxywh(10,10,100,30));
//	nameEditLabel->setText("Library Name");
//	nameEditLabel->show();

//	QLineEdit *nameEdit = new QLineEdit(editWidget);
//	nameEdit->setFont(font);
//	nameEdit->setGeometry(adaptxywh(110,10,180,30));
//	nameEdit->show();

//	QPushButton *save = new QPushButton(editWidget);
//	save->setGeometry(adaptxywh(150-40,50,80,30));
//	save->setFont(font);
//	save->setText("Save");
//	connect(save,&QPushButton::released,[this,editWidget,nameEdit,row]()
//	{
//		if(true==nameEdit->text().isEmpty())
//		{
//			message_output("Name can not be empty");
//			return;
//		}

//		FaceLibraryPackage::GetInstance()->editFaceLibraryName(m_ip,m_username,m_password,m_faceLibraryTable->item(row,0)->text(),nameEdit->text());
//		editWidget->close();

//		updateFaceLibraryInfo();
//	});

	//	editWidget->show();
}

void FaceLibrary::slot_add()
{
	func_lock

	NameWidget *nameWidget = new NameWidget;
	nameWidget->setModal(true);
	nameWidget->setWindowFlags(Qt::CustomizeWindowHint | Qt::WindowCloseButtonHint);
	nameWidget->show();
	connect(nameWidget,&NameWidget::signal_name,[this](QString name)
	{
		func_lock

		HttpManagerErcode ercode = FaceLibraryPackage::GetInstance()->addFaceLibraryName(m_ip,m_username,m_password,name);
		check_http_ercode_no_return(ercode,"Edit face library name failed!")

		message_output("Add face library successfully!");

		updateFaceLibraryInfo();
	});
}

void FaceLibrary::slot_export()
{
	func_lock

	QString fileName = QFileDialog::getSaveFileName(this, tr("Save File"),"",tr("FaceLibrary (*.bin)"));
	if(true == fileName.isEmpty())
		return;

	QByteArray data;
	HttpManagerErcode ercode = FaceLibraryPackage::GetInstance()->exportFaceLibrary(m_ip,m_username,m_password,data);
	check_http_ercode_no_return(ercode,"Export face library failed!")

	QFile file(fileName);
	file.open(QIODevice::WriteOnly);
	file.write(data,data.length());
	file.close();

	message_output("Export face library successfully!");
}

void FaceLibrary::slot_import()
{
	func_lock

	QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"),"",tr("Face Library (*.bin)"));
	if(true == fileName.isEmpty())
		return;

	QFile file(fileName);
	file.open(QIODevice::ReadOnly);
	QByteArray data;
	data = file.readAll();
	file.close();

	HttpManagerErcode ercode = FaceLibraryPackage::GetInstance()->importFaceLibrary(m_ip,m_username,m_password,fileName.mid(fileName.lastIndexOf("/")+1),data);
	check_http_ercode_no_return(ercode,"Import face library failed!")

	message_output("Import successfully! The device will reboot!");
}

void FaceLibrary::slot_removeAll()
{
	func_lock

	warning_box("Confirm delete all face library?")

	FaceLibraryList::iterator iter;
	for(iter = m_allFaceLibrary.begin()+1;iter!=m_allFaceLibrary.end();++iter)
	{
		HttpManagerErcode ercode = FaceLibraryPackage::GetInstance()->deleteFaceLibrary(m_ip,m_username,m_password,iter->libraryID);
		check_http_ercode_no_return(ercode,"Delete face library failed")
	}

	message_output("Delete all face lbrary successfully!");

	//删除完成之后刷新脸库界面
	updateFaceLibraryInfo();
}
