﻿#include "src/inc/gui/facelibrary/facelibraryitemshowwidget.h"

#include "src/inc/aifacepch.h"
#include "src/inc/httpmanager/facelibrarypackage.h"

#include <QTableWidget>
#include <QPushButton>
#include <QMessageBox>
#include <QComboBox>
#include <QLabel>
#include <QLineEdit>
#include <QDateTimeEdit>

FaceLibraryItemShowWidget::FaceLibraryItemShowWidget(QWidget *parent) : QWidget(parent)
{
	this->setGeometry(adaptxywh(100,100,1300,700));
//	this->setAttribute(Qt::WA_DeleteOnClose);

	initGui();
}

void FaceLibraryItemShowWidget::init(const QString &mac, const QString &ip, const QString &username, const QString &password)
{
	m_ip = ip;
	m_username = username;
	m_password = password;
}

void FaceLibraryItemShowWidget::setSourceLibraryID(const QString &faceLibraryID,const QString &faceLibraryName)
{
	m_sourceFaceLibraryID = faceLibraryID;
	m_libraryName->setText("unBindGroup"==faceLibraryName?QStringLiteral("未绑定脸库"):faceLibraryName);

	//更新页码信息
	updatePageInfo();
	//更新当前页脸库条目
	updateItems(10,1);
}

void FaceLibraryItemShowWidget::setTargetLibraryID(const QString &faceLibraryID)
{
	m_targetFaceLibraryID = faceLibraryID;
}

void FaceLibraryItemShowWidget::initGui()
{
	QFont font("Microsoft YaHei",adaptx(10));

	m_libraryName = new QLabel(this);
	check_pointer_no_return(m_libraryName)
	m_libraryName->setGeometry(this->width()/2-adaptx(300)/2,adapty(10),adaptx(300),adapty(40));
	m_libraryName->setAlignment(Qt::AlignCenter);
	m_libraryName->setFont(QFont("Microsoft YaHei",adaptx(20)));

	m_tableWidget = new QTableWidget(10,7,this);
	check_pointer_no_return(m_tableWidget)
	m_tableWidget->setGeometry(adaptxywh(50,100,1200,550));
	QStringList header{"Name","Sex","Birthday","city","Certificate Type","IDCard","Model Status"};
	m_tableWidget->setHorizontalHeaderLabels(header);
	m_tableWidget->setSelectionBehavior(QAbstractItemView::SelectRows);
	m_tableWidget->setSelectionMode(QAbstractItemView::MultiSelection);
//	m_faceLibraryTable->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
//	m_tableWidget->setColumnWidth(0,adaptx(50));
	m_tableWidget->setColumnWidth(0,adaptx(200));
	m_tableWidget->setColumnWidth(1,adaptx(50));
	m_tableWidget->setColumnWidth(2,adaptx(150));
	m_tableWidget->setColumnWidth(3,adaptx(150));
	m_tableWidget->setColumnWidth(4,adaptx(150));
	m_tableWidget->setColumnWidth(5,adaptx(250));
	m_tableWidget->setColumnWidth(6,adaptx(226));
//	m_tableWidget->setColumnWidth(7,adaptx(100));

	//页码信息
	QLabel *pageSIzeLabel{nullptr};
	pageSIzeLabel = new QLabel(this);
	check_pointer_no_return(pageSIzeLabel)
	pageSIzeLabel->setGeometry(adaptxywh(50,660,70,30));
	pageSIzeLabel->setFont(font);
	pageSIzeLabel->setText("Page size:");

	//每页容纳的
	m_pageSize = new QComboBox(this);
	check_pointer_no_return(m_pageSize)
	m_pageSize->setGeometry(adaptxywh(130,660,50,30));
	m_pageSize->setFont(font);
	QStringList pageSizeItems{"10","20","30","40","50"};
	m_pageSize->addItems(pageSizeItems);
	connect(m_pageSize,&QComboBox::currentTextChanged,[this](QString size)
	{
		func_lock

		updatePageInfo();
		updateItems(size.toInt(),1);
	});

	m_sumPageNum = new QLabel(this);
	check_pointer_no_return(m_sumPageNum)
	m_sumPageNum->setGeometry(adaptxywh(200,660,250,30));
	m_sumPageNum->setFont(font);

	m_page = new QComboBox(this);
	check_pointer_no_return(m_page)
	m_page->setGeometry(adaptxywh(460,660,50,30));
	m_page->setFont(font);
	connect(m_page,static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),[this](int index)
	{
		func_lock

		updateItems(m_pageSize->currentText().toInt(),index+1);
	});

	QPushButton *ok{nullptr};
	ok = new QPushButton(this);
	check_pointer_no_return(ok);
	ok->setGeometry(adaptxywh(1150,660,80,30));
	ok->setFont(font);
	ok->setText("import");
	connect(ok,&QPushButton::released,[this]()
	{
		func_lock

		//获取所有选中的行号
		QVector<int> indexes;//保存选中行的索引
		QItemSelectionModel *selections = m_tableWidget->selectionModel(); //返回当前的选择模式
		QModelIndexList selectedsList = selections->selectedIndexes(); //返回所有选定的模型项目索引列表
		for (int i = 0; i < selectedsList.count(); i++)
		{
			int row = selectedsList.at(i).row();
			if(false == indexes.contains(row))
				indexes.push_back(row);
		}

		QVector<QVector<QString>> datas;
		for(const int &index:indexes)
		{
			if(index >= m_infos.length())
				continue;
#ifdef UniqueID_Support
			datas.append({m_sourceFaceLibraryID,m_targetFaceLibraryID,m_infos[index].uniqueID});
#else
			datas.append({m_sourceFaceLibraryID,m_targetFaceLibraryID,m_infos[index].personID});
#endif
		}

		HttpManagerErcode ercode = FaceLibraryPackage::GetInstance()->bindPersonToFaceLibrary(m_ip,m_username,m_password,datas);
		check_http_ercode_no_return(ercode,"Import from other library failed!")

		message_output("Import from other library successfully!");
		this->close();

		emit signal_updateFaceLibraryItem();
		emit signal_updateFaceLibrary();
	});
}

void FaceLibraryItemShowWidget::updatePageInfo(const bool &searchFlag, const QString &name,const QString &sex,const QString &certificateType,const QString &certificateID,const QString &country,const QString address,const QString &startBirthday,const QString &stopBirthday)
{
	func_lock

	//获得识别记录总数
	QString itemCount;

	HttpManagerErcode ercode{HttpErcode_None};
	if(true == searchFlag)
		ercode = FaceLibraryPackage::GetInstance()->getFaceLibraryCount(m_ip,m_username,m_password,m_sourceFaceLibraryID,itemCount,
																		 name,sex,certificateType,certificateID,country,address,startBirthday,stopBirthday);
	else
		ercode = FaceLibraryPackage::GetInstance()->getFaceLibraryCount(m_ip,m_username,m_password,m_sourceFaceLibraryID,itemCount);
	check_http_ercode_no_return(ercode,"Get face library count failed!")

	//计算总页数
	int sumPage = itemCount.toInt()/m_pageSize->currentText().toInt()+1;
	//显示总页数
	m_sumPageNum->setText("Sum item count:"+itemCount+"      Sum page:"+QString::number(sumPage));
	m_page->clear();
	for(int i = 0;i<sumPage;++i)
		m_page->addItem(QString::number(i+1));

	m_tableWidget->setRowCount(m_pageSize->currentText().toInt());
}

void FaceLibraryItemShowWidget::updateItems(const int &pageSize, const int &pageIndex,
										const bool &searchFlag, const QString &name,const QString &sex,const QString &certificateType,const QString &certificateID,const QString &country,const QString address,const QString &startBirthday,const QString &stopBirthday)
{
	func_lock

	HttpManagerErcode ercode{HttpErcode_None};

	//根据脸库ID获取脸库中的所有条目
	m_infos.clear();
	//判断是否是搜索
	if(true==searchFlag)
		ercode = FaceLibraryPackage::GetInstance()->getFaceLibraryByLibraryID(m_ip,m_username,m_password,m_sourceFaceLibraryID,pageSize,pageIndex,m_infos,
																				   name,sex,certificateType,certificateID,country,address,startBirthday,stopBirthday);
	else
		ercode = FaceLibraryPackage::GetInstance()->getFaceLibraryByLibraryID(m_ip,m_username,m_password,m_sourceFaceLibraryID,pageSize,pageIndex,m_infos);
	check_http_ercode_no_return(ercode,"Get face library by library ID failed!")

	//清除表中的信息
	m_tableWidget->clearContents();

	//将脸库信息添加到列表中
	FaceLibraryItemInfoList::Iterator iter;
	for(iter = m_infos.begin();iter != m_infos.end();++iter)
	{
		int offset = iter-m_infos.begin();

//		QTableWidgetItem *index = new QTableWidgetItem(QString::number(offset));
//		check_pointer_no_return(index)
//		index->setFlags(index->flags() & (~Qt::ItemIsEditable));
//		m_tableWidget->setItem(offset,0,index);

		QTableWidgetItem *name = new QTableWidgetItem(iter->name);
		check_pointer_no_return(name)
		name->setFlags(name->flags() & (~Qt::ItemIsEditable));
		m_tableWidget->setItem(offset,0,name);

		QTableWidgetItem *sex = new QTableWidgetItem(iter->sex);
		check_pointer_no_return(sex)
		sex->setFlags(sex->flags() & (~Qt::ItemIsEditable));
		m_tableWidget->setItem(offset,1,sex);

		QTableWidgetItem *birthday = new QTableWidgetItem(iter->birthday.mid(0,iter->birthday.lastIndexOf(" ")));
		check_pointer_no_return(birthday)
		birthday->setFlags(birthday->flags() & (~Qt::ItemIsEditable));
		m_tableWidget->setItem(offset,2,birthday);

		QTableWidgetItem *address = new QTableWidgetItem(iter->address);
		check_pointer_no_return(address)
		address->setFlags(address->flags() & (~Qt::ItemIsEditable));
		m_tableWidget->setItem(offset,3,address);

		QTableWidgetItem *certificateType = new QTableWidgetItem(iter->certificateType);
		check_pointer_no_return(certificateType)
		certificateType->setFlags(certificateType->flags() & (~Qt::ItemIsEditable));
		m_tableWidget->setItem(offset,4,certificateType);

		QTableWidgetItem *certificateID = new QTableWidgetItem(iter->certificateID);
		check_pointer_no_return(certificateID)
		certificateID->setFlags(certificateID->flags() & (~Qt::ItemIsEditable));
		m_tableWidget->setItem(offset,5,certificateID);

		QTableWidgetItem *modelStatus = new QTableWidgetItem(iter->modelStatus);
		check_pointer_no_return(modelStatus)
		modelStatus->setFlags(modelStatus->flags() & (~Qt::ItemIsEditable));
		m_tableWidget->setItem(offset,6,modelStatus);
	}
}
