﻿#include "src/inc/gui/facelibrary/facelibraryitemwidget.h"

#include "src/inc/aifacepch.h"
#include "src/inc/httpmanager/facelibrarypackage.h"
#include "src/inc/gui/facelibrary/personitemeditwidget.h"
#include "src/inc/gui/facelibrary/personitemaddwidget.h"
#include "src/inc/gui/facelibrary/facelibraryitemshowwidget.h"
#include "src/inc/gui/status/statuswidget.h"

#include <QTableWidget>
#include <QPushButton>
#include <QMessageBox>
#include <QComboBox>
#include <QLabel>
#include <QLineEdit>
#include <QDateTimeEdit>
#include <QFileDialog>
#include <QRegExp>

#include <thread>

FaceLibraryItemWidget::FaceLibraryItemWidget(QWidget *parent) : QWidget(parent)
{
	this->setGeometry(adaptxywh(100,100,1300,700));
//	this->setAttribute(Qt::WA_DeleteOnClose);

	initGui();
}

void FaceLibraryItemWidget::init(const QString &mac, const QString &ip, const QString &username, const QString &password)
{
	m_ip = ip;
	m_username = username;
	m_password = password;
}

void FaceLibraryItemWidget::setFaceLibraryID(const QString &faceLibraryID,const QString &faceLibraryName)
{
	m_faceLibraryID = faceLibraryID;
	m_libraryName->setText("unBindGroup"==faceLibraryName?QStringLiteral("未绑定脸库"):faceLibraryName);

	//更新页码信息
	updatePageInfo();
	//更新当前页脸库条目
	updateItems(10,1);
}

void FaceLibraryItemWidget::setUseForUnBindGroup(const bool &isUseForUnBindGroup)
{
	if(true == isUseForUnBindGroup)
	{
		m_removeAll->setGeometry(adaptxywh(50+1200-110,60,100,30));
		m_removeAll->show();

		m_remove->setGeometry(adaptxywh(50+1200-220,60,100,30));
		m_remove->show();

		m_bind->setGeometry(adaptxywh(50+1200-330,60,100,30));
		m_bind->show();

		m_refresh->setGeometry(adaptxywh(50+1200-440,60,100,30));
		m_refresh->show();

		m_search->setGeometry(adaptxywh(50+1200-550,60,100,30));
		m_search->show();

		m_add->hide();
		m_addMulti->hide();
	}
	else
	{
		m_remove->setGeometry(adaptxywh(50+1200-110,60,100,30));
		m_remove->show();

		m_add->setGeometry(adaptxywh(50+1200-220,60,100,30));
		m_add->show();

		m_addMulti->setGeometry(adaptxywh(50+1200-330,60,100,30));
		m_addMulti->show();

		m_refresh->setGeometry(adaptxywh(50+1200-440,60,100,30));
		m_refresh->show();

		m_search->setGeometry(adaptxywh(50+1200-550,60,100,30));
		m_search->show();

		m_removeAll->hide();
		m_bind->hide();
	}
}

void FaceLibraryItemWidget::initGui()
{
	QFont font("Microsoft YaHei",adaptx(10));

	m_libraryName = new QLabel(this);
	check_pointer_no_return(m_libraryName)
	m_libraryName->setGeometry(this->width()/2-adaptx(300)/2,adapty(10),adaptx(300),adapty(40));
	m_libraryName->setAlignment(Qt::AlignCenter);
	m_libraryName->setFont(QFont("Microsoft YaHei",adaptx(20)));

	m_tableWidget = new QTableWidget(10,8,this);
	check_pointer_no_return(m_tableWidget)
	m_tableWidget->setGeometry(adaptxywh(50,100,1200,550));
	QStringList header{"Name","Sex","Birthday","city","Certificate Type","IDCard","Model Status","Edit"};
	m_tableWidget->setHorizontalHeaderLabels(header);
	m_tableWidget->setSelectionBehavior(QAbstractItemView::SelectRows);
	m_tableWidget->setSelectionMode(QAbstractItemView::MultiSelection);
//	m_faceLibraryTable->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
//	m_tableWidget->setColumnWidth(0,adaptx(50));
	m_tableWidget->setColumnWidth(0,adaptx(200));
	m_tableWidget->setColumnWidth(1,adaptx(50));
	m_tableWidget->setColumnWidth(2,adaptx(150));
	m_tableWidget->setColumnWidth(3,adaptx(150));
	m_tableWidget->setColumnWidth(4,adaptx(150));
	m_tableWidget->setColumnWidth(5,adaptx(250));
	m_tableWidget->setColumnWidth(6,adaptx(126));
	m_tableWidget->setColumnWidth(7,adaptx(100));

	//批量添加人员信息
	m_addMulti = new QPushButton(this);
	check_pointer_no_return(m_addMulti)
	m_addMulti->setGeometry(adaptxywh(50+1200-770,60,100,30));
	m_addMulti->setFont(font);
	m_addMulti->setText("Multi Add");
	connect(m_addMulti,&QPushButton::released,this,&FaceLibraryItemWidget::slot_addMulitPersonInfo);
	m_addMulti->hide();

	//添加人员信息
//	QPushButton *add{nullptr};
	m_add = new QPushButton(this);
	check_pointer_no_return(m_add)
	m_add->setGeometry(adaptxywh(50+1200-660,60,100,30));
	m_add->setFont(font);
	m_add->setText("Add");
	connect(m_add,&QPushButton::released,this,&FaceLibraryItemWidget::slot_addPersonInfo);

	//绑定
	m_bind = new QPushButton(this);
	check_pointer_no_return(m_bind);
	m_bind->setGeometry(adaptxywh(50+1200-220,60,100,30));
	m_bind->setFont(font);
	m_bind->setText("Bind");
	connect(m_bind,&QPushButton::released,this,&FaceLibraryItemWidget::slot_bind);

	//搜索按钮
//	QPushButton *search{nullptr};
	m_search = new QPushButton(this);
	check_pointer_no_return(m_search)
	m_search->setGeometry(adaptxywh(50+1200-440,60,100,30));
	m_search->setFont(font);
	m_search->setText("Search");
	connect(m_search,&QPushButton::released,this,&FaceLibraryItemWidget::slot_search);

	//刷新
//	QPushButton *m_refresh{nullptr};
	m_refresh = new QPushButton(this);
	check_pointer_no_return(m_refresh)
	m_refresh->setGeometry(adaptxywh(50+1200-330,60,100,30));
	m_refresh->setFont(font);
	m_refresh->setText("Refresh");
	connect(m_refresh,&QPushButton::released,[this]()
	{
		func_lock

		m_searchContent.clear();
		updateItems(m_pageSize->currentText().toInt(),m_page->currentIndex()+1);
	});

	//删除人员信息
//	QPushButton *remove{nullptr};
	m_remove = new QPushButton(this);
	check_pointer_no_return(m_remove)
	m_remove->setGeometry(adaptxywh(50+1200-220,60,100,30));
	m_remove->setFont(font);
	m_remove->setText("Remove");
	connect(m_remove,&QPushButton::released,this,&FaceLibraryItemWidget::slot_removeItem);

	m_removeAll = new QPushButton(this);
	check_pointer_no_return(m_removeAll)
	m_removeAll->setGeometry(adaptxywh(50+1200-110,60,100,30));
	m_removeAll->setFont(font);
	m_removeAll->setText("Remove All");
	connect(m_removeAll,&QPushButton::released,this,&FaceLibraryItemWidget::slot_removeAll);

	//页码信息
	QLabel *pageSIzeLabel{nullptr};
	pageSIzeLabel = new QLabel(this);
	check_pointer_no_return(pageSIzeLabel)
	pageSIzeLabel->setGeometry(adaptxywh(50,660,70,30));
	pageSIzeLabel->setFont(font);
	pageSIzeLabel->setText("Page size:");

	//每页容纳的
	m_pageSize = new QComboBox(this);
	check_pointer_no_return(m_pageSize)
	m_pageSize->setGeometry(adaptxywh(130,660,50,30));
	m_pageSize->setFont(font);
	QStringList pageSizeItems{"10","20","30","40","50"};
	m_pageSize->addItems(pageSizeItems);
	connect(m_pageSize,&QComboBox::currentTextChanged,[this](QString size)
	{
		func_lock

		updatePageInfo();
		updateItems(size.toInt(),1);
	});

	m_sumPageNum = new QLabel(this);
	check_pointer_no_return(m_sumPageNum)
	m_sumPageNum->setGeometry(adaptxywh(200,660,250,30));
	m_sumPageNum->setFont(font);

	m_page = new QComboBox(this);
	check_pointer_no_return(m_page)
	m_page->setGeometry(adaptxywh(460,660,50,30));
	m_page->setFont(font);
	connect(m_page,static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),[this](int index)
	{
		func_lock

		updateItems(m_pageSize->currentText().toInt(),index+1);
	});
}

void FaceLibraryItemWidget::updatePageInfo(const bool &searchFlag, const QString &name,const QString &sex,const QString &certificateType,const QString &certificateID,const QString &country,const QString address,const QString &startBirthday,const QString &stopBirthday)
{	
	func_lock

	//获得识别记录总数
	QString itemCount;

	HttpManagerErcode ercode{HttpErcode_None};
	if(true == searchFlag)
		ercode = FaceLibraryPackage::GetInstance()->getFaceLibraryCount(m_ip,m_username,m_password,m_faceLibraryID,itemCount,
																		 name,sex,certificateType,certificateID,country,address,startBirthday,stopBirthday);
	else
		ercode = FaceLibraryPackage::GetInstance()->getFaceLibraryCount(m_ip,m_username,m_password,m_faceLibraryID,itemCount);
	check_http_ercode_no_return(ercode,"Get face library count failed!")

	//计算总页数
	int sumPage = (0==itemCount.toInt()?1:(itemCount.toInt()-1)/m_pageSize->currentText().toInt()+1);
	//显示总页数
	m_sumPageNum->setText("Sum item count:"+itemCount+"      Sum page:"+QString::number(sumPage));
	m_page->disconnect();
	m_page->clear();
	for(int i = 0;i<sumPage;++i)
		m_page->addItem(QString::number(i+1));
	connect(m_page,static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),[this](int index)
	{
		func_lock

		updateItems(m_pageSize->currentText().toInt(),index+1);
	});

	m_tableWidget->setRowCount(m_pageSize->currentText().toInt());
}

void FaceLibraryItemWidget::updateItems(const int &pageSize, const int &pageIndex,
										const bool &searchFlag, const QString &name,const QString &sex,const QString &certificateType,const QString &certificateID,const QString &country,const QString address,const QString &startBirthday,const QString &stopBirthday)
{
	func_lock

	HttpManagerErcode ercode{HttpErcode_None};

	//根据脸库ID获取脸库中的所有条目
	m_infos.clear();
	//判断是否是搜索
	if(true==searchFlag)
		ercode = FaceLibraryPackage::GetInstance()->getFaceLibraryByLibraryID(m_ip,m_username,m_password,m_faceLibraryID,pageSize,pageIndex,m_infos,
																				   name,sex,certificateType,certificateID,country,address,startBirthday,stopBirthday);
	else
		ercode = FaceLibraryPackage::GetInstance()->getFaceLibraryByLibraryID(m_ip,m_username,m_password,m_faceLibraryID,pageSize,pageIndex,m_infos);
	check_http_ercode_no_return(ercode,"Get face library by library ID failed!")

	//清除表中的信息
	m_tableWidget->clearContents();

	//将脸库信息添加到列表中
	FaceLibraryItemInfoList::Iterator iter;
	for(iter = m_infos.begin();iter != m_infos.end();++iter)
	{
		int offset = iter-m_infos.begin();

//		QTableWidgetItem *index = new QTableWidgetItem(QString::number(offset));
//		check_pointer_no_return(index)
//		index->setFlags(index->flags() & (~Qt::ItemIsEditable));
//		m_tableWidget->setItem(offset,0,index);

		QTableWidgetItem *name = new QTableWidgetItem(iter->name);
		check_pointer_no_return(name)
		name->setFlags(name->flags() & (~Qt::ItemIsEditable));
		m_tableWidget->setItem(offset,0,name);

		QTableWidgetItem *sex = new QTableWidgetItem(iter->sex);
		check_pointer_no_return(sex)
		sex->setFlags(sex->flags() & (~Qt::ItemIsEditable));
		m_tableWidget->setItem(offset,1,sex);

		QTableWidgetItem *birthday = new QTableWidgetItem(iter->birthday.mid(0,iter->birthday.lastIndexOf(" ")));
		check_pointer_no_return(birthday)
		birthday->setFlags(birthday->flags() & (~Qt::ItemIsEditable));
		m_tableWidget->setItem(offset,2,birthday);

		QTableWidgetItem *address = new QTableWidgetItem(iter->address);
		check_pointer_no_return(address)
		address->setFlags(address->flags() & (~Qt::ItemIsEditable));
		m_tableWidget->setItem(offset,3,address);

		QTableWidgetItem *certificateType = new QTableWidgetItem(iter->certificateType);
		check_pointer_no_return(certificateType)
		certificateType->setFlags(certificateType->flags() & (~Qt::ItemIsEditable));
		m_tableWidget->setItem(offset,4,certificateType);

		QTableWidgetItem *certificateID = new QTableWidgetItem(iter->certificateID);
		check_pointer_no_return(certificateID)
		certificateID->setFlags(certificateID->flags() & (~Qt::ItemIsEditable));
		m_tableWidget->setItem(offset,5,certificateID);

		QTableWidgetItem *modelStatus = new QTableWidgetItem(iter->modelStatus);
		check_pointer_no_return(modelStatus)
		modelStatus->setFlags(modelStatus->flags() & (~Qt::ItemIsEditable));
		m_tableWidget->setItem(offset,6,modelStatus);

		QPushButton *edit = new QPushButton(this);
		check_pointer_no_return(edit)
		edit->setText("Edit");
		m_tableWidget->setCellWidget(offset,7,edit);

		FaceLibraryItemInfo info = *iter;
		connect(edit,&QPushButton::released,[this,info]()
		{
			func_lock

			//脸库条目详情页面
			PersonItemEditWidget *personItemEditWidget = new PersonItemEditWidget;
			check_pointer_no_return(personItemEditWidget)
			personItemEditWidget->initUserInfo("",m_ip,m_username,m_password);
			personItemEditWidget->setPersonInfo(info);
			personItemEditWidget->show();
			connect(personItemEditWidget,&PersonItemEditWidget::signal_updateFaceLibrary,[this]()
			{
				func_lock

				QString oldIndex = m_page->currentText();
				updateItems(10,oldIndex.toInt());
			});
		});
	}
}

void FaceLibraryItemWidget::slot_editPersonInfo()
{
	func_lock

	PersonItemEditWidget *personItemEditWidget{nullptr};
	personItemEditWidget = new PersonItemEditWidget;
	check_pointer_no_return(personItemEditWidget)
	personItemEditWidget->initUserInfo("",m_ip,m_username,m_password);
	personItemEditWidget->show();
}

void FaceLibraryItemWidget::slot_addPersonInfo()
{
	func_lock

	//创建选择添加方式窗体
	QDialog *addSelect{nullptr};
	addSelect = new QDialog;
	check_pointer_no_return(addSelect)
	addSelect->setWindowFlags(Qt::CustomizeWindowHint | Qt::WindowCloseButtonHint);
	addSelect->setGeometry(adaptxywh(100+1300/2-150,100+700/2-50,300,150));
	addSelect->show();

	QFont font("Microsoft YaHei",adaptx(10));

	QLabel *importModeLabel{nullptr};
	importModeLabel = new QLabel(addSelect);
	check_pointer_no_return(importModeLabel);
	importModeLabel->setFont(font);
	importModeLabel->setGeometry(adaptxywh(10,10,100,30));
	importModeLabel->setText("Import mode");
	importModeLabel->show();

	QComboBox *importModeBox{nullptr};
	importModeBox = new QComboBox(addSelect);
	check_pointer_no_return(importModeBox)
	importModeBox->setFont(font);
	importModeBox->setGeometry(adaptxywh(110,10,180,30));
	importModeBox->addItem("From face library");
	importModeBox->addItem("Custom");
	importModeBox->show();

	QLabel *secondLabel{nullptr};
	secondLabel = new QLabel(addSelect);
	check_pointer_no_return(importModeLabel);
	secondLabel->setFont(font);
	secondLabel->setGeometry(adaptxywh(10,50,100,30));
	secondLabel->setText("From");
	secondLabel->show();

	QComboBox *secondBox{nullptr};
	secondBox = new QComboBox(addSelect);
	check_pointer_no_return(secondBox)
	secondBox->setFont(font);
	secondBox->setGeometry(adaptxywh(110,50,180,30));
	secondBox->show();

	//获取所有脸库名称
	FaceLibraryList items;
	HttpManagerErcode ercode = FaceLibraryPackage::GetInstance()->getFaceLibraryList(m_ip,m_username,m_password,items);
	check_http_ercode_no_return(ercode,"Get face library list failed!")
	for(const FaceLibraryItem &item:items)
	{
		if(item.libraryID == m_faceLibraryID)
			continue;
		secondBox->addItem(item.libraryName=="unBindGroup"?QStringLiteral("未绑定脸库"):item.libraryName);
	}

	connect(importModeBox,static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),[this,secondBox](int index)
	{
		func_lock

		if(0 == index)			//从其他脸库导入
		{
//			secondLabel->setText("Face library");
			secondBox->clear();

			//获取所有脸库名称
			FaceLibraryList items;
			HttpManagerErcode ercode = FaceLibraryPackage::GetInstance()->getFaceLibraryList(m_ip,m_username,m_password,items);
			check_http_ercode_no_return(ercode,"Get face library list failed!")
			for(const FaceLibraryItem &item:items)
			{
				if(item.libraryID == m_faceLibraryID)
					continue;

				secondBox->addItem(item.libraryName=="unBindGroup"?QStringLiteral("未绑定脸库"):item.libraryName);
			}
		}
		else if(1 == index)	//自行添加，比如从本地图片添加
		{
			secondBox->clear();
			secondBox->addItem("Local image");
		}
	});

	QPushButton *sure{nullptr};
	sure = new QPushButton(addSelect);
	check_pointer_no_return(sure);
	sure->setGeometry(adaptxywh(150-40,100,80,30));
	sure->setFont(font);
	sure->setText("OK");
	sure->show();
	connect(sure,&QPushButton::released,[this,addSelect,importModeBox,secondBox]()
	{
		func_lock

		if(0 == importModeBox->currentIndex())		//从脸库导入
		{
			FaceLibraryItemShowWidget *showWidget = new FaceLibraryItemShowWidget;
			connect(showWidget,&FaceLibraryItemShowWidget::signal_updateFaceLibraryItem,[this](){updatePageInfo();updateItems(10,1);});
			connect(showWidget,&FaceLibraryItemShowWidget::signal_updateFaceLibrary,this,&FaceLibraryItemWidget::signal_updateFaceLibrary);
			showWidget->show();

			//获取所有脸库名称
			FaceLibraryList items;
			HttpManagerErcode ercode = FaceLibraryPackage::GetInstance()->getFaceLibraryList(m_ip,m_username,m_password,items);
			check_http_ercode_no_return(ercode,"Get face library list failed!")

			QString libraryID;
			QString libraryName;
			for(const FaceLibraryItem &item:items)
			{
				if(0 == secondBox->currentIndex())
				{
					libraryID = item.libraryID;
					libraryName = item.libraryName;
					break;
				}
				else if(secondBox->currentText() == item.libraryName)
				{
					libraryID = item.libraryID;
					libraryName = item.libraryName;
					break;
				}
			}
			showWidget->init("",m_ip,m_username,m_password);
			showWidget->setSourceLibraryID(libraryID,libraryName);
			showWidget->setTargetLibraryID(m_faceLibraryID);
		}
		else if(1==importModeBox->currentIndex())		//自行导入
		{
			PersonItemAddWidget *personItemWidget{nullptr};
			personItemWidget = new PersonItemAddWidget;
			check_pointer_no_return(personItemWidget)

			connect(personItemWidget,&PersonItemAddWidget::signal_updateFaceLibrary,[this]()
			{
				func_lock

				updateItems(10,1);
				emit signal_updateFaceLibrary();
			});
			check_pointer_no_return(personItemWidget)
			personItemWidget->initUserInfo("",m_ip,m_username,m_password);
			personItemWidget->setLibraryID(m_faceLibraryID);
			personItemWidget->show();
		}

		addSelect->close();
	});
}

void FaceLibraryItemWidget::slot_removeItem()
{
	func_lock

	warning_box("Confirm delete?")

	//确认是否从所有脸库中删除
	QString isDeleteFromAllLibrary{"false"};
	if("1" != m_faceLibraryID)
	{
		QMessageBox::StandardButton select = QMessageBox::warning(this, "Warning",
																  "Delete from all library?",
																  QMessageBox::Yes | QMessageBox::No);
		if (select == QMessageBox::Yes)
			isDeleteFromAllLibrary = "true";
		else if (select == QMessageBox::No)
			isDeleteFromAllLibrary = "false";
	}

	//获取当前选中的行
	QList<QTableWidgetItem *> selectItems = m_tableWidget->selectedItems();
	if(true == selectItems.isEmpty())
	{
		message_output("Please select an item!");
		return;
	}

	//获取所有选中的行号
	QVector<int> indexes;//保存选中行的索引
	QItemSelectionModel *selections = m_tableWidget->selectionModel(); //返回当前的选择模式
	QModelIndexList selectedsList = selections->selectedIndexes(); //返回所有选定的模型项目索引列表
	for (int i = 0; i < selectedsList.count(); i++)
	{
		int row = selectedsList.at(i).row();
		if(false == indexes.contains(row))
			indexes.push_back(row);
	}

	QVector<QVector<QString>> datas;
	for(const int &rowIndex:indexes)
	{
		if(rowIndex >= m_infos.length())
			continue;

#ifdef UniqueID_Support
		datas.append({m_infos[rowIndex].libraryID,m_infos[rowIndex].uniqueID});
#else
		datas.append({m_infos[rowIndex].libraryID,m_infos[rowIndex].personID});
#endif
	}
	HttpManagerErcode ercode = FaceLibraryPackage::GetInstance()->deletePersonInfo(m_ip,m_username,m_password,datas,isDeleteFromAllLibrary);
	check_http_ercode_no_return(ercode,"Delete person info failed!")

	updateItems(m_pageSize->currentText().toInt(),1);

	message_output("Delete successfully!");

	emit signal_updateFaceLibrary();
}

void FaceLibraryItemWidget::slot_bind()
{
	func_lock

	//获取当前选中项
	QList<QTableWidgetItem*> selectItems = m_tableWidget->selectedItems();
	if(true == selectItems.isEmpty())
	{
		message_output("Not select any item!");
		return;
	}

	QWidget *bindWidget{nullptr};
	bindWidget = new QWidget;
	check_pointer_no_return(bindWidget);
	bindWidget->setGeometry(adaptxywh(100+1300/2-150,100+700/2-50,300,100));
//	bindWidget->setAttribute(Qt::WA_DeleteOnClose);

	QFont font("Microsoft YaHei",adaptx(10));

	QLabel *nameEditLabel{nullptr};
	nameEditLabel = new QLabel(bindWidget);
	check_pointer_no_return(nameEditLabel)
	nameEditLabel->setFont(font);
	nameEditLabel->setGeometry(adaptxywh(10,10,100,30));
	nameEditLabel->setText("Library Name");
	nameEditLabel->show();

	QComboBox *libraryBox{nullptr};
	libraryBox = new QComboBox(bindWidget);
	check_pointer_no_return(libraryBox)
	libraryBox->setFont(font);
	libraryBox->setGeometry(adaptxywh(110,10,180,30));
	libraryBox->show();

	QPushButton *bind{nullptr};
	bind = new QPushButton(bindWidget);
	check_pointer_no_return(bind)
	bind->setGeometry(adaptxywh(150-40,50,80,30));
	bind->setFont(font);
	bind->setText("Bind");
	bind->show();

	bindWidget->show();

	//获取所有脸库名称
	FaceLibraryList items;
	HttpManagerErcode ercode = FaceLibraryPackage::GetInstance()->getFaceLibraryList(m_ip,m_username,m_password,items);
	check_http_ercode_no_return(ercode,"Get face library list failed!")
	for(const FaceLibraryItem &item:items)
	{
		if("unBindGroup"==item.libraryName)
			continue;
		libraryBox->addItem(item.libraryName);
	}

	connect(bind,&QPushButton::released,[this,items,libraryBox,bindWidget]()
	{
		func_lock

		//获取所有选中的行号
		QVector<int> indexes;//保存选中行的索引
		QItemSelectionModel *selections = m_tableWidget->selectionModel(); //返回当前的选择模式
		QModelIndexList selectedsList = selections->selectedIndexes(); //返回所有选定的模型项目索引列表
		for (int i = 0; i < selectedsList.count(); i++)
		{
			int row = selectedsList.at(i).row();
			if(false == indexes.contains(row))
				indexes.push_back(row);
		}

		QVector<QVector<QString>> datas;
		QString targetLibraryID = items[libraryBox->currentIndex()+1].libraryID;
		for(const int &row:indexes)
		{
			if(row >= m_infos.length())
				continue;

#ifdef UniqueID_Support
			QString uniqueID = m_infos.at(row).uniqueID;
			datas.append({m_faceLibraryID,targetLibraryID,uniqueID});
#else
			QString personID = m_infos.at(row).personID;
			datas.append({m_faceLibraryID,targetLibraryID,personID});
#endif
		}
		HttpManagerErcode ercode = FaceLibraryPackage::GetInstance()->bindPersonToFaceLibrary(m_ip,m_username,m_password,datas);
		check_http_ercode_no_return(ercode,"Bind person to face library failed!")

		message_output("Bind successfully!");
		bindWidget->close();

		updatePageInfo();
		updateItems(10,1);

		emit signal_updateFaceLibrary();
	});
}

void FaceLibraryItemWidget::slot_search()
{	
	func_lock

	QFont font("Microsoft YaHei",adaptx(10));

	QDialog *searchWidget{nullptr};
	searchWidget = new QDialog;
	check_pointer_no_return(searchWidget)
	searchWidget->setModal(true);
	searchWidget->setGeometry(adaptxywh(100+1300/2-1020/2,100+700/2-150,1020,300));

	//姓名
	QLabel *name{nullptr};
	name = new QLabel(searchWidget);
	check_pointer_no_return(name)
	name->setGeometry(adaptxywh(30,30,100,30));
	name->setFont(font);
	name->setText("Name");
	name->show();

	QLineEdit *nameEdit{nullptr};
	nameEdit = new QLineEdit(searchWidget);
	check_pointer_no_return(nameEdit)
	nameEdit->setGeometry(adaptxywh(130,30,200,30));
	nameEdit->setFont(font);
	nameEdit->show();

	//生日
	QLabel *birthdayStart{nullptr};
	birthdayStart = new QLabel(searchWidget);
	check_pointer_no_return(birthdayStart)
	birthdayStart->setGeometry(adaptxywh(360,30,100,30));
	birthdayStart->setFont(font);
	birthdayStart->setText("Birthday Start");
	birthdayStart->show();

	QDateTimeEdit *birthdayStartEdit{nullptr};
	birthdayStartEdit = new QDateTimeEdit(searchWidget);
	check_pointer_no_return(birthdayStartEdit)
	birthdayStartEdit->setGeometry(adaptxywh(460,30,200,30));
	birthdayStartEdit->setDisplayFormat("yyyy-MM-dd");
	birthdayStartEdit->setFont(font);
	birthdayStartEdit->show();

	QLabel *birthdayEnd{nullptr};
	birthdayEnd = new QLabel(searchWidget);
	check_pointer_no_return(birthdayEnd)
	birthdayEnd->setGeometry(adaptxywh(690,30,100,30));
	birthdayEnd->setFont(font);
	birthdayEnd->setText("Birthday End");
	birthdayEnd->show();

	QDateTimeEdit *birthdayEndEdit{nullptr};
	birthdayEndEdit = new QDateTimeEdit(searchWidget);
	check_pointer_no_return(birthdayEndEdit)
	birthdayEndEdit->setGeometry(adaptxywh(790,30,200,30));
	birthdayEndEdit->setDisplayFormat("yyyy-MM-dd");
	birthdayEndEdit->setFont(font);
	birthdayEndEdit->show();

	//国家
	QLabel *country{nullptr};
	country = new QLabel(searchWidget);
	check_pointer_no_return(country)
	country->setGeometry(adaptxywh(30,90,100,30));
	country->setFont(font);
	country->setText("Country");
	country->show();

	QLineEdit *countryEdit{nullptr};
	countryEdit = new QLineEdit(searchWidget);
	check_pointer_no_return(countryEdit)
	countryEdit->setGeometry(adaptxywh(130,90,200,30));
	countryEdit->setFont(font);
	countryEdit->show();

	//省份
	QLabel *province{nullptr};
	province = new QLabel(searchWidget);
	check_pointer_no_return(province)
	province->setGeometry(adaptxywh(360,90,100,30));
	province->setFont(font);
	province->setText("Province");
	province->show();

	QLineEdit *provinceEdit{nullptr};
	provinceEdit = new QLineEdit(searchWidget);
	check_pointer_no_return(provinceEdit)
	provinceEdit->setGeometry(adaptxywh(460,90,200,30));
	provinceEdit->setFont(font);
	provinceEdit->show();

	//城市
	QLabel *city{nullptr};
	city = new QLabel(searchWidget);
	check_pointer_no_return(city)
	city->setGeometry(adaptxywh(690,90,100,30));
	city->setFont(font);
	city->setText("City");
	city->show();

	QLineEdit *cityEdit{nullptr};
	cityEdit = new QLineEdit(searchWidget);
	check_pointer_no_return(cityEdit)
	cityEdit->setGeometry(adaptxywh(790,90,200,30));
	cityEdit->setFont(font);
	cityEdit->show();

	//证件类型
	QLabel *certificateType{nullptr};
	certificateType = new QLabel(searchWidget);
	check_pointer_no_return(certificateType)
	certificateType->setGeometry(adaptxywh(30,150,100,30));
	certificateType->setFont(font);
	certificateType->setText("Certificate Type");
	certificateType->show();

	QComboBox *certificateTypeBox{nullptr};
	certificateTypeBox = new QComboBox(searchWidget);
	check_pointer_no_return(certificateTypeBox)
	certificateTypeBox->setGeometry(adaptxywh(130,150,200,30));
	certificateTypeBox->setFont(font);
	QStringList certificateTypeItems{"IC","passport","custom","other"};
	certificateTypeBox->addItems(certificateTypeItems);
	certificateTypeBox->show();

	//性别
	QLabel *sex{nullptr};
	sex = new QLabel(searchWidget);
	check_pointer_no_return(sex)
	sex->setGeometry(adaptxywh(360,150,100,30));
	sex->setFont(font);
	sex->setText("Sex");
	sex->show();

	QComboBox *sexBox{nullptr};
	sexBox = new QComboBox(searchWidget);
	check_pointer_no_return(sexBox)
	sexBox->setGeometry(adaptxywh(460,150,200,30));
	sexBox->setFont(font);
	QStringList sexItems{"male","female"};
	sexBox->addItems(sexItems);
	sexBox->show();

	//证件号码
	QLabel *certificateID{nullptr};
	certificateID = new QLabel(searchWidget);
	check_pointer_no_return(certificateID)
	certificateID->setGeometry(adaptxywh(690,150,100,30));
	certificateID->setFont(font);
	certificateID->setText("Certificate ID");
	certificateID->show();

	QLineEdit *certificateIDEdit{nullptr};
	certificateIDEdit = new QLineEdit(searchWidget);
	check_pointer_no_return(certificateIDEdit)
	certificateIDEdit->setGeometry(adaptxywh(790,150,200,30));
	certificateIDEdit->setFont(font);
	certificateIDEdit->show();

	QPushButton *search{nullptr};
	search = new QPushButton(searchWidget);
	check_pointer_no_return(search)
	search->setGeometry(adaptxywh(1020/2-80,210,80,30));
	search->setText("Search");
	search->show();
	connect(search,&QPushButton::released,[=]()
	{
		func_lock

		//赋值搜索字段
		m_searchContent.searchFlag=true;
		m_searchContent.name=nameEdit->text();
		m_searchContent.sex=sexBox->currentText();
		m_searchContent.certificateType=certificateTypeBox->currentText();
		m_searchContent.certificateID=certificateIDEdit->text();
		m_searchContent.country=countryEdit->text();
		m_searchContent.address=provinceEdit->text()+"#"+cityEdit->text();
		m_searchContent.address=m_searchContent.address=="#"?"":m_searchContent.address;
		m_searchContent.startBirthday=birthdayStartEdit->text().replace("-","");
		m_searchContent.stopBirthday=birthdayEndEdit->text().replace("-","");

		//更新页码信息
		updatePageInfo(m_searchContent.searchFlag,m_searchContent.name,m_searchContent.sex,m_searchContent.certificateType,m_searchContent.certificateID,m_searchContent.country,m_searchContent.address,m_searchContent.startBirthday,m_searchContent.stopBirthday);
		//更新条目信息
		updateItems(10,1,m_searchContent.searchFlag,m_searchContent.name,m_searchContent.sex,m_searchContent.certificateType,m_searchContent.certificateID,m_searchContent.country,m_searchContent.address,m_searchContent.startBirthday,m_searchContent.stopBirthday);

		searchWidget->close();
	});

	searchWidget->show();
}

void FaceLibraryItemWidget::slot_addMulitPersonInfo()
{
	func_lock

	warning_box(QStringLiteral("Name format: 名称#S性别#B生日#C国家#P省份#城市#T证件类型#M证件号码.jpg\nEg: 张三#S1#B1990-01-01#C中国#P湖北省#武汉市#T1#M330501190001016222.jpg\nSex: 0. 男 1. 女\nCertificateType: 0. 身份证 1. 护照 2. 其他"))

	QStringList files = QFileDialog::getOpenFileNames(
								  nullptr,
								  "Select one or more files to import",
								  "",
								  "Images (*.jpg *.jpeg)");

	if(true == files.empty())
		return;

	StatusWidget *statusWidget = new StatusWidget;
	statusWidget->setGeometry(adaptxywh(100+1300/2-200/2,100+700/2-100/2,200,100));
	statusWidget->show();

//	const auto multiAdd = [this,files]()
	{
		HttpManagerErcode ercode{HttpErcode_None};

		statusWidget->setMessage("Sum: "+QString::number(files.length())+"\n"+"Succeed: 0\nFailed:0");

		//判断文件名称是否合法
		//名称	eg:张三
		QString namePattern{"(\\S+)"};
		//#S性别	eg:#S女
		QString sexPattern{"(#S\\S+)"};
		//#B生日	eg:#B1990-12-10
		QString birthdayPattern{"(#B\\d{4}-\\d{2}-\\d{2})"};
		//#C国家	eg:中国
		QString countryPattern{"(#C\\S+)"};
		//#P省份#城市	eg:#P湖北省#武汉市
		QString provincePattern{"(#P\\D+#\\S+)"};
		//#T证件类型
		QString certificateTypePattern{"(#T\\S+)"};
		//#M证件号码
		QString certificateIDPattern{"(#M\\d+)"};
		//文件后缀
		QString format{".\\S+"};

		QString pattern = namePattern+sexPattern+birthdayPattern+countryPattern+provincePattern+certificateTypePattern+certificateIDPattern+format;
		QRegExp reg(pattern);

		//去除后缀的文件路径
		QString filePath;
		//人员信息结构体
		FaceLibraryItemInfo item;
		//图片数据
		QByteArray imageData;
		//成功个数
		int addNum{0};
		//失败个数
		int failedNum{0};
		for(const QString &fileName:files)
		{
			QString name = fileName.mid(fileName.lastIndexOf("/")+1);
			//判断文件名是否符合规则
			if(false == reg.exactMatch(fileName))
			{
				++failedNum;
				continue;
			}

			//提取值
			QRegExp pickUpPattern;
			int pos;

			pickUpPattern.setPattern("(#\\S){,1}\\S+#S");
			pos = pickUpPattern.indexIn(name,0);
			item.name = pickUpPattern.cap(0).replace("#S","");

			pickUpPattern.setPattern("#S\\S+#B");
			pos = pickUpPattern.indexIn(name,0);
			item.sex = pickUpPattern.cap(0).replace("#S","").replace("#B","");

			pickUpPattern.setPattern("#B\\S+#C");
			pos = pickUpPattern.indexIn(name,0);
			item.birthday = pickUpPattern.cap(0).replace("#B","").replace("#C","").replace("-","");

			pickUpPattern.setPattern("#C\\S+#P");
			pos = pickUpPattern.indexIn(name,0);
			item.country = pickUpPattern.cap(0).replace("#C","").replace("#P","");

			pickUpPattern.setPattern("#P\\S+#T");
			pos = pickUpPattern.indexIn(name,0);
			item.address = pickUpPattern.cap(0).replace("#P","").replace("#T","");

			pickUpPattern.setPattern("#T\\S+#M");
			pos = pickUpPattern.indexIn(name,0);
			item.certificateType = pickUpPattern.cap(0).replace("#T","").replace("#M","");

			pickUpPattern.setPattern("#M\\S+\\.");
			pos = pickUpPattern.indexIn(name,0);
			item.certificateID = pickUpPattern.cap(0).replace("#M","").replace(".","");

			item.libraryID=m_faceLibraryID;

			//图片文件
			QFile imageFile(fileName);
			imageFile.open(QIODevice::ReadOnly);
			imageData = imageFile.readAll();

			QString personID,uniqueID;
			ercode = FaceLibraryPackage::GetInstance()->addPersonInfo(m_ip,m_username,m_password,item,imageData,personID,uniqueID);
			if(HttpErcode_None != ercode)
				++failedNum;
			else
				++addNum;

			imageFile.close();

			if(0!=failedNum)		//失败原因可能是：图片中不存在人脸，图片在某些脸库中已经存在，图片中存在多张人脸，图片名称不正确等
				statusWidget->setMessage("Sum: "+QString::number(files.length())+"\n"+"Succeed: "+QString::number(addNum)+"\nFailed: "+QString::number(failedNum)+"\nPlease check the picture!");
			else
				statusWidget->setMessage("Sum: "+QString::number(files.length())+"\n"+"Succeed: "+QString::number(addNum)+"\nFailed: "+QString::number(failedNum));
		}

		if(0!=failedNum)		//失败原因可能是：图片中不存在人脸，图片在某些脸库中已经存在，图片中存在多张人脸，图片名称不正确等
			statusWidget->setMessage("Sum: "+QString::number(files.length())+"\n"+"Succeed: "+QString::number(addNum)+"\nFailed: "+QString::number(failedNum)+"\nPlease check the picture!");
		else
			statusWidget->setMessage("Sum: "+QString::number(files.length())+"\n"+"Succeed: "+QString::number(addNum)+"\nFailed: "+QString::number(failedNum));

		statusWidget->setFinished();
	}
//	std::thread(multiAdd).detach();

	updatePageInfo();
	updateItems(10,1);

	emit signal_updateFaceLibrary();
}

void FaceLibraryItemWidget::slot_removeAll()
{
	warning_box("Confirm to delete all?")

	HttpManagerErcode ercode = FaceLibraryPackage::GetInstance()->deleteFaceLibrary(m_ip,m_username,m_password,m_faceLibraryID,"true");
	check_http_ercode_no_return(ercode,"Delete all failed")

	message_output("Delete all succesfully");

	updatePageInfo();
	updateItems(10,1);

	emit signal_updateFaceLibrary();
}
