﻿#ifndef FACELIBRARYITEMWIDGET_H
#define FACELIBRARYITEMWIDGET_H

#include "src/inc/aifaceobject.h"

#include <QWidget>
#include <QPushButton>

QT_BEGIN_NAMESPACE
class QTableWidget;
class QLabel;
class QComboBox;
class QPushButton;
class StatusWidget;
QT_END_NAMESPACE

class FaceLibraryItemWidget : public QWidget
{
	Q_OBJECT
public:
	explicit FaceLibraryItemWidget(QWidget *parent = nullptr);

	/**
	 * @brief init					初始化
	 * @param mac			[in]设备mac地址
	 * @param ip				[in]设备IP地址
	 * @param username	[in]用户名
	 * @param password	[in]密码
	 */
	void init(const QString &mac,const QString &ip,const QString &username,const QString &password);

	/**
	 * @brief setFaceLibraryID		设置脸库ID
	 * @param faceLibraryID			[in]脸库ID
	 */
	void setFaceLibraryID(const QString &faceLibraryID,const QString &faceLibraryName);

	/**
	 * @brief enableBind		此窗体用于未绑定脸库
	 */
	void setUseForUnBindGroup(const bool &isUseForUnBindGroup = true);

private:
	QLabel *m_libraryName{nullptr};
	QTableWidget *m_tableWidget{nullptr};

	QString m_faceLibraryID;

	QString m_ip;
	QString m_username;
	QString m_password;

	FaceLibraryItemInfoList m_infos;

	QComboBox *m_pageSize{nullptr};
	QComboBox *m_page{nullptr};
	QLabel *m_sumPageNum{nullptr};

	QPushButton *m_bind{nullptr};
	QPushButton *m_addMulti{nullptr};
	QPushButton *m_add{nullptr};
	QPushButton *m_search{nullptr};
	QPushButton *m_refresh{nullptr};
	QPushButton *m_remove{nullptr};
	QPushButton *m_removeAll{nullptr};

//	struct LibraryButton
//	{
//		virtual void show() = 0;
//		virtual void hide() = 0;

//		QPushButton *refresh{nullptr};
//		QPushButton *remove{nullptr};
//	};
//	struct UnBindLibraryButton:public LibraryButton
//	{
//		void show() override
//		{
//			bind->show();
//			removeAll->show();
//		}
//		void hide() override
//		{
//			bind->hide();
//			removeAll->hide();
//		}

//		QPushButton *bind{nullptr};
//		QPushButton *removeAll{nullptr};
//	}m_unbindLibraryButton;
//	struct OtherLibraryButton:public LibraryButton
//	{
//		void show() override
//		{
//			addMulti->show();
//			add->show();
//		}
//		void hide() override
//		{
//			addMulti->hide();
//			add->hide();
//		}

//		QPushButton *addMulti{nullptr};
//		QPushButton *add{nullptr};
//	}m_otherLibraryButton;

	//搜索字段
	using SearchContent = struct SearchContent
	{
		void clear()
		{
			searchFlag = true;

			name.clear();
			sex.clear();
			certificateType.clear();
			certificateID.clear();
			country.clear();
			address.clear();
			startBirthday.clear();
			stopBirthday.clear();
		}

		bool searchFlag{false};

		QString name;
		QString sex;
		QString certificateType;
		QString certificateID;
		QString country;
		QString address;
		QString startBirthday;
		QString stopBirthday;
	};
	SearchContent m_searchContent;

//	StatusWidget *m_statusWidget{nullptr};

private:
	/**
	 * @brief initGui		创建表格
	 */
	void initGui();

	/**
	 * @brief updatePageInfo		更新页信息
	 * 以下为搜索字段
	 * @param searchFlag										[in]当前是否处于搜索状态，如果否，则以下值都无效
	 * @param name												[in]名字
	 * @param sex													[in]性别
	 * @param certificateType								[in]证件类型
	 * @param certificateID									[in]证件ID
	 * @param country											[in]国家
	 * @param address											[in]地址
	 * @param startBirthday									[in]生日起始时间
	 * @param stopBirthday									[in]生日停止时间
	 */
	void updatePageInfo(const bool &searchFlag=false, const QString &Name="",const QString &sex="unknown",const QString &certificateType="other",const QString &certificateID="",const QString &country="",const QString address="",const QString &startBirthday="00000000",const QString &stopBirthday="00000000");

	/**
	 * @brief updateItems		更新表格数据
	 * 以下为搜索字段
	 * @param searchFlag										[in]当前是否处于搜索状态，如果否，则以下值都无效
	 * @param name												[in]名字
	 * @param sex													[in]性别
	 * @param certificateType								[in]证件类型
	 * @param certificateID									[in]证件ID
	 * @param country											[in]国家
	 * @param address											[in]地址
	 * @param startBirthday									[in]生日起始时间
	 * @param stopBirthday									[in]生日停止时间
	 */
	void updateItems(const int &pageSize,const int &pageIndex,
					 const bool &searchFlag=false, const QString &Name="",const QString &sex="unknown",const QString &certificateType="other",const QString &certificateID="",const QString &country="",const QString address="",const QString &startBirthday="00000000",const QString &stopBirthday="00000000");

signals:
	void signal_updateFaceLibrary();

public slots:
	void slot_editPersonInfo();
	void slot_addMulitPersonInfo();
	void slot_addPersonInfo();
	void slot_removeItem();
	void slot_bind();
	void slot_search();
	void slot_removeAll();
};

#endif // FACELIBRARYITEMWIDGET_H
