﻿#include "src/inc/gui/facelibrary/personitemeditwidget.h"

#include "src/inc/aifacepch.h"
#include "src/inc/httpmanager/facelibrarypackage.h"

#include <QLabel>
#include <QComboBox>
#include <QLineEdit>
#include <QPushButton>
#include <QFileDialog>
#include <QBuffer>
#include <QDateTimeEdit>

PersonItemEditWidget::PersonItemEditWidget(QWidget *parent) : QDialog(parent)
{
	this->setGeometry(adaptxywh(320,200,870,500));
//	this->setAttribute(Qt::WA_DeleteOnClose);
	this->setWindowTitle("Edit");
	this->setModal(true);

	initGui();
}

void PersonItemEditWidget::initGui()
{
	QFont font("Microsoft YaHei",adaptx(10));

	QLabel *nameLabel{nullptr};
	nameLabel = new QLabel(this);
	check_pointer_no_return(nameLabel)
	nameLabel->setGeometry(adaptxywh(30,20,65,40));
	nameLabel->setFont(font);
	nameLabel->setText("Name");

	m_nameEdit = new QLineEdit(this);
	m_nameEdit->setGeometry(adaptxywh(95,20,170,40));
	m_nameEdit->setFont(font);

	QLabel *sexLabel{nullptr};
	sexLabel = new QLabel(this);
	check_pointer_no_return(sexLabel)
	sexLabel->setGeometry(adaptxywh(295,20,65,40));
	sexLabel->setFont(font);
	sexLabel->setText("Sex");

	m_sexBox = new QComboBox(this);
	m_sexBox->setGeometry(adaptxywh(360,20,170,40));
	m_sexBox->setFont(font);
	QStringList sexItems{"male","female"};
	m_sexBox->addItems(sexItems);

	QLabel *birthday{nullptr};
	birthday = new QLabel(this);
	check_pointer_no_return(sexLabel)
	birthday->setGeometry(adaptxywh(560,20,65,40));
	birthday->setFont(font);
	birthday->setText("Birthday");

	m_birthdayEdit = new QDateTimeEdit(this);
	m_birthdayEdit->setGeometry(adaptxywh(665,20,170,40));
	m_birthdayEdit->setFont(font);
	m_birthdayEdit->setDisplayFormat("yyyy-MM-dd");

	QLabel *countryLabel{nullptr};
	countryLabel = new QLabel(this);
	check_pointer_no_return(countryLabel)
	countryLabel->setGeometry(adaptxywh(30,80,65,40));
	countryLabel->setFont(font);
	countryLabel->setText("Country");

	m_countryEdit = new QLineEdit(this);
	m_countryEdit->setGeometry(adaptxywh(95,80,170,40));
	m_countryEdit->setFont(font);

	QLabel *provinceLabel{nullptr};
	provinceLabel = new QLabel(this);
	check_pointer_no_return(provinceLabel)
	provinceLabel->setGeometry(adaptxywh(295,80,65,40));
	provinceLabel->setFont(font);
	provinceLabel->setText("Province");

	m_provinceEdit = new QLineEdit(this);
	m_provinceEdit->setGeometry(adaptxywh(365,80,170,40));
	m_provinceEdit->setFont(font);

	QLabel *cityLabel{nullptr};
	cityLabel = new QLabel(this);
	check_pointer_no_return(cityLabel)
	cityLabel->setGeometry(adaptxywh(560,80,65,40));
	cityLabel->setFont(font);
	cityLabel->setText("City");

	m_cityEdit = new QLineEdit(this);
	m_cityEdit->setGeometry(adaptxywh(665,80,170,40));
	m_cityEdit->setFont(font);

	QLabel *certificateType{nullptr};
	certificateType = new QLabel(this);
	check_pointer_no_return(certificateType)
	certificateType->setGeometry(adaptxywh(30,140,65,40));
	certificateType->setFont(font);
	certificateType->setWordWrap(true);
	certificateType->setText("Certificate Type");

	m_certificateTypeBox = new QComboBox(this);
	m_certificateTypeBox->setGeometry(adaptxywh(95,140,170,40));
	m_certificateTypeBox->setFont(font);
	QStringList certificateTypeItems{"IC","passport","custom","other"};
	m_certificateTypeBox->addItems(certificateTypeItems);

	QLabel *certificateIDLabel{nullptr};
	certificateIDLabel = new QLabel(this);
	check_pointer_no_return(certificateIDLabel)
	certificateIDLabel->setGeometry(adaptxywh(295,140,65,40));
	certificateIDLabel->setFont(font);
	certificateIDLabel->setWordWrap(true);
	certificateIDLabel->setText("Certificate ID");

	m_certificateIDEdit = new QLineEdit(this);
	m_certificateIDEdit->setGeometry(adaptxywh(365,140,170,40));
	m_certificateIDEdit->setFont(font);

	m_pictureLabel = new QLabel(this);
	m_pictureLabel->setGeometry(adaptxywh(870/2-150,190,300,300));

	QPushButton *change{nullptr};
	change = new QPushButton(this);
	check_pointer_no_return(change)
	change->setGeometry(adaptxywh(100,300,100,50));
	change->setFont(font);
	change->setText("Change Image");
	connect(change,&QPushButton::released,this,&PersonItemEditWidget::slot_changeImage);

	QPushButton *save{nullptr};
	save = new QPushButton(this);
	check_pointer_no_return(save)
	save->setGeometry(adaptxywh(650,300,100,50));
	save->setFont(font);
	save->setText("Save");
	connect(save,&QPushButton::released,this,&PersonItemEditWidget::slot_save);
}

void PersonItemEditWidget::slot_changeImage()
{
	func_lock

	if("1"==m_itemInfo.libraryID)
	{
		message_output("UnBindGroup not support change image!");
		return;
	}

	QString fileName = QFileDialog::getOpenFileName(this,tr("Open Image"), "./", tr("Image Files (*.jpg *.jpeg)"));

	if(true==fileName.isEmpty())
		return;

	QString format = fileName.mid(fileName.lastIndexOf(".")+1);

	QPixmap image(fileName);
	m_faceImage.clear();
	QBuffer buffer(&m_faceImage);
	buffer.open(QIODevice::WriteOnly);
	image.save(&buffer,format.toLocal8Bit().data());

	m_pictureLabel->setPixmap(image.scaled(m_pictureLabel->width(),m_pictureLabel->height()));
}

void PersonItemEditWidget::slot_save()
{
	func_lock

	HttpManagerErcode ercode{HttpErcode_None};

	m_itemInfo.name=m_nameEdit->text();
	m_itemInfo.sex=m_sexBox->currentText();
	m_itemInfo.certificateType=m_certificateTypeBox->currentText();
	m_itemInfo.certificateID=m_certificateIDEdit->text();
	m_itemInfo.country=m_countryEdit->text();
	m_itemInfo.address=m_provinceEdit->text()+"#"+m_cityEdit->text();

	QString birthday=m_birthdayEdit->text().replace("-","")+"T000000";
	m_itemInfo.birthday=birthday;
	ercode = FaceLibraryPackage::GetInstance()->setPersonInfo(m_ip,m_username,m_password,m_itemInfo);
	check_http_ercode_no_return(ercode,"Save person info failed!")

	if("1"==m_itemInfo.libraryID);		//未绑定脸库无法修改图片信息
	else
	{
#ifdef UniqueID_Support
		ercode = FaceLibraryPackage::GetInstance()->setPersonImage(m_ip,m_username,m_password,m_itemInfo.uniqueID,m_itemInfo.faceImageID,"temp.jpeg",m_faceImage);
#else
		ercode = FaceLibraryPackage::GetInstance()->setPersonImage(m_ip,m_username,m_password,m_itemInfo.personID,m_itemInfo.faceImageID,"temp.jpeg",m_faceImage);
#endif
		check_http_ercode_no_return(ercode,"Save image info failed!")
	}

	message_output("Edit person info successfully");
	this->close();

	emit signal_updateFaceLibrary();
}

void PersonItemEditWidget::initUserInfo(const QString &mac, const QString &ip, const QString &username, const QString &password)
{
	m_ip=ip;
	m_username=username;
	m_password=password;
}

void PersonItemEditWidget::setPersonInfo(const FaceLibraryItemInfo &itemInfo)
{
	m_itemInfo=itemInfo;

	QByteArray imageData;
#ifdef UniqueID_Support
	HttpManagerErcode ercode = FaceLibraryPackage::GetInstance()->getPersonImageInfo(m_ip,m_username,m_password,itemInfo.uniqueID,itemInfo.faceImageID,imageData);
#else
	HttpManagerErcode ercode = FaceLibraryPackage::GetInstance()->getPersonImageInfo(m_ip,m_username,m_password,itemInfo.personID,itemInfo.faceImageID,imageData);
#endif
	check_http_ercode_no_return(ercode,"Get person image info failed")

	m_nameEdit->setText(itemInfo.name);
	m_sexBox->setCurrentText(itemInfo.sex);
	m_birthdayEdit->setDateTime(QDateTime::fromString(itemInfo.birthday.mid(0,itemInfo.birthday.lastIndexOf(" ")),"yyyy-MM-dd"));
	m_countryEdit->setText(itemInfo.country);
	if(true!=itemInfo.address.isEmpty())
	{
		//从设备获取的地址格式：省名#市名	eg:湖北#武汉
		if(false == itemInfo.address.contains("#"))
			m_cityEdit->setText(itemInfo.address);
		else
		{
			m_provinceEdit->setText(itemInfo.address.split("#")[0]);
			m_cityEdit->setText(itemInfo.address.split("#")[1]);
		}
	}
	m_certificateTypeBox->setCurrentText(itemInfo.certificateType);
	m_certificateIDEdit->setText(itemInfo.certificateID);

	m_faceImage=imageData;
	QPixmap pixmap;
	pixmap.loadFromData(imageData);
	m_pictureLabel->setPixmap(pixmap);
}

void PersonItemEditWidget::setImageInfo(const QByteArray &imageData)
{
	m_faceImage=imageData;
	QPixmap pixmap;
	pixmap.loadFromData(imageData);
	m_pictureLabel->setPixmap(pixmap);
}
