﻿#ifndef PERSONITEMEDITWIDGET_H
#define PERSONITEMEDITWIDGET_H

#include "src/inc/aifaceobject.h"

#include <QDialog>

QT_BEGIN_NAMESPACE
class QLabel;
class QLineEdit;
class QComboBox;
class QDateTimeEdit;
QT_END_NAMESPACE

class PersonItemEditWidget : public QDialog
{
	Q_OBJECT
public:
	explicit PersonItemEditWidget(QWidget *parent = nullptr);

	/**
	 * @brief init					初始化
	 * @param mac			[in]设备mac地址
	 * @param ip				[in]设备IP地址
	 * @param username	[in]用户名
	 * @param password	[in]密码
	 */
	void initUserInfo(const QString &mac,const QString &ip,const QString &username,const QString &password);

	/**
	 * @brief setPersonID		设置人物ID
	 * @param personID			[in]人物ID
	 * @param imageID			[in]图片ID
	 */
	void setPersonInfo(const FaceLibraryItemInfo &itemInfo);

	/**
	 * @brief setImageInfo		设置图片信息
	 * @param imageData		[in]图片二进制数据
	 */
	void setImageInfo(const QByteArray &imageData);

private:
	/**
	 * @brief initGui		初始化界面
	 */
	void initGui();

protected:
	QLineEdit *m_nameEdit{nullptr};
	QComboBox *m_sexBox{nullptr};
	QDateTimeEdit *m_birthdayEdit{nullptr};
	QLineEdit *m_countryEdit{nullptr};
	QLineEdit *m_provinceEdit{nullptr};
	QLineEdit *m_cityEdit{nullptr};
	QComboBox *m_certificateTypeBox{nullptr};
	QLineEdit *m_certificateIDEdit{nullptr};

	QLabel *m_pictureLabel{nullptr};

	QString m_ip;
	QString m_username;
	QString m_password;

	FaceLibraryItemInfo m_itemInfo;
	QByteArray m_faceImage;

signals:
	void signal_updateFaceLibrary();

public slots:
	void slot_changeImage();
	virtual void slot_save();
};

#endif // PERSONITEMEDITWIDGET_H
