﻿#include "src/inc/gui/facerecognition/facerecognizewidget.h"

#include "src/inc/aifacepch.h"
#include "src/inc/aifaceobject.h"
#include "src/inc/httpmanager/facerecognizepackage.h"
#include "src/inc/gui/facerecognition/recordplanwidget.h"

#include <QLabel>
#include <QComboBox>
#include <QFont>
#include <QLineEdit>
#include <QTableWidget>
#include <QHeaderView>
#include <QCheckBox>
#include <QPushButton>

FaceRecognizeWidget::FaceRecognizeWidget(QWidget *parent) : QWidget(parent)
{
	this->setGeometry(adaptxywh(320,150,500,600));
//	this->setAttribute(Qt::WA_DeleteOnClose);
	this->setWindowTitle("FaceRecognize");

	initGui();
}

void FaceRecognizeWidget::initGui()
{
	QFont font("Microsoft YaHei",adaptx(10));

	QLabel *rule{nullptr};
	rule = new QLabel(this);
	check_pointer_no_return(rule)
	rule->setGeometry(adaptxywh(30,30,150,30));
	rule->setFont(font);
	rule->setText("Rule");

	m_ruleBox = new QComboBox(this);
	m_ruleBox->setGeometry(adaptxywh(180,30,200,30));
	m_ruleBox->setFont(font);
	QStringList ruleItems{"Rule one","Rule two","Rule three","Rule four"};
	m_ruleBox->addItems(ruleItems);
	connect(m_ruleBox,static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),[this](int index){updateFaceRecoRule(index);});

	QLabel *isEnable{nullptr};
	isEnable = new QLabel(this);
	check_pointer_no_return(isEnable)
	isEnable->setGeometry(adaptxywh(30,70,150,30));
	isEnable->setFont(font);
	isEnable->setText("Enable");

	m_isEnableBox = new QComboBox(this);
	m_isEnableBox->setGeometry(adaptxywh(180,70,200,30));
	m_isEnableBox->setFont(font);
	QStringList isEnableItems{"Enable","Disable"};
	m_isEnableBox->addItems(isEnableItems);

	QLabel *identificationRule{nullptr};
	identificationRule = new QLabel(this);
	check_pointer_no_return(identificationRule)
	identificationRule->setGeometry(adaptxywh(30,110,150,30));
	identificationRule->setFont(font);
	identificationRule->setText("Identification Rule");

	m_identificationRuleBox = new QComboBox(this);
	m_identificationRuleBox->setGeometry(adaptxywh(180,110,200,30));
	m_identificationRuleBox->setFont(font);
	QStringList identificationRuleItems{"Comparison pass","Comparison fail"};
	m_identificationRuleBox->addItems(identificationRuleItems);

	QLabel *alignmentLimit{nullptr};
	alignmentLimit = new QLabel(this);
	check_pointer_no_return(alignmentLimit)
	alignmentLimit->setGeometry(adaptxywh(30,150,150,30));
	alignmentLimit->setFont(font);
	alignmentLimit->setText("Alignment limit");

	m_compareUpLimitBox = new QComboBox(this);
	m_compareUpLimitBox->setGeometry(adaptxywh(180,150,200,30));
	m_compareUpLimitBox->setFont(font);
	QStringList compareUpLimitItems{"1","2","3","4","5"};
	m_compareUpLimitBox->addItems(compareUpLimitItems);

	QLabel *identificationThreshold{nullptr};
	identificationThreshold = new QLabel(this);
	check_pointer_no_return(identificationThreshold)
	identificationThreshold->setGeometry(adaptxywh(30,190,150,30));
	identificationThreshold->setFont(font);
	identificationThreshold->setText("Identification threshold");

	m_identificationThresholdEdit = new QLineEdit(this);
	m_identificationThresholdEdit->setGeometry(adaptxywh(180,190,150,30));
	m_identificationThresholdEdit->setFont(font);
	m_identificationThresholdEdit->setText("70");

	QLabel *identificationThresholdSuggest{nullptr};
	identificationThresholdSuggest = new QLabel(this);
	check_pointer_no_return(identificationThresholdSuggest)
	identificationThresholdSuggest->setGeometry(adaptxywh(340,190,100,30));
	identificationThresholdSuggest->setFont(font);
	identificationThresholdSuggest->setText("Suggest: 60-95");

	QLabel *faceLibrary{nullptr};
	faceLibrary = new QLabel(this);
	check_pointer_no_return(faceLibrary)
	faceLibrary->setGeometry(adaptxywh(30,230,150,90));
	faceLibrary->setFont(font);
	faceLibrary->setText("Face Library");

	m_faceLibraryTable = new QTableWidget(5,2,this);
	m_faceLibraryTable->setGeometry(adaptxywh(180,230,250,90));
	QStringList header{"Name","Enable"};
	m_faceLibraryTable->setHorizontalHeaderLabels(header);
	m_faceLibraryTable->setSelectionBehavior(QAbstractItemView::SelectRows);
	m_faceLibraryTable->setSelectionMode(QAbstractItemView::SingleSelection);
	m_faceLibraryTable->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);

	QLabel *linkageOptions{nullptr};
	linkageOptions = new QLabel(this);
	check_pointer_no_return(linkageOptions)
	linkageOptions->setGeometry(adaptxywh(30,320,150,30));
	linkageOptions->setFont(font);
	linkageOptions->setText("Linkage Options");

	m_alarm1 = new QCheckBox(this);
	m_alarm1->setGeometry(adaptxywh(180,320,150,30));
	m_alarm1->setFont(font);
	m_alarm1->setText("Trigger Alarmout 1");

	m_alarm2 = new QCheckBox(this);
	m_alarm2->setGeometry(adaptxywh(340,320,150,30));
	m_alarm2->setFont(font);
	m_alarm2->setText("Trigger Alarmout 2");

	QLabel *alarmPlan{nullptr};
	alarmPlan = new QLabel(this);
	check_pointer_no_return(alarmPlan)
	alarmPlan->setGeometry(adaptxywh(30,360,150,30));
	alarmPlan->setFont(font);
	alarmPlan->setText("Alarm Plan");

	m_alarmPlanBox = new QComboBox(this);
	m_alarmPlanBox->setGeometry(adaptxywh(180,360,200,30));
	m_alarmPlanBox->setFont(font);
	QStringList alarmPlanItems{"All Day","Manual Input"};
	m_alarmPlanBox->addItems(alarmPlanItems);
	connect(m_alarmPlanBox,static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),[this](int index){m_recordPlanWidget->setEditEnable(index);});

	m_recordPlanWidget = new RecordPlanWidget(this);
	m_recordPlanWidget->setGeometry(adaptxywh(10,400,480,160));
	m_recordPlanWidget->setRowItems({"Sun","Sat","Fri","Thu","Wed","Tue","Mon"});
	m_recordPlanWidget->setColumeItems({"00","01","02","03","04","05","06","07","08","09","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24"});
	m_recordPlanWidget->updateItems();

	QPushButton *save{nullptr};
	save = new QPushButton(this);
	check_pointer_no_return(save)
	save->setGeometry(adaptxywh(210,560,80,30));
	save->setFont(font);
	save->setText("Save");
	connect(save,&QPushButton::released,this,&FaceRecognizeWidget::slot_save);
}

void FaceRecognizeWidget::initUserInfo(const QString &mac, const QString &ip, const QString &username, const QString &password)
{
	m_ip=ip;
	m_username=username;
	m_password=password;

	updateFaceRecoRule();
}

void FaceRecognizeWidget::updateFaceRecoRule(const int &index)
{
	func_lock

	HttpManagerErcode ercode{HttpErcode_None};

	RecoRuleList recoRuleList;
	ercode = FaceRecognizePackage::GetInstance()->getRecoRuleList(m_ip,m_username,m_password,recoRuleList);
	check_http_ercode_no_return(ercode,"Get recognize rule failed!")

	if(true==recoRuleList.isEmpty())
		return;

	RecoRuleList::Iterator iter = recoRuleList.begin()+index;
//	for(iter=recoRuleList.begin();iter!=recoRuleList.end();++iter)
	{
//		int offset = iter-recoRuleList.begin();

		//从设备获取的是否启用字段的值为"true"或者"false"，转换到界面上显示为"Enable"或者"Disable"
		//是否启用
		m_isEnableBox->setCurrentText(("true"==iter->isEnable?"Enable":"Disable"));
		//识别规则
		m_identificationRuleBox->setCurrentText(iter->recognitionRule);
		m_compareUpLimitBox->setCurrentText(iter->compareLimit);
		m_identificationThresholdEdit->setText(iter->identificationThreshold);

		m_faceLibraryTable->setRowCount(iter->faceLibraryNodes.length());
		m_faceLibraryNodes.clear();
		m_faceLibraryNodes.append(iter->faceLibraryNodes);
		QVector<RecoRuleFaceLibraryNode>::iterator inIter;
		for(inIter=iter->faceLibraryNodes.begin();inIter!=iter->faceLibraryNodes.end();++inIter)
		{
			int inOffset = inIter-iter->faceLibraryNodes.begin();

			QTableWidgetItem *nameItem{nullptr};
			nameItem = new QTableWidgetItem(inIter->name);
			check_pointer_no_return(nameItem)
			m_faceLibraryTable->setItem(inOffset,0,nameItem);

			QCheckBox *isEnable = new QCheckBox(this);
			if("true"==inIter->enable)
				isEnable->setCheckState(Qt::Checked);
			else
				isEnable->setCheckState(Qt::Unchecked);
			m_faceLibraryTable->setCellWidget(inOffset,1,isEnable);
		}

		if("1"==iter->alarm1Linkage)
			m_alarm1->setCheckState(Qt::Checked);
		else
			m_alarm1->setCheckState(Qt::Unchecked);
		if("1"==iter->alarm2Linkage)
			m_alarm2->setCheckState(Qt::Checked);
		else
			m_alarm2->setCheckState(Qt::Unchecked);

		m_alarmPlanBox->setCurrentIndex("false"==iter->recordPlan?1:0);
		m_recordPlanWidget->setEditEnable(m_alarmPlanBox->currentIndex());
		//GUI以小时为单位分格，设备以半小时为单位，但约定设备适应小时单位制，
		//如某小时启用录像，则设备返回当前小时是否录像的值为11，反之为00
		//而GUI以小时单位值用来表示某小时是否启用录像的值为1或0，所以设备值仅需取两个1中的一个足以
		QVector<QString> recordTimeBlocks;
		for(int i=0;i<7;++i)
		{
			QString day24Block;
			for(int j=0;j<iter->recordTimeBlocks[i].length();j+=2)
				day24Block+=iter->recordTimeBlocks[i][j];
			recordTimeBlocks.append(day24Block);
		}
		m_recordPlanWidget->setItemStatus(recordTimeBlocks);
	}
}

void FaceRecognizeWidget::slot_save()
{
	func_lock

	HttpManagerErcode ercode{HttpErcode_None};

	RecoRuleList recoRuleList;
	ercode = FaceRecognizePackage::GetInstance()->getRecoRuleList(m_ip,m_username,m_password,recoRuleList);
	check_http_ercode_no_return(ercode,"Get recognize rule failed!")

	RecoRule &recoRule = recoRuleList[m_ruleBox->currentIndex()];

	recoRule.isEnable=(m_isEnableBox->currentText()=="Enable"?"true":"false");
	recoRule.recognitionRule=m_identificationRuleBox->currentText();
	recoRule.compareLimit=m_compareUpLimitBox->currentText();
	if(m_identificationThresholdEdit->text().toInt()<0 || m_identificationThresholdEdit->text().toInt()>100)
	{
		message_output("Value must less than 100 and more than 0");
		return;
	}
	recoRule.identificationThreshold=m_identificationThresholdEdit->text();
	recoRule.faceLibraryNodes.clear();
	for(int i=0;i<m_faceLibraryTable->rowCount();++i)
	{
		RecoRuleFaceLibraryNode node;
		node.id=m_faceLibraryNodes[i].id;
		node.name=m_faceLibraryNodes[i].name;
		node.enable=qobject_cast<QCheckBox *>(m_faceLibraryTable->cellWidget(i,1))->checkState()==Qt::Checked?"true":"false";

		recoRule.faceLibraryNodes.append(node);
	}
	recoRule.alarm1Linkage=m_alarm1->checkState()==Qt::Checked?"1":"0";
	recoRule.alarm2Linkage=m_alarm2->checkState()==Qt::Checked?"1":"0";
	recoRule.recordPlan=(0==m_alarmPlanBox->currentIndex()?"true":"false");
	QVector<QString> plans;
	m_recordPlanWidget->getItemStatus(plans);
//		for(const QString &plan:plans)
//		{
//			qDebug()<<plan.length();
//			qDebug()<<plan;
//		}
	recoRule.recordTimeBlocks.clear();
	recoRule.recordTimeBlocks.append(plans);

//		recoRuleList.append(recoRule);

	ercode = FaceRecognizePackage::GetInstance()->setRecoRuleList(m_ip,m_username,m_password,recoRuleList);
	check_http_ercode_no_return(ercode,"Set recognize rule failed!")

	message_output("Save recognize rule successfully!");

	this->close();
}
