﻿#include "src/inc/gui/facerecognition/recordplanwidget.h"

#include "src/inc/aifacepch.h"

#include <QPainter>
#include <QMouseEvent>

RecordPlanWidget::RecordPlanWidget(QWidget *parent) : QWidget(parent)
{

}

void RecordPlanWidget::setEditEnable(const bool &flag)
{
	m_isAllowEdit = flag;
}

void RecordPlanWidget::setRowItems(const QVector<QString> &rowItems)
{
	m_rowItems.clear();
	m_rowItems.append(rowItems);
}

void RecordPlanWidget::setColumeItems(const QVector<QString> &columeItems)
{
	m_columeItems.clear();
	m_columeItems.append(columeItems);
}

void RecordPlanWidget::updateItems()
{
	m_timeBlockItems.clear();

	//计算方块大小
	int rowBlockSize = (this->width()-adaptx(50))/m_columeItems.length();
	int columeBlockSize=(this->height()-adapty(30))/m_rowItems.length();

	m_blockSize = rowBlockSize<columeBlockSize?rowBlockSize:columeBlockSize;

	for(int i=0;i<m_rowItems.length();++i)
	{
		QVector<QPair<QRect,bool>> items;
		for(int j=0;j<m_columeItems.length()-1;++j)
		{
			QRect rect(adaptx(50)+j*m_blockSize,adapty(30)+i*m_blockSize,m_blockSize,m_blockSize);
			items.append(QPair<QRect,bool>(rect,false));
		}
		m_timeBlockItems.append(items);
	}

	update();
}

void RecordPlanWidget::setItemStatus(const QVector<QString> &itemStatus)
{
	if(itemStatus.length()!=m_rowItems.length())
		return;

	QVector<QString>::const_iterator iter;
	for(iter=itemStatus.begin();iter!=itemStatus.end();++iter)
	{
		if(iter->length()!=m_columeItems.length()-1)
			return;
	}

	for(iter=itemStatus.begin();iter!=itemStatus.end();++iter)
	{
		int offset = iter-itemStatus.begin();

		for(int i=0;i<iter->length();++i)
			m_timeBlockItems[offset][i].second=(iter->at(i)=="1"?true:false);
	}

	update();
}

void RecordPlanWidget::getItemStatus(QVector<QString> &itemStatus)
{
	for(const QVector<QPair<QRect,bool>> &timeBlockitem:m_timeBlockItems)
	{
		QString status;
		for(const QPair<QRect,bool> &timeBlock:timeBlockitem)
			status+=(true==timeBlock.second?"11":"00");

		itemStatus.append(status);
	}
}

void RecordPlanWidget::mousePressEvent(QMouseEvent *event)
{
	if(false==m_isAllowEdit)
		return;

	m_pressPos = event->pos();

	for(QVector<QPair<QRect,bool>> &timeBlockitem:m_timeBlockItems)
	{
		for(QPair<QRect,bool> &timeBlock:timeBlockitem)
		{
			if(true==timeBlock.first.contains(event->pos()))
			{
				m_currentBlockStatus=!timeBlock.second;
				break;
			}
		}
	}
}

void RecordPlanWidget::mouseMoveEvent(QMouseEvent *event)
{
	if(false==m_isAllowEdit)
		return;

	//计算鼠标拉动覆盖的区域
	int minX = event->x() > m_pressPos.x()?m_pressPos.x():event->x();
	int minY = event->y() > m_pressPos.y()?m_pressPos.y():event->y();
	int width = qAbs(event->x()-m_pressPos.x());
	int height = qAbs(event->y()-m_pressPos.y());
	QRect rect(minX,minY,width,height);

	for(QVector<QPair<QRect,bool>> &timeBlockitem:m_timeBlockItems)
	{
		for(QPair<QRect,bool> &timeBlock:timeBlockitem)
		{
			if(true==rect.contains(timeBlock.first.topLeft())||true==rect.contains(timeBlock.first.topRight())||true==rect.contains(timeBlock.first.bottomLeft())||true==rect.contains(timeBlock.first.bottomRight()))
			{
				timeBlock.second = m_currentBlockStatus;
			}
		}
	}

	update();
}

void RecordPlanWidget::mouseReleaseEvent(QMouseEvent *event)
{
	if(false == m_isAllowEdit)
		return;

	if(qAbs(event->x()-m_pressPos.x())>4&&qAbs(event->y()-m_pressPos.y())>4)
	{
		for(QVector<QPair<QRect,bool>> &timeBlockitem:m_timeBlockItems)
		{
			for(QPair<QRect,bool> &timeBlock:timeBlockitem)
			{
				if(true==timeBlock.first.contains(event->pos()))
				{
					timeBlock.second = m_currentBlockStatus;
					return;
				}
			}
		}
	}

	for(QVector<QPair<QRect,bool>> &timeBlockitem:m_timeBlockItems)
	{
		for(QPair<QRect,bool> &timeBlock:timeBlockitem)
		{
			if(true==timeBlock.first.contains(event->pos()))
				timeBlock.second=!timeBlock.second;
		}
	}

	update();
}

void RecordPlanWidget::paintEvent(QPaintEvent *event)
{
	QPainter painter(this);

	//绘制横线
	for(int i=0;i<m_rowItems.length()+1;++i)
		painter.drawLine(adaptx(50),adapty(30)+i*m_blockSize,adaptx(50)+(m_columeItems.length()-1)*m_blockSize,adapty(30)+i*m_blockSize);

	//绘制竖线
	for(int i=0;i<m_columeItems.length();++i)
		painter.drawLine(adaptx(50)+i*m_blockSize,adapty(25),adaptx(50)+i*m_blockSize,adapty(30)+(m_rowItems.length())*m_blockSize);

	QFont font("Microsoft YaHei",adaptx(8));
	painter.setFont(font);
	//绘制Y轴标尺
	for(int i=0;i<m_rowItems.length();++i)
		painter.drawText(QRect(adaptx(0),adapty(30)+i*m_blockSize,adaptx(50),m_blockSize),Qt::AlignVCenter|Qt::AlignLeft,m_rowItems[i]);

	//绘制X轴标尺
	for(int i=0;i<m_columeItems.length();++i)
		painter.drawText(QRect(adaptx(50)+i*m_blockSize-m_blockSize/2+adaptx(2),adapty(0),m_blockSize,adapty(25)),Qt::AlignVCenter|Qt::AlignLeft,m_columeItems[i]);

	//绘制块
	for(const QVector<QPair<QRect,bool>> &rowItems:m_timeBlockItems)
	{
		for(const QPair<QRect,bool> &item:rowItems)
		{
			if(true==item.second)
				painter.setBrush(QColor(Qt::green));
			else
				painter.setBrush(QColor(Qt::gray));

			painter.drawRect(item.first);
		}
	}
}
