﻿#include "src/inc/gui/recognizerecord/recognizerecordwidget.h"

#include "src/inc/aifacepch.h"
#include "src/inc/aifaceobject.h"

#include <QTableWidget>
#include <QPushButton>
#include <QHeaderView>
#include <QComboBox>
#include <QLabel>
#include <QLineEdit>
#include <QDateTimeEdit>
#include <QDialog>

#include "src/inc/httpmanager/facelibrarypackage.h"
#include "src/inc/httpmanager/recognizerecordpackage.h"
#include "src/inc/gui/recognizerecord/recorddetail.h"

RecognizeRecordWidget::RecognizeRecordWidget(QWidget *parent) : QWidget(parent)
{
	this->setGeometry(adaptxywh(100,100,1300,700));
//	this->setAttribute(Qt::WA_DeleteOnClose);
	this->setWindowTitle("Recognize Record");

	initGui();
}

void RecognizeRecordWidget::initUserInfo(const QString &mac, const QString &ip, const QString &username, const QString &password)
{
	m_ip=ip;
	m_username=username;
	m_password=password;

	updatePageInfo();
	updateRecord(m_pageSize->currentText().toInt(),1);
}

void RecognizeRecordWidget::initGui()
{
	QFont font("Microsoft YaHei",adaptx(10));

	m_tableWidget = new QTableWidget(10,7,this);
	m_tableWidget->setGeometry(adaptxywh(50,100,1200,550));
	QStringList header{"Index","Rule","Face Library","Name","Recognize Time","Recognize Result","Detail"};
	m_tableWidget->setHorizontalHeaderLabels(header);
	m_tableWidget->setSelectionBehavior (QAbstractItemView::SelectRows);
	m_tableWidget->setSelectionMode(QAbstractItemView::SingleSelection);
	m_tableWidget->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);

	QLabel *pageSIzeLabel{nullptr};
	pageSIzeLabel = new QLabel(this);
	check_pointer_no_return(pageSIzeLabel)
	pageSIzeLabel->setGeometry(adaptxywh(50,660,70,30));
	pageSIzeLabel->setFont(font);
	pageSIzeLabel->setText("Page size:");

	m_pageSize = new QComboBox(this);
	m_pageSize->setGeometry(adaptxywh(130,660,50,30));
	m_pageSize->setFont(font);
	QStringList pageSizeItems{"10","20","30","40","50"};
	m_pageSize->addItems(pageSizeItems);
	connect(m_pageSize,&QComboBox::currentTextChanged,[this](QString size)
	{
		func_lock

		updatePageInfo();
		updateRecord(size.toInt(),1);
	});

	m_sumPageNum = new QLabel(this);
	m_sumPageNum->setGeometry(adaptxywh(200,660,250,30));
	m_sumPageNum->setFont(font);

	m_page = new QComboBox(this);
	m_page->setGeometry(adaptxywh(460,660,50,30));
	m_page->setFont(font);
	connect(m_page,static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),[this](int index)
	{
		func_lock

		updateRecord(m_pageSize->currentText().toInt(),index+1);
	});

	//查询
	QPushButton *search{nullptr};
	search = new QPushButton(this);
	check_pointer_no_return(search)
	search->setGeometry(adaptxywh(50+1200-220,60,100,30));
	search->setFont(font);
	search->setText("Search");
	connect(search,&QPushButton::released,this,&RecognizeRecordWidget::slot_search);

	//刷新
	QPushButton *refresh{nullptr};
	refresh = new QPushButton(this);
	check_pointer_no_return(refresh)
	refresh->setGeometry(adaptxywh(50+1200-110,60,100,30));
	refresh->setFont(font);
	refresh->setText("Refresh");
	connect(refresh,&QPushButton::released,this,&RecognizeRecordWidget::slot_refresh);
}

void RecognizeRecordWidget::updatePageInfo(const bool &searchFlag,const QString &libraryID,const QString &peopleName,const QString &startTime,const QString &endTime,const QString &recognitionResultType,const QString &ruleID)
{
	func_lock

	//获得识别记录总数
	QString recordCount;

	HttpManagerErcode ercode{HttpErcode_None};
	if(true==searchFlag)
		ercode = RecognizeRecordPackage::GetInstance()->getRecordRecordCount(m_ip,m_username,m_password,recordCount,
																			 libraryID,peopleName,startTime.isEmpty()?"00000000":startTime,endTime.isEmpty()?"00000000":endTime,recognitionResultType,ruleID);
	else
		ercode = RecognizeRecordPackage::GetInstance()->getRecordRecordCount(m_ip,m_username,m_password,recordCount);

	//计算总页数
	int sumPage = (0==recordCount.toInt()?1:(recordCount.toInt()-1)/m_pageSize->currentText().toInt()+1);
	//显示总页数
	m_sumPageNum->setText("Sum record count:"+recordCount+"      Sum page:"+QString::number(sumPage));
	m_page->disconnect();
	m_page->clear();
	for(int i=0;i<sumPage;++i)
		m_page->addItem(QString::number(i+1));
	connect(m_page,static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),[this](int index)
	{
		func_lock

		updateRecord(m_pageSize->currentText().toInt(),index+1);
	});

	m_tableWidget->setRowCount(m_pageSize->currentText().toInt());
}


void RecognizeRecordWidget::updateRecord(const int &pageSize,const int &pageIndex,const bool &searchFlag,const QString &libraryID,const QString &peopleName,const QString &startTime,const QString &endTime,const QString &recognitionResultType,const QString &ruleID)
{
	func_lock

	RecognitionRecordList recordList;

	HttpManagerErcode ercode{HttpErcode_None};
	if(true == searchFlag)		//带搜索字段的获取
		ercode = RecognizeRecordPackage::GetInstance()->getRecognizeRecord(m_ip,m_username,m_password,QString::number(pageSize),QString::number(pageIndex),recordList,
																  libraryID,peopleName,startTime.isEmpty()?"000000000000":startTime,endTime.isEmpty()?"000000000000":endTime,recognitionResultType,ruleID);
	else		//不带搜索字段的获取
		ercode = RecognizeRecordPackage::GetInstance()->getRecognizeRecord(m_ip,m_username,m_password,QString::number(pageSize),QString::number(pageIndex),recordList);
	check_http_ercode_no_return(ercode,"Get regonize record failed!")

	m_tableWidget->clearContents();

	if(true==recordList.isEmpty())
		return;

//	m_tableWidget->setRowCount(recordList.length());
	RecognitionRecordList::iterator iter;
	for(iter=recordList.begin();iter!=recordList.end();++iter)
	{
		int offset = iter-recordList.begin();

		QTableWidgetItem *logIDItem{nullptr};
		logIDItem = new QTableWidgetItem(iter->logID);
		check_pointer_no_return(logIDItem)
		logIDItem->setFlags(logIDItem->flags() & (~Qt::ItemIsEditable));
		m_tableWidget->setItem(offset,0,logIDItem);

		QTableWidgetItem *ruleItem{nullptr};
		ruleItem = new QTableWidgetItem("Rule "+QString(iter->ruleID));
		check_pointer_no_return(ruleItem)
		ruleItem->setFlags(ruleItem->flags() & (~Qt::ItemIsEditable));
		m_tableWidget->setItem(offset,1,ruleItem);

		QTableWidgetItem *faceLibraryItem{nullptr};
		faceLibraryItem = new QTableWidgetItem(iter->libraryName);
		check_pointer_no_return(faceLibraryItem)
		faceLibraryItem->setFlags(faceLibraryItem->flags() & (~Qt::ItemIsEditable));
		m_tableWidget->setItem(offset,2,faceLibraryItem);

		QTableWidgetItem *nameItem{nullptr};
		nameItem = new QTableWidgetItem(iter->peopleName);
		check_pointer_no_return(nameItem)
		nameItem->setFlags(nameItem->flags() & (~Qt::ItemIsEditable));
		m_tableWidget->setItem(offset,3,nameItem);

		//转换"yyyymmddThhmmss"到"yyyy-mm-dd hh:mm:ss"
		iter->time.insert(4,"-");
		iter->time.insert(7,"-");
		iter->time.replace("T"," ");
		iter->time.insert(13,":");
		iter->time.insert(16,":");
		QTableWidgetItem *recognizeTimeItem{nullptr};
		recognizeTimeItem = new QTableWidgetItem(iter->time);
		check_pointer_no_return(recognizeTimeItem)
		recognizeTimeItem->setFlags(recognizeTimeItem->flags() & (~Qt::ItemIsEditable));
		m_tableWidget->setItem(offset,4,recognizeTimeItem);

		QTableWidgetItem *recognizeResultItem{nullptr};
		recognizeResultItem = new QTableWidgetItem(iter->isPass=="0"?"Pass":"Failure");
		check_pointer_no_return(recognizeResultItem)
		recognizeResultItem->setFlags(recognizeResultItem->flags() & (~Qt::ItemIsEditable));
		m_tableWidget->setItem(offset,5,recognizeResultItem);

		QPushButton *detail{nullptr};
		detail = new QPushButton(this);
		check_pointer_no_return(detail)
		detail->setText("Detail");
		m_tableWidget->setCellWidget(offset,6,detail);
		QString libraryName=iter->libraryName;
		QString similar = iter->similar;
		QString logID= iter->logID;
		QString snapshotPath=iter->snapshotPath;
		QString recognizeTime=iter->time;
		connect(detail,&QPushButton::released,[this,logID,libraryName,similar,snapshotPath,recognizeTime]()
		{
			func_lock

			RecordDetail *recordDetail{nullptr};
			recordDetail = new RecordDetail;
			check_pointer_no_return(recordDetail)
			recordDetail->initUserInfo("",m_ip,m_username,m_password);
			recordDetail->updateRecordDetail(logID,libraryName,similar,recognizeTime,snapshotPath);
			recordDetail->show();
		});
	}
}

void RecognizeRecordWidget::slot_search()
{
	func_lock

	QFont font("Microsoft YaHei",adaptx(10));

	QDialog *searchWidget{nullptr};
	searchWidget = new QDialog;
	check_pointer_no_return(searchWidget)
	searchWidget->setModal(true);
//	searchWidget->setAttribute(Qt::WA_DeleteOnClose);
	searchWidget->setGeometry(adaptxywh(100+1300/2-1020/2,100+700/2-210/2,1020,210));

	//人脸库
	QLabel *library{nullptr};
	library = new QLabel(searchWidget);
	check_pointer_no_return(library)
	library->setGeometry(adaptxywh(30,30,100,30));
	library->setFont(font);
	library->setText("Library");
	library->show();

	QComboBox *libraryBox{nullptr};
	libraryBox = new QComboBox(searchWidget);
	check_pointer_no_return(libraryBox)
	libraryBox->setGeometry(adaptxywh(130,30,200,30));
	libraryBox->setFont(font);
	libraryBox->addItem("All");

	//得到所有的脸库
	FaceLibraryList faceLibraryList;
	HttpManagerErcode ercode = FaceLibraryPackage::GetInstance()->getFaceLibraryList(m_ip,m_username,m_password,faceLibraryList);
	check_http_ercode_no_return(ercode,"Get face library failed!")
	for(const FaceLibraryItem &item:faceLibraryList)
		libraryBox->addItem(item.libraryName);
	libraryBox->show();

	//开始时间
	QLabel *startTime{nullptr};
	startTime = new QLabel(searchWidget);
	check_pointer_no_return(startTime)
	startTime->setGeometry(adaptxywh(360,30,100,30));
	startTime->setFont(font);
	startTime->setText("Start Time");
	startTime->show();

	QDateTimeEdit *startTimeEdit{nullptr};
	startTimeEdit = new QDateTimeEdit(searchWidget);
	check_pointer_no_return(startTimeEdit)
	startTimeEdit->setGeometry(adaptxywh(460,30,200,30));
	startTimeEdit->setFont(font);
	startTimeEdit->setDisplayFormat("yyyy-MM-dd hh:mm:ss");
	startTimeEdit->show();

	//结束时间
	QLabel *endTime{nullptr};
	endTime = new QLabel(searchWidget);
	check_pointer_no_return(endTime)
	endTime->setGeometry(adaptxywh(690,30,100,30));
	endTime->setFont(font);
	endTime->setText("End Time");
	endTime->show();

	QDateTimeEdit *endTimeEdit{nullptr};
	endTimeEdit = new QDateTimeEdit(searchWidget);
	check_pointer_no_return(endTimeEdit)
	endTimeEdit->setGeometry(adaptxywh(790,30,200,30));
	endTimeEdit->setFont(font);
	endTimeEdit->setDisplayFormat("yyyy-MM-dd hh:mm:ss");
	endTimeEdit->show();

	//姓名
	QLabel *name{nullptr};
	name = new QLabel(searchWidget);
	check_pointer_no_return(name)
	name->setGeometry(adaptxywh(30,90,100,30));
	name->setFont(font);
	name->setText("Name");
	name->show();

	QLineEdit *nameEdit{nullptr};
	nameEdit = new QLineEdit(searchWidget);
	check_pointer_no_return(nameEdit)
	nameEdit->setGeometry(adaptxywh(130,90,200,30));
	nameEdit->setFont(font);
	nameEdit->show();

	//规则
	QLabel *rule{nullptr};
	rule = new QLabel(searchWidget);
	check_pointer_no_return(rule)
	rule->setGeometry(adaptxywh(360,90,100,30));
	rule->setFont(font);
	rule->setText("Rule");
	rule->show();

	QComboBox *ruleBox{nullptr};
	ruleBox = new QComboBox(searchWidget);
	check_pointer_no_return(ruleBox)
	ruleBox->setGeometry(adaptxywh(460,90,200,30));
	ruleBox->setFont(font);
	QStringList ruleItems{"all","rule 1","rule 2","rule 3","rule 4"};
	ruleBox->addItems(ruleItems);
	ruleBox->show();

	//识别类型
	QLabel *recognizeType{nullptr};
	recognizeType = new QLabel(searchWidget);
	check_pointer_no_return(recognizeType)
	recognizeType->setGeometry(adaptxywh(690,90,100,30));
	recognizeType->setFont(font);
	recognizeType->setWordWrap(true);
	recognizeType->setText("Comparison Type");
	recognizeType->show();

	QComboBox *recognizeTypeBox{nullptr};
	recognizeTypeBox = new QComboBox(searchWidget);
	check_pointer_no_return(recognizeTypeBox)
	recognizeTypeBox->setGeometry(adaptxywh(790,90,200,30));
	recognizeTypeBox->setFont(font);
	QStringList recognizeTypeItems{"all","Comparison pass","Comparison fail"};
	recognizeTypeBox->addItems(recognizeTypeItems);
	recognizeTypeBox->show();

	QPushButton *search{nullptr};
	search = new QPushButton(searchWidget);
	check_pointer_no_return(search)
	search->setGeometry(adaptxywh(1020/2-80,150,80,30));
	search->setText("Search");
	search->show();

	searchWidget->show();

	connect(search,&QPushButton::released,[=]()
	{
		func_lock

		//置搜索标志
		m_searchContent.searchFlag=true;
		//如果是在所有脸库中查询，则查询时传入的脸库ID为-1
		if(0 != libraryBox->currentIndex())
			m_searchContent.libraryID=faceLibraryList[libraryBox->currentIndex()].libraryID;
		else
			m_searchContent.libraryID = "-1";
		//姓名
		m_searchContent.peopleName=nameEdit->text();
		//查询起始时间
		m_searchContent.startTime=startTimeEdit->text();
		m_searchContent.startTime.replace("-","");
		//查询结束时间
		m_searchContent.endTime=endTimeEdit->text();
		m_searchContent.endTime.replace("-","");
		//识别类型（识别通过）(识别不通过)
		m_searchContent.recognizeType=recognizeTypeBox->currentText();
		//识别规则ID，如果查询所有识别规则，则查询时查询ID传入-1
		if(0 == ruleBox->currentIndex())
			m_searchContent.ruleID = "-1";
		else
			m_searchContent.ruleID = QString::number(ruleBox->currentIndex());

		updatePageInfo(m_searchContent.searchFlag,m_searchContent.libraryID,m_searchContent.peopleName,m_searchContent.startTime,m_searchContent.endTime,m_searchContent.recognizeType,m_searchContent.ruleID);
		updateRecord(m_pageSize->currentText().toInt(),1,m_searchContent.searchFlag,m_searchContent.libraryID,m_searchContent.peopleName,m_searchContent.startTime,m_searchContent.endTime,m_searchContent.recognizeType,m_searchContent.ruleID);

		searchWidget->close();
	});
}

void RecognizeRecordWidget::slot_refresh()
{
	func_lock

	m_searchContent.clear();

	updatePageInfo();
	updateRecord(m_pageSize->currentText().toInt(),1);
}
