﻿#include "src/inc/gui/recognizerecord/recorddetail.h"

#include "src/inc/aifacepch.h"
#include "src/inc/httpmanager/recognizerecordpackage.h"

#include <QLabel>

RecordDetail::RecordDetail(QWidget *parent) : QWidget(parent)
{
	this->setGeometry(adaptxywh(100+1300/2-200,150,400,580));
//	this->setAttribute(Qt::WA_DeleteOnClose);
	this->setWindowTitle("Record Detail");

	initGui();
}

void RecordDetail::initUserInfo(const QString &mac, const QString &ip, const QString &username, const QString &password)
{
	m_ip=ip;
	m_username=username;
	m_password=password;
}

void RecordDetail::updateRecordDetail(const QString &logID,const QString &faceLibraryName,const QString &similar,const QString &recognizeTime,const QString &snapshotPath)
{
	func_lock

	HttpManagerErcode ercode{HttpErcode_None};

	RecognitionRecordDetail detail;
	ercode = RecognizeRecordPackage::GetInstance()->getRecognizeRecordDetail(m_ip,m_username,m_password,logID,detail);
	check_http_ercode_no_return(ercode,"Get recognize record detail failed")

	const auto judgeEmpty=[](const QString &text)->QString
	{
		if(true==text.isEmpty())
			return "Unknown";

		if("unknown"==text)
			return "Unknown";

		if("other"==text)
			return "Unknown";

		if("00000000T000000"==text)
			return "Unknown";

		return text;
	};

	m_faceLibraryName->setText(judgeEmpty(faceLibraryName));
	m_similar->setText(judgeEmpty(similar));
	m_name->setText(judgeEmpty(detail.name));
	m_birthday->setText(judgeEmpty(detail.birthday));
	m_sex->setText(judgeEmpty(detail.sex));
	m_certificateType->setText(judgeEmpty(detail.certificateType));
	m_certificateID->setText(judgeEmpty(detail.certificateID));
	m_country->setText(judgeEmpty(detail.country));
	m_recognizeTime->setText(judgeEmpty(recognizeTime));

	QByteArray captureImage;
	ercode = RecognizeRecordPackage::GetInstance()->getSnapshot(m_ip,m_username,m_password,snapshotPath,captureImage);
	check_http_ercode_no_return(ercode,"Get snap shot failed!")
	QPixmap capturePixmap;
	capturePixmap.loadFromData(captureImage);
	m_captureImage->setPixmap(capturePixmap.scaled(m_captureImage->width(),m_captureImage->height()));

	QByteArray compareImage;
	ercode = RecognizeRecordPackage::GetInstance()->getComparedImage(m_ip,m_username,m_password,logID,compareImage);
	check_http_ercode_no_return(ercode,"Get compare image failed!");
	QPixmap comparePixmap;
	comparePixmap.loadFromData(compareImage);
	m_compareImage->setPixmap(comparePixmap.scaled(m_compareImage->width(),m_compareImage->height()));
}

void RecordDetail::initGui()
{
	QFont font("Microsoft YaHei",adaptx(10));

	//左侧
	QLabel *captureImageLabel{nullptr};
	captureImageLabel = new QLabel(this);
	check_pointer_no_return(captureImageLabel)
	captureImageLabel->setGeometry(adaptxywh(20,20,180,30));
	captureImageLabel->setFont(font);
	captureImageLabel->setAlignment(Qt::AlignCenter);
	captureImageLabel->setText("Captured Image");
	captureImageLabel->setStyleSheet("border:1px solid gray;");

	m_captureImage = new QLabel(this);
	m_captureImage->setGeometry(adaptxywh(20,50,180,180));
	m_captureImage->setFont(font);
	m_captureImage->setStyleSheet("border:1px solid gray;");

	QLabel *faceLibraryNameLabel{nullptr};
	faceLibraryNameLabel = new QLabel(this);
	check_pointer_no_return(faceLibraryNameLabel)
	faceLibraryNameLabel->setGeometry(adaptxywh(20,230,180,30));
	faceLibraryNameLabel->setFont(font);
	faceLibraryNameLabel->setAlignment(Qt::AlignCenter);
	faceLibraryNameLabel->setText("Library Time");
	faceLibraryNameLabel->setStyleSheet("border:1px solid gray;");

	QLabel *similarLabel{nullptr};
	similarLabel = new QLabel(this);
	check_pointer_no_return(faceLibraryNameLabel)
	similarLabel->setGeometry(adaptxywh(20,260,180,30));
	similarLabel->setFont(font);
	similarLabel->setAlignment(Qt::AlignCenter);
	similarLabel->setText("Similar");
	similarLabel->setStyleSheet("border:1px solid gray;");

	QLabel *nameLabel{nullptr};
	nameLabel = new QLabel(this);
	check_pointer_no_return(nameLabel)
	nameLabel->setGeometry(adaptxywh(20,290,180,30));
	nameLabel->setFont(font);
	nameLabel->setAlignment(Qt::AlignCenter);
	nameLabel->setText("Name");
	nameLabel->setStyleSheet("border:1px solid gray;");

	QLabel *birthdayLabel{nullptr};
	birthdayLabel = new QLabel(this);
	check_pointer_no_return(birthdayLabel)
	birthdayLabel->setGeometry(adaptxywh(20,320,180,30));
	birthdayLabel->setFont(font);
	birthdayLabel->setAlignment(Qt::AlignCenter);
	birthdayLabel->setText("Birthday");
	birthdayLabel->setStyleSheet("border:1px solid gray;");

	QLabel *sexLabel{nullptr};
	sexLabel = new QLabel(this);
	check_pointer_no_return(birthdayLabel)
	sexLabel->setGeometry(adaptxywh(20,350,180,30));
	sexLabel->setFont(font);
	sexLabel->setAlignment(Qt::AlignCenter);
	sexLabel->setText("Sex");
	sexLabel->setStyleSheet("border:1px solid gray;");

	QLabel *certificateTypeLabel{nullptr};
	certificateTypeLabel = new QLabel(this);
	check_pointer_no_return(certificateTypeLabel)
	certificateTypeLabel->setGeometry(adaptxywh(20,380,180,30));
	certificateTypeLabel->setFont(font);
	certificateTypeLabel->setAlignment(Qt::AlignCenter);
	certificateTypeLabel->setText("CertificateType");
	certificateTypeLabel->setStyleSheet("border:1px solid gray;");

	QLabel *certificateIDLabel{nullptr};
	certificateIDLabel = new QLabel(this);
	check_pointer_no_return(certificateIDLabel)
	certificateIDLabel->setGeometry(adaptxywh(20,410,180,30));
	certificateIDLabel->setFont(font);
	certificateIDLabel->setAlignment(Qt::AlignCenter);
	certificateIDLabel->setText("CertificateID");
	certificateIDLabel->setStyleSheet("border:1px solid gray;");

	QLabel *countryLabel{nullptr};
	countryLabel = new QLabel(this);
	check_pointer_no_return(certificateIDLabel)
	countryLabel->setGeometry(adaptxywh(20,440,180,30));
	countryLabel->setFont(font);
	countryLabel->setAlignment(Qt::AlignCenter);
	countryLabel->setText("Country");
	countryLabel->setStyleSheet("border:1px solid gray;");

	QLabel *provinceLabel{nullptr};
	provinceLabel = new QLabel(this);
	check_pointer_no_return(provinceLabel)
	provinceLabel->setGeometry(adaptxywh(20,470,180,30));
	provinceLabel->setFont(font);
	provinceLabel->setAlignment(Qt::AlignCenter);
	provinceLabel->setText("Province");
	provinceLabel->setStyleSheet("border:1px solid gray;");

	QLabel *cityLabel{nullptr};
	cityLabel = new QLabel(this);
	check_pointer_no_return(cityLabel)
	cityLabel->setGeometry(adaptxywh(20,500,180,30));
	cityLabel->setFont(font);
	cityLabel->setAlignment(Qt::AlignCenter);
	cityLabel->setText("City");
	cityLabel->setStyleSheet("border:1px solid gray;");

	QLabel *recognizeLabel{nullptr};
	recognizeLabel = new QLabel(this);
	check_pointer_no_return(recognizeLabel)
	recognizeLabel->setGeometry(adaptxywh(20,530,180,30));
	recognizeLabel->setFont(font);
	recognizeLabel->setAlignment(Qt::AlignCenter);
	recognizeLabel->setText("Recognize Time");
	recognizeLabel->setStyleSheet("border:1px solid gray;");

	//右侧
	QLabel *compareImageLabel{nullptr};
	compareImageLabel = new QLabel(this);
	check_pointer_no_return(compareImageLabel)
	compareImageLabel->setGeometry(adaptxywh(200,20,180,30));
	compareImageLabel->setFont(font);
	compareImageLabel->setAlignment(Qt::AlignCenter);
	compareImageLabel->setText("Compare Image");
	compareImageLabel->setStyleSheet("border:1px solid gray;");

	m_compareImage = new QLabel(this);
	m_compareImage->setGeometry(adaptxywh(200,50,180,180));
	m_compareImage->setFont(font);
	m_compareImage->setStyleSheet("border:1px solid gray;");

	m_faceLibraryName = new QLabel(this);
	m_faceLibraryName->setGeometry(adaptxywh(200,230,180,30));
	m_faceLibraryName->setFont(font);
	m_faceLibraryName->setAlignment(Qt::AlignCenter);
	m_faceLibraryName->setText("Unknown");
	m_faceLibraryName->setStyleSheet("border:1px solid gray;");

	m_similar = new QLabel(this);
	m_similar->setGeometry(adaptxywh(200,260,180,30));
	m_similar->setFont(font);
	m_similar->setAlignment(Qt::AlignCenter);
	m_similar->setText("Unknown");
	m_similar->setStyleSheet("border:1px solid gray;");

	m_name = new QLabel(this);
	m_name->setGeometry(adaptxywh(200,290,180,30));
	m_name->setFont(font);
	m_name->setAlignment(Qt::AlignCenter);
	m_name->setText("Unknown");
	m_name->setStyleSheet("border:1px solid gray;");

	m_birthday = new QLabel(this);
	m_birthday->setGeometry(adaptxywh(200,320,180,30));
	m_birthday->setFont(font);
	m_birthday->setAlignment(Qt::AlignCenter);
	m_birthday->setText("Unknown");
	m_birthday->setStyleSheet("border:1px solid gray;");

	m_sex = new QLabel(this);
	m_sex->setGeometry(adaptxywh(200,350,180,30));
	m_sex->setFont(font);
	m_sex->setAlignment(Qt::AlignCenter);
	m_sex->setText("Unknown");
	m_sex->setStyleSheet("border:1px solid gray;");

	m_certificateType = new QLabel(this);
	m_certificateType->setGeometry(adaptxywh(200,380,180,30));
	m_certificateType->setFont(font);
	m_certificateType->setAlignment(Qt::AlignCenter);
	m_certificateType->setText("Unknown");
	m_certificateType->setStyleSheet("border:1px solid gray;");

	m_certificateID = new QLabel(this);
	m_certificateID->setGeometry(adaptxywh(200,410,180,30));
	m_certificateID->setFont(font);
	m_certificateID->setAlignment(Qt::AlignCenter);
	m_certificateID->setText("Unknown");
	m_certificateID->setStyleSheet("border:1px solid gray;");

	m_country = new QLabel(this);
	m_country->setGeometry(adaptxywh(200,440,180,30));
	m_country->setFont(font);
	m_country->setAlignment(Qt::AlignCenter);
	m_country->setText("Unknown");
	m_country->setStyleSheet("border:1px solid gray;");

	m_province = new QLabel(this);
	m_province->setGeometry(adaptxywh(200,470,180,30));
	m_province->setFont(font);
	m_province->setAlignment(Qt::AlignCenter);
	m_province->setText("Unknown");
	m_province->setStyleSheet("border:1px solid gray;");

	m_city = new QLabel(this);
	m_city->setGeometry(adaptxywh(200,500,180,30));
	m_city->setFont(font);
	m_city->setAlignment(Qt::AlignCenter);
	m_city->setText("Unknown");
	m_city->setStyleSheet("border:1px solid gray;");

	m_recognizeTime = new QLabel(this);
	m_recognizeTime->setGeometry(adaptxywh(200,530,180,30));
	m_recognizeTime->setFont(font);
	m_recognizeTime->setAlignment(Qt::AlignCenter);
	m_recognizeTime->setText("Unknown");
	m_recognizeTime->setStyleSheet("border:1px solid gray;");
}


