﻿#include "src/inc/gui/snapshot/snapshotitemwidget.h"

#include "src/inc/aifacepch.h"
#include "src/inc/httpmanager/facelibrarypackage.h"
#include "src/inc/gui/facelibrary/personitemaddwidget.h"

#include <QLabel>
#include <QFont>
#include <QPushButton>
#include <QComboBox>

SnapShotItemWidget::SnapShotItemWidget(QWidget *parent) : QWidget(parent)
{

}

void SnapShotItemWidget::setImageData(const QByteArray &imageData)
{
	m_faceImageData = imageData;

	QPixmap pixmap;
	pixmap.loadFromData(imageData);
	m_image->setPixmap(pixmap.scaled(this->width(),this->width()));
}

void SnapShotItemWidget::setImageInfo(const QString &imageInfo)
{
	m_info->setText(imageInfo);
}

void SnapShotItemWidget::enterEvent(QEvent *event)
{
	m_addToFaceLibrary->show();
}

void SnapShotItemWidget::leaveEvent(QEvent *event)
{
	m_addToFaceLibrary->hide();
}

void SnapShotItemWidget::slot_addToFaceLibrary()
{
	func_lock

	QDialog *addWidget{nullptr};
	addWidget = new QDialog;
	check_pointer_no_return(addWidget);
	addWidget->setGeometry(adaptxywh(100+1300/2-150,100+700/2-50,300,100));
//	addWidget->setAttribute(Qt::WA_DeleteOnClose);

	QFont font("Microsoft YaHei",adaptx(10));

	QLabel *nameEditLabel{nullptr};
	nameEditLabel = new QLabel(addWidget);
	check_pointer_no_return(nameEditLabel)
	nameEditLabel->setFont(font);
	nameEditLabel->setGeometry(adaptxywh(10,10,100,30));
	nameEditLabel->setText("Library Name");
	nameEditLabel->show();

	QComboBox *libraryBox{nullptr};
	libraryBox = new QComboBox(addWidget);
	check_pointer_no_return(libraryBox)
	libraryBox->setFont(font);
	libraryBox->setGeometry(adaptxywh(110,10,180,30));
	libraryBox->show();

	QPushButton *stock{nullptr};
	stock = new QPushButton(addWidget);
	check_pointer_no_return(stock)
	stock->setGeometry(adaptxywh(150-40,50,80,30));
	stock->setFont(font);
	stock->setText("Stock");
	stock->show();

	addWidget->show();

	//获取所有脸库名称
	FaceLibraryList items;
	HttpManagerErcode ercode = FaceLibraryPackage::GetInstance()->getFaceLibraryList(m_ip,m_username,m_password,items);
	check_http_ercode_no_return(ercode,"Get face library list failed!")
	for(const FaceLibraryItem &item:items)
	{
		if("unBindGroup"==item.libraryName)
			continue;
		libraryBox->addItem(item.libraryName);
	}

	connect(stock,&QPushButton::released,[this,items,addWidget,libraryBox]()
	{
		func_lock

		QString libraryID = items.at(libraryBox->currentIndex()+1).libraryID;

		PersonItemAddWidget *personInfoAddWidget{nullptr};
		personInfoAddWidget = new PersonItemAddWidget;
//		personInfoAddWidget->setAttribute(Qt::WA_DeleteOnClose);
		personInfoAddWidget->initUserInfo("",m_ip,m_username,m_password);
		personInfoAddWidget->setLibraryID(libraryID);
		personInfoAddWidget->setImageInfo(m_faceImageData);
		personInfoAddWidget->show();

		connect(personInfoAddWidget,&PersonItemAddWidget::signal_updateFaceLibrary,[addWidget]()
		{
			func_lock

			addWidget->close();
		});
	});
}

void SnapShotItemWidget::initGui()
{
	QFont font("Microsoft YaHei",adaptx(8));

	m_image = new QLabel(this);
	check_pointer_no_return(m_image)
	m_image->setGeometry(0,0,this->width(),this->width());
	m_image->show();

	m_info = new QLabel(this);
	check_pointer_no_return(m_info)
	m_info->setGeometry(0,this->width()+adapty(2),this->width(),this->height()-this->width());
	m_info->setFont(font);
	m_info->show();

	m_addToFaceLibrary = new QPushButton(m_image);
	check_pointer_no_return(m_addToFaceLibrary)
	m_addToFaceLibrary->setGeometry(m_image->width()/2-adaptx(60)/2,m_image->width()/2-adaptx(30)/2,adaptx(60),adapty(30));
	m_addToFaceLibrary->setFont(font);
	m_addToFaceLibrary->setText("Stock");
	m_addToFaceLibrary->hide();
	connect(m_addToFaceLibrary,&QPushButton::released,this,&SnapShotItemWidget::slot_addToFaceLibrary);
}

void SnapShotItemWidget::init(const QString &mac, const QString &ip, const QString &username, const QString &password)
{
	m_ip = ip;
	m_username = username;
	m_password = password;
}
