﻿#include "src/inc/gui/snapshot/snapshotwidget.h"

#include "src/inc/aifacepch.h"
#include "src/inc/gui/snapshot/snapshotitemwidget.h"

#include <QLabel>
#include <QListWidget>
#include <QScrollArea>

SnapshotWidget *SnapshotWidget::m_snapshotWidget{nullptr};

SnapshotWidget::SnapshotWidget(QWidget *parent):QWidget(parent)
{
	this->setGeometry(adaptxywh(0,370,1300,460));
	this->setWindowTitle("Snapshot");

	initGui();

	m_imageSize = adaptx(110);

	connect(this,&SnapshotWidget::signal_addBasicPeopleDetectEvent,this,&SnapshotWidget::slot_addBasicPeopleDetectEvent);
	connect(this,&SnapshotWidget::signal_addBasicFaceDetectEvent,this,&SnapshotWidget::slot_addBasicFaceDetectEvent);
	connect(this,&SnapshotWidget::signal_addAdvancedFaceDetectEvent,this,&SnapshotWidget::slot_addAdvancedFaceDetectEvent);
	connect(this,&SnapshotWidget::signal_addAdvancedFaceRecognizeImage,this,&SnapshotWidget::slot_addAdvancedFaceRecognizeImage);
}

void SnapshotWidget::init(const QString &mac, const QString &ip, const QString &username, const QString &password)
{
	m_ip = ip;
	m_username = username;
	m_password = password;
}

void SnapshotWidget::slot_addBasicPeopleDetectEvent(QString eventInfo)
{	
	m_basicEventWidget->addItem(eventInfo);
}

void SnapshotWidget::slot_addBasicFaceDetectEvent(QString eventInfo)
{
	m_basicEventWidget->addItem(eventInfo);
}

void SnapshotWidget::slot_addAdvancedFaceDetectEvent(QString time,QString faceTemperature,QString trackID,QByteArray imageData)
{
	QFont font("Microsoft YaHei",adaptx(8));

	int x = adapty(10)+(m_imageSize+adaptx(10))*m_detectImageCount;

	SnapShotItemWidget *item{nullptr};
	item = new SnapShotItemWidget(m_advancedFaceDetectEventWidget);
	item->setGeometry(x,adapty(10),m_imageSize,m_imageSize+adapty(75));
	item->init("",m_ip,m_username,m_password);
	item->initGui();
	item->setImageData(imageData);
	item->setImageInfo(time+"\nTemper: "+faceTemperature+"\nTrackID: "+trackID);
	item->show();

//	QLabel *image{nullptr};
//	image = new QLabel(m_advancedFaceDetectEventWidget);
//	check_pointer_no_return(image)
//	image->setGeometry(x,adapty(10),m_imageSize,m_imageSize);
//	image->show();

//	QLabel *info{nullptr};
//	info = new QLabel(m_advancedFaceDetectEventWidget);
//	check_pointer_no_return(info)
//	info->setGeometry(adaptxywh(x,adapty(10)+m_imageSize,m_imageSize,adapty(40)));
//	info->setWordWrap(true);
//	info->setFont(font);
//	info->setText(time);
//	info->show();

	int width = x+m_imageSize+adaptx(10);
	if(x+m_imageSize+adaptx(10)>m_advancedFaceDetectEventArea->width())
		m_advancedFaceDetectEventWidget->resize(width,m_advancedFaceDetectEventWidget->height());

	++m_detectImageCount;
}

void SnapshotWidget::slot_addAdvancedFaceRecognizeImage(QString time,QString name,QString similar,QString faceTemperature,QByteArray imageData)
{
	QFont font("Microsoft YaHei",adaptx(8));

	int x = adapty(10)+(m_imageSize+adaptx(10))*m_recognizeImageCount;

	SnapShotItemWidget *item{nullptr};
	item = new SnapShotItemWidget(m_advancedFaceRecognizeEventWidget);
	item->setGeometry(x,adapty(10),m_imageSize,m_imageSize+adapty(75));
	item->init("",m_ip,m_username,m_password);
	item->initGui();
	item->setImageData(imageData);
	item->setImageInfo(time+"\nName: "+name+"\nSimilar:"+similar+"\nTemper:"+faceTemperature);
	item->show();

//	QLabel *image{nullptr};
//	image =new QLabel(m_advancedFaceRecognizeEventWidget);
//	check_pointer_no_return(image)
//	image->setGeometry(x,adapty(10),m_imageSize,m_imageSize);
//	image->show();

//	QLabel *info{nullptr};
//	info = new QLabel(m_advancedFaceRecognizeEventWidget);
//	check_pointer_no_return(info)
//	info->setGeometry(x,adapty(12)+m_imageSize,m_imageSize,adapty(40));
//	info->setFont(font);
//	info->setText(time+"\nName: "+name+"\nSimilar:"+similar);
//	info->show();

	if(x+m_imageSize+adaptx(10)>m_advancedFaceRecognizeEventWidget->width())
		m_advancedFaceRecognizeEventWidget->resize(x+m_imageSize+adaptx(10),m_advancedFaceRecognizeEventWidget->height());

//	QPixmap pixmap;
//	pixmap.loadFromData(imageData);
//	image->setPixmap(pixmap.scaled(m_imageSize,m_imageSize));

	++m_recognizeImageCount;
}

void SnapshotWidget::addBasicPeopleDetectEvent(const QString &eventInfo)
{
	emit signal_addBasicPeopleDetectEvent(eventInfo);
}

void SnapshotWidget::addBasicFaceDetectEvent(const QString &eventInfo)
{
	emit signal_addBasicFaceDetectEvent(eventInfo);
}

void SnapshotWidget::addAdvancedFaceDetectEvent(const QString &time, const QString &faceTemperature, const QString &trackID, const QByteArray &imageData)
{
	emit signal_addAdvancedFaceDetectEvent(time,faceTemperature,trackID,imageData);
}

void SnapshotWidget::addAdvancedFaceRecognizeEvent(const QString &time, const QString &name, const QString &similar, const QString &faceTemperature, const QByteArray &imageData)
{
	emit signal_addAdvancedFaceRecognizeImage(time,name,similar,faceTemperature,imageData);
}

void SnapshotWidget::showGui()
{
	this->show();
}

void SnapshotWidget::hideGui()
{
	this->hide();
}

void SnapshotWidget::initGui()
{
	QFont font("Microsoft YaHei",adaptx(10));

	QLabel *basicPeopleDetectLabel{nullptr};
	basicPeopleDetectLabel = new QLabel(this);
	check_pointer_no_return(basicPeopleDetectLabel)
	basicPeopleDetectLabel->setGeometry(adaptxywh(1010,0,260,30));
	basicPeopleDetectLabel->setFont(font);
	basicPeopleDetectLabel->setAlignment(Qt::AlignCenter);
	basicPeopleDetectLabel->setText("Basic Event");

	//基础版人形侦测事件
	m_basicEventWidget = new QListWidget(this);
	check_pointer_no_return(m_basicEventWidget)
	m_basicEventWidget->setGeometry(adaptxywh(1010,30,260,370));

	QLabel *advancedFaceDetectLabel{nullptr};
	advancedFaceDetectLabel = new QLabel(this);
	check_pointer_no_return(advancedFaceDetectLabel)
	advancedFaceDetectLabel->setGeometry(adaptxywh(30,0,940,30));
	advancedFaceDetectLabel->setFont(font);
	advancedFaceDetectLabel->setAlignment(Qt::AlignCenter);
	advancedFaceDetectLabel->setText("The face image detected");

	//高级人脸侦测事件
	m_advancedFaceDetectEventArea = new QScrollArea(this);
	check_pointer_no_return(m_advancedFaceDetectEventArea)
	m_advancedFaceDetectEventArea->setGeometry(adaptxywh(30,30,940,170));
	m_advancedFaceDetectEventArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	m_advancedFaceDetectEventWidget = new QWidget(this);
	check_pointer_no_return(m_advancedFaceDetectEventArea)
	m_advancedFaceDetectEventWidget->setGeometry(adaptxywh(0,0,940,200));
	m_advancedFaceDetectEventArea->setWidget(m_advancedFaceDetectEventWidget);

	QLabel *advancedFaceRecognizeLabel{nullptr};
	advancedFaceRecognizeLabel = new QLabel(this);
	check_pointer_no_return(advancedFaceRecognizeLabel)
	advancedFaceRecognizeLabel->setGeometry(adaptxywh(30,200,940,30));
	advancedFaceRecognizeLabel->setFont(font);
	advancedFaceRecognizeLabel->setAlignment(Qt::AlignCenter);
	advancedFaceRecognizeLabel->setText("The compare face image in face library");

	//高级人脸识别事件
	m_advancedFaceRecognizeEventArea = new QScrollArea(this);
	check_pointer_no_return(m_advancedFaceRecognizeEventArea)
	m_advancedFaceRecognizeEventArea->setGeometry(adaptxywh(30,230,940,170));
	m_advancedFaceRecognizeEventArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	m_advancedFaceRecognizeEventWidget=new QWidget(this);
	check_pointer_no_return(m_advancedFaceRecognizeEventWidget)
	m_advancedFaceRecognizeEventWidget->setGeometry(adaptxywh(0,200,940,200));
	m_advancedFaceRecognizeEventArea->setWidget(m_advancedFaceRecognizeEventWidget);
}
