﻿#include "src/inc/gui/widget.h"

#include "src/inc/aifacepch.h"
#include "src/inc/aifaceobject.h"
#include "src/inc/package/videopackage.h"
#include "src/inc/devicemanager/devicemanager.h"
#include "src/inc/httpmanager/httpmanagerpch.h"
#include "src/inc/httpmanager/aifacepackage.h"
#include "src/inc/gui/facedetect/aifaceconfigwidget.h"
#include "src/inc/gui/facelibrary/facelibrary.h"
#include "src/inc/gui/facerecognition/facerecognizewidget.h"
#include "src/inc/gui/recognizerecord/recognizerecordwidget.h"
#include "src/inc/gui/snapshot/snapshotwidget.h"

#include <QTableWidget>
#include <QScrollBar>
#include <QHeaderView>
#include <QCheckBox>
#include <QFont>
#include <QLabel>
#include <QLineEdit>
#include <QComboBox>
#include <QPushButton>
#include <QMessageBox>

#include <QRegExp>

Widget::Widget(QWidget *parent)
	: QWidget(parent)
{
	this->setGeometry(adaptxywh(100,100,1300,800));

	initDeviceGui();
	initUsernamePasswordGui();
	initSubscribeEventGui();
	initAIFaceConfigGui();
	initFaceLibraryGui();
	initFaceRecgnitionGui();
	initRecognizeRecordGui();
	slot_openFaceSnapshot();

	//启动设备探测服务
	vncp::VideoPackage::GetInstance()->startDeviceDetectService(5555,5555,1500);
	connect(vncp::VideoPackage::GetInstance(),&vncp::VideoPackage::signal_deviceUpdate,this,&Widget::slot_updateDevices);

	connect(this,&Widget::destroyed,[](){qDebug()<<"Gui Exited!";});
}

Widget::~Widget()
{
	//停止设备探测服务
	vncp::VideoPackage::GetInstance()->stopDeviceDetectService();
}

void Widget::initDeviceGui()
{
	//字体
	QFont font("Microsoft YaHei",adaptx(10));

	m_dvr = new QCheckBox(this);
	m_dvr->setGeometry(adaptxywh(50,10,100,30));
	m_dvr->setText("DVR");
	m_dvr->setFont(font);
	m_dvr->setChecked(true);
	connect(m_dvr,&QCheckBox::released,this,[&]()
	{
//		m_deviceList->clearContents();
		showDevices();
	});

	m_nvr = new QCheckBox(this);
	m_nvr->setGeometry(adaptxywh(170,10,100,30));
	m_nvr->setText("NVR");
	m_nvr->setFont(font);
	m_nvr->setChecked(true);
	connect(m_nvr,&QCheckBox::released,this,[&]()
	{
//		m_deviceList->clearContents();
		showDevices();
	});

	m_ipc = new QCheckBox(this);
	m_ipc->setGeometry(adaptxywh(290,10,100,30));
	m_ipc->setText("IPC");
	m_ipc->setFont(font);
	m_ipc->setChecked(true);
	connect(m_ipc,&QCheckBox::released,this,[&]()
	{
//		m_deviceList->clearContents();
		showDevices();
	});

	m_ipd =new  QCheckBox(this);
	m_ipd->setGeometry(adaptxywh(410,10,100,30));
	m_ipd->setText("IPD");
	m_ipd->setFont(font);
	m_ipd->setChecked(true);
	connect(m_ipd,&QCheckBox::released,this,[&]()
	{
		m_deviceList->clearContents();
		showDevices();
	});

	//创建设备列表框
	m_deviceList=new QTableWidget(10,5,this);
	m_deviceList->setGeometry(adaptxywh(30,50,940,320));
	QStringList header{"DeviceType","MAC","IP","ROM Version","BootTime"};
	m_deviceList->setHorizontalHeaderLabels(header);
	m_deviceList->setSelectionBehavior (QAbstractItemView::SelectRows);
	m_deviceList->setSelectionMode(QAbstractItemView::SingleSelection);
	m_deviceList->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
	connect(m_deviceList,&QTableWidget::itemPressed,[this](QTableWidgetItem *)
	{
		//获取当前选中项
		QList<QTableWidgetItem*> selectItems = m_deviceList->selectedItems();
		if(true == selectItems.isEmpty())
			return;

		m_selectDeviceMAC=selectItems.at(1)->text();
		m_selectDeviceIP=selectItems.at(2)->text();
	});
}

void Widget::initUsernamePasswordGui()
{
	QFont font("Microsoft YaHei",adaptx(10));
	QFontMetrics fontMetrics(font);
	int w;

	w = fontMetrics.boundingRect("Username").width();
	QLabel *usernameLabel{nullptr};
	usernameLabel = new QLabel(this);
	check_pointer_no_return(usernameLabel)
	usernameLabel->setGeometry(adaptx(540),adapty(10),w,adapty(30));
	usernameLabel->setText("Username");
	usernameLabel->setFont(font);

	m_username = new QLineEdit(this);
	m_username->setGeometry(adaptx(540)+w+adaptx(10),adapty(10),adaptx(100),adapty(30));
	m_username->setText("admin");
	m_username->setFont(font);

	w = fontMetrics.boundingRect("Password").width();
	QLabel *passwordLabel = new QLabel(this);
	passwordLabel->setGeometry(m_username->x()+m_username->width()+adaptx(60),adapty(10),w,adapty(30));
	passwordLabel->setText("Password");
	passwordLabel->setFont(font);

	m_password = new QLineEdit(this);
	m_password->setGeometry(passwordLabel->x()+passwordLabel->width()+adaptx(10),adapty(10),adaptx(100),adapty(30));
	m_password->setText("");
	m_password->setFont(font);
}

void Widget::initSubscribeEventGui()
{
	QFont font("Microsoft YaHei",adaptx(10));
//	QFontMetrics fontMetrics(font);
//	int w;

//	w = fontMetrics.boundingRect("Event Type").width();
//	QLabel *eventLabel{nullptr};
//	eventLabel = new QLabel(this);
//	check_pointer_no_return(eventLabel)
//	eventLabel->setGeometry(adaptx(1010),adapty(50),w,adapty(30));
//	eventLabel->setFont(font);
//	eventLabel->setText("Event Type");

//	m_eventTypes = new QComboBox(this);
//	m_eventTypes->setGeometry(eventLabel->x()+eventLabel->width()+adaptx(20),adapty(50),adaptx(160),adapty(30));
//	m_eventTypes->setFont(font);
//	/*
//	 * PeopleDetect：探测到人形
//	 * Face Detect Lite：探测到人脸
//	 * Face Detect：探测到人脸，传回识别到的人脸图片
//	 * Face Recognition：识别到人脸库中的人脸
//	*/
//	QStringList eventTypes{"People Detect","Face Detect Lite","Face Detect","Face Recognition"};
//	m_eventTypes->addItems(eventTypes);

	//订阅人脸事件按钮
	QPushButton *subscribe{nullptr};
	subscribe = new QPushButton(this);
	check_pointer_no_return(subscribe)
	subscribe->setGeometry(adaptxywh(1010,50,260,30));
	subscribe->setFont(font);
	subscribe->setText("Login");
	connect(subscribe,&QPushButton::released,this,&Widget::slot_subscribeEvent);

	//取消订阅人脸时间按钮
	QPushButton *unSubscribe{nullptr};
	unSubscribe = new QPushButton(this);
	check_pointer_no_return(unSubscribe)
	unSubscribe->setGeometry(adaptxywh(1010,100,260,30));
	unSubscribe->setFont(font);
	unSubscribe->setText("Logout");
	connect(unSubscribe,&QPushButton::released,this,&Widget::slot_unSubscribeEvent);
}

void Widget::initAIFaceConfigGui()
{
	QFont font("Microsoft YaHei",adaptx(10));

	//人脸配置按钮
	QPushButton *aiFaceConfig{nullptr};
	aiFaceConfig = new QPushButton(this);
	check_pointer_no_return(aiFaceConfig)
	aiFaceConfig->setGeometry(adaptxywh(1010,150,260,30));
	aiFaceConfig->setFont(font);
	aiFaceConfig->setText("AIFace Config");
	connect(aiFaceConfig,&QPushButton::released,this,&Widget::slot_openAIFaceConfigWidget);
}

void Widget::initFaceLibraryGui()
{
	QFont font("Microsoft YaHei",adaptx(10));

	QPushButton *faceLibrary{nullptr};
	faceLibrary = new QPushButton(this);
	check_pointer_no_return(faceLibrary)
	faceLibrary->setGeometry(adaptxywh(1010,200,260,30));
	faceLibrary->setFont(font);
	faceLibrary->setText("Face Library");
	connect(faceLibrary,&QPushButton::released,this,&Widget::slot_openFaceLibrary);
}

void Widget::initFaceRecgnitionGui()
{
	QFont font("Microsoft YaHei",adaptx(10));

	QPushButton *faceRecgnition{nullptr};
	faceRecgnition = new QPushButton(this);
	check_pointer_no_return(faceRecgnition)
	faceRecgnition->setGeometry(adaptxywh(1010,250,260,30));
	faceRecgnition->setFont(font);
	faceRecgnition->setText("Face Recognition");
	connect(faceRecgnition,&QPushButton::released,this,&Widget::slot_openFaceRecognition);
}

void Widget::initRecognizeRecordGui()
{
	QFont font("Microsoft YaHei",adaptx(10));

	QPushButton *recognizeRecord{nullptr};
	recognizeRecord = new QPushButton(this);
	check_pointer_no_return(recognizeRecord)
	recognizeRecord->setGeometry(adaptxywh(1010,300,260,30));
	recognizeRecord->setFont(font);
	recognizeRecord->setText("Recognize Record");
	connect(recognizeRecord,&QPushButton::released,this,&Widget::slot_openRecognizeRecord);
}

void Widget::initSnapshotGui()
{
//	QFont font("Microsoft YaHei",adaptx(10));

//	QPushButton *snapshot{nullptr};
//	snapshot = new QPushButton(this);
//	check_pointer_no_return(snapshot)
//	snapshot->setGeometry(adaptxywh(1010,350,200,30));
//	snapshot->setFont(font);
//	snapshot->setText("Face Snapshot");
//	connect(snapshot,&QPushButton::released,this,&Widget::slot_openFaceSnapshot);
}


void Widget::showDevices()
{
	QList<QTableWidgetItem *> selectItems = m_deviceList->selectedItems();
	if(true == selectItems.empty())
	{
		m_selectDeviceMAC.clear();
		m_selectDeviceIP.clear();
	}
	else
	{
		m_selectDeviceMAC=selectItems.at(1)->text();
		m_selectDeviceIP=selectItems.at(2)->text();
	}

	m_deviceList->clearContents();

	//根据设备个数，调整列表大小
	if(m_devices.length()>m_deviceList->rowCount())
		m_deviceList->setRowCount(m_devices.length());

	//如果某个设备掉线，去除登录状态
	QVector<QString>::iterator iter;
	for(iter = m_loginDeviceMac.begin();iter!=m_loginDeviceMac.end();)
	{
		bool isLost{true};
		for(const DeviceInfo &info:m_devices)
		{
			if(info.mac==*iter)
				isLost=false;
		}

		if(true == isLost)
			iter=m_loginDeviceMac.erase(iter);
		else
			++iter;
	}

	//将设备填充到设备列表中
	int count{0};
	for(const DeviceInfo &deviceInfo:m_devices)
	{
		//如果没有选中任何类型的设备，则直接退出添加
		if(false == m_dvr->isChecked()&&false == m_nvr->isChecked()&&false == m_ipc->isChecked()&&false == m_ipd->isChecked())
			break;

		if(DeviceType_DVR==deviceInfo.deviceType&&false==m_dvr->isChecked())
			continue;
		if(DeviceType_NVR==deviceInfo.deviceType&&false==m_nvr->isChecked())
			continue;
		if(DeviceType_IPC==deviceInfo.deviceType&&false==m_ipc->isChecked())
			continue;
		if(DeviceType_IPD==deviceInfo.deviceType&&false==m_ipd->isChecked())
			continue;

		QTableWidgetItem *deviceNameItem{nullptr};
		deviceNameItem = new QTableWidgetItem(deviceInfo.deviceName);
		check_pointer_no_return(deviceNameItem)
		deviceNameItem->setFlags(deviceNameItem->flags() & (~Qt::ItemIsEditable));
		m_deviceList->setItem(count,0,deviceNameItem);

		QTableWidgetItem *macItem{nullptr};
		macItem = new QTableWidgetItem(deviceInfo.mac);
		check_pointer_no_return(macItem)
		macItem->setFlags(macItem->flags() & (~Qt::ItemIsEditable));
		m_deviceList->setItem(count,1,macItem);

		QTableWidgetItem *ipItem{nullptr};
		ipItem = new QTableWidgetItem(deviceInfo.ip);
		check_pointer_no_return(ipItem)
		ipItem->setFlags(ipItem->flags() & (~Qt::ItemIsEditable));
		m_deviceList->setItem(count,2,ipItem);

		QTableWidgetItem *romVersionItem{nullptr};
		romVersionItem= new QTableWidgetItem(deviceInfo.romVersion);
		check_pointer_no_return(romVersionItem)
		romVersionItem->setFlags(romVersionItem->flags() & (~Qt::ItemIsEditable));
		m_deviceList->setItem(count,3,romVersionItem);

		QTableWidgetItem *bootTimeItem{nullptr};
		bootTimeItem= new QTableWidgetItem(deviceInfo.bootTime);
		check_pointer_no_return(bootTimeItem)
		bootTimeItem->setFlags(bootTimeItem->flags() & (~Qt::ItemIsEditable));
		m_deviceList->setItem(count,4,bootTimeItem);

		if(deviceInfo.mac==m_selectDeviceMAC)
		{
			int scrollPos = m_deviceList->verticalScrollBar()->value();
			m_deviceList->selectRow(count);
			m_deviceList->verticalScrollBar()->setValue(scrollPos);
		}

		if(true == m_loginDeviceMac.contains(deviceInfo.mac))
		{
			deviceNameItem->setForeground(Qt::red);
			macItem->setForeground(Qt::red);
			ipItem->setForeground(Qt::red);
			romVersionItem->setForeground(Qt::red);
			bootTimeItem->setForeground(Qt::red);

			m_deviceList->update();
		}

		++count;
	}
}

void Widget::slot_updateDevices(QVector<DeviceInfo> devices)
{
	m_devices.clear();
	m_devices.append(devices);

	showDevices();
}

void Widget::slot_subscribeEvent()
{	
	//如果未选中任何设备，则直接返回
	if(true==m_selectDeviceMAC.isEmpty())
	{
		message_output("Please select a device");
		return;
	}

	//用户名密码
	QString username=m_username->text();
	QString password=m_password->text();

//	m_selectDeviceIP="192.168.6.27";
//	username="admin";

	//判断设备是否支持AI人脸
	bool isSupportAIFace{false};
	HttpManagerErcode httpErcode = AIFacePackage::GetInstance()->isSupportAIFace(m_selectDeviceIP,username,password,isSupportAIFace);
	check_http_ercode_no_return(httpErcode,"Judge support ai face failed")

	if(false==isSupportAIFace)
	{
		QMessageBox msgBox;
		msgBox.setText("AI Face is not supported on this device.");
		msgBox.exec();

		return;
	}

	int ercode{0};

	int userID{-1};
	ercode = vncp::VideoPackage::GetInstance()->loginDevice(m_selectDeviceMAC,m_selectDeviceIP,6060,username,password,userID);
	check_ercode_no_return(ercode,"Login device failed!")

	if(true == m_loginDeviceMac.contains(m_selectDeviceMAC))
	{
		message_output("Already login!");
		return;
	}

	ercode = vncp::VideoPackage::GetInstance()->subscribeFaceEvent(userID);
	check_ercode_no_return(ercode,"Subscribe face event failed!")

	//自动订阅人脸有关事件
//	switch(m_eventTypes->currentIndex())
//	{
//	case 0:
//		ercode = vncp::VideoPackage::GetInstance()->subscribeBasicPeopleDetectEvent(userID);
//		check_ercode_no_return(ercode,"Subscribe basic people detect event failed!")
//		break;
//	case 1:
//		ercode = vncp::VideoPackage::GetInstance()->subscribeBasicFaceDetectEvent(userID);
//		check_ercode_no_return(ercode,"Subscribe basic face detect event failed!")
//		break;
//	case 2:
//		ercode = vncp::VideoPackage::GetInstance()->subscribeAdvancedFaceDetectEvent(userID);
//		check_ercode_no_return(ercode,"Subscribe advanced face detect event failed!")
//		break;
//	case 3:
//		ercode = vncp::VideoPackage::GetInstance()->subscribeAdvancedFaceRecognizeEvent(userID);
//		check_ercode_no_return(ercode,"Subscribe advanced face recognize event failed!")
//		break;
//	}

	m_loginDeviceMac.append(m_selectDeviceMAC);

	//图片将会入库到当前登录的设备中
	SnapshotWidget::GetInstance()->init("",m_selectDeviceIP,m_username->text(),m_password->text());

	message_output("login successfully!");
	showDevices();
}

void Widget::slot_unSubscribeEvent()
{
	//如果未选中任何设备，则直接返回
	if(true==m_selectDeviceMAC.isEmpty())
	{
		message_output("Please select a device");
		return;
	}

	//用户名密码
//	QString username=m_username->text();
//	QString password=m_password->text();

//	m_selectDeviceIP="192.168.6.27";
//	username="admin";

	//判断设备是否支持AI人脸
//	bool isSupportAIFace{false};
//	HttpManagerErcode httpErcode = AIFacePackage::GetInstance()->isSupportAIFace(m_selectDeviceIP,username,password,isSupportAIFace);
//	check_http_ercode_no_return(httpErcode,"Judge support ai face failed")

//	if(false==isSupportAIFace)
//	{
//		QMessageBox msgBox;
//		msgBox.setText("AI Face is not supported on this device.");
//		msgBox.exec();

//		return;
//	}

	if(false == m_loginDeviceMac.contains(m_selectDeviceMAC))
	{
		message_output("Not login");
		return;
	}

	int ercode{0};

	int userID{-1};
//	ercode = vncp::VideoPackage::GetInstance()->loginDevice(m_selectDeviceMAC,m_selectDeviceIP,6060,username,password,userID);
//	check_ercode_no_return(ercode,"Login device failed!")

	ercode = UserManager::GetInstance()->getDeviceUserID(m_selectDeviceMAC,userID);
	check_ercode_no_return(ercode,"Get userID failed!")

	ercode = vncp::VideoPackage::GetInstance()->unsubscribeFaceEvent(userID);
	check_ercode_no_return(ercode,"Unsubscribe face event failed!")

	//自动订阅人脸有关事件
//	switch(m_eventTypes->currentIndex())
//	{
//	case 0:
//		ercode = vncp::VideoPackage::GetInstance()->unSubscribeBasePeopleDetectEvent(userID);
//		check_ercode_no_return(ercode,"Unsubscribe basic people detect event failed!")
//		break;
//	case 1:
//		ercode = vncp::VideoPackage::GetInstance()->unSubscribeBasicFaceDetectEvent(userID);
//		check_ercode_no_return(ercode,"Unsubscribe basic face detect event failed!")
//		break;
//	case 2:
//		ercode = vncp::VideoPackage::GetInstance()->unSubscribeAdvancedFaceDetectEvent(userID);
//		check_ercode_no_return(ercode,"Unsubscribe advanced face detect event failed!")
//		break;
//	case 3:
//		ercode = vncp::VideoPackage::GetInstance()->unSubscribeAdvancedFaceRecognizeEvent(userID);
//		check_ercode_no_return(ercode,"Unsubscribe advanced face recognize event failed!")
//		break;
//	}

	m_loginDeviceMac.removeOne(m_selectDeviceMAC);

	message_output("logout successfully!");
	showDevices();
}

void Widget::slot_openAIFaceConfigWidget()
{
	func_lock

	//如果未选中任何设备，则直接返回
	if(true==m_selectDeviceMAC.isEmpty())
	{
		message_output("Please select a device");
		return;
	}

	//用户名密码
	QString username=m_username->text();
	QString password=m_password->text();

//	m_selectDeviceIP="192.168.6.27";
//	username="admin";

	//判断设备是否支持AI人脸
	bool isSupportAIFace{false};
	HttpManagerErcode httpErcode = AIFacePackage::GetInstance()->isSupportAIFace(m_selectDeviceIP,username,password,isSupportAIFace);
	check_http_ercode_no_return(httpErcode,"Judge support ai face failed")

	if(false==isSupportAIFace)
	{
		QMessageBox msgBox;
		msgBox.setText("AI Face is not supported on this device.");
		msgBox.exec();

		return;
	}

	AIFaceConfigWidget *aiFaceConfigWidget{nullptr};
	aiFaceConfigWidget = new AIFaceConfigWidget;
	check_pointer_no_return(aiFaceConfigWidget)
	aiFaceConfigWidget->show();

//	aiFaceConfigWidget->init(m_selectDeviceMAC,"192.168.6.27",username,password);
	aiFaceConfigWidget->init(m_selectDeviceMAC,m_selectDeviceIP,username,password);
}

void Widget::slot_openFaceLibrary()
{
	func_lock

	//如果未选中任何设备，则直接返回
	if(true==m_selectDeviceMAC.isEmpty())
	{
		message_output("Please select a device");

		return;
	}

	//用户名密码
	QString username=m_username->text();
	QString password=m_password->text();

//	m_selectDeviceIP="192.168.6.27";
//	username="admin";

	//判断设备是否支持AI人脸
	bool isSupportAIFace{false};
	HttpManagerErcode httpErcode = AIFacePackage::GetInstance()->isSupportAIFace(m_selectDeviceIP,username,password,isSupportAIFace);
	check_http_ercode_no_return(httpErcode,"Judge support ai face failed")

	if(false==isSupportAIFace)
	{
		QMessageBox msgBox;
		msgBox.setText("AI Face is not supported on this device.");
		msgBox.exec();

		return;
	}

	FaceLibrary *faceLibrary{nullptr};
	faceLibrary = new FaceLibrary;
	check_pointer_no_return(faceLibrary)
	faceLibrary->show();
//	faceLibrary->init(m_selectDeviceMAC,"192.168.6.27",m_username->text(),m_password->text());
	faceLibrary->init(m_selectDeviceMAC,m_selectDeviceIP,m_username->text(),m_password->text());
	faceLibrary->updateFaceLibraryInfo();
}

void Widget::slot_openFaceRecognition()
{
	func_lock

	//如果未选中任何设备，则直接返回
	if(true==m_selectDeviceMAC.isEmpty())
	{
		message_output("Please select a device");
		return;
	}

	//用户名密码
	QString username=m_username->text();
	QString password=m_password->text();

//	m_selectDeviceIP="192.168.6.27";
//	username="admin";

	//判断设备是否支持AI人脸
	bool isSupportAIFace{false};
	HttpManagerErcode httpErcode = AIFacePackage::GetInstance()->isSupportAIFace(m_selectDeviceIP,username,password,isSupportAIFace);
	check_http_ercode_no_return(httpErcode,"Judge support ai face failed")

	if(false==isSupportAIFace)
	{
		QMessageBox msgBox;
		msgBox.setText("AI Face is not supported on this device.");
		msgBox.exec();

		return;
	}

	FaceRecognizeWidget *faceRecognizeWidget{nullptr};
	faceRecognizeWidget = new FaceRecognizeWidget;
	check_pointer_no_return(faceRecognizeWidget)
	faceRecognizeWidget->show();
//	faceRecognizeWidget->initUserInfo(m_selectDeviceMAC,"192.168.6.27",m_username->text(),m_password->text());
	faceRecognizeWidget->initUserInfo(m_selectDeviceMAC,m_selectDeviceIP,m_username->text(),m_password->text());
}

void Widget::slot_openRecognizeRecord()
{
	func_lock

	//如果未选中任何设备，则直接返回
	if(true==m_selectDeviceMAC.isEmpty())
	{
		message_output("Please select a device");
		return;
	}

	//用户名密码
	QString username=m_username->text();
	QString password=m_password->text();

//	m_selectDeviceIP="192.168.6.27";
//	username="admin";

	//判断设备是否支持AI人脸
	bool isSupportAIFace{false};
	HttpManagerErcode httpErcode = AIFacePackage::GetInstance()->isSupportAIFace(m_selectDeviceIP,username,password,isSupportAIFace);
	check_http_ercode_no_return(httpErcode,"Judge support ai face failed")

	if(false==isSupportAIFace)
	{
		QMessageBox msgBox;
		msgBox.setText("AI Face is not supported on this device.");
		msgBox.exec();

		return;
	}

	RecognizeRecordWidget *recognizeRecordWidget{nullptr};
	recognizeRecordWidget = new RecognizeRecordWidget;
	check_pointer_no_return(recognizeRecordWidget)
	recognizeRecordWidget->show();
//	recognizeRecordWidget->initUserInfo(m_selectDeviceMAC,"192.168.6.27",m_username->text(),m_password->text());
	recognizeRecordWidget->initUserInfo(m_selectDeviceMAC,m_selectDeviceIP,m_username->text(),m_password->text());
}

void Widget::slot_openFaceSnapshot()
{
	SnapshotWidget::GetInstance(this)->showGui();
}
