﻿#ifndef WIDGET_H
#define WIDGET_H

#include <QWidget>
#include <QTimer>

#include "src/inc/aifaceobject.h"

QT_BEGIN_NAMESPACE
class QTableWidget;
class QCheckBox;
class QLineEdit;
class QPushButton;
class QComboBox;
QT_END_NAMESPACE

class Widget : public QWidget
{
	Q_OBJECT

public:
	Widget(QWidget *parent = 0);
	~Widget();

private:
	/**
	 * @brief initDeviceGui		创建设备界面部分
	 */
	void initDeviceGui();
	/**
	 * @brief initUsernamePasswordGui		创建设备登陆用户名密码界面
	 */
	void initUsernamePasswordGui();
	/**
	 * @brief initSubscribeEventGui		创建订阅事件界面
	 */
	void initSubscribeEventGui();
	/**
	 * @brief initAIFaceConfigGui		创建人脸探测配置界面
	 */
	void initAIFaceConfigGui();
	/**
	 * @brief initFaceLibraryGui			创建脸库管理界面
	 */
	void initFaceLibraryGui();
	/**
	 * @brief initFaceRecgnitionGui		创建人脸识别规则界面
	 */
	void initFaceRecgnitionGui();
	/**
	 * @brief initRecognizeRecordGui		创建识别记录界面
	 */
	void initRecognizeRecordGui();
	/**
	 * @brief initSnapshotGui		创建人脸抓拍图片GUI
	 */
	void initSnapshotGui();

	//显示探测到的设备
	void showDevices();

private:
	//显示所有探测到的设备
	QTableWidget *m_deviceList{nullptr};
	//设备过滤
	QCheckBox *m_dvr{nullptr};
	QCheckBox *m_nvr{nullptr};
	QCheckBox *m_ipc{nullptr};
	QCheckBox *m_ipd{nullptr};
	//被选中的设备
	QString m_selectDeviceMAC;
	QString m_selectDeviceIP;
	QVector<QString> m_loginDeviceMac;
	//探测到的设备，保存用于显示过滤器
	QVector<DeviceInfo> m_devices;

	//用户名
	QLineEdit *m_username{nullptr};
	//密码
	QLineEdit *m_password{nullptr};

	//事件订阅
	QComboBox *m_eventTypes{nullptr};

private slots:
	void slot_updateDevices(QVector<DeviceInfo>);

	void slot_subscribeEvent();
	void slot_unSubscribeEvent();
	void slot_openAIFaceConfigWidget();
	void slot_openFaceLibrary();
	void slot_openFaceRecognition();
	void slot_openRecognizeRecord();
	void slot_openFaceSnapshot();
};

#endif // WIDGET_H
