﻿#include "src/inc/httpmanager/aifacepackage.h"

#include "src/inc/restclient/xmlreader.h"
#include "src/inc/restclient/xmlwritter.h"
#include "src/inc/restclient/restclient.h"

#include <QNetworkRequest>
#include <QNetworkAccessManager>
#include <QNetworkReply>

AIFacePackage::AIFacePackage()
{

}

HttpManagerErcode AIFacePackage::isSupportAIFace(const QString &ip,const QString &username,const QString &password,bool &isSupport)
{
	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/System/DeviceCap");

	//设置请求头
	setHeader(ip,username,password);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Get,response,httpWaitTime);

	check_status_code(statusCode)

	isSupport = false;

	//解析返回的XML
	XmlReader xmlReader;
	xmlReader.setContent(response);

//	for(int i = 0;i<20;++i)
//	{
		QVector<KeyNode> keys{{"Device"},{"FunctionListAboutChannel",4},{"ChannelList"},{"Channel"},{"SmartEvent"}};
		QString value;
		XmlErcode ercode = xmlReader.readNodeValue(keys,value);
		//能力字段中包含"SmartEvent"，代表支持AIFace
		if(XmlErcode_None == ercode)
		{
			isSupport = true;
//			break;
		}
//	}

	return HttpErcode_None;
}

HttpManagerErcode AIFacePackage::getStreamResolution(const QString &ip, const QString &username, const QString &password, int &width, int &height)
{
	RestClient::GetInstance()->clear();

	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/Streams/1/1");

	//设置请求头
	setHeader(ip,username,password);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Get,response,httpWaitTime);

	check_status_code(statusCode)

	//解析返回的XML
	XmlReader xmlReader;
	xmlReader.setContent(response);
	XmlErcode ercode{XmlErcode_None};
	QVector<KeyNode> keys{{"StreamConfig"},{"ResolutionWidth",3}};
	QString widthValue;
	ercode = xmlReader.readNodeValue(keys,widthValue);
	keys.clear();
	keys={{"StreamConfig"},{"ResolutionHeigth",4}};
	QString heightValue;
	ercode = xmlReader.readNodeValue(keys,heightValue);
	//能力字段中包含"SmartEvent"，代表支持AIFace
	if(XmlErcode_None==ercode)
	{
		width=widthValue.toInt();
		height=heightValue.toInt();
		return HttpErcode_None;
	}
	else
		return HttpErcode_CannotAnalysisResponse;
}

HttpManagerErcode AIFacePackage::getMaxFaceResolutionLimit(const QString &ip, const QString &username, const QString &password, int &width, int &height)
{
	RestClient::GetInstance()->clear();

	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/System/DeviceCap");

	//设置请求头
	setHeader(ip,username,password);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Get,response,httpWaitTime);

	check_status_code(statusCode)

	//解析返回的XML
	XmlReader xmlReader;
	xmlReader.setContent(response);
	XmlErcode ercode{XmlErcode_None};
	KeyNodeList keys{{"Device"},{"FunctionListAboutChannel"},{"ChannelList"},{"Channel"},{"FaceSnapFilterRegion"},{"MaxDetectFaceRegionSize"},{"Width"}};
	QString maxFaceWidthValue;
	ercode = xmlReader.readNodeValue(keys,maxFaceWidthValue);
	keys.clear();
	keys={{"Device"},{"FunctionListAboutChannel"},{"ChannelList"},{"Channel"},{"FaceSnapFilterRegion"},{"MaxDetectFaceRegionSize"},{"Height"}};
	QString maxFaceHeightValue;
	ercode = xmlReader.readNodeValue(keys,maxFaceHeightValue);

	//能力字段中包含"SmartEvent"，代表支持AIFace
	if(XmlErcode_None==ercode)
	{
		width=maxFaceWidthValue.toInt();
		height=maxFaceHeightValue.toInt();
		return HttpErcode_None;
	}
	else
		return HttpErcode_CannotAnalysisResponse;
}

HttpManagerErcode AIFacePackage::getMinFaceResolutionLimit(const QString &ip, const QString &username, const QString &password, int &width, int &height)
{
	RestClient::GetInstance()->clear();

	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/System/DeviceCap");

	//设置请求头
	setHeader(ip,username,password);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Get,response,httpWaitTime);

	check_status_code(statusCode)

	//解析返回的XML
	XmlReader xmlReader;
	xmlReader.setContent(response);
	XmlErcode ercode{XmlErcode_None};
	KeyNodeList keys{{"Device"},{"FunctionListAboutChannel",4},{"ChannelList"},{"Channel"},{"FaceSnapFilterRegion"},{"MinDetectFaceRegionSize"},{"Width"}};
	QString minFaceWidthValue;
	ercode = xmlReader.readNodeValue(keys,minFaceWidthValue);
	keys.clear();
	keys={{"Device"},{"FunctionListAboutChannel",4},{"ChannelList"},{"Channel"},{"FaceSnapFilterRegion"},{"MinDetectFaceRegionSize"},{"Height"}};
	QString minFaceHeightValue;
	ercode = xmlReader.readNodeValue(keys,minFaceHeightValue);

	//能力字段中包含"SmartEvent"，代表支持AIFace
	if(XmlErcode_None==ercode)
	{
		width=minFaceWidthValue.toInt();
		height=minFaceHeightValue.toInt();
		return HttpErcode_None;
	}
	else
		return HttpErcode_CannotAnalysisResponse;
}

HttpManagerErcode AIFacePackage::getCurrentDetectRegion(const QString &ip, const QString &username, const QString &password, int &x, int &y, int &width, int &height)
{
	RestClient::GetInstance()->clear();

	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/Face/1/DetectRegion");

	//设置请求头
	setHeader(ip,username,password);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Get,response,httpWaitTime);

	check_status_code(statusCode)

	//解析返回的XML
	XmlReader xmlReader;
	xmlReader.setContent(response);
	XmlErcode ercode{XmlErcode_None};
	KeyNodeList keys;
	keys={{"FaceDetectRegion"},{"Region"},{"TopLeftX",1}};
	QString topLeftX;
	ercode = xmlReader.readNodeValue(keys,topLeftX);
	keys={{"FaceDetectRegion"},{"Region"},{"TopLeftY",2}};
	QString topLeftY;
	ercode = xmlReader.readNodeValue(keys,topLeftY);
	keys={{"FaceDetectRegion"},{"Region"},{"BottomRightX",3}};
	QString bottomRightX;
	ercode = xmlReader.readNodeValue(keys,bottomRightX);
	keys={{"FaceDetectRegion"},{"Region"},{"BottomRightY",4}};
	QString bottomRightY;
	ercode = xmlReader.readNodeValue(keys,bottomRightY);

	if(XmlErcode_None==ercode)
	{
		x=topLeftX.toInt();
		y=topLeftY.toInt();
		width=bottomRightX.toInt()-x;
		height=bottomRightY.toInt()-y;
		return HttpErcode_None;
	}
	else
		return HttpErcode_CannotAnalysisResponse;
}

HttpManagerErcode AIFacePackage::getCurrentMaxFaceResolution(const QString &ip, const QString &username, const QString &password, int &width, int &height)
{
	RestClient::GetInstance()->clear();

	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/Face/1/FilterRegionSize");

	//设置请求头
	setHeader(ip,username,password);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Get,response,httpWaitTime);

	check_status_code(statusCode)

	//解析返回的XML
	XmlReader xmlReader;
	xmlReader.setContent(response);
	XmlErcode ercode{XmlErcode_None};
	KeyNodeList keys;
	keys={{"FaceFilterRegionSize"},{"MaxFaceRegionSize"},{"Width",0}};
	QString maxFaceWidthValue;
	ercode = xmlReader.readNodeValue(keys,maxFaceWidthValue);
	keys={{"FaceFilterRegionSize"},{"MaxFaceRegionSize"},{"Height",1}};
	QString maxFaceHeightValue;
	ercode = xmlReader.readNodeValue(keys,maxFaceHeightValue);

	if(XmlErcode_None==ercode)
	{
		width=maxFaceWidthValue.toInt();
		height=maxFaceHeightValue.toInt();
		return HttpErcode_None;
	}
	else
		return HttpErcode_CannotAnalysisResponse;
}

HttpManagerErcode AIFacePackage::getCurrentMinFaceResolution(const QString &ip, const QString &username, const QString &password, int &width, int &height)
{
	RestClient::GetInstance()->clear();

	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/Face/1/FilterRegionSize");

	//设置请求头
	setHeader(ip,username,password);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Get,response,httpWaitTime);

	check_status_code(statusCode)

	//解析返回的XML
	XmlReader xmlReader;
	xmlReader.setContent(response);
	XmlErcode ercode{XmlErcode_None};
	KeyNodeList keys;
	keys={{"FaceFilterRegionSize"},{"MinFaceRegionSize",1},{"Width",0}};
	QString minFaceWidthValue;
	ercode = xmlReader.readNodeValue(keys,minFaceWidthValue);
	keys={{"FaceFilterRegionSize"},{"MinFaceRegionSize",1},{"Height",1}};
	QString minFaceHeightValue;
	ercode = xmlReader.readNodeValue(keys,minFaceHeightValue);

	if(XmlErcode_None==ercode)
	{
		width=minFaceWidthValue.toInt();
		height=minFaceHeightValue.toInt();
		return HttpErcode_None;
	}
	else
		return HttpErcode_CannotAnalysisResponse;
}

HttpManagerErcode AIFacePackage::setFaceDetectRegion(const QString &ip, const QString username, const QString &password, const int &x, const int &y, const int &width, const int &height)
{
	RestClient::GetInstance()->clear();

	RestClient::GetInstance()->setUrl("http://"+ip+"/Face/1/DetectRegion");

	setHeader(ip,username,password);

	XmlWritter xmlWritter;
	xmlWritter.addHead("");
	xmlWritter.addNode(XmlNodeType_Root,{"FaceDetectRegion"});
	xmlWritter.addNode(XmlNodeType_Parent,{"Region"},{"FaceDetectRegion"});
	xmlWritter.addNode(XmlNodeType_Child,{"ID",0},{"Region"},QString::number(0));
	xmlWritter.addNode(XmlNodeType_Child,{"TopLeftX",1},{"Region"},QString::number(x));
	xmlWritter.addNode(XmlNodeType_Child,{"TopLeftY",2},{"Region"},QString::number(y));
	xmlWritter.addNode(XmlNodeType_Child,{"BottomRightX",3},{"Region"},QString::number(x+width));
	xmlWritter.addNode(XmlNodeType_Child,{"BottomRightY",4},{"Region"},QString::number(y+height));

	QByteArray body;
	xmlWritter.exportXml(body);
	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Put,response,httpWaitTime);

	check_status_code(statusCode)

	return analysisResponseStatusCode(response);
}

HttpManagerErcode AIFacePackage::setFaceFilterSize(const QString &ip, const QString username, const QString &password, const int &maxWidth, const int &maxHeight,const int &minWidth,const int &minHeight)
{
	RestClient::GetInstance()->clear();

	RestClient::GetInstance()->setUrl("http://"+ip+"/Face/1/FilterRegionSize");

	setHeader(ip,username,password);

	XmlWritter xmlWritter;
	xmlWritter.addHead("");
	xmlWritter.addNode(XmlNodeType_Root,{"FaceFilterRegionSize"});
	xmlWritter.addNode(XmlNodeType_Parent,{"MaxFaceRegionSize",0},{"FaceFilterRegionSize"},"");
	xmlWritter.addNode(XmlNodeType_Child,{"Width",0},{"MaxFaceRegionSize",0},QString::number(maxWidth));
	xmlWritter.addNode(XmlNodeType_Child,{"Height",0},{"MaxFaceRegionSize",0},QString::number(maxHeight));
	xmlWritter.addNode(XmlNodeType_Parent,{"MinFaceRegionSize",1},{"FaceFilterRegionSize"});
	xmlWritter.addNode(XmlNodeType_Child,{"Width",0},{"MinFaceRegionSize",1},QString::number(minWidth));
	xmlWritter.addNode(XmlNodeType_Child,{"Height",1},{"MinFaceRegionSize",1},QString::number(minHeight));

	QByteArray body;
	xmlWritter.exportXml(body);
	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Put,response,httpWaitTime);

	check_status_code(statusCode)

	return analysisResponseStatusCode(response);
}

HttpManagerErcode AIFacePackage::getCurrentSnapConfig(const QString &ip, const QString &username, const QString &password, QVector<QString> &values)
{
	RestClient::GetInstance()->clear();

	RestClient::GetInstance()->setUrl("http://"+ip+"/Face/1/Config");

	setHeader(ip,username,password);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Get,response,httpWaitTime);

	check_status_code(statusCode)

	//解析返回的XML
	XmlReader xmlReader;
	xmlReader.setContent(response);
	XmlErcode ercode{XmlErcode_None};
	KeyNodeList keys;
	keys={{"FaceConfig"},{"SnapshotMode"}};
	QString snapShotMode;
	ercode = xmlReader.readNodeValue(keys,snapShotMode);
	values.append(snapShotMode);
	if("auto"==snapShotMode)
	{
		keys={{"FaceConfig"},{"ModeAuto",2},{"SnapshotCount",0}};
		QString snapShotCount;
		ercode=xmlReader.readNodeValue(keys,snapShotCount);
		values.append(snapShotCount);
		keys={{"FaceConfig"},{"ModeAuto",2},{"SnapshotInterval",1}};
		QString snapShotInterval;
		ercode=xmlReader.readNodeValue(keys,snapShotInterval);
		values.append(snapShotInterval);
		keys={{"FaceConfig"},{"CutoutMode",1}};
		QString mapConfig;
		ercode=xmlReader.readNodeValue(keys,mapConfig);
		values.append(mapConfig);
	}
	else if("optimal"==snapShotMode)
	{
		keys={{"FaceConfig"},{"ModeOptimal",4},{"SnapshotCount",0}};
		QString snapShotCount;
		ercode=xmlReader.readNodeValue(keys,snapShotCount);
		values.append(snapShotCount);
		keys={{"FaceConfig"},{"CutoutMode",1}};
		QString mapConfig;
		ercode=xmlReader.readNodeValue(keys,mapConfig);
		values.append(mapConfig);
	}
	else if("fast"==snapShotMode)
	{
		keys={{"FaceConfig"},{"ModeFast",3},{"SnapshotCount",0}};
		QString snapShotCount;
		ercode=xmlReader.readNodeValue(keys,snapShotCount);
		values.append(snapShotCount);
		keys={{"FaceConfig"},{"CutoutMode",1}};
		QString mapConfig;
		ercode=xmlReader.readNodeValue(keys,mapConfig);
		values.append(mapConfig);
	}
	else if("custom"==snapShotMode)
	{
		keys={{"FaceConfig"},{"ModeCustom",5},{"SnapshotCount",0}};
		QString snapShotCount;
		ercode=xmlReader.readNodeValue(keys,snapShotCount);
		values.append(snapShotCount);
		keys={{"FaceConfig"},{"ModeCustom",5},{"PicQualityThreshold",1}};
		QString qualityThreshold;
		ercode=xmlReader.readNodeValue(keys,qualityThreshold);
		values.append(qualityThreshold);
		keys={{"FaceConfig"},{"ModeCustom",5},{"ObliqueAngle",2},{"MaxValue"}};
		QString obliqueAngle;
		ercode=xmlReader.readNodeValue(keys,qualityThreshold);
		values.append(qualityThreshold);
		keys={{"FaceConfig"},{"ModeCustom",5},{"PitchAngle",3},{"MaxValue"}};
		QString pitchAngle;
		ercode=xmlReader.readNodeValue(keys,pitchAngle);
		values.append(pitchAngle);
		keys={{"FaceConfig"},{"ModeCustom",5},{"SideAngle",4},{"MaxValue"}};
		QString sideAngle;
		ercode=xmlReader.readNodeValue(keys,sideAngle);
		values.append(sideAngle);
		keys={{"FaceConfig"},{"ModeCustom",5},{"FuzzyDegree",5},{"MaxValue"}};
		QString fuzzyDegree;
		ercode=xmlReader.readNodeValue(keys,fuzzyDegree);
		values.append(fuzzyDegree);
		keys={{"FaceConfig"},{"CutoutMode",1}};
		QString mapConfig;
		ercode=xmlReader.readNodeValue(keys,mapConfig);
		values.append(mapConfig);
	}

	return HttpErcode_None;
}

HttpManagerErcode AIFacePackage::getSnapConfig(const QString &ip, const QString &username, const QString &password, const QString &snapShotMode, QVector<QString> &values)
{
	RestClient::GetInstance()->clear();

	RestClient::GetInstance()->setUrl("http://"+ip+"/Face/1/Config");

	setHeader(ip,username,password);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Get,response,httpWaitTime);

	check_status_code(statusCode)

	//解析返回的XML
	XmlReader xmlReader;
	xmlReader.setContent(response);
	XmlErcode ercode{XmlErcode_None};
	KeyNodeList keys;
	if("auto"==snapShotMode)
	{
		keys={{"FaceConfig"},{"ModeAuto",2},{"SnapshotCount",0}};
		QString snapShotCount;
		ercode=xmlReader.readNodeValue(keys,snapShotCount);
		values.append(snapShotCount);
		keys={{"FaceConfig"},{"ModeAuto",2},{"SnapshotInterval",1}};
		QString snapShotInterval;
		ercode=xmlReader.readNodeValue(keys,snapShotInterval);
		values.append(snapShotInterval);
		keys={{"FaceConfig"},{"CutoutMode",1}};
		QString mapConfig;
		ercode=xmlReader.readNodeValue(keys,mapConfig);
		values.append(mapConfig);
	}
	else if("optimal"==snapShotMode)
	{
		keys={{"FaceConfig"},{"ModeOptimal",4},{"SnapshotCount",0}};
		QString snapShotCount;
		ercode=xmlReader.readNodeValue(keys,snapShotCount);
		values.append(snapShotCount);
		keys={{"FaceConfig"},{"CutoutMode",1}};
		QString mapConfig;
		ercode=xmlReader.readNodeValue(keys,mapConfig);
		values.append(mapConfig);
	}
	else if("fast"==snapShotMode)
	{
		keys={{"FaceConfig"},{"ModeFast",3},{"SnapshotCount",0}};
		QString snapShotCount;
		ercode=xmlReader.readNodeValue(keys,snapShotCount);
		values.append(snapShotCount);
		keys={{"FaceConfig"},{"CutoutMode",1}};
		QString mapConfig;
		ercode=xmlReader.readNodeValue(keys,mapConfig);
		values.append(mapConfig);
	}
	else if("custom"==snapShotMode)
	{
		keys={{"FaceConfig"},{"ModeCustom",5},{"SnapshotCount",0}};
		QString snapShotCount;
		ercode=xmlReader.readNodeValue(keys,snapShotCount);
		values.append(snapShotCount);
		keys={{"FaceConfig"},{"ModeCustom",5},{"PicQualityThreshold",1}};
		QString qualityThreshold;
		ercode=xmlReader.readNodeValue(keys,qualityThreshold);
		values.append(qualityThreshold);
		keys={{"FaceConfig"},{"ModeCustom",5},{"ObliqueAngle",2},{"MaxValue"}};
		QString obliqueAngle;
		ercode=xmlReader.readNodeValue(keys,qualityThreshold);
		values.append(qualityThreshold);
		keys={{"FaceConfig"},{"ModeCustom",5},{"PitchAngle",3},{"MaxValue"}};
		QString pitchAngle;
		ercode=xmlReader.readNodeValue(keys,pitchAngle);
		values.append(pitchAngle);
		keys={{"FaceConfig"},{"ModeCustom",5},{"SideAngle",4},{"MaxValue"}};
		QString sideAngle;
		ercode=xmlReader.readNodeValue(keys,sideAngle);
		values.append(sideAngle);
		keys={{"FaceConfig"},{"ModeCustom",5},{"FuzzyDegree",5},{"MaxValue"}};
		QString fuzzyDegree;
		ercode=xmlReader.readNodeValue(keys,fuzzyDegree);
		values.append(fuzzyDegree);
		keys={{"FaceConfig"},{"CutoutMode",1}};
		QString mapConfig;
		ercode=xmlReader.readNodeValue(keys,mapConfig);
		values.append(mapConfig);
	}

	return HttpErcode_None;
}

HttpManagerErcode AIFacePackage::setSnapConfig(const QString &ip, const QString &username, const QString &password, const QVector<QString> &values)
{
	RestClient::GetInstance()->clear();

	RestClient::GetInstance()->setUrl("http://"+ip+"/Face/1/Config");

	setHeader(ip,username,password);

	XmlWritter xmlWritter;
	xmlWritter.addHead("");
	xmlWritter.addNode(XmlNodeType_Root,{"FaceConfig"});
	xmlWritter.addNode(XmlNodeType_Child,{"SnapshotMode"},{"FaceConfig"},values[0]);
	xmlWritter.addNode(XmlNodeType_Child,{"CutoutMode"},{"FaceConfig"},values[1]);
	xmlWritter.addNode(XmlNodeType_Parent,{"ModeAuto"},{"FaceConfig"});
	xmlWritter.addNode(XmlNodeType_Child,{"SnapshotCount"},{"ModeAuto"},values[2]);
	xmlWritter.addNode(XmlNodeType_Child,{"SnapshotInterval"},{"ModeAuto"},values[3]);
	xmlWritter.addNode(XmlNodeType_Parent,{"ModeFast"},{"FaceConfig"});
	xmlWritter.addNode(XmlNodeType_Child,{"SnapshotCount"},{"ModeFast"},values[4]);
	xmlWritter.addNode(XmlNodeType_Parent,{"ModeOptimal"},{"FaceConfig"});
	xmlWritter.addNode(XmlNodeType_Child,{"SnapshotCount"},{"ModeOptimal"},values[5]);
	xmlWritter.addNode(XmlNodeType_Parent,{"ModeCustom"},{"FaceConfig"});
	xmlWritter.addNode(XmlNodeType_Child,{"SnapshotCount"},{"ModeCustom"},values[6]);
	xmlWritter.addNode(XmlNodeType_Child,{"PicQualityThreshold"},{"ModeCustom"},values[7]);
	xmlWritter.addNode(XmlNodeType_Parent,{"ObliqueAngle"},{"ModeCustom"});
	xmlWritter.addNode(XmlNodeType_Child,{"MaxValue"},{"ObliqueAngle"},values[8]);
	xmlWritter.addNode(XmlNodeType_Child,{"MinValue"},{"ObliqueAngle"},"0");
	xmlWritter.addNode(XmlNodeType_Parent,{"PitchAngle"},{"ModeCustom"});
	xmlWritter.addNode(XmlNodeType_Child,{"MaxValue"},{"PitchAngle"},values[9]);
	xmlWritter.addNode(XmlNodeType_Child,{"MinValue"},{"PitchAngle"},"0");
	xmlWritter.addNode(XmlNodeType_Parent,{"SideAngle"},{"ModeCustom"});
	xmlWritter.addNode(XmlNodeType_Child,{"MaxValue"},{"SideAngle"},values[10]);
	xmlWritter.addNode(XmlNodeType_Child,{"MinValue"},{"SideAngle"},"0");
	xmlWritter.addNode(XmlNodeType_Parent,{"FuzzyDegree"},{"ModeCustom"});
	xmlWritter.addNode(XmlNodeType_Child,{"MaxValue"},{"FuzzyDegree"},values[11]);
	xmlWritter.addNode(XmlNodeType_Child,{"MinValue"},{"FuzzyDegree"},"0");

	QByteArray body;
	xmlWritter.exportXml(body);
	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Put,response,httpWaitTime);

	check_status_code(statusCode)

	return analysisResponseStatusCode(response);
}

HttpManagerErcode AIFacePackage::getOverlayConfig(const QString &ip, const QString &username, const QString &password, QString &value)
{
	RestClient::GetInstance()->clear();

	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/Face/1/OverlayInfo");

	//设置请求头
	setHeader(ip,username,password);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Get,response,httpWaitTime);

	check_status_code(statusCode)

	//解析返回的XML
	XmlReader xmlReader;
	xmlReader.setContent(response);
	XmlErcode ercode{XmlErcode_None};
	KeyNodeList keys{{"FaceOverlayInfo"},{"FaceTrackFrame",1},{"EnableFaceTrackFrame"}};
	ercode = xmlReader.readNodeValue(keys,value);

	return HttpErcode_None;
}

HttpManagerErcode AIFacePackage::setOverlayConfig(const QString &ip, const QString &username, const QString &password, const QString &value)
{
	RestClient::GetInstance()->clear();

	RestClient::GetInstance()->setUrl("http://"+ip+"/Face/1/OverlayInfo");

	setHeader(ip,username,password);

	XmlWritter xmlWritter;
	xmlWritter.addHead("");
	xmlWritter.addNode(XmlNodeType_Root,{"FaceOverlayInfo"});
	xmlWritter.addNode(XmlNodeType_Child,{"EnableFaceStatistcs"},{"FaceOverlayInfo"},"false");
	xmlWritter.addNode(XmlNodeType_Parent,{"FaceTrackFrame"},{"FaceOverlayInfo"});
	xmlWritter.addNode(XmlNodeType_Child,{"EnableFaceTrackFrame"},{"FaceTrackFrame"},value);
	xmlWritter.addNode(XmlNodeType_Child,{"EnableBackEndSuperposition"},{"FaceTrackFrame"},"false");

	QByteArray body;
	xmlWritter.exportXml(body);
	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Put,response,httpWaitTime);

	check_status_code(statusCode)

	return analysisResponseStatusCode(response);
}

void AIFacePackage::setHeader(const QString &ip,const QString &username,const QString &password)
{
	//设置请求头
	//发送HTTP请求的时候需要进行用户名密码验证：需要发送请求头"Authorization"，值为："Basic "+"用户名:密码"的Base64加密值
	RestClient::GetInstance()->addRequestParam("Authorization",QString("Basic ")+QString(username+":"+password).toLocal8Bit().toBase64());
	RestClient::GetInstance()->addRequestParam("User-Agent","PostmanRuntime/7.19.0");
	RestClient::GetInstance()->addRequestParam("Accept","*/*");
	RestClient::GetInstance()->addRequestParam("Accept-Encoding","gzip, deflate");
	RestClient::GetInstance()->addRequestParam("Host",ip);
	RestClient::GetInstance()->addRequestParam("Connection","Keep-Alive");
	RestClient::GetInstance()->addRequestParam("Cache-Control","no-cache");
}
