﻿#include "src/inc/httpmanager/facelibrarypackage.h"

#include "src/inc/restclient/restclient.h"
#include "src/inc/restclient/xmlreader.h"
#include "src/inc/restclient/xmlwritter.h"

#include <QFile>
#include <QTextCodec>

FaceLibraryPackage::FaceLibraryPackage()
{

}

HttpManagerErcode FaceLibraryPackage::getFaceLibraryList(const QString &ip, const QString &username, const QString &password, FaceLibraryList &values)
{
	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/FaceGroups/QueryAll");

	//设置请求头
	setHeader(ip,username,password);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Put,response,httpWaitTime);

	check_status_code(statusCode)

	//解析返回的XML
	XmlReader xmlReader;
	xmlReader.setContent(response);
	int i{0};
	XmlErcode ercode{XmlErcode_None};
	while(true)
	{
		KeyNodeList keys;
		FaceLibraryItem item;
		keys={{"FaceGroups"},{"FaceGroup",i},{"GroupID",0}};
		ercode = xmlReader.readNodeValue(keys,item.libraryID);
		if(XmlErcode_None != ercode)
			break;
		keys={{"FaceGroups"},{"FaceGroup",i},{"GroupName",1}};
		ercode = xmlReader.readNodeValue(keys,item.libraryName);
		keys={{"FaceGroups"},{"FaceGroup",i},{"CreateTime",2}};
		ercode = xmlReader.readNodeValue(keys,item.createTime);
		item.createTime.replace("T"," ");
		item.createTime.insert(4,"-");
		item.createTime.insert(7,"-");
		item.createTime.insert(13,":");
		item.createTime.insert(16,":");
		keys={{"FaceGroups"},{"FaceGroup",i},{"PersonInfoCount",3}};
		ercode = xmlReader.readNodeValue(keys,item.imageCount);

		values.append(item);
		++i;
	}

	return HttpErcode_None;
}

HttpManagerErcode FaceLibraryPackage::getFaceLibraryCount(const QString &ip, const QString &username, const QString &password, const QString &libraryID, QString &itemCount,
											 const QString &name,const QString &sex,const QString &certificateType,const QString &certificateID,const QString &country,const QString address,const QString &startBirthday,const QString &stopBirthday)
{
	RestClient::GetInstance()->clear();

	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/FaceGroup/QueryPersonInfoListCount");

	//设置请求头
	setHeader(ip,username,password);
	RestClient::GetInstance()->addRequestParam("Content-Type","application/x-www-form-urlencoded; charset=UTF-8");
	RestClient::GetInstance()->addRequestParam("Accept-Encoding","gzip, deflate");
	RestClient::GetInstance()->addRequestParam("Accept-Language","en-US,en;q=0.9,zh-CN;q=0.8,zh;q=0.7");

	XmlWritter xmlWritter;
	xmlWritter.addHead("");
	xmlWritter.addNode(XmlNodeType_Root,{"PersonInfoFilter"});
	xmlWritter.addNode(XmlNodeType_Child,{"GroupID"},{"PersonInfoFilter"},libraryID);
	xmlWritter.addNode(XmlNodeType_Child,{"Name"},{"PersonInfoFilter"},name.toUtf8());
	xmlWritter.addNode(XmlNodeType_Child,{"Sex"},{"PersonInfoFilter"},sex);
	xmlWritter.addNode(XmlNodeType_Child,{"CertificateType"},{"PersonInfoFilter"},certificateType);
	xmlWritter.addNode(XmlNodeType_Child,{"CertificateID"},{"PersonInfoFilter"},certificateID);
	xmlWritter.addNode(XmlNodeType_Child,{"Country"},{"PersonInfoFilter"},country.toUtf8());
	xmlWritter.addNode(XmlNodeType_Child,{"Address"},{"PersonInfoFilter"},address.toUtf8());
	xmlWritter.addNode(XmlNodeType_Child,{"StartBirthday"},{"PersonInfoFilter"},startBirthday);
	xmlWritter.addNode(XmlNodeType_Child,{"StopBirthday"},{"PersonInfoFilter"},stopBirthday);

	QByteArray body;
	xmlWritter.exportXml(body);
	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Put,response,httpWaitTime);

	check_status_code(statusCode)

	XmlReader xmlReader;
	xmlReader.setContent(response);

	KeyNodeList keys={{"PersonInfoList"},{"PersonCount"}};
	XmlErcode ercode = xmlReader.readNodeValue(keys,itemCount);

	return HttpErcode_None;
}

HttpManagerErcode FaceLibraryPackage::getFaceLibraryByLibraryID(const QString &ip, const QString &username, const QString &password, const QString &libraryID, const int &pageSize,const int &pageIndex,FaceLibraryItemInfoList &infos,
												   const QString &name,const QString &sex,const QString &certificateType,const QString &certificateID,const QString &country,const QString address,const QString &startBirthday,const QString &stopBirthday)
{
	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/FaceGroup/QueryPersonInfoList");

	//设置请求头
	setHeader(ip,username,password);
	RestClient::GetInstance()->addRequestParam("Content-Type","application/x-www-form-urlencoded; charset=UTF-8");
	RestClient::GetInstance()->addRequestParam("Accept-Encoding","gzip, deflate");
	RestClient::GetInstance()->addRequestParam("Accept-Language","en-US,en;q=0.9,zh-CN;q=0.8,zh;q=0.7");

	XmlWritter xmlWritter;
	xmlWritter.addHead("");
	xmlWritter.addNode(XmlNodeType_Root,{"PersonInfoFilter"});
	xmlWritter.addNode(XmlNodeType_Child,{"GroupID"},{"PersonInfoFilter"},libraryID);
	xmlWritter.addNode(XmlNodeType_Child,{"Name"},{"PersonInfoFilter"},name.toUtf8());
	xmlWritter.addNode(XmlNodeType_Child,{"Sex"},{"PersonInfoFilter"},sex);
	xmlWritter.addNode(XmlNodeType_Child,{"CertificateType"},{"PersonInfoFilter"},certificateType);
	xmlWritter.addNode(XmlNodeType_Child,{"CertificateID"},{"PersonInfoFilter"},certificateID);
	xmlWritter.addNode(XmlNodeType_Child,{"Country"},{"PersonInfoFilter"},country.toUtf8());
	xmlWritter.addNode(XmlNodeType_Child,{"Address"},{"PersonInfoFilter"},address.toUtf8());
	xmlWritter.addNode(XmlNodeType_Child,{"StartBirthday"},{"PersonInfoFilter"},startBirthday);
	xmlWritter.addNode(XmlNodeType_Child,{"StopBirthday"},{"PersonInfoFilter"},stopBirthday);
	xmlWritter.addNode(XmlNodeType_Child,{"Pagesize"},{"PersonInfoFilter"},QString::number(pageSize));
	xmlWritter.addNode(XmlNodeType_Child,{"Pagenum"},{"PersonInfoFilter"},QString::number(pageIndex));

	QByteArray body;
	xmlWritter.exportXml(body);
	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Put,response,httpWaitTime);

//	qDebug()<<response;

	check_status_code(statusCode)

	XmlReader xmlReader;
	xmlReader.setContent(response);
	XmlErcode ercode{XmlErcode_None};
	int i{0};
	KeyNodeList keys;
	while(true)
	{
		FaceLibraryItemInfo info;

		keys={{"PersonInfoList"},{"PersonInfo",i},{"Name",0}};
		ercode = xmlReader.readNodeValue(keys,info.name);
		if(XmlErcode_None!=ercode)
			break;
		keys={{"PersonInfoList"},{"PersonInfo",i},{"Sex",1}};
		ercode = xmlReader.readNodeValue(keys,info.sex);
		keys={{"PersonInfoList"},{"PersonInfo",i},{"CertificateType",2}};
		ercode = xmlReader.readNodeValue(keys,info.certificateType);
		keys={{"PersonInfoList"},{"PersonInfo",i},{"CertificateID",3}};
		ercode = xmlReader.readNodeValue(keys,info.certificateID);
#ifdef UniqueID_Support
		keys={{"PersonInfoList"},{"PersonInfo",i},{"UniqueID"}};
		ercode = xmlReader.readNodeValue(keys,info.uniqueID);
#else
		keys={{"PersonInfoList"},{"PersonInfo",i},{"PersonID",4}};
		ercode = xmlReader.readNodeValue(keys,info.personID);
#endif
		keys={{"PersonInfoList"},{"PersonInfo",i},{"GroupID",5}};
		ercode = xmlReader.readNodeValue(keys,info.libraryID);
		keys={{"PersonInfoList"},{"PersonInfo",i},{"Country",6}};
		ercode = xmlReader.readNodeValue(keys,info.country);
		keys={{"PersonInfoList"},{"PersonInfo",i},{"Address",7}};
		ercode = xmlReader.readNodeValue(keys,info.address);
		keys={{"PersonInfoList"},{"PersonInfo",i},{"Birthday",8}};
		ercode = xmlReader.readNodeValue(keys,info.birthday);
		info.birthday.replace("T"," ");
		info.birthday.insert(4,"-");
		info.birthday.insert(7,"-");
		info.birthday.insert(13,":");
		info.birthday.insert(16,":");
		keys={{"PersonInfoList"},{"PersonInfo",i},{"FaceImageID",9}};
		ercode = xmlReader.readNodeValue(keys,info.faceImageID);
		keys={{"PersonInfoList"},{"PersonInfo",i},{"ModelState",10}};
		ercode = xmlReader.readNodeValue(keys,info.modelStatus);

		infos.append(info);
		++i;
	}

	return HttpErcode_None;
}

#ifdef UniqueID_Support
HttpManagerErcode FaceLibraryPackage::getPersonImageInfo(const QString &ip, const QString &username, const QString &password, const QString &uniqueID, const QString &imageID, QByteArray &imageData)
{
	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/FaceGroup/QueryPersonImageInfo");

	//设置请求头
	setHeader(ip,username,password);

	XmlWritter xmlWritter;
	xmlWritter.addHead("");
	xmlWritter.addNode(XmlNodeType_Root,{"PersonFaceImageInfo"});
	xmlWritter.addNode(XmlNodeType_Child,{"UniqueID"},{"PersonFaceImageInfo"},uniqueID);
	xmlWritter.addNode(XmlNodeType_Child,{"PictureID"},{"PersonFaceImageInfo"},imageID);

	QByteArray body;
	xmlWritter.exportXml(body);
	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Post,response,httpWaitTime);

	check_status_code(statusCode)

	imageData=response;

	return HttpErcode_None;
}
#else
HttpManagerErcode FaceLibraryPackage::getPersonImageInfo(const QString &ip, const QString &username, const QString &password, const QString &personID, const QString &imageID, QByteArray &imageData)
{
	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/FaceGroup/QueryPersonImageInfo");

	//设置请求头
	setHeader(ip,username,password);

	XmlWritter xmlWritter;
	xmlWritter.addHead("");
	xmlWritter.addNode(XmlNodeType_Root,{"PersonFaceImageInfo"});
	xmlWritter.addNode(XmlNodeType_Child,{"PersonID"},{"PersonFaceImageInfo"},personID);
	xmlWritter.addNode(XmlNodeType_Child,{"PictureID"},{"PersonFaceImageInfo"},imageID);

	QByteArray body;
	xmlWritter.exportXml(body);
	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Post,response,httpWaitTime);

	check_status_code(statusCode)

	imageData=response;

	return HttpErcode_None;
}
#endif

HttpManagerErcode FaceLibraryPackage::setPersonInfo(const QString &ip, const QString &username, const QString &password, const FaceLibraryItemInfo &info)
{
	RestClient::GetInstance()->clear();

	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/FaceGroup/CheckPersonInfo");

	//设置请求头
	setHeader(ip,username,password);
	RestClient::GetInstance()->addRequestParam("Content-Type","application/x-www-form-urlencoded; charset=UTF-8");
	RestClient::GetInstance()->addRequestParam("Accept-Encoding","gzip, deflate");
	RestClient::GetInstance()->addRequestParam("Accept-Language","en-US,en;q=0.9,zh-CN;q=0.8,zh;q=0.7");

	XmlWritter xmlWritter;
	xmlWritter.addHead("");
	xmlWritter.addNode(XmlNodeType_Root,{"PersonInfo"});
#ifdef UniqueID_Support
	xmlWritter.addNode(XmlNodeType_Child,{"UniqueID"},{"PersonInfo"},info.uniqueID);
#else
	xmlWritter.addNode(XmlNodeType_Child,{"PersonID"},{"PersonInfo"},info.personID);
#endif
	xmlWritter.addNode(XmlNodeType_Child,{"Name"},{"PersonInfo"},info.name.toUtf8());
	xmlWritter.addNode(XmlNodeType_Child,{"Sex"},{"PersonInfo"},info.sex);
	xmlWritter.addNode(XmlNodeType_Child,{"CertificateType"},{"PersonInfo"},info.certificateType);
	xmlWritter.addNode(XmlNodeType_Child,{"CertificateID"},{"PersonInfo"},info.certificateID);
	xmlWritter.addNode(XmlNodeType_Child,{"Country"},{"PersonInfo"},info.country.toUtf8());
	xmlWritter.addNode(XmlNodeType_Child,{"Address"},{"PersonInfo"},info.address.toUtf8());
	xmlWritter.addNode(XmlNodeType_Child,{"Birthday"},{"PersonInfo"},info.birthday);

	QByteArray body;
	xmlWritter.exportXml(body);
	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Put,response,httpWaitTime);

	check_status_code(statusCode)
			
	HttpManagerErcode ercode = analysisResponseStatusCode(response);
	if(HttpErcode_None != ercode)
		return ercode;

	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/FaceGroup/ModifyPersonInfo");

	//设置请求头
	setHeader(ip,username,password);
	RestClient::GetInstance()->addRequestParam("Content-Type","application/x-www-form-urlencoded; charset=UTF-8");
	RestClient::GetInstance()->addRequestParam("Accept-Encoding","gzip, deflate");
	RestClient::GetInstance()->addRequestParam("Accept-Language","en-US,en;q=0.9,zh-CN;q=0.8,zh;q=0.7");

	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	statusCode=RestClient::GetInstance()->sendRequest(RequestType_Put,response,httpWaitTime);

	check_status_code(statusCode)

	return analysisResponseStatusCode(response);
}

HttpManagerErcode FaceLibraryPackage::setPersonImage(const QString &ip, const QString &username, const QString &password, const QString &personID, const QString &imageID, const QString &imageName,const QByteArray &imageData)
{
	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/FaceGroup/UpdateFaceImageInfoByPerson");

	//设置请求头
	setHeader(ip,username,password);

	const QByteArray boundary = "boundary=---------------------------7e3d513720998";
	RestClient::GetInstance()->addRequestParam("Content-Type","multipart/form-data; boundary=---------------------------7e3d513720998");

	/*
	-----------------------------7e3d513720998
	Content-Disposition: form-data; name="PersonID"

	 */
	QByteArray body;
	body.append(boundary);
	body.append("\r\n");
	body.append("Content-Disposition: form-data; name=\"PersonID\"\r\n\r\n");

	/*
	179
	 */
	body.append(personID);
	body.append("\r\n");

	/*
	-----------------------------7e3d513720998
	*/
	body.append(boundary);
	body.append("\r\n");

	/*
	Content-Disposition: form-data; name="PictureID"

	 */
	body.append("Content-Disposition: form-data; name=\"PictureID\"\r\n\r\n");

	/*
	1
	*/
	body.append(imageID);
	body.append("\r\n");

	/*
	-----------------------------7e3d513720998
	*/
	body.append(boundary);
	body.append("\r\n");

	/*
	Content-Disposition: form-data; name="file"; filename="temp.jpeg"
	Content-Type: image/jpeg

	*/
	body.append("Content-Disposition: form-data; name=\"file\"; filename=\""+imageName+"\"\r\n");
	body.append("Content-Type: image/jpeg\r\n");
	body.append("\r\n");

	body.append(imageData);

	/*
	-----------------------------7e3d513720998--

	*/
	body.append("\r\n");
	body.append(boundary);
	body.append("--");
//	body.append("\r\n");

	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Post,response,httpWaitTime);

	check_status_code(statusCode)

	return analysisResponseStatusCode(response);
}

HttpManagerErcode FaceLibraryPackage::deletePersonInfo(const QString &ip, const QString &username, const QString &password, const QVector<QVector<QString>> &datas, const QString &isDeleteFromAllLibrary)
{
	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/FaceGroup/DeletePersonList");

	//设置请求头
	setHeader(ip,username,password);

	XmlWritter xmlWritter;
	xmlWritter.addHead("");
	xmlWritter.addNode(XmlNodeType_Root,{"PersonInfoList"});
	QVector<QVector<QString>>::const_iterator iter;
	for(iter = datas.begin();iter != datas.end();++iter)
	{
		int offset = iter-datas.begin();

		xmlWritter.addNode(XmlNodeType_Parent,{"PersonInfo",offset},{"PersonInfoList"});
		xmlWritter.addNode(XmlNodeType_Child,{"GroupID"},{"PersonInfo",offset},iter->at(0));
#ifdef UniqueID_Support
		xmlWritter.addNode(XmlNodeType_Child,{"UniqueID"},{"PersonInfo",offset},iter->at(1));
#else
		xmlWritter.addNode(XmlNodeType_Child,{"PersonID"},{"PersonInfo",offset},iter->at(1));
#endif
		xmlWritter.addNode(XmlNodeType_Child,{"IsDeletePicture"},{"PersonInfo",offset},isDeleteFromAllLibrary);
	}


	QByteArray body;
	xmlWritter.exportXml(body);

	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Put,response,httpWaitTime);

	check_status_code(statusCode)

	return analysisResponseStatusCode(response);
}

HttpManagerErcode FaceLibraryPackage::addPersonInfo(const QString &ip, const QString &username, const QString &password, const FaceLibraryItemInfo &info, const QByteArray &imageData, QString &personID, QString &uniqueID)
{
	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/FaceGroup/AddPersonInfoAndFaceImage");

	//设置请求头
	setHeader(ip,username,password);
	RestClient::GetInstance()->addRequestParam("Content-Type","application/x-www-form-urlencoded; charset=UTF-8");
	RestClient::GetInstance()->addRequestParam("Accept-Encoding","gzip, deflate");
	RestClient::GetInstance()->addRequestParam("Accept-Language","en-US,en;q=0.9,zh-CN;q=0.8,zh;q=0.7");

	const QByteArray boundary = "---------------------------7e3d513720998";
	RestClient::GetInstance()->addRequestParam("Content-Type","multipart/form-data; boundary=---------------------------7e3d513720998");


	QByteArray body;
	/*
	-----------------------------7e33321a613aa
	Content-Disposition: form-data; name="IgnoreError"
	*/
	body.append(boundary);
	body.append("\r\n");
	body.append("Content-Disposition: form-data; name=\"IgnoreError\"\r\n\r\n");

	/*
	false
	 */
	body.append("false");
	body.append("\r\n");

	/*
	-----------------------------7e33321a613aa
	Content-Disposition: form-data; name="Name"
	 */
	body.append(boundary);
	body.append("\r\n");
	body.append("Content-Disposition: form-data; name=\"Name\"\r\n\r\n");

	/*
	name
	*/
	QByteArray name = info.name.toUtf8();
	body.append(name);
	body.append("\r\n");

	/*
	-----------------------------7e33321a613aa
	Content-Disposition: form-data; name="Sex"
	*/
	body.append(boundary);
	body.append("\r\n");
	body.append("Content-Disposition: form-data; name=\"Sex\"\r\n\r\n");

	/*
	male
	*/
	body.append(info.sex.toLower());
	body.append("\r\n");

	/*
	-----------------------------7e33321a613aa
	Content-Disposition: form-data; name="CertificateType"
	*/
	body.append(boundary);
	body.append("\r\n");
	body.append("Content-Disposition: form-data; name=\"CertificateType\"\r\n\r\n");

	/*
	IC
	 */
	body.append(info.certificateType);
	body.append("\r\n");

	/*
	-----------------------------7e33321a613aa
	Content-Disposition: form-data; name="CertificateID"
	*/
	body.append(boundary);
	body.append("\r\n");
	body.append("Content-Disposition: form-data; name=\"CertificateID\"\r\n\r\n");

	/*
	120000000000000000
	 */
	body.append(info.certificateID);
	body.append("\r\n");

	/*
	-----------------------------7e33321a613aa
	Content-Disposition: form-data; name="GroupID"
	*/
	body.append(boundary);
	body.append("\r\n");
	body.append("Content-Disposition: form-data; name=\"GroupID\"\r\n\r\n");

	/*
	2
	 */
	body.append(info.libraryID);
	body.append("\r\n");

	/*
	-----------------------------7e33321a613aa
	Content-Disposition: form-data; name="Country"
	*/
	body.append(boundary);
	body.append("\r\n");
	body.append("Content-Disposition: form-data; name=\"Country\"\r\n\r\n");

	/*
	中国
	 */
	QByteArray country = info.country.toUtf8();
	body.append(country);
	body.append("\r\n");

	/*
	-----------------------------7e3fa103093e
	Content-Disposition: form-data; name="Address"
	*/
	body.append(boundary);
	body.append("\r\n");
	body.append("Content-Disposition: form-data; name=\"Address\"\r\n\r\n");

	/*
	湖北省#武汉市
	 */
	QByteArray address = info.address.toUtf8();
	body.append(address);
	body.append("\r\n");

	/*
	-----------------------------7e3fa103093e
	Content-Disposition: form-data; name="Birthday"
	*/
	body.append(boundary);
	body.append("\r\n");
	body.append("Content-Disposition: form-data; name=\"Birthday\"\r\n\r\n");

	/*
	20191112
	 */
	QByteArray birthday = info.birthday.toLocal8Bit();
	body.append(birthday);
	body.append("\r\n");

	/*
	-----------------------------7e3fa103093e
	Content-Disposition: form-data; name="FaceImageID"
	*/
	body.append(boundary);
	body.append("\r\n");
	body.append("Content-Disposition: form-data; name=\"FaceImageID\"\r\n\r\n");

	/*
	20191112
	 */
	QByteArray faceImageID = "1";
	body.append(faceImageID);
	body.append("\r\n");

	/*
	 -----------------------------7e3b493093e
	Content-Disposition: form-data; name="file"; filename="temp.jpeg"
	Content-Type: image/jpeg
	*/
	body.append(boundary);
	body.append("\r\n");
	body.append("Content-Disposition: form-data; name=\"file\"; filename=\"temp.jpeg\"\r\n");
	body.append("Content-Type: image/jpeg\r\n");
	body.append("\r\n");

	body.append(imageData);

	/*
	-----------------------------7e3d513720998--

	*/
	body.append("\r\n");
	body.append(boundary);
	body.append("--");
//	body.append("\r\n");

	RestClient::GetInstance()->addRequestParam("Content-Length",QString::number(body.length()));
	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Post,response,httpWaitTime);

	check_status_code(statusCode)

	XmlReader xmlReader;
	xmlReader.setContent(response);

	if(static_cast<HttpManagerErcode>(4111) == statusCode)
	{
		QString value;
		QVector<KeyNode> keys{{"ResponseStatus"},{"PersonID",1}};
		xmlReader.readNodeValue(keys,value);
		personID = value;
		keys = {{"ResponseStatus"},{"UniqueID",2}};
		xmlReader.readNodeValue(keys,value);
		uniqueID = value;
	}

	return analysisResponseStatusCode(response);
}

HttpManagerErcode FaceLibraryPackage::deleteFaceLibrary(const QString &ip, const QString &username, const QString &password, const QString &groupID,const QString &isDelBindFace)
{
	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/FaceGroups/Delete");

	//设置请求头
	setHeader(ip,username,password);

	XmlWritter xmlWritter;
	xmlWritter.addHead("");
	xmlWritter.addNode(XmlNodeType_Root,{"FaceGroup"});
	xmlWritter.addNode(XmlNodeType_Child,{"GroupID"},{"FaceGroup"},groupID);
	xmlWritter.addNode(XmlNodeType_Child,{"DelBindFace"},{"FaceGroup"},isDelBindFace);

	QByteArray body;
	xmlWritter.exportXml(body);

	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Put,response,httpWaitTime);

	check_status_code(statusCode)

	return analysisResponseStatusCode(response);
}

HttpManagerErcode FaceLibraryPackage::editFaceLibraryName(const QString &ip, const QString &username, const QString &password, const QString &groupID, const QString &groupName)
{
	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/FaceGroups/Modify");

	//设置请求头
	setHeader(ip,username,password);
	RestClient::GetInstance()->addRequestParam("Content-Type","application/x-www-form-urlencoded; charset=UTF-8");
	RestClient::GetInstance()->addRequestParam("Accept-Encoding","gzip, deflate");
	RestClient::GetInstance()->addRequestParam("Accept-Language","en-US,en;q=0.9,zh-CN;q=0.8,zh;q=0.7");

	XmlWritter xmlWritter;
	xmlWritter.addHead("");
	xmlWritter.addNode(XmlNodeType_Root,{"FaceGroup"});
	xmlWritter.addNode(XmlNodeType_Child,{"GroupID"},{"FaceGroup"},groupID);
	xmlWritter.addNode(XmlNodeType_Child,{"GroupName"},{"FaceGroup"},groupName.toUtf8());

	QByteArray body;
	xmlWritter.exportXml(body);

	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Put,response,httpWaitTime);

	check_status_code(statusCode)

	return analysisResponseStatusCode(response);
}

HttpManagerErcode FaceLibraryPackage::addFaceLibraryName(const QString &ip, const QString &username, const QString &password, const QString &name)
{
	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/FaceGroups/Create");

	//设置请求头
	setHeader(ip,username,password);
	RestClient::GetInstance()->addRequestParam("Content-Type","application/x-www-form-urlencoded; charset=UTF-8");
	RestClient::GetInstance()->addRequestParam("Accept-Encoding","gzip, deflate");
	RestClient::GetInstance()->addRequestParam("Accept-Language","en-US,en;q=0.9,zh-CN;q=0.8,zh;q=0.7");

	XmlWritter xmlWritter;
	xmlWritter.addHead("");
	xmlWritter.addNode(XmlNodeType_Root,{"FaceGroup"});
	xmlWritter.addNode(XmlNodeType_Child,{"GroupName"},{"FaceGroup"},name.toUtf8());

	QByteArray body;
	xmlWritter.exportXml(body);

	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Put,response,httpWaitTime);

	check_status_code(statusCode)

	return analysisResponseStatusCode(response);
}

HttpManagerErcode FaceLibraryPackage::bindPersonToFaceLibrary(const QString &ip, const QString &username, const QString &password, const QVector<QVector<QString>> &datas)
{
	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/FaceGroup/BindPersonInfo");

	//设置请求头
	setHeader(ip,username,password);

	XmlWritter xmlWritter;
	xmlWritter.addHead("");
	xmlWritter.addNode(XmlNodeType_Root,{"PersonInfoList"});
	QVector<QVector<QString>>::const_iterator iter;
	for(iter = datas.begin();iter != datas.end();++iter)
	{
		int offset = iter - datas.begin();

		xmlWritter.addNode(XmlNodeType_Parent,{"PersonInfo",offset},{"PersonInfoList"});
		xmlWritter.addNode(XmlNodeType_Child,{"SrcGroupID"},{"PersonInfo",offset},iter->at(0));
		xmlWritter.addNode(XmlNodeType_Child,{"DstGroupID"},{"PersonInfo",offset},iter->at(1));
#ifdef UniqueID_Support
		xmlWritter.addNode(XmlNodeType_Child,{"UniqueID"},{"PersonInfo",offset},iter->at(2));
#else
		xmlWritter.addNode(XmlNodeType_Child,{"PersonID"},{"PersonInfo",offset},iter->at(2));
#endif
	}

	QByteArray body;
	xmlWritter.exportXml(body);

	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Put,response,httpWaitTime);

	check_status_code(statusCode)

	return analysisResponseStatusCode(response);
}

HttpManagerErcode FaceLibraryPackage::exportFaceLibrary(const QString &ip, const QString &username, const QString &password, QByteArray &data)
{
	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/Common/DeviceFileDownload?FileType=11&Path=1575273539354");

	//设置请求头
	setHeader(ip,username,password);

	//发送请求
	QByteArray response;
	//文件上传可能需要耗时较长，同步等待时间为最大值
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Get,response);

	check_status_code(statusCode)

	data = response;

	return HttpErcode_None;
}

HttpManagerErcode FaceLibraryPackage::importFaceLibrary(const QString &ip, const QString &username, const QString &password, const QString &filename, const QByteArray &data)
{
	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/Common/DeviceFileUpload");

	//设置请求头
	setHeader(ip,username,password);
	RestClient::GetInstance()->addRequestParam("Content-Type","multipart/form-data; boundary=---------------------------7e33a61bb0920");
	RestClient::GetInstance()->addRequestParam("FileType","11");
	RestClient::GetInstance()->addRequestParam("FilePath","1");

	QByteArray body;

	/*
	-----------------------------7e33a61bb0920
	 */
	body.append("-----------------------------7e33a61bb0920");
	body.append("\r\n");

	/*
	Content-Disposition: form-data; name="file"; filename="faceReco (4).bin"
	Content-Type: application/octet-stream
	 */
	body.append("Content-Disposition: form-data; name=\"file\"; filename=\""+filename+"\"");
	body.append("\r\n");
	body.append("Content-Type: application/octet-stream");
	body.append("\r\n\r\n");

	body.append(data);

	body.append("\r\n");
	body.append("-----------------------------7e33a61bb0920--\r\n");

	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	//文件上传可能需要耗时较长，同步等待时间为最大值
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Put,response);

	check_status_code(statusCode)

	return analysisResponseStatusCode(response);
}

void FaceLibraryPackage::setHeader(const QString &ip,const QString &username,const QString &password)
{
	//设置请求头
	//发送HTTP请求的时候需要进行用户名密码验证：需要发送请求头"Authorization"，值为："Basic "+"用户名:密码"的Base64加密值
	RestClient::GetInstance()->addRequestParam("Authorization",QString("Basic ")+QString(username+":"+password).toLocal8Bit().toBase64());
	RestClient::GetInstance()->addRequestParam("User-Agent","PostmanRuntime/7.19.0");
	RestClient::GetInstance()->addRequestParam("Accept","*/*");
	RestClient::GetInstance()->addRequestParam("Accept-Encoding","gzip, deflate");
	RestClient::GetInstance()->addRequestParam("Host",ip);
	RestClient::GetInstance()->addRequestParam("Connection","Keep-Alive");
	RestClient::GetInstance()->addRequestParam("Cache-Control","no-cache");
}
