﻿#include "src/inc/httpmanager/facerecognizepackage.h"

#include "src/inc/restclient/restclient.h"
#include "src/inc/restclient/xmlreader.h"
#include "src/inc/restclient/xmlwritter.h"

FaceRecognizePackage::FaceRecognizePackage()
{

}

HttpManagerErcode FaceRecognizePackage::getRecoRuleList(const QString &ip, const QString &username, const QString &password, RecoRuleList &recoRuleList)
{
	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/FaceReco/1/RecoRuleList");

	//设置请求头
	setHeader(ip,username,password);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Get,response,httpWaitTime);

	check_status_code(statusCode)

	XmlReader xmlReader;
	xmlReader.setContent(response);

	int recoRuleIndex{0};
	while(true)
	{
		RecoRule recoRule;

		KeyNodeList keys;
		keys={{"RecoRuleList"},{"RecoRule",recoRuleIndex},{"Enable",0}};
		XmlErcode ercode = xmlReader.readNodeValue(keys,recoRule.isEnable);
		if(XmlErcode_None!=ercode)
			break;
		keys={{"RecoRuleList"},{"RecoRule",recoRuleIndex},{"RecognitionRule",1}};
		xmlReader.readNodeValue(keys,recoRule.recognitionRule);
		keys={{"RecoRuleList"},{"RecoRule",recoRuleIndex},{"CompareLimit",2}};
		xmlReader.readNodeValue(keys,recoRule.compareLimit);
		keys={{"RecoRuleList"},{"RecoRule",recoRuleIndex},{"RecoThresholdScheduleList",6},{"RecoThresholdSchedule",0},{"RecoThreshold",0}};
		xmlReader.readNodeValue(keys,recoRule.identificationThreshold);

		int faceGroupIndex{0};
		while(true)
		{
			RecoRuleFaceLibraryNode node;
			keys={{"RecoRuleList"},{"RecoRule",recoRuleIndex},{"FaceGroupList",5},{"FaceGroup",faceGroupIndex},{"GroupID",0}};
			ercode = xmlReader.readNodeValue(keys,node.id);
			if(XmlErcode_None!=ercode)
				break;
			keys={{"RecoRuleList"},{"RecoRule",recoRuleIndex},{"FaceGroupList",5},{"FaceGroup",faceGroupIndex},{"Enable",1}};
			xmlReader.readNodeValue(keys,node.enable);
			keys={{"RecoRuleList"},{"RecoRule",recoRuleIndex},{"FaceGroupList",5},{"FaceGroup",faceGroupIndex},{"GroupName",2}};
			xmlReader.readNodeValue(keys,node.name);

			recoRule.faceLibraryNodes.append(node);
			++faceGroupIndex;
		}

		QString alarmValue;
		keys={{"RecoRuleList"},{"RecoRule",recoRuleIndex},{"Trigger",3},{"AlarmOut",6},{"AlarmOutMask",1}};
		xmlReader.readNodeValue(keys,alarmValue);
		recoRule.alarm1Linkage=alarmValue[0];
		recoRule.alarm2Linkage=alarmValue[1];

		keys={{"RecoRuleList"},{"RecoRule",recoRuleIndex},{"Schedule",4},{"AllDay",0}};
		xmlReader.readNodeValue(keys,recoRule.recordPlan);
		for(int i=0;i<7;++i)
		{
			QString timeBlock;
			keys={{"RecoRuleList"},{"RecoRule",recoRuleIndex},{"Schedule",4},{"TimeBlockList",1},{"TimeBlock_"+QString::number(i),i}};
			xmlReader.readNodeValue(keys,timeBlock);
			recoRule.recordTimeBlocks.append(timeBlock);
		}

		recoRuleList.append(recoRule);

		++recoRuleIndex;
	}

	return HttpErcode_None;
}

HttpManagerErcode FaceRecognizePackage::setRecoRuleList(const QString &ip, const QString &username, const QString &password, const RecoRuleList &recoRuleList)
{
	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/FaceReco/1/RecoRuleList");

	//设置请求头
	setHeader(ip,username,password);

	XmlWritter xmlWritter;
	xmlWritter.addHead("");
	xmlWritter.addNode(XmlNodeType_Root,{"RecoRuleList"});

	RecoRuleList::const_iterator iter;
	for(iter=recoRuleList.begin();iter!=recoRuleList.end();++iter)
	{
		int offset=iter-recoRuleList.begin();

		xmlWritter.addNode(XmlNodeType_Parent,{"RecoRule",offset},{"RecoRuleList"});
		xmlWritter.addNode(XmlNodeType_Child,{"Enable"},{"RecoRule",offset},iter->isEnable);
		xmlWritter.addNode(XmlNodeType_Child,{"RecognitionRule"},{"RecoRule",offset},iter->recognitionRule);
		xmlWritter.addNode(XmlNodeType_Child,{"CompareLimit"},{"RecoRule",offset},iter->compareLimit);
		xmlWritter.addNode(XmlNodeType_Parent,{"Trigger",offset},{"RecoRule",offset});
		xmlWritter.addNode(XmlNodeType_Parent,{"NotifyAMS",offset},{"Trigger",offset});
		xmlWritter.addNode(XmlNodeType_Child,{"Enable",offset},{"NotifyAMS",offset},"false");
		xmlWritter.addNode(XmlNodeType_Parent,{"BeepAlert",offset},{"Trigger",offset});
		xmlWritter.addNode(XmlNodeType_Child,{"Enable",offset},{"BeepAlert",offset},"false");
		xmlWritter.addNode(XmlNodeType_Parent,{"FullScreen",offset},{"Trigger",offset});
		xmlWritter.addNode(XmlNodeType_Child,{"Enable",offset},{"FullScreen",offset},"false");
		xmlWritter.addNode(XmlNodeType_Parent,{"Mail",offset},{"Trigger",offset});
		xmlWritter.addNode(XmlNodeType_Child,{"Enable",offset},{"Mail",offset},"false");
		xmlWritter.addNode(XmlNodeType_Parent,{"Ftp",offset},{"Trigger",offset});
		xmlWritter.addNode(XmlNodeType_Child,{"Enable",offset},{"Ftp",offset},"false");
		xmlWritter.addNode(XmlNodeType_Parent,{"Push",offset},{"Trigger",offset});
		xmlWritter.addNode(XmlNodeType_Child,{"Enable",offset},{"Push",offset},"false");
		xmlWritter.addNode(XmlNodeType_Parent,{"AlarmOut",offset},{"Trigger",offset});
		xmlWritter.addNode(XmlNodeType_Child,{"Enable",offset},{"AlarmOut",offset},"false");
		xmlWritter.addNode(XmlNodeType_Child,{"AlarmOutMask",offset},{"AlarmOut",offset},iter->alarm1Linkage+iter->alarm2Linkage);
		xmlWritter.addNode(XmlNodeType_Parent,{"Snapshot",offset},{"Trigger",offset});
		xmlWritter.addNode(XmlNodeType_Child,{"Enable",offset},{"Snapshot",offset},"false");
		xmlWritter.addNode(XmlNodeType_Child,{"SnapshotMask",offset},{"Snapshot",offset},"1");
		xmlWritter.addNode(XmlNodeType_Parent,{"Record",offset},{"Trigger",offset});
		xmlWritter.addNode(XmlNodeType_Child,{"Enable",offset},{"Record",offset},"false");
		xmlWritter.addNode(XmlNodeType_Child,{"RecordMask",offset},{"Record",offset},"1");
		xmlWritter.addNode(XmlNodeType_Parent,{"PTZ",offset},{"Trigger",offset});
		xmlWritter.addNode(XmlNodeType_Child,{"Enable",offset},{"PTZ",offset},"false");
		xmlWritter.addNode(XmlNodeType_Parent,{"PTZActionList",offset},{"PTZ",offset});
		xmlWritter.addNode(XmlNodeType_Parent,{"PTZAction",offset},{"PTZActionList",offset});
		xmlWritter.addNode(XmlNodeType_Child,{"ChannelID",offset},{"PTZAction",offset},"1");
		xmlWritter.addNode(XmlNodeType_Child,{"ActionName",offset},{"PTZAction",offset},"none");
		xmlWritter.addNode(XmlNodeType_Child,{"ActionNum",offset},{"PTZAction",offset},"0");
		xmlWritter.addNode(XmlNodeType_Parent,{"Schedule",offset},{"RecoRule",offset});
		xmlWritter.addNode(XmlNodeType_Child,{"AllDay"},{"Schedule",offset},iter->recordPlan);
		xmlWritter.addNode(XmlNodeType_Parent,{"TimeBlockList",offset},{"Schedule",offset});
		QVector<QString>::const_iterator timeBlockIter;
		for(timeBlockIter=iter->recordTimeBlocks.begin();timeBlockIter!=iter->recordTimeBlocks.end();++timeBlockIter)
		{
			int offset = timeBlockIter-iter->recordTimeBlocks.begin();

			xmlWritter.addNode(XmlNodeType_Child,{"TimeBlock_"+QString::number(offset)},{"TimeBlockList",offset},*timeBlockIter);
		}
		xmlWritter.addNode(XmlNodeType_Parent,{"FaceGroupList",offset},{"RecoRule",offset});
		QVector<RecoRuleFaceLibraryNode>::const_iterator faceLibraryIter;
		for(faceLibraryIter=iter->faceLibraryNodes.begin();faceLibraryIter!=iter->faceLibraryNodes.end();++faceLibraryIter)
		{
			int offset=faceLibraryIter-iter->faceLibraryNodes.begin();

			xmlWritter.addNode(XmlNodeType_Parent,{"FaceGroup",offset},{"FaceGroupList",offset});
			xmlWritter.addNode(XmlNodeType_Child,{"GroupID"},{"FaceGroup",offset},faceLibraryIter->id);
			xmlWritter.addNode(XmlNodeType_Child,{"Enable"},{"FaceGroup",offset},faceLibraryIter->enable);
			xmlWritter.addNode(XmlNodeType_Child,{"GroupName"},{"FaceGroup",offset},faceLibraryIter->name);
		}
		xmlWritter.addNode(XmlNodeType_Parent,{"RecoThresholdScheduleList",offset},{"RecoRule",offset});
		xmlWritter.addNode(XmlNodeType_Parent,{"RecoThresholdSchedule",offset},{"RecoThresholdScheduleList",offset});
		xmlWritter.addNode(XmlNodeType_Child,{"RecoThreshold"},{"RecoThresholdSchedule",offset},iter->identificationThreshold);
		xmlWritter.addNode(XmlNodeType_Child,{"StartTime"},{"RecoThresholdSchedule",offset},"000000");
		xmlWritter.addNode(XmlNodeType_Child,{"StopTime"},{"RecoThresholdSchedule",offset},"000000");
	}

	QByteArray body;
	xmlWritter.exportXml(body);
	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Put,response,httpWaitTime);

	check_status_code(statusCode)

	return analysisResponseStatusCode(response);
}

void FaceRecognizePackage::setHeader(const QString &ip,const QString &username,const QString &password)
{
	//设置请求头
	//发送HTTP请求的时候需要进行用户名密码验证：需要发送请求头"Authorization"，值为："Basic "+"用户名:密码"的Base64加密值
	RestClient::GetInstance()->addRequestParam("Authorization",QString("Basic ")+QString(username+":"+password).toLocal8Bit().toBase64());
	RestClient::GetInstance()->addRequestParam("User-Agent","PostmanRuntime/7.19.0");
	RestClient::GetInstance()->addRequestParam("Accept","*/*");
	RestClient::GetInstance()->addRequestParam("Accept-Encoding","gzip, deflate");
	RestClient::GetInstance()->addRequestParam("Host",ip);
	RestClient::GetInstance()->addRequestParam("Connection","Keep-Alive");
	RestClient::GetInstance()->addRequestParam("Cache-Control","no-cache");
}
