﻿#include "src/inc/httpmanager/recognizerecordpackage.h"

#include "src/inc/restclient/restclient.h"
#include "src/inc/restclient/xmlreader.h"
#include "src/inc/restclient/xmlwritter.h"

RecognizeRecordPackage::RecognizeRecordPackage()
{

}

HttpManagerErcode RecognizeRecordPackage::getRecordRecordCount(const QString &ip, const QString &username, const QString &password, QString &recordCount,
												  const QString &libraryID,const QString &peopleName,const QString &startTime,const QString &endTime,const QString &recognitionResultType,const QString &ruleID)
{
	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/FaceRecognition/QueryRecordCount");

	//设置请求头
	setHeader(ip,username,password);
	RestClient::GetInstance()->addRequestParam("Content-Type","application/x-www-form-urlencoded; charset=UTF-8");
	RestClient::GetInstance()->addRequestParam("Accept-Encoding","gzip, deflate");
	RestClient::GetInstance()->addRequestParam("Accept-Language","en-US,en;q=0.9,zh-CN;q=0.8,zh;q=0.7");

	XmlWritter xmlWritter;
	xmlWritter.addHead("");
	xmlWritter.addNode(XmlNodeType_Root,{"FaceRecognitionFilter"});
	xmlWritter.addNode(XmlNodeType_Child,{"GroupID"},{"FaceRecognitionFilter"},libraryID);
	xmlWritter.addNode(XmlNodeType_Child,{"PeopleName"},{"FaceRecognitionFilter"},peopleName);
	xmlWritter.addNode(XmlNodeType_Child,{"StartTime"},{"FaceRecognitionFilter"},startTime);
	xmlWritter.addNode(XmlNodeType_Child,{"StopTime"},{"FaceRecognitionFilter"},endTime);
	xmlWritter.addNode(XmlNodeType_Child,{"RecognitionResultType"},{"FaceRecognitionFilter"},recognitionResultType);
	xmlWritter.addNode(XmlNodeType_Child,{"RuleID"},{"FaceRecognitionFilter"},ruleID);

	QByteArray body;
	xmlWritter.exportXml(body);
	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Put,response,httpWaitTime);

	check_status_code(statusCode)

	XmlReader xmlReader;
	xmlReader.setContent(response);
	KeyNodeList keys={{"FaceRecognition"},{"RecognitionRecordCount"}};
	XmlErcode ercode = xmlReader.readNodeValue(keys,recordCount);

	return HttpErcode_None;
}

HttpManagerErcode RecognizeRecordPackage::getRecognizeRecord(const QString &ip, const QString &username, const QString &password, const QString &pageSize, const QString &pageIndex, RecognitionRecordList &recordList,
												const QString &libraryID,const QString &peopleName,const QString &startTime,const QString &endTime,const QString &recognitionResultType,const QString &ruleID)
{
	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/FaceRecognition/QueryRecordList");

	//设置请求头
	setHeader(ip,username,password);

	XmlWritter xmlWritter;
	xmlWritter.addHead("");
	xmlWritter.addNode(XmlNodeType_Root,{"FaceRecognitionFilter"});
	xmlWritter.addNode(XmlNodeType_Child,{"GroupID"},{"FaceRecognitionFilter"},libraryID);
	xmlWritter.addNode(XmlNodeType_Child,{"PeopleName"},{"FaceRecognitionFilter"},peopleName.toUtf8());
	xmlWritter.addNode(XmlNodeType_Child,{"StartTime"},{"FaceRecognitionFilter"},startTime);
	xmlWritter.addNode(XmlNodeType_Child,{"StopTime"},{"FaceRecognitionFilter"},endTime);
	xmlWritter.addNode(XmlNodeType_Child,{"Pagesize"},{"FaceRecognitionFilter"},pageSize);
	xmlWritter.addNode(XmlNodeType_Child,{"Pagenum"},{"FaceRecognitionFilter"},pageIndex);
	xmlWritter.addNode(XmlNodeType_Child,{"IsHasPath"},{"FaceRecognitionFilter"});
	xmlWritter.addNode(XmlNodeType_Child,{"RecognitionResultType"},{"FaceRecognitionFilter"},recognitionResultType);
	xmlWritter.addNode(XmlNodeType_Child,{"RuleID"},{"FaceRecognitionFilter"},ruleID);

	QByteArray body;
	xmlWritter.exportXml(body);
	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Put,response,httpWaitTime);

	check_status_code(statusCode)

	XmlReader xmlReader;
	xmlReader.setContent(response);
	KeyNodeList keys;
	int recognitionIndex{0};
	while(true)
	{
		RecognitionRecord record;

		keys={{"RecognitionRecordList"},{"RecognitionRecord",recognitionIndex},{"LogID",0}};
		XmlErcode ercode = xmlReader.readNodeValue(keys,record.logID);
		if(XmlErcode_None != ercode)
			break;
		keys={{"RecognitionRecordList"},{"RecognitionRecord",recognitionIndex},{"PersonID",1}};
		xmlReader.readNodeValue(keys,record.personID);
		keys={{"RecognitionRecordList"},{"RecognitionRecord",recognitionIndex},{"GroupName",2}};
		xmlReader.readNodeValue(keys,record.libraryName);
		keys={{"RecognitionRecordList"},{"RecognitionRecord",recognitionIndex},{"Time",3}};
		xmlReader.readNodeValue(keys,record.time);
		keys={{"RecognitionRecordList"},{"RecognitionRecord",recognitionIndex},{"Pass",4}};
		xmlReader.readNodeValue(keys,record.isPass);
		keys={{"RecognitionRecordList"},{"RecognitionRecord",recognitionIndex},{"SnapshotPath",5}};
		xmlReader.readNodeValue(keys,record.snapshotPath);
		keys={{"RecognitionRecordList"},{"RecognitionRecord",recognitionIndex},{"Similar",6}};
		xmlReader.readNodeValue(keys,record.similar);
		keys={{"RecognitionRecordList"},{"RecognitionRecord",recognitionIndex},{"RuleID",7}};
		xmlReader.readNodeValue(keys,record.ruleID);
		keys={{"RecognitionRecordList"},{"RecognitionRecord",recognitionIndex},{"PeopleName",8}};
		xmlReader.readNodeValue(keys,record.peopleName);

		recordList.append(record);
		++recognitionIndex;
	}

	return HttpErcode_None;
}

HttpManagerErcode RecognizeRecordPackage::getRecognizeRecordDetail(const QString &ip, const QString &username, const QString &password, const QString &logID, RecognitionRecordDetail &detail)
{
	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/FaceRecognition/DetailByRecognitionRecord");

	//设置请求头
	setHeader(ip,username,password);

	XmlWritter xmlWritter;
	xmlWritter.addHead("");
	xmlWritter.addNode(XmlNodeType_Root,{"RecognitionInfo"});
	xmlWritter.addNode(XmlNodeType_Child,{"LogID"},{"RecognitionInfo"},logID);
	xmlWritter.addNode(XmlNodeType_Child,{"IsPicture"},{"RecognitionInfo"},"false");

	QByteArray body;
	xmlWritter.exportXml(body);
	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Put,response,httpWaitTime);

	check_status_code(statusCode)

	XmlReader xmlReader;
	xmlReader.setContent(response);
	xmlReader.readNodeValue({{"RecognitionInfo"},{"Name",0}},detail.name);
	xmlReader.readNodeValue({{"RecognitionInfo"},{"Sex",1}},detail.sex);
	xmlReader.readNodeValue({{"RecognitionInfo"},{"CertificateType",2}},detail.certificateType);
	xmlReader.readNodeValue({{"RecognitionInfo"},{"CertificateID",3}},detail.certificateID);
	xmlReader.readNodeValue({{"RecognitionInfo"},{"PersonID",4}},detail.personID);
	xmlReader.readNodeValue({{"RecognitionInfo"},{"Country",5}},detail.country);
	xmlReader.readNodeValue({{"RecognitionInfo"},{"Address",6}},detail.address);
	xmlReader.readNodeValue({{"RecognitionInfo"},{"Birthday",7}},detail.birthday);
	xmlReader.readNodeValue({{"RecognitionInfo"},{"RecoResult",8}},detail.recoResult);

	return HttpErcode_None;
}

HttpManagerErcode RecognizeRecordPackage::getSnapshot(const QString &ip, const QString &username, const QString &password, const QString &snapshotPath, QByteArray &image)
{
	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/FaceRecognition/SnapshotByRecognitionRecord");

	//设置请求头
	setHeader(ip,username,password);

	XmlWritter xmlWritter;
	xmlWritter.addHead("");
	xmlWritter.addNode(XmlNodeType_Root,{"RecognitionInfo"});
	xmlWritter.addNode(XmlNodeType_Child,{"SnapshotPath"},{"RecognitionInfo"},snapshotPath);

	QByteArray body;
	xmlWritter.exportXml(body);
	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Post,response,httpWaitTime);

	check_status_code(statusCode)

	image=response;

	return HttpErcode_None;
}

HttpManagerErcode RecognizeRecordPackage::getComparedImage(const QString &ip, const QString &username, const QString &password, const QString &logID, QByteArray &image)
{
	RestClient::GetInstance()->clear();

	//获取设备能力
	//设置URL
	RestClient::GetInstance()->setUrl("http://"+ip+"/FaceRecognition/RecoResultImageByRecognitionRecord");

	//设置请求头
	setHeader(ip,username,password);

	XmlWritter xmlWritter;
	xmlWritter.addHead("");
	xmlWritter.addNode(XmlNodeType_Root,{"RecognitionInfo"});
	xmlWritter.addNode(XmlNodeType_Child,{"LogID"},{"RecognitionInfo"},logID);

	QByteArray body;
	xmlWritter.exportXml(body);
	RestClient::GetInstance()->setRequestBody(body);

	//发送请求
	QByteArray response;
	int statusCode=RestClient::GetInstance()->sendRequest(RequestType_Post,response,httpWaitTime);

	check_status_code(statusCode)

	image=response;

	return HttpErcode_None;
}

void RecognizeRecordPackage::setHeader(const QString &ip,const QString &username,const QString &password)
{
	//设置请求头
	//发送HTTP请求的时候需要进行用户名密码验证：需要发送请求头"Authorization"，值为："Basic "+"用户名:密码"的Base64加密值
	RestClient::GetInstance()->addRequestParam("Authorization",QString("Basic ")+QString(username+":"+password).toLocal8Bit().toBase64());
	RestClient::GetInstance()->addRequestParam("User-Agent","PostmanRuntime/7.19.0");
	RestClient::GetInstance()->addRequestParam("Accept","*/*");
	RestClient::GetInstance()->addRequestParam("Accept-Encoding","gzip, deflate");
	RestClient::GetInstance()->addRequestParam("Host",ip);
	RestClient::GetInstance()->addRequestParam("Connection","Keep-Alive");
	RestClient::GetInstance()->addRequestParam("Cache-Control","no-cache");
}
