﻿#include "src/inc/package/videopackage.h"

#include "lib/sdk/VideoNetClient/include/VideoNetClient.h"
#ifdef _WIN32
#pragma comment(lib,"lib/sdk/VideoNetClient/lib/x86/VideoNetClient.lib")
#else
#pragma comment(lib,"lib/sdk/VideoNetClient/lib/x64/VideoNetClient.lib")
#endif

#include "lib/sdk/VideoPlay/include/VideoPlay.h"
#pragma comment(lib,"lib/sdk/VideoPlay/lib/VideoPlay.lib")

#include <Windows.h>
#pragma comment(lib,"Gdi32.lib")

#include <QWidget>
#include <QPixmap>
#include <QDateTime>
#include <QTimeZone>

#include "src/inc/usermanager/userinfo.h"
#include "src/inc/gui/snapshot/snapshotwidget.h"

namespace vncp
{

VideoPackage::VideoPackage()
{
	qRegisterMetaType<QVector<DeviceInfo>>("QVector<DeviceInfo>");

	int ercode{0};

	//启动VideoNetClient服务
	ercode=VideoNetClient_Start();
	check_ercode_no_return(ercode,"VideoNetClient_Start failed!")
}

VideoPackage::~VideoPackage()
{
	int ercode{0};

	//停止VideoNetClient服务
	ercode=VideoNetClient_Stop();
	check_ercode_no_return(ercode,"VideoNetClient_Stop failed!")
}

void VideoPackage::setCallback(VideoNetClientCallback *callback)
{
	m_callback=callback;
}

void CALLBACK deviceProbeCallback(LPDeviceProbeConfigV2 cConfig)
{
	//解析设备信息
	DeviceType deviceType=DeviceAnalysiser::GetInstance()->getDeviceType(cConfig);
	QString deviceName=DeviceAnalysiser::GetInstance()->getDeviceName(cConfig);
	QString mac=DeviceAnalysiser::GetInstance()->getMACAddress(cConfig);
	QString ip=DeviceAnalysiser::GetInstance()->getIPAddress(cConfig);
	QString romVersion=DeviceAnalysiser::GetInstance()->getROMVersion(cConfig);
	QString bootTime=DeviceAnalysiser::GetInstance()->getBootTime(cConfig);

	//将设备添加到设备管理类中
	DeviceManager::GetInstance()->addDevice(deviceType,deviceName,mac,ip,romVersion,bootTime);
}

int VideoPackage::startDeviceDetectService(const int &beginPort, const int &endPort, const int &updateTime)
{
	if(nullptr!=m_deviceUpdater)
		return -1;

	int ercode{0};

	//设备探测
	DeviceProbeParameter deviceProbeParameter;
	deviceProbeParameter.m_wBeginPort = static_cast<unsigned short>(beginPort);
	deviceProbeParameter.m_wEndPort = static_cast<unsigned short>(endPort);
	ercode=VideoNetClient_DeviceProbeStartV2(&deviceProbeParameter,deviceProbeCallback);
	check_ercode(ercode,"VideoNetClient_DeviceProbeStartV2 failed!")

	const auto expression=[&,beginPort,endPort,updateTime]()->void
	{
		while(true==m_deviceUpdateFlag)
		{
			DeviceManager::GetInstance()->clearDevices();

			//刷新设备探测
			DeviceProbeParameter deviceProbeParameter;
			deviceProbeParameter.m_wBeginPort = static_cast<unsigned short>(beginPort);
			deviceProbeParameter.m_wEndPort = static_cast<unsigned short>(endPort);
			ercode=VideoNetClient_DeviceProbeRefreshV2(&deviceProbeParameter);
			check_ercode_no_return(ercode,"VideoNetClient_DeviceProbeRefreshV2")

			//等待1000毫秒，通知界面层获取最新探测到的设备列表
			asyncWait(1000,[&](){
				QVector<DeviceInfo> devices;
				DeviceManager::GetInstance()->getDevices(devices);
				emit signal_deviceUpdate(devices);
			});

			std::this_thread::sleep_for(std::chrono::milliseconds(updateTime));
		}
	};
	m_deviceUpdateFlag=true;
	m_deviceUpdater=new std::thread(expression);

	return ercode;
}

int VideoPackage::stopDeviceDetectService()
{
	if(nullptr==m_deviceUpdater)
		return -1;

	int ercode{0};

	ercode = VideoNetClient_DeviceProbeStopV2();
	check_ercode(ercode,"VideoNetClient_DeviceProbeStopV2 failed")

	m_deviceUpdateFlag=false;
	m_deviceUpdater->join();
	delete m_deviceUpdater;
	m_deviceUpdater=nullptr;

	return ercode;
}

int VideoPackage::loginDevice(const QString &mac,const QString &ip, const int &port, const QString &username, const QString &password, int &userID)
{
	int ercode{0};

//	QString ip=DeviceManager::GetInstance()->getIPbyMAC(mac);
//	if(true==ip.isEmpty())
//	{
//		error_output("Device is not exist!")
//		return -1;
//	}

	if(AFErcode_None == UserManager::GetInstance()->getDeviceUserID(mac,userID))
		return ercode;

	//创建登录参数
	UserLoginPara loginPara;
	strncpy(loginPara.sServerIP,ip.toStdString().c_str(),sizeof(loginPara.sServerIP));
	loginPara.dwCommandPort=static_cast<DWORD>(port);
	strncpy(loginPara.sUName,username.toStdString().c_str(),sizeof(loginPara.sUName));
	strncpy(loginPara.sUPass,password.toStdString().c_str(),sizeof(loginPara.sUPass));

	//登录设备
	HUSER hUser{INVALID_HANDLE_VALUE};
	ercode = VideoNetClient_UserLogin(&hUser,&loginPara);
	check_ercode(ercode,"VideoNetClient_UserLogin failed")

	static int s_userID{0};

	UserInfo userInfo;
	userInfo.hUser=hUser;

	ercode=UserManager::GetInstance()->addUser(s_userID,hUser,mac,ip);
	check_ercode(ercode,"Add user failed!")

	userID=s_userID++;

	return ercode;
}

int VideoPackage::logoutDevice(const int &userID)
{
	int ercode{0};

	HUSER hUser{INVALID_HANDLE_VALUE};
	ercode=UserManager::GetInstance()->getHUser(userID,hUser);
	check_ercode(ercode,"Get hUser failed!")

	ercode = VideoNetClient_UserLogout(hUser);
	check_ercode(ercode,"VideoNetClient_UserLogout failed!")

	ercode=UserManager::GetInstance()->removeUser(userID);
	check_ercode(ercode,"UserID not exist!")

	return ercode;
}

int CALLBACK CB_UserEventEx(IN HUSER hUser, IN DWORD dwEventType, IN DWORD dwSubEventType, IN DWORD dwParam1, IN DWORD dwParam2, IN DWORD dwParam3, IN const LPBuffer cBuffer, IN DWORD dwUserData)
{
	Q_UNUSED(dwParam1)
	Q_UNUSED(dwParam2)
	Q_UNUSED(dwParam3)
	Q_UNUSED(dwUserData)

	if(USEREVENT_AI_EVENT_PUSH == dwEventType)
	{
		EventFaceRecognition *event = reinterpret_cast<EventFaceRecognition *>(cBuffer->pBuffer);

		switch(dwSubEventType)
		{
		case EVENT_AI_PEOPLE_DETECT:						/*!< 人体检测事件                          	*/
		{
			unsigned long offsetTime = event->dwTimeSec-8*3600;
			QString eventInfo = "People Detect, "+QDateTime::fromTime_t(offsetTime).toString("yyyy-MM-dd hh:mm:ss");
			SnapshotWidget::GetInstance()->addBasicPeopleDetectEvent(eventInfo);
		}
			break;
		case EVENT_AI_FACE_DETECT_LITE:							/*!< 基础版本人脸检测事件					*/
		{
			unsigned long offsetTime = event->dwTimeSec-8*3600;
			QString eventInfo = "People Detect, "+QDateTime::fromTime_t(offsetTime).toString("yyyy-MM-dd hh:mm:ss");
			SnapshotWidget::GetInstance()->addBasicFaceDetectEvent(eventInfo);
		}
			break;
		case EVENT_AI_FACE_DETECT:								/*!< 高阶版本人脸检测事件（EventFaceRecognition）*/
		{
//			qDebug()<<"face detect";

			int imageDataLength{0};
			if(0==event->dwType)
				imageDataLength = event->cFace.cCaptureInfo.cFaceImage.dwLength;
			else
				imageDataLength = event->cFace.cMatchInfo.cImageInfo.dwLength;

			QString faceTemperature;
			if(0 == event->cTemperatureUnit)
				faceTemperature = QString::number(event->fFaceTemperature)+QStringLiteral("℃");
			else
				faceTemperature = QString::number(event->fFaceTemperature)+QStringLiteral("℉");

			unsigned long offsetTime = event->dwTimeSec-8*3600;
			SnapshotWidget::GetInstance()->addAdvancedFaceDetectEvent(QDateTime::fromSecsSinceEpoch(offsetTime).toString("yyyy-MM-dd hh:mm:ss"),
																	  faceTemperature,
																	  QString::number(event->cFace.cCaptureInfo.llFaceTraceId),
																	  QByteArray(reinterpret_cast<char *>(cBuffer->pBuffer)+sizeof(EventFaceRecognition),imageDataLength));
		}
			break;
		case EVENT_AI_FACE_RECOGNITION:							/*!< 人脸识别事件（EventFaceRecognition）   */
		{
//			qDebug()<<"face recoginze";
			EventFaceRecognition *event = reinterpret_cast<EventFaceRecognition *>(cBuffer->pBuffer);
			int imageDataLength{0};
			QString name;
			QString similar;
			if(0==event->dwType)
				imageDataLength = event->cFace.cCaptureInfo.cFaceImage.dwLength;
			else
			{
				imageDataLength = event->cFace.cMatchInfo.cImageInfo.dwLength;
				name=QString::fromUtf8(reinterpret_cast<char *>(event->cFace.cMatchInfo.cFaceName));
				similar=QString::number(event->cFace.cMatchInfo.fMatch);
			}

			QString faceTemperature;
			if(0 == event->cTemperatureUnit)
				faceTemperature = QString::number(event->fFaceTemperature)+QStringLiteral("℃");
			else
				faceTemperature = QString::number(event->fFaceTemperature)+QStringLiteral("℉");

			unsigned long offsetTime = event->dwTimeSec-8*3600;
			SnapshotWidget::GetInstance()->addAdvancedFaceRecognizeEvent(QDateTime::fromSecsSinceEpoch(offsetTime).toString("yyyy-MM-dd hh:mm:ss"),
													name,
													similar,
													faceTemperature,
													QByteArray(reinterpret_cast<char *>(cBuffer->pBuffer)+sizeof(EventFaceRecognition),imageDataLength));
		}
			break;
		}
	}

	return 0;
}

int VideoPackage::subscribeDeviceEvent(const int &userID, const DeviceEventType &deviceEventType)
{
	int ercode{0};

	HUSER hUser{INVALID_HANDLE_VALUE};
	ercode=UserManager::GetInstance()->getHUser(userID,hUser);
	check_ercode(ercode,"Get hUser failed!")

	ercode=VideoNetClient_SubscribeEventEx(hUser,deviceEventType,CB_UserEventEx,0);
	check_ercode(ercode,"Subscribe event failed!")

	return ercode;
}

int VideoPackage::subscribeFaceEvent(const int &userID)
{
	int ercode{0};

	HUSER hUser{INVALID_HANDLE_VALUE};
	ercode=UserManager::GetInstance()->getHUser(userID,hUser);
	check_ercode(ercode,"Get hUser failed!")

	ercode=VideoNetClient_SubscribeEventEx(hUser,USEREVENT_AI_EVENT_PUSH|EVENT_AI_PEOPLE_DETECT|EVENT_AI_FACE_DETECT_LITE|EVENT_AI_FACE_DETECT|EVENT_AI_FACE_RECOGNITION,CB_UserEventEx,0);
	check_ercode(ercode,"Subscribe face event failed!")

	return ercode;
}

int VideoPackage::unsubscribeFaceEvent(const int &userID)
{
	int ercode{0};

	HUSER hUser{INVALID_HANDLE_VALUE};
	ercode=UserManager::GetInstance()->getHUser(userID,hUser);
	check_ercode(ercode,"Get hUser failed!")

	ercode=VideoNetClient_UnSubscribeEvent(hUser,USEREVENT_AI_EVENT_PUSH|EVENT_AI_PEOPLE_DETECT|EVENT_AI_FACE_DETECT_LITE|EVENT_AI_FACE_DETECT|EVENT_AI_FACE_RECOGNITION);
	check_ercode(ercode,"Unsubscribe face event failed!")

	return ercode;
}

int VideoPackage::subscribeBasicPeopleDetectEvent(const int &userID)
{
	int ercode{0};

	HUSER hUser{INVALID_HANDLE_VALUE};
	ercode=UserManager::GetInstance()->getHUser(userID,hUser);
	check_ercode(ercode,"Get hUser failed!")

	ercode=VideoNetClient_SubscribeEventEx(hUser,USEREVENT_AI_EVENT_PUSH|EVENT_AI_PEOPLE_DETECT,CB_UserEventEx,0);
	check_ercode(ercode,"Subscribe event failed!")

	return ercode;
}

int VideoPackage::unSubscribeBasePeopleDetectEvent(const int &userID)
{
	int ercode{0};

	HUSER hUser{INVALID_HANDLE_VALUE};
	ercode=UserManager::GetInstance()->getHUser(userID,hUser);
	check_ercode(ercode,"Get hUser failed!")

	ercode=VideoNetClient_UnSubscribeEvent(hUser,USEREVENT_AI_EVENT_PUSH|EVENT_AI_PEOPLE_DETECT);
	check_ercode(ercode,"Unsubscribe event failed!")

	return ercode;
}

int VideoPackage::subscribeBasicFaceDetectEvent(const int &userID)
{
	int ercode{0};

	HUSER hUser{INVALID_HANDLE_VALUE};
	ercode=UserManager::GetInstance()->getHUser(userID,hUser);
	check_ercode(ercode,"Get hUser failed!")

	ercode=VideoNetClient_SubscribeEventEx(hUser,USEREVENT_AI_EVENT_PUSH|EVENT_AI_FACE_DETECT_LITE,CB_UserEventEx,0);
	check_ercode(ercode,"Subscribe event failed!")

	return ercode;
}

int VideoPackage::unSubscribeBasicFaceDetectEvent(const int &userID)
{
	int ercode{0};

	HUSER hUser{INVALID_HANDLE_VALUE};
	ercode=UserManager::GetInstance()->getHUser(userID,hUser);
	check_ercode(ercode,"Get hUser failed!")

	ercode=VideoNetClient_UnSubscribeEvent(hUser,USEREVENT_AI_EVENT_PUSH|EVENT_AI_FACE_DETECT_LITE);
	check_ercode(ercode,"Unsubscribe event failed!")

	return ercode;
}

int VideoPackage::subscribeAdvancedFaceDetectEvent(const int &userID)
{
	int ercode{0};

	HUSER hUser{INVALID_HANDLE_VALUE};
	ercode=UserManager::GetInstance()->getHUser(userID,hUser);
	check_ercode(ercode,"Get hUser failed!")

	ercode=VideoNetClient_SubscribeEventEx(hUser,USEREVENT_AI_EVENT_PUSH|EVENT_AI_FACE_DETECT,CB_UserEventEx,0);
	check_ercode(ercode,"Subscribe event failed!")

	return ercode;
}

int VideoPackage::unSubscribeAdvancedFaceDetectEvent(const int &userID)
{
	int ercode{0};

	HUSER hUser{INVALID_HANDLE_VALUE};
	ercode=UserManager::GetInstance()->getHUser(userID,hUser);
	check_ercode(ercode,"Get hUser failed!")

	ercode=VideoNetClient_UnSubscribeEvent(hUser,USEREVENT_AI_EVENT_PUSH|EVENT_AI_FACE_DETECT);
	check_ercode(ercode,"Unsubscribe event failed!")

	return ercode;
}

int VideoPackage::subscribeAdvancedFaceRecognizeEvent(const int userID)
{
	int ercode{0};

	HUSER hUser{INVALID_HANDLE_VALUE};
	ercode=UserManager::GetInstance()->getHUser(userID,hUser);
	check_ercode(ercode,"Get hUser failed!")

	ercode=VideoNetClient_SubscribeEventEx(hUser,USEREVENT_AI_EVENT_PUSH|EVENT_AI_FACE_RECOGNITION,CB_UserEventEx,0);
	check_ercode(ercode,"Subscribe event failed!")

	return ercode;
}

int VideoPackage::unSubscribeAdvancedFaceRecognizeEvent(const int userID)
{
	int ercode{0};

	HUSER hUser{INVALID_HANDLE_VALUE};
	ercode=UserManager::GetInstance()->getHUser(userID,hUser);
	check_ercode(ercode,"Get hUser failed!")

	ercode=VideoNetClient_UnSubscribeEvent(hUser,USEREVENT_AI_EVENT_PUSH|EVENT_AI_FACE_RECOGNITION);
	check_ercode(ercode,"Unsubscribe event failed!")

	return ercode;
}

inline bool isControlFrame(const DWORD &frameType)
{
	return ((0xFFFF0000&(frameType))>0x00000000);
}

int CALLBACK streamCallback(IN HSTREAM hStream, IN const StreamMediaFrame *cFrame, IN DWORD dwUserData)
{
	Q_UNUSED(hStream)

	if(true==isControlFrame(cFrame->dwFrameType));
	else
	{
		if(0==cFrame->cFrameBuffer.dwBufLen)
			return 1;

		VideoPlay_InputData(static_cast<LONG>(dwUserData),cFrame->cFrameBuffer.pBuffer,cFrame->cFrameBuffer.dwBufLen);
	}

	return 0;
}

void CALLBACK DrawFunEx(long nPort, HDC hdc, LONG nUser)
{
	QVector<QRect> rects;
	UserManager::GetInstance()->getRectsByPort(nPort,rects);

	HPEN hPen[3];
	hPen[0] = CreatePen(PS_SOLID,2,RGB(0xFF,0,0));
	hPen[1] = CreatePen(PS_SOLID,2,RGB(0,0xFF,0));
	hPen[2] = CreatePen(PS_SOLID,2,RGB(0,0,0xFF));

	QVector<QRect>::iterator iter;
	for(iter = rects.begin();iter != rects.end();++iter)
	{
		int offset = iter-rects.begin();

		HPEN oldPen = (HPEN)SelectObject(hdc,hPen[offset]);
		//将矩形使用四根线条绘制出来

		//上
		MoveToEx(hdc,iter->topLeft().x(),iter->topLeft().y(),NULL);
		LineTo(hdc,iter->topRight().x(),iter->topRight().y());

		//下
		MoveToEx(hdc,iter->bottomLeft().x(),iter->bottomLeft().y(),NULL);
		LineTo(hdc,iter->bottomRight().x(),iter->bottomRight().y());

		//左
		MoveToEx(hdc,iter->topLeft().x(),iter->topLeft().y(),NULL);
		LineTo(hdc,iter->bottomLeft().x(),iter->bottomLeft().y());

		//右
		MoveToEx(hdc,iter->topRight().x(),iter->topRight().y(),NULL);
		LineTo(hdc,iter->bottomRight().x(),iter->bottomRight().y());

		SelectObject(hdc,oldPen);
	}

	DeleteObject(hPen[0]);
	DeleteObject(hPen[1]);
	DeleteObject(hPen[2]);
}

int VideoPackage::startRealStream(const int &userID, const int &channelID, QWidget *widget)
{
	int ercode{0};

	HUSER hUser;
	ercode = UserManager::GetInstance()->getHUser(userID,hUser);
	check_ercode(ercode,"Device not login!")

	//锁定通道
	long port{0};
	ercode = VideoPlay_LockNewPort(&port);
	check_bool(ercode,"VideoPlay_LockNewPort failed!")
	//设置实时流模式
	ercode = VideoPlay_SetStreamOpenMode(port,STREAME_REALTIME);
	check_bool(ercode,"VideoPlay_SetStreamOpenMode failed!")
	//打开实时流
	const int videoPlayBufferSize{2097152};		//2MByte=2*1024*1024=2097152(Byte);
	unsigned char bytHeader[] = {0x00, 0x00, 0x01, 0xB6, 0x48, 0x32, 0x36, 0x34, 0x00, 0x00, 0x00, 0x13, 0x01, 0x01, 0x00, 0x00, 0x05, 0xD0, 0x02};
	ercode = VideoPlay_OpenStream(port,bytHeader,sizeof(bytHeader),videoPlayBufferSize);
	check_bool(ercode,"VideoPlay_OpenStream")
	//启动视频解码播放
	ercode = VideoPlay_Play(port,reinterpret_cast<HWND>(widget->winId()));
	check_bool(ercode,"VideoPlay_Play")
	//启动音频流播放
	ercode = VideoPlay_PlaySound(port);
	check_bool(ercode,"VideoPlay_PlaySound failed!")

	//注册绘制回调函数，用于在视频流上绘制
	ercode = VideoPlay_RigisterDrawFun_Ex(port,DrawFunEx,0);
	check_bool(ercode,"VideoPlay_RigisterDrawFun_Ex failed!")

	//构建实时流连接参数
	RealStreamPara realStreamParam{};
	realStreamParam.dwChannel = static_cast<DWORD>(channelID);
	realStreamParam.eMediaType = eMainVideoAndSound;
	realStreamParam.eTransferMode = eGeneralTCP;
	realStreamParam.wMediaPort = 6066;

	//连接实时流
	HSTREAM hStream;
	ercode = VideoNetClient_RealStreamConnect(&hStream,hUser,&realStreamParam);
	check_ercode(ercode,"RealStream connect failed!")

	//设置实时流回调
	ercode = VideoNetClient_StreamMediaCB(hStream,streamCallback,static_cast<DWORD>(port));
	check_ercode(ercode,"RealStream callback set failed!")

	//启动实时流
	ercode = VideoNetClient_StreamMediaControl(hStream,eTaskStart);
	check_ercode(ercode,"RealStream start failed!")

	UserManager::GetInstance()->setRealStreamHandle(userID,channelID,hStream);
	UserManager::GetInstance()->setRealStreamPort(userID,channelID,port);

	return AFErcode_None;
}

int VideoPackage::stopRealStream(const int &userID, const int &channelID)
{
	int ercode{0};

	HUSER hUser;
	ercode = UserManager::GetInstance()->getHUser(userID,hUser);
	check_ercode(ercode,"UserID not exist!")

	HSTREAM hStream;
	ercode=UserManager::GetInstance()->getRealStreamHandle(userID,channelID,hStream);
	check_ercode(ercode,"ChannelID not exist")

	//停止实时流
	ercode = VideoNetClient_StreamMediaControl(hStream,eTaskStop);
	check_ercode(ercode,"RealStream 0 stop failed!")

	//断开实时流
	ercode = VideoNetClient_RealStreamDisconnect(hStream);
	check_ercode(ercode,"RealStream 0 disconnect failed!")

	//停止播放音频流
	ercode = VideoPlay_StopSound();
	check_bool(ercode,"VideoPlay_PlaySound failed!")

	int port;
	UserManager::GetInstance()->getRealStreamPort(userID,channelID,port);

	ercode = VideoPlay_ResetSourceBuffer(port);
	check_bool(ercode,"VideoPlay_ResetSourceBuffer failed!")
	ercode = VideoPlay_CloseStream(port);
	check_bool(ercode,"VideoPlay_CloseStream")
	ercode = VideoPlay_UnlockPort(port);
	check_bool(ercode,"VideoPlay_UnlockPort")

	return AFErcode_None;
}

int VideoPackage::addRectOnStream(const int &userID,const int &channelID,const QVector<QRect> &rects)
{
	UserManager::GetInstance()->clearRectsOnStream(userID,channelID);

	for(const QRect &rect:rects)
		UserManager::GetInstance()->addRectOnStream(userID,channelID,rect);

	return 0;
}

}


