﻿#include "src/inc/restclient/restclient.h"

#include <QNetworkRequest>
#include <QNetworkReply>
#include <QTime>
#include <QPair>
#include <QtConcurrent>
#include <QFuture>
#include <QThread>

RestClient::RestClient()
{	
	m_manager.reset(new QNetworkAccessManager);
}

void RestClient::setUrl(const QString &url)
{
	m_url=url;
}

void RestClient::addRequestParam(const QString &key,const QString &value)
{
	m_params.append(QPair<QString,QString>(key,value));
}

void RestClient::addRequestParam(const QString &key, const int &value)
{
	m_params.append(QPair<QString,QString>(key,QString::number(value)));
}

void RestClient::setRequestBody(const QByteArray &body)
{
	m_body=body;
}

int RestClient::sendRequest(const RequestType &requestType, QByteArray &response, const int &waitTime)
{
	QNetworkRequest request;
	request.setUrl(m_url);
	for(const QPair<QString,QString> &node:m_params)
		request.setRawHeader(node.first.toLocal8Bit(),node.second.toLocal8Bit());

	QNetworkReply *reply{nullptr};
	switch(requestType)
	{
	case RequestType_Get:
		reply=m_manager->get(request);
		break;
	case RequestType_Put:
		reply=m_manager->put(request,m_body);
		break;
	case RequestType_Post:
		reply=m_manager->post(request,m_body);
		break;
	default:
		break;
	}

	QTimer timer;
	QEventLoop eventLoop;
	connect(reply,&QNetworkReply::finished,&eventLoop,&QEventLoop::quit);
	connect(&timer,&QTimer::timeout,&eventLoop,&QEventLoop::quit);
	timer.start(waitTime);
	eventLoop.exec();

	if(true==timer.isActive())
	{
		timer.stop();

		int statusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
		switch(statusCode)
		{
			case 301:
			{
				QUrl newUrl = reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl();
				response=newUrl.toString().toLocal8Bit();
				break;
			}
			case 200:
				response = reply->readAll();
				break;
		}

		return statusCode;
	}
	else
	{
		reply->abort();
		return -1;
	}
}

void RestClient::clear()
{
	m_url.clear();
	m_params.clear();
	m_body.clear();
}
