﻿#include "src/inc/usermanager/userinfo.h"

UserManager::UserManager()
{

}

AIFaceErcode UserManager::addUser(const int &userID, const UserInfo &userInfo)
{
	if(true==m_userInfo.contains(userID))
		check_ercode(AFErcode_UserAlreadyExist,"UserID already exist!")

	m_userInfo.insert(userID,userInfo);

	return AFErcode_None;
}

AIFaceErcode UserManager::removeUser(const int &userID)
{
	if(false==m_userInfo.contains(userID))
		check_ercode(AFErcode_UserNotExist,"UserID not exist!")

	m_userInfo.remove(userID);

	return AFErcode_None;
}

AIFaceErcode UserManager::getHUser(const int &userID, HUSER &hUser)
{
	if(false==m_userInfo.contains(userID))
		check_ercode(AFErcode_UserNotExist,"UserID not exist!")

	hUser=m_userInfo[userID].hUser;

	return AFErcode_None;
}

AIFaceErcode UserManager::getMAC(const HANDLE &hUser, QString &mac)
{
	QMap<int,UserInfo>::iterator iter;
	for(iter=m_userInfo.begin();iter!=m_userInfo.end();++iter)
	{
		if(hUser==iter->hUser)
		{
			mac=iter->mac;
			return AFErcode_MACNotExist;
		}
	}

	return AFErcode_None;
}

AIFaceErcode UserManager::getDeviceUserID(const QString &mac,int &userID)
{
	QMap<int,UserInfo>::iterator iter;
	for(iter=m_userInfo.begin();iter!=m_userInfo.end();++iter)
	{
		if(mac==iter->mac)
		{
			userID=iter.key();
			return AFErcode_None;
		}
	}

	return AFErcode_MACNotExist;
}

AIFaceErcode UserManager::setRealStreamHandle(const int &userID, const int &channelID, const HANDLE &hStream)
{
	if(false==m_userInfo.contains(userID))
		check_ercode(AFErcode_UserNotExist,"UserID not exist!")

	m_userInfo[userID].channelRealStream[channelID]=hStream;

	return AFErcode_None;
}

AIFaceErcode UserManager::getRealStreamHandle(const int &userID, const int &channelID, HANDLE &hStream)
{
	if(false==m_userInfo.contains(userID))
		check_ercode(AFErcode_UserNotExist,"UserID not exist!")

	if(false==m_userInfo[userID].channelRealStream.contains(channelID))
		check_ercode(AFErcode_ChannelNotExist,"ChannelID not exist!")

	hStream=m_userInfo[userID].channelRealStream[channelID];

	return AFErcode_None;
}

AIFaceErcode UserManager::setRealStreamPort(const int &userID, const int &channelID, const int &port)
{
	if(false==m_userInfo.contains(userID))
		check_ercode(AFErcode_UserNotExist,"UserID not exist!")

	m_userInfo[userID].channelRealStreamPort[channelID]=port;

	return AFErcode_None;
}

AIFaceErcode UserManager::getRealStreamPort(const int &userID, const int &channelID, int &port)
{
	if(false==m_userInfo.contains(userID))
		check_ercode(AFErcode_UserNotExist,"UserID not exist!")

	if(false==m_userInfo[userID].channelRealStream.contains(channelID))
		check_ercode(AFErcode_ChannelNotExist,"ChannelID not exist!")

	port=m_userInfo[userID].channelRealStreamPort[channelID];

	return AFErcode_None;
}

AIFaceErcode UserManager::addRectOnStream(const int &userID, const int &channelID, const QRect &rect)
{
	if(false==m_userInfo.contains(userID))
		check_ercode(AFErcode_UserNotExist,"UserID not exist!")

	if(false==m_userInfo[userID].channelRealStream.contains(channelID))
		check_ercode(AFErcode_ChannelNotExist,"ChannelID not exist!")

	m_userInfo[userID].channelRealStreamRects[channelID].append(rect);

	return AFErcode_None;
}

AIFaceErcode UserManager::clearRectsOnStream(const int &userID, const int &channelID)
{
	if(false==m_userInfo.contains(userID))
		check_ercode(AFErcode_UserNotExist,"UserID not exist!")

	if(false==m_userInfo[userID].channelRealStream.contains(channelID))
		check_ercode(AFErcode_ChannelNotExist,"ChannelID not exist!")

	m_userInfo[userID].channelRealStreamRects[channelID].clear();

	return AFErcode_None;
}

AIFaceErcode UserManager::getRectsByPort(const int &port, QVector<QRect> &rects)
{
	//通过Port得到UserID和ChannelID
	int userID{-1};
	int channelID{-1};

	QMap<int,UserInfo>::iterator outIter;
	for(outIter=m_userInfo.begin();outIter!=m_userInfo.end();++outIter)
	{
		QMap<int,int>::iterator inIter;
		for(inIter=outIter->channelRealStreamPort.begin();inIter!=outIter->channelRealStreamPort.end();++inIter)
		{
			if(inIter.value()==port)
			{
				userID=outIter.key();
				channelID=inIter.key();

				break;
			}
		}
	}

	if(-1==userID&&-1==channelID)
		return AFErcode_PortNotExist;

	rects.append(m_userInfo[userID].channelRealStreamRects[channelID]);

	return AFErcode_None;
}
